@@ -316,21 +316,39 @@ namespace CBinding
 			return (exitCode == 0);
 		}
 
+		static readonly string[] libraryExtensions = { ".so", ".a", ".dll", ".dylib" };
+		/// <summary>
+		/// Checks whether a library can be linked with -lbasename
+		/// </summary>
+		/// <remarks>
+		/// This should return true iff directory is empty or in 
+		/// the configured library paths, and library is of the form blah
+		/// or libblah.(a|so|dll|dylib), 
+		/// </remarks>
 		internal bool IsStandardLibrary(CProjectConfiguration configuration,
 		                                string directory, string library,
 		                                ref string std_lib)
 		{
-			if((String.IsNullOrEmpty(directory) || configuration.LibPaths.Contains(directory)) == false)
+			std_lib = library;
+			
+			if(!(String.IsNullOrEmpty(directory) || 
+			    configuration.LibPaths.Contains(directory)))
 				return false;
-
-			if((library.Contains(".") == false || library.StartsWith("lib")) == false)
-				return false;
-
-			if(library.Contains(".") == false)
-				std_lib = library;
-			else
-				std_lib = library.Substring(3, library.IndexOf('.') - 3);
-
+				
+			string libraryExtension = Path.GetExtension (library);
+			
+			foreach (string extension in libraryExtensions)
+			{
+				if (libraryExtension.Equals (extension, StringComparison.OrdinalIgnoreCase)) {
+					if (library.StartsWith("lib", StringComparison.OrdinalIgnoreCase)) {
+						std_lib = std_lib.Substring(3);
+						return true;
+					} else {
+						return false;
+					}
+				}
+			}
+			
 			return true;
 		}
 		
@@ -384,6 +402,7 @@ namespace CBinding
 			
 			ParseCompilerOutput (errorOutput, cr);
 			ParseLinkerOutput (errorOutput, cr);
+			CheckReturnCode (exitCode, cr);
 		}
 		
 		private void MakeStaticLibrary (Project project,
@@ -408,6 +427,7 @@ namespace CBinding
 			
 			ParseCompilerOutput (errorOutput, cr);
 			ParseLinkerOutput (errorOutput, cr);
+			CheckReturnCode (exitCode, cr);
 		}
 		
 		private void MakeSharedLibrary(Project project,
@@ -460,6 +480,7 @@ namespace CBinding
 			
 			ParseCompilerOutput (errorOutput, cr);
 			ParseLinkerOutput (errorOutput, cr);
+			CheckReturnCode (exitCode, cr);
 		}
 		
 		int ExecuteCommand (string command, string args, string baseDirectory, IProgressMonitor monitor, out string errorOutput)
@@ -540,6 +561,7 @@ namespace CBinding
 			int exitCode = ExecuteCommand ((use_ccache ? "ccache" : compilerCommand), compiler_args, configuration.OutputDirectory, monitor, out errorOutput);
 			
 			ParseCompilerOutput (errorOutput, cr);
+			CheckReturnCode (exitCode, cr);
 			return exitCode == 0;
 		}
 		
@@ -653,28 +675,29 @@ namespace CBinding
 		private CompilerError CreateErrorFromErrorString (string errorString, TextReader reader)
 		{
 			CompilerError error = new CompilerError ();
+			string warning = GettextCatalog.GetString("warning");
 			
-			Match withColMatch = withColRegex.Match (errorString);
+			Match match = withColRegex.Match (errorString);
 			
-			if (withColMatch.Success)
+			if (match.Success)
 			{
-				error.FileName = withColMatch.Groups["file"].Value;
-				error.Line = int.Parse (withColMatch.Groups["line"].Value);
-				error.Column = int.Parse (withColMatch.Groups["column"].Value);
-				error.IsWarning = withColMatch.Groups["level"].Value.Equals ("warning");
-				error.ErrorText = withColMatch.Groups["message"].Value;
+				error.FileName = match.Groups["file"].Value;
+				error.Line = int.Parse (match.Groups["line"].Value);
+				error.Column = int.Parse (match.Groups["column"].Value);
+				error.IsWarning = match.Groups["level"].Value.Equals (warning, StringComparison.Ordinal);
+				error.ErrorText = match.Groups["message"].Value;
 				
 				return error;
 			}
 			
-			Match noColMatch = noColRegex.Match (errorString);
+			match = noColRegex.Match (errorString);
 			
-			if (noColMatch.Success)
+			if (match.Success)
 			{
-				error.FileName = noColMatch.Groups["file"].Value;
-				error.Line = int.Parse (noColMatch.Groups["line"].Value);
-				error.IsWarning = noColMatch.Groups["level"].Value.Equals ("warning");
-				error.ErrorText = noColMatch.Groups["message"].Value;
+				error.FileName = match.Groups["file"].Value;
+				error.Line = int.Parse (match.Groups["line"].Value);
+				error.IsWarning = match.Groups["level"].Value.Equals (warning, StringComparison.Ordinal);
+				error.ErrorText = match.Groups["message"].Value;
 				
 				// Multi-line error message? attempt to parse it into a single error.
 				if (error.ErrorText.StartsWith ("(") && !error.ErrorText.EndsWith (")"))
@@ -760,5 +783,25 @@ namespace CBinding
 				return false;
 			}
 		}
+		
+		/// <summary>
+		/// Checks a compilation return code, 
+		/// and adds an error result if the compiler results
+		/// show no errors.
+		/// </summary>
+		/// <param name="returnCode">
+		/// A <see cref="System.Int32"/>: A process return code
+		/// </param>
+		/// <param name="cr">
+		/// A <see cref="CompilerResults"/>: The return code from a compilation run
+		/// </param>
+		void CheckReturnCode (int returnCode, CompilerResults cr)
+		{
+			cr.NativeCompilerReturnValue = returnCode;
+			if (0 != returnCode && 0 == cr.Errors.Count) { 
+				cr.Errors.Add (new CompilerError (string.Empty, 0, 0, string.Empty,
+				                                  GettextCatalog.GetString ("Build failed - check build output for details")));
+			}
+		}
 	}
 }
