@@ -78,7 +78,7 @@ namespace CBinding
 		// -1 means the cursor is outside the method parameter list
 		// 0 means no parameter entered
 		// > 0 is the index of the parameter (1-based)
-		public int GetCurrentParameterIndex (ICodeCompletionContext ctx)
+		public int GetCurrentParameterIndex (CodeCompletionContext ctx)
 		{
 			int cursor = editor.CursorPosition;
 			int i = ctx.TriggerOffset;
@@ -93,6 +93,8 @@ namespace CBinding
 			int parameterIndex = 1;
 			
 			while (i++ < cursor) {
+				if (i >= editor.TextLength)
+					break;
 				char ch = editor.GetCharAt (i);
 				if (ch == ',')
 					parameterIndex++;
@@ -105,7 +107,7 @@ namespace CBinding
 		
 		// Returns the markup to use to represent the specified method overload
 		// in the parameter information window.
-		public string GetMethodMarkup (int overload, string[] parameterMarkup)
+		public string GetMethodMarkup (int overload, string[] parameterMarkup, int currentParameter)
 		{
 			Function function = functions[overload];
 			string paramTxt = string.Join (", ", parameterMarkup);
@@ -114,7 +116,7 @@ namespace CBinding
 			string prename = null;
 			
 			if (len > 0)
-				prename = function.FullName.Substring (0, len + 2);
+				prename = GLib.Markup.EscapeText (function.FullName.Substring (0, len + 2));
 			
 			string cons = string.Empty;
 			
@@ -129,7 +131,7 @@ namespace CBinding
 		{
 			Function function = functions[overload];
 			
-			return function.Parameters[paramIndex];
+			return GLib.Markup.EscapeText (function.Parameters[paramIndex]);
 		}
 		
 		// Returns the number of parameters of the specified method
