@@ -10,7 +10,7 @@ namespace MonoDevelop.Deployment.Linux
 {
 	public class DotDesktopDisplayBinding: IDisplayBinding
 	{
-		public bool CanCreateContentForFile (string fileName)
+		public bool CanCreateContentForUri (string fileName)
 		{
 			return fileName.EndsWith (".desktop");
 		}
@@ -20,7 +20,7 @@ namespace MonoDevelop.Deployment.Linux
 			return mimetype == "application/x-desktop";
 		}
 
-		public IViewContent CreateContentForFile (string fileName)
+		public IViewContent CreateContentForUri (string fileName)
 		{
 			return new DotDesktopView ();
 		}
@@ -30,7 +30,7 @@ namespace MonoDevelop.Deployment.Linux
 			return new DotDesktopView ();
 		}
 
-		public string DisplayName {
+		public string Name {
 			get { return "Desktop Entry"; }
 		}
 	}
