@@ -5,6 +5,7 @@ using System.Collections.Specialized;
 using System.Xml;
 using Gtk;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui;
 
 namespace MonoDevelop.Deployment.Linux
@@ -130,7 +131,7 @@ namespace MonoDevelop.Deployment.Linux
 			sortedmt.Sort ();
 			
 			foreach (string mt in sortedmt) {
-				string desc = IdeApp.Services.PlatformService.GetMimeTypeDescription (mt);
+				string desc = DesktopService.GetMimeTypeDescription (mt);
 				storeMimeTypes.AppendValues (mt, desc);
 			}
 			
@@ -227,6 +228,7 @@ namespace MonoDevelop.Deployment.Linux
 		protected virtual void OnButtonAddCategoriesClicked(object sender, System.EventArgs e)
 		{
 			MenuCategorySelectorDialog dlg = new MenuCategorySelectorDialog ();
+			dlg.TransientFor = IdeApp.Workbench.RootWindow;
 			if (dlg.Run () == (int) Gtk.ResponseType.Ok) {
 				foreach (string s in dlg.Selection)
 					entry.Categories.Add (s);
@@ -302,7 +304,7 @@ namespace MonoDevelop.Deployment.Linux
 					if (!entry.MimeTypes.Contains (mt)) {
 						entry.MimeTypes.Add (mt);
 						storeMimeTypes.SetValue (iter, 0, mt);
-						storeMimeTypes.SetValue (iter, 1, IdeApp.Services.PlatformService.GetMimeTypeDescription (mt));
+						storeMimeTypes.SetValue (iter, 1, DesktopService.GetMimeTypeDescription (mt));
 						
 						if (!string.IsNullOrEmpty (oldmt))
 							// It is a modification. Remove the old name.
