@@ -7,7 +7,7 @@ using Gtk;
 
 namespace MonoDevelop.Deployment.Linux
 {
-	public class LinuxIntegrationProjectFeature: ICombineEntryFeature
+	public class LinuxIntegrationProjectFeature: ISolutionItemFeature
 	{
 		public string Title {
 			get { return GettextCatalog.GetString ("Unix Integration"); }
@@ -17,27 +17,27 @@ namespace MonoDevelop.Deployment.Linux
 			get { return GettextCatalog.GetString ("Set options for generating files to better integrate the application or library in a Unix system."); }
 		}
 
-		public bool SupportsCombineEntry (Combine parentCombine, CombineEntry entry)
+		public bool SupportsSolutionItem (SolutionFolder parentCombine, SolutionItem entry)
 		{
 			return entry is DotNetProject;
 		}
 		
-		public Widget CreateFeatureEditor (Combine parentCombine, CombineEntry entry)
+		public Widget CreateFeatureEditor (SolutionFolder parentCombine, SolutionItem entry)
 		{
 			return new BasicOptionPanelWidget ((DotNetProject) entry, true);
 		}
 
-		public void ApplyFeature (Combine parentCombine, CombineEntry entry, Widget editor)
+		public void ApplyFeature (SolutionFolder parentCombine, SolutionItem entry, Widget editor)
 		{
 			((BasicOptionPanelWidget)editor).Store ();
 		}
 		
-		public string Validate (Combine parentCombine, CombineEntry entry, Gtk.Widget editor)
+		public string Validate (SolutionFolder parentCombine, SolutionItem entry, Gtk.Widget editor)
 		{
 			return ((BasicOptionPanelWidget)editor).Validate ();
 		}
 		
-		public bool IsEnabled (Combine parentCombine, CombineEntry entry) 
+		public bool IsEnabled (SolutionFolder parentCombine, SolutionItem entry) 
 		{
 			return false;
 		}
