@@ -59,13 +59,9 @@ namespace MonoDevelop.Deployment.Gui
 			
 			//IFileReplacePolicy is not likely to be running in the GUI thread
 			//so use some DispatchService magic to synchronously call the dialog in the GUI thread
-			DialogDelegate del = delegate {
-				return MessageService.ShowCustomDialog (new FileReplaceDialog (response, source, sourceModified.ToString (), target, targetModified.ToString ()));
-			};
-			if (!DispatchService.IsGuiThread)
-				response = (FileReplaceDialog.ReplaceResponse) DispatchService.GuiDispatch (del).DynamicInvoke ();
-			else
-				response = (FileReplaceDialog.ReplaceResponse) del.DynamicInvoke ();
+			DispatchService.GuiSyncDispatch (delegate {
+				response = (FileReplaceDialog.ReplaceResponse) MessageService.ShowCustomDialog (new FileReplaceDialog (response, source, sourceModified.ToString (), target, targetModified.ToString ()));
+			});
 			
 			switch (response) {
 			case FileReplaceDialog.ReplaceResponse.Replace:
