@@ -7,7 +7,7 @@ using Gtk;
 
 namespace MonoDevelop.Deployment.Gui
 {
-	internal class PackagingFeature: ICombineEntryFeature
+	internal class PackagingFeature: ISolutionItemFeature
 	{
 		public string Title {
 			get { return GettextCatalog.GetString ("Packaging"); }
@@ -17,27 +17,27 @@ namespace MonoDevelop.Deployment.Gui
 			get { return GettextCatalog.GetString ("Add a Packaging Project to the solution for generating different kinds of packages for the new project."); }
 		}
 
-		public bool SupportsCombineEntry (Combine parentCombine, CombineEntry entry)
+		public bool SupportsSolutionItem (SolutionFolder parentCombine, SolutionItem entry)
 		{
 			return ((entry is Project) || (entry is PackagingProject)) && parentCombine != null;
 		}
 		
-		public Widget CreateFeatureEditor (Combine parentCombine, CombineEntry entry)
+		public Widget CreateFeatureEditor (SolutionFolder parentCombine, SolutionItem entry)
 		{
 			return new PackagingFeatureWidget (parentCombine, entry);
 		}
 
-		public void ApplyFeature (Combine parentCombine, CombineEntry entry, Widget editor)
+		public void ApplyFeature (SolutionFolder parentCombine, SolutionItem entry, Widget editor)
 		{
 			((PackagingFeatureWidget)editor).ApplyFeature ();
 		}
 		
-		public string Validate (Combine parentCombine, CombineEntry entry, Gtk.Widget editor)
+		public string Validate (SolutionFolder parentCombine, SolutionItem entry, Gtk.Widget editor)
 		{
 			return null;
 		}
 		
-		public bool IsEnabled (Combine parentCombine, CombineEntry entry) 
+		public bool IsEnabled (SolutionFolder parentCombine, SolutionItem entry) 
 		{
 			return entry is PackagingProject;
 		}
