@@ -28,6 +28,7 @@ using Mono.Addins;
 using MonoDevelop.Projects;
 using MonoDevelop.Deployment;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Assemblies;
 
 namespace MonoDevelop.Autotools
 {
@@ -59,14 +60,25 @@ namespace MonoDevelop.Autotools
 		public MakefileType MakefileType {
 			get { return makefileType; }
 		}
-		
-		string base_dir;
-		public string BaseDirectory {
+
+		FilePath base_dir;
+		public FilePath BaseDirectory {
 			get {
 				return base_dir;
 			}
 		}
 		
+		IList<Switch> switches;
+		public IList<Switch> Switches {
+			get {
+				return switches;
+			} 
+			set {
+				switches = value;
+			}
+		}
+		
+		
 		string libdir = "lib";
 		public string LibraryDirectory {
 			get {
