@@ -35,11 +35,13 @@ using System.Text;
 using System.Text.RegularExpressions;
 
 using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Ide.Gui;
-using MonoDevelop.Projects.Serialization;
+using MonoDevelop.Core.Serialization;
 using MonoDevelop.Projects;
+using MonoDevelop.Core.Assemblies;
 
 namespace MonoDevelop.Autotools
 {
@@ -55,8 +57,10 @@ namespace MonoDevelop.Autotools
 		CustomMakefile makefile;
 		
 		[ItemProperty ("ExcludedFiles")]
-		[ProjectPathItemProperty("File", Scope=1)]
+		[ProjectPathItemProperty("File", Scope="*")]
 		List<string> excludedFiles = new List<string> ();
+		
+		IAssemblyContext assemblyContext;
 
 		public MakefileData ()
 		{
@@ -64,6 +68,21 @@ namespace MonoDevelop.Autotools
 			buildTargetName = "all";
 			cleanTargetName = "clean";
 			executeTargetName = String.Empty;
+			
+			assemblyContext = GetMonoRuntimeContext ();
+			if (assemblyContext == null)
+				integrationEnabled = false;
+		}
+		
+		internal static IAssemblyContext GetMonoRuntimeContext ()
+		{
+			if (Runtime.SystemAssemblyService.CurrentRuntime.RuntimeId == "Mono")
+				return Runtime.SystemAssemblyService.CurrentRuntime.AssemblyContext;
+			else {
+				foreach (TargetRuntime r in Runtime.SystemAssemblyService.GetTargetRuntimes ("Mono"))
+					return r.AssemblyContext;
+			}
+			return null;
 		}
 
 		public Project OwnerProject {
@@ -76,22 +95,24 @@ namespace MonoDevelop.Autotools
 							dirty = true;
 					};
 					
-					value.ReferenceRemovedFromProject += refhandler;
-					value.ReferenceAddedToProject += refhandler;
+					if (value is DotNetProject) {
+						((DotNetProject)value).ReferenceRemovedFromProject += refhandler;
+						((DotNetProject)value).ReferenceAddedToProject += refhandler;
+					}
 
 					ProjectFileEventHandler filehandler = delegate (object sender, ProjectFileEventArgs e) {
 						MakefileVar var = null;
 						switch (e.ProjectFile.BuildAction) {
-						case BuildAction.Compile:
+						case "Compile":
 							var = BuildFilesVar;
 							break;
-						case BuildAction.FileCopy:
+						case "Content":
 							var = DeployFilesVar;
 							break;
-						case BuildAction.EmbedAsResource:
+						case "EmbeddedResource":
 							var = ResourcesVar;
 							break;
-						case BuildAction.Nothing:
+						case "None":
 							var = OthersVar;
 							break;
 						}
@@ -120,6 +141,10 @@ namespace MonoDevelop.Autotools
 			get { return integrationEnabled; }
 			set { integrationEnabled = value;}
 		}
+
+		public bool SupportsIntegration {
+			get { return IntegrationEnabled && !PropertyService.IsWindows; }
+		}
 		
 		[ItemProperty (DefaultValue = "")]
 		public string RelativeMakefileName {
@@ -415,14 +440,14 @@ namespace MonoDevelop.Autotools
 			}
 		}
 		
-		public bool IsFileIntegrationEnabled (BuildAction fileType)
+		public bool IsFileIntegrationEnabled (string buildAction)
 		{
 			if (IntegrationEnabled) {
-				if (fileType == BuildAction.Compile)
+				if (buildAction == BuildAction.Compile)
 					return BuildFilesVar.Sync;
-				else if (fileType == BuildAction.EmbedAsResource)
+				else if (buildAction == BuildAction.EmbeddedResource)
 					return ResourcesVar.Sync;
-				else if (fileType == BuildAction.FileCopy)
+				else if (buildAction == BuildAction.Content)
 					return DeployFilesVar.Sync;
 				else
 					return OthersVar.Sync;
@@ -491,6 +516,16 @@ namespace MonoDevelop.Autotools
 				@"^\s*(?<file>[^:]*):(?<line>\d*){1}(:(?<column>\d*))?\s*:\s*error\s*:\s(?<message>.*)",
 				@"^\s*(?<file>[^:]*):(?<line>\d*){1}(:(?<column>\d*))?\s*:\s*warning\s*:\s(?<message>.*)"
 			};
+			
+			compilerMessageRegex ["Boo"] = new string [] {
+				@"(^\s*(?<file>.*)\((?<line>\d*){1}(,(?<column>\d*[\+]*))?\)(:|)\s+)*(?=BCE)(?<number>.*?):\s(?<message>.*)",
+				@"(^\s*(?<file>.*)\((?<line>\d*){1}(,(?<column>\d*[\+]*))?\)(:|)\s+)*(?=BCW)(?<number>.*?):\sWARNING:\s(?<message>.*)"
+			};
+
+			compilerMessageRegex ["Vala"] = new string [] {
+				@"((^\s*(?<file>.*):(?<line>\d*)\.(?<column>\d*)-\d*\.\d*: error: (?<message>.*))|(^\s*(?<file>[^:]*):(?<line>\d*){1}(:(?<column>\d*))?\s*:\s*error\s*:\s(?<message>.*)))",
+				@"((^\s*(?<file>.*):(?<line>\d*)\.(?<column>\d*)-\d*\.\d*: warning: (?<message>.*))|(^\s*(?<file>[^:]*):(?<line>\d*){1}(:(?<column>\d*))?\s*:\s*warning\s*:\s(?<message>.*)))"
+			};
 		}
 
 		void InitBuildVars ()
@@ -630,12 +665,11 @@ namespace MonoDevelop.Autotools
 
 			//FIXME: Improve the message
 			if (promptForRemoval) {
-				int choice = IdeApp.Services.MessageService.ShowCustomDialog (
-						GettextCatalog.GetString ("Enable Makefile integration"),
-						GettextCatalog.GetString ("Enabling Makefile integration. You can choose to have either the Project or the Makefile be used as the master copy. This is done only when enabling this feature. After this, the Makefile will be taken as the master copy."),
-						"Project", "Makefile");
-
-				if (choice == 0) {
+				AlertButton projectButton = new AlertButton ("_Project");
+				AlertButton makefileButton = new AlertButton ("_Makefile");
+				AlertButton choice = MessageService.AskQuestion (GettextCatalog.GetString ("Enabling Makefile integration. You can choose to have either the Project or the Makefile be used as the master copy. This is done only when enabling this feature. After this, the Makefile will be taken as the master copy."),
+				                                                 projectButton, makefileButton);
+				if (choice == projectButton) {
 					//Sync Project --> Makefile
 					dirty = true;
 					return;
@@ -676,14 +710,14 @@ namespace MonoDevelop.Autotools
 					AbsoluteConfigureInPath, OwnerProject.Name, e.ToString ());
 
 				monitor.ReportWarning (GettextCatalog.GetString (
-					"Error trying to read configure.in ('{0'}) for project '{1}':\n{2}",
+					"Error trying to read configure.in ('{0}') for project '{1}':\n{2}",
 					AbsoluteConfigureInPath, OwnerProject.Name, e.Message));
 			}
 
 			ReadFiles (BuildFilesVar, BuildAction.Compile, "Build", promptForRemoval);
-			ReadFiles (DeployFilesVar, BuildAction.FileCopy, "Deploy", promptForRemoval);
-			ReadFiles (OthersVar, BuildAction.Nothing, "Others", promptForRemoval);
-			ReadFiles (ResourcesVar, BuildAction.EmbedAsResource, "Resources", promptForRemoval);
+			ReadFiles (DeployFilesVar, BuildAction.Content, "Deploy", promptForRemoval);
+			ReadFiles (OthersVar, BuildAction.None, "Others", promptForRemoval);
+			ReadFiles (ResourcesVar, BuildAction.EmbeddedResource, "Resources", promptForRemoval);
 
 			if (!SyncReferences)
 				return;
@@ -699,15 +733,23 @@ namespace MonoDevelop.Autotools
 					ProjectRefVar.Extra.Clear ();
 
 					existingGacRefs = new Dictionary<string, ProjectReference> ();
+					requiredPackageVersions = new Dictionary<string,string> ();
 					newGacRefs = new Dictionary<string, ProjectReference> ();
 
 					List<ProjectReference> toRemove = new List<ProjectReference> ();
-					foreach (ProjectReference pref in OwnerProject.ProjectReferences) {
+					foreach (ProjectReference pref in dotnetProject.References) {
 						if (pref.ReferenceType == ReferenceType.Gac) {
-							string [] files = pref.GetReferencedFileNames ();
+							string [] files = pref.GetReferencedFileNames (ConfigurationSelector.Default);
 							if (files == null)
 								continue;
 
+							if (pref.ReferenceType == ReferenceType.Gac) {
+								// Store the package version required by this reference. We'll use
+								// the same version when trying to match references coming from the makefile
+								SystemAssembly asm = assemblyContext.GetAssemblyFromFullName (pref.StoredReference, pref.Package != null ? pref.Package.Name : null, dotnetProject.TargetFramework);
+								if (asm != null && asm.Package != null)
+									requiredPackageVersions [asm.Package.Name] = asm.Package.Version;
+							}
 							// this should help normalize paths like /foo//bar/../
 							string fullpath = Path.GetFullPath (files [0]);
 							if (existingGacRefs.ContainsKey (fullpath))
@@ -719,7 +761,7 @@ namespace MonoDevelop.Autotools
 
 					// Remove the repeats
 					foreach (ProjectReference pref in toRemove)
-						OwnerProject.ProjectReferences.Remove (pref);
+						dotnetProject.References.Remove (pref);
 
 					ReadReferences (GacRefVar, ReferenceType.Gac, "Gac References", dotnetProject);
 
@@ -732,11 +774,11 @@ namespace MonoDevelop.Autotools
 					
 					//Resolve References
 					//Required when UpdateProject gets called by ui
-					if (ownerProject.RootCombine != null)
-						ResolveProjectReferences (ownerProject.RootCombine, monitor);
+					if (ownerProject.ParentSolution != null)
+						ResolveProjectReferences (ownerProject.ParentSolution.RootFolder, monitor);
 
 					foreach (ProjectReference pr in existingGacRefs.Values)
-						ownerProject.ProjectReferences.Remove (pr);
+						dotnetProject.References.Remove (pr);
 
 					existingGacRefs.Clear ();
 					newGacRefs.Clear ();
@@ -753,7 +795,7 @@ namespace MonoDevelop.Autotools
 			this.monitor = null;
 		}
 
-		void ReadFiles (MakefileVar fileVar, BuildAction buildAction, string id, bool promptForRemoval)
+		void ReadFiles (MakefileVar fileVar, string buildAction, string id, bool promptForRemoval)
 		{
 			try { 
 				fileVar.SaveEnabled = true;
@@ -766,7 +808,7 @@ namespace MonoDevelop.Autotools
 			}
 		}
 
-		void ReadFilesActual (MakefileVar fileVar, BuildAction buildAction, string id, bool promptForRemoval)
+		void ReadFilesActual (MakefileVar fileVar, string buildAction, string id, bool promptForRemoval)
 		{
 			fileVar.Extra.Clear ();
 			if (!fileVar.Sync || String.IsNullOrEmpty (fileVar.Name))
@@ -793,7 +835,7 @@ namespace MonoDevelop.Autotools
 				len = fileVar.Prefix.Length;
 
 			Dictionary<string, ProjectFile> existingFiles = new Dictionary<string, ProjectFile> ();
-			foreach (ProjectFile pf in ownerProject.ProjectFiles) {
+			foreach (ProjectFile pf in ownerProject.Files) {
 				if (pf.BuildAction == buildAction)
 					existingFiles [ownerProject.GetAbsoluteChildPath (pf.FilePath)] = pf;
 			}
@@ -811,10 +853,10 @@ namespace MonoDevelop.Autotools
 						//FIXME: If it doesn't match, then?
 						fname = fname.Substring (len);
 
-					fname = UnescapeString (fname);
+					fname = FromMakefilePath (fname);
 
 					string resourceId = null;
-					if (buildAction == BuildAction.EmbedAsResource && fname.IndexOf (',') >= 0) {
+					if (buildAction == BuildAction.EmbeddedResource && fname.IndexOf (',') >= 0) {
 						string [] tmp = fname.Split (new char [] {','}, 2);
 						fname = tmp [0];
 						if (tmp.Length > 1)
@@ -859,7 +901,7 @@ namespace MonoDevelop.Autotools
 					}
 
 					ProjectFile pf = ownerProject.AddFile (absPath, buildAction);
-					if (buildAction == BuildAction.EmbedAsResource && resourceId != null)
+					if (buildAction == BuildAction.EmbeddedResource && resourceId != null)
 						pf.ResourceId = resourceId;
 				} catch (Exception e) {
 					LoggingService.LogError (e.ToString ());
@@ -870,13 +912,14 @@ namespace MonoDevelop.Autotools
 			if (existingFiles.Count > 0) {
 				foreach (ProjectFile file in existingFiles.Values) {
 					if (!IsFileExcluded (file.FilePath))
-						ownerProject.ProjectFiles.Remove (file);
+						ownerProject.Files.Remove (file);
 				}
 			}
 		}
 
 		Dictionary<string, ProjectReference> existingGacRefs = null;
 		Dictionary<string, ProjectReference> newGacRefs = null;
+		Dictionary<string, string> requiredPackageVersions = null;
 
 		void ReadReferences (MakefileVar refVar, ReferenceType refType, string id, DotNetProject project)
 		{
@@ -946,7 +989,7 @@ namespace MonoDevelop.Autotools
 				}
 
 				string pkgVarName = rname.Substring (2, rname.Length - 3).Replace ("_LIBS", String.Empty);
-				List<string> pkgNames = ConfiguredPackages.GetNamesFromVarName (pkgVarName);
+				List<PackageContent> pkgNames = ConfiguredPackages.GetPackageContentFromVarName (pkgVarName);
 				if (pkgNames == null) {
 					 LoggingService.LogWarning  ("Package named '{0}' not found in configure.in. Ignoring reference to '{1}'.",
 						pkgVarName, rname);
@@ -955,16 +998,18 @@ namespace MonoDevelop.Autotools
 				}
 
 				bool added = false;
-				foreach (string pkgName in pkgNames) {
-					if (ReferencedPackages.Contains (pkgName)) {
+				foreach (PackageContent packageContent in pkgNames) {
+					if (ReferencedPackages.Contains (packageContent.Name)) {
 						added = true;
 						continue;
 					}
 
 					// Add all successfully added packages to ReferencedPackages
-					if (LoadPackageReference (pkgName, project, refVar.Prefix)) {
-						ReferencedPackages.Add (pkgName);
-						added = true;
+					foreach (string referencedName in packageContent.AllReferencedNames) {
+						if (LoadPackageReference (referencedName, project, refVar.Prefix)) {
+							ReferencedPackages.Add (referencedName);
+							added = true;
+						}
 					}
 				}
 
@@ -1018,18 +1063,24 @@ namespace MonoDevelop.Autotools
 				if (TryGetExistingGacRef (fullpath) != null)
 					continue;
 
-				// Check that its a valid assembly
-				string fullname = null;
-				try {
-					fullname = AssemblyName.GetAssemblyName (fullpath).FullName;
-				} catch (FileNotFoundException) {
-				} catch (BadImageFormatException) {
-				}
+				if (refname.IndexOf (Path.DirectorySeparatorChar) < 0) {
+					// Check that its a valid assembly
+					string fullname = null;
+					try {
+						fullname = AssemblyName.GetAssemblyName (fullpath).FullName;
+					} catch (FileNotFoundException) {
+					} catch (BadImageFormatException) {
+					}
 
-				// Valid assembly, From a package, add as Gac
-				if (fullname != null && Runtime.SystemAssemblyService.GetPackageFromPath (fullpath) != null) {
-					AddNewGacReference (project, fullname, fullpath);
-					continue;
+					// Valid assembly, From a package, add as Gac
+					SystemPackage pkg = assemblyContext.GetPackageFromPath (fullpath);
+					if (fullname != null && pkg != null) {
+						SystemAssembly sa = assemblyContext.GetAssemblyFromFullName (fullname, pkg.Name, project.TargetFramework);
+						if (sa != null) {
+							AddNewGacReference (project, sa);
+							continue;
+						}
+					}
 				}
 
 				//Else add to unresolved project refs, avoid repeats
@@ -1040,52 +1091,29 @@ namespace MonoDevelop.Autotools
 
 		bool LoadPackageReference (string pkgName, DotNetProject project, string prefix)
 		{
-			SystemPackage pkg = Runtime.SystemAssemblyService.GetPackage (pkgName);
+			SystemPackage pkg = null;
+			string packageVersion;
+			if (requiredPackageVersions.TryGetValue (pkgName, out packageVersion))
+				pkg = assemblyContext.GetPackage (pkgName, packageVersion);
+			if (pkg == null)
+				pkg = assemblyContext.GetPackage (pkgName);
+			
 			if (pkg == null) {
 				LoggingService.LogWarning ("No package named '{0}' found. Ignoring.", pkgName);
 				return false;
 			}
 
-			foreach (string s in pkg.Assemblies) {
-				try {
-					string fullpath = Path.GetFullPath (s);
-					if (TryGetExistingGacRef (fullpath) != null)
-						continue;
+			foreach (SystemAssembly sa in pkg.Assemblies) {
+				if (TryGetExistingGacRef (sa.Location) != null)
+					continue;
 
-					//Get fullname of the assembly
-					string fullname = AssemblyName.GetAssemblyName (s).FullName;
-					AddNewGacReference (project, fullname, fullpath);
-				} catch {
-					//Ignore
-					LoggingService.LogWarning ("Invalid assembly reference ({0}) found in package {1}. Ignoring.",
-						s, pkg.Name);
-				}
+				//Get fullname of the assembly
+				AddNewGacReference (project, sa);
 			}
 
 			return true;
 		}
 
-		// maps "System" -> full name
-		// for assemblies from the GAC (SystemPackage.IsCorePackage)
-		// GetAssemblyNameForVersion should be used to get the exact assembly name
-		// for a ClrVersion
-		static Dictionary<string, string> corePackageAssemblyNames;
-		public static Dictionary<string, string> CorePackageAssemblyNames {
-			get {
-				if (corePackageAssemblyNames == null) {
-					corePackageAssemblyNames = new Dictionary<string, string> ();
-					foreach (string asm in Runtime.SystemAssemblyService.GetAssemblyFullNames ()) {
-						SystemPackage pkg = Runtime.SystemAssemblyService.GetPackageFromFullName (asm);
-						if (pkg != null && pkg.IsCorePackage)
-							//asm.IndexOf(,) should be > 0 as this is a fullname
-							corePackageAssemblyNames [asm.Substring (0, asm.IndexOf (',')).Trim ()] = asm;
-					}
-				}
-
-				return corePackageAssemblyNames;
-			}
-		}
-
 		ProjectReference ParseReferenceAsGac (string rname, DotNetProject project)
 		{
 			string aname = rname;
@@ -1093,26 +1121,19 @@ namespace MonoDevelop.Autotools
 				//-r:Mono.Posix.dll
 				aname = rname.Substring (0, rname.Length - 4);
 
-			string fullname;
-			if (!CorePackageAssemblyNames.TryGetValue (aname, out fullname)) {
-				fullname = Runtime.SystemAssemblyService.GetAssemblyFullName (aname);
-
-				//fullname can be null
-				CorePackageAssemblyNames [aname] = fullname;
-			}
+			string fullname = assemblyContext.GetAssemblyFullName (aname, project.TargetFramework);
 			if (fullname == null)
 				return null;
 
-			fullname = Runtime.SystemAssemblyService.GetAssemblyNameForVersion (fullname, project.ClrVersion);
-			if (fullname == null)
+			SystemAssembly asm = assemblyContext.GetAssemblyForVersion (fullname, null, project.TargetFramework);
+			if (asm == null)
 				return null;
 
-			string fullpath = Runtime.SystemAssemblyService.GetAssemblyLocation (fullname);
-			ProjectReference pref = TryGetExistingGacRef (fullpath);
+			ProjectReference pref = TryGetExistingGacRef (asm.Location);
 			if (pref != null)
 				return pref;
 
-			return AddNewGacReference (project, fullname, fullpath);
+			return AddNewGacReference (project, asm);
 		}
 
 		ProjectReference TryGetExistingGacRef (string fullpath)
@@ -1131,26 +1152,26 @@ namespace MonoDevelop.Autotools
 			return null;
 		}
 
-		ProjectReference AddNewGacReference (Project project, string fullname, string fullpath)
+		ProjectReference AddNewGacReference (DotNetProject project, SystemAssembly sa)
 		{
-			ProjectReference pref = new ProjectReference (ReferenceType.Gac, fullname);
-			project.ProjectReferences.Add (pref);
-			newGacRefs [fullpath] = pref;
+			ProjectReference pref = new ProjectReference (sa);
+			project.References.Add (pref);
+			newGacRefs [sa.Location] = pref;
 
 			return pref;
 		}
 
-		public static void ResolveProjectReferences (Combine c, IProgressMonitor monitor)
+		public static void ResolveProjectReferences (SolutionFolder folder, IProgressMonitor monitor)
 		{
-			Dictionary<string, Project> projects = new Dictionary<string, Project> ();
-			foreach (Project p in c.GetAllProjects ()) {
-				string filename = p.GetOutputFileName ();
+			Dictionary<string, DotNetProject> projects = new Dictionary<string, DotNetProject> ();
+			foreach (DotNetProject p in folder.GetAllItems<DotNetProject> ()) {
+				string filename = p.GetOutputFileName (ConfigurationSelector.Default);
 				// Can be null for Generic projects
 				if (!String.IsNullOrEmpty (filename))
 					projects [filename] = p;
 			}
 
-			foreach (Project sproj in projects.Values) {
+			foreach (DotNetProject sproj in projects.Values) {
 				MakefileData mdata = sproj.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
 				if (mdata == null)
 					continue;
@@ -1160,12 +1181,12 @@ namespace MonoDevelop.Autotools
 					Dictionary<string, ProjectReference> asmProjectRefs =
 						new Dictionary<string, ProjectReference> ();
 
-					foreach (ProjectReference pr in sproj.ProjectReferences) {
+					foreach (ProjectReference pr in sproj.References) {
 						if (pr.ReferenceType != ReferenceType.Assembly &&
 							pr.ReferenceType != ReferenceType.Project)
 							continue;
 						
-						string [] files = pr.GetReferencedFileNames ();
+						string [] files = pr.GetReferencedFileNames (ConfigurationSelector.Default);
 						if (files.Length > 0)
 							asmProjectRefs [files [0]] = pr;
 					}
@@ -1179,7 +1200,7 @@ namespace MonoDevelop.Autotools
 						} else {
 							// Try as a project ref
 							if (projects.ContainsKey (refstr)) {
-								sproj.ProjectReferences.Add (new ProjectReference (projects [refstr]));
+								sproj.References.Add (new ProjectReference (projects [refstr]));
 								toRemove.Add (refstr);
 							}
 						}
@@ -1190,11 +1211,11 @@ namespace MonoDevelop.Autotools
 
 					// Add all remaining unresolved refs as Assembly refs
 					foreach (string s in mdata.UnresolvedReferences.Keys)
-						sproj.ProjectReferences.Add (new ProjectReference (ReferenceType.Assembly, s));
+						sproj.References.Add (new ProjectReference (ReferenceType.Assembly, s));
 						
 					// Remove asm/project refs not found in UnresolvedReferences
 					foreach (ProjectReference pr in asmProjectRefs.Values)
-						sproj.ProjectReferences.Remove (pr);
+						sproj.References.Remove (pr);
 
 					mdata.UnresolvedReferences.Clear ();
 				}
@@ -1301,9 +1322,9 @@ namespace MonoDevelop.Autotools
 			//FIXME: If anything fails while writing, skip completely
 			//FIXME: All the file vars must be distinct
 			WriteFiles (BuildFilesVar, BuildAction.Compile, makeRelative, "Build");
-			WriteFiles (DeployFilesVar, BuildAction.FileCopy, makeRelative, "Deploy");
-			WriteFiles (OthersVar, BuildAction.Nothing, makeRelative, "Others");
-			WriteFiles (ResourcesVar, BuildAction.EmbedAsResource, makeRelative, "Resources");
+			WriteFiles (DeployFilesVar, BuildAction.Content, makeRelative, "Deploy");
+			WriteFiles (OthersVar, BuildAction.None, makeRelative, "Others");
+			WriteFiles (ResourcesVar, BuildAction.EmbeddedResource, makeRelative, "Resources");
 
 			if (SyncReferences && SaveReferences) {
 				Makefile.ClearVariableValue (GacRefVar.Name);
@@ -1342,7 +1363,7 @@ namespace MonoDevelop.Autotools
 			this.monitor = null;
 		}
 
-		bool WriteFiles (MakefileVar fileVar, BuildAction action, bool makeRelative, string id)
+		bool WriteFiles (MakefileVar fileVar, string buildAction, bool makeRelative, string id)
 		{
 			if (!fileVar.Sync || !fileVar.SaveEnabled)
 				return false;
@@ -1357,12 +1378,12 @@ namespace MonoDevelop.Autotools
 			}
 
 			List<string> files = new List<string> ();
-			foreach (ProjectFile pf in OwnerProject.ProjectFiles) {
+			foreach (ProjectFile pf in OwnerProject.Files) {
 				if (pf.Subtype != Subtype.Code)
 					continue;
 				if (IsFileExcluded (pf.FilePath))
 					continue;
-				if (pf.BuildAction == action) {
+				if (pf.BuildAction == buildAction) {
 					string str = null;
 					if (makeRelative)
 						//Files are relative to the Makefile
@@ -1371,7 +1392,6 @@ namespace MonoDevelop.Autotools
 						str = pf.RelativePath;
 
 					string unescapedFileName = Path.GetFileName (str);
-					str = EscapeString (str);
 
 					if (EncodeValues [fileVar.Name]) {
 						if (pf.IsExternalToProject)
@@ -1379,9 +1399,10 @@ namespace MonoDevelop.Autotools
 						else
 							str = EncodeFileName (str, "srcdir", false);
 					}
+					str = ToMakefilePath (str);
 
 					// Emit the resource ID only when it is different from the file name
-					if (pf.BuildAction == BuildAction.EmbedAsResource && pf.ResourceId != null && pf.ResourceId.Length > 0 && pf.ResourceId != unescapedFileName)
+					if (pf.BuildAction == BuildAction.EmbeddedResource && pf.ResourceId != null && pf.ResourceId.Length > 0 && pf.ResourceId != unescapedFileName)
 						str = String.Format ("{0}{1},{2}", fileVar.Prefix, str, EscapeString (pf.ResourceId));
 					else
 						str = String.Format ("{0}{1}", fileVar.Prefix, str);
@@ -1421,7 +1442,7 @@ namespace MonoDevelop.Autotools
 			//	key -> pkgname, emit as -pkg:$key
 			Dictionary<string, bool> hasAcSubstPackages = new Dictionary<string, bool> ();
 
-			foreach (ProjectReference pr in OwnerProject.ProjectReferences) {
+			foreach (ProjectReference pr in ((DotNetProject)OwnerProject).References) {
 				if (pr.ReferenceType != refType)
 					continue;
 
@@ -1469,10 +1490,10 @@ namespace MonoDevelop.Autotools
 			//Gac ref can be a full name OR a path!
 			//FIXME: Use GetReferencedFileName and GetPackageFromPath ?
 			string fullname = pr.Reference;
-			SystemPackage pkg = Runtime.SystemAssemblyService.GetPackageFromFullName (pr.Reference);
+			SystemPackage pkg = pr.Package;
 			if (pkg == null) {
 				//reference could be a path
-				pkg = Runtime.SystemAssemblyService.GetPackageFromPath (Path.GetFullPath (pr.Reference));
+				pkg = assemblyContext.GetPackageFromPath (Path.GetFullPath (pr.Reference));
 				if (pkg != null) {
 					//Path
 					try {
@@ -1488,7 +1509,7 @@ namespace MonoDevelop.Autotools
 			}
 
 			if (pkg == null)
-				return AsmRefToString (pr.GetReferencedFileNames () [0], refVar, false);
+				return AsmRefToString (pr.GetReferencedFileNames (ConfigurationSelector.Default) [0], refVar, false);
 
 			// Reference is from a package
 
@@ -1501,7 +1522,7 @@ namespace MonoDevelop.Autotools
 			if (UseAutotools)
 				//Check whether ref'ed in configure.in
 				varname = ConfiguredPackages.GetVarNameFromName (pkg.Name);
-
+			
 			if (varname == null) {
 				//Package not referenced in configure.in
 				//Or not a autotools based project,
@@ -1512,9 +1533,9 @@ namespace MonoDevelop.Autotools
 						//Warn only if UseAutotools
 						string msg = GettextCatalog.GetString (
 							"A reference to the pkg-config package '{0}' is being emitted to the Makefile, " +
-							"because at least one assembly from the package is used in the project. However, " +
+							"because at least one assembly from the package is used in the project '{1}'. However, " +
 							"this dependency is not specified in the configure.in file, so you might need to " +
-							"add it to ensure that the project builds successfully on other systems.", pkg.Name);
+							"add it to ensure that the project builds successfully on other systems.", pkg.Name, pr.OwnerProject.Name);
 						LoggingService.LogWarning (msg);
 						monitor.ReportWarning (msg);
 					}
@@ -1538,19 +1559,19 @@ namespace MonoDevelop.Autotools
 		{
 			if (EncodeValues [refVar.Name]) {
 				if (!isBuiltAssembly)
-					return EncodeFileName (reference, "top_srcdir", true);
+					return ToMakefilePath (EncodeFileName (reference, "top_srcdir", true));
 				else if (!reference.StartsWith (BaseDirectory))
 					//Reference is external to this project
-					return EncodeFileName (reference, "top_builddir", true);
+					return ToMakefilePath (EncodeFileName (reference, "top_builddir", true));
 			}
 
 			// !external and !encode
-			return GetRelativePath (reference);
+			return ToMakefilePath (GetRelativePath (reference));
 		}
 
 		string ProjectRefToString (ProjectReference pr, MakefileVar refVar)
 		{
-			string [] tmp = pr.GetReferencedFileNames ();
+			string [] tmp = pr.GetReferencedFileNames (ConfigurationSelector.Default);
 			if (tmp == null || tmp.Length == 0)
 				//Reference not found, ignoring
 				return null;
@@ -1558,7 +1579,33 @@ namespace MonoDevelop.Autotools
 			return AsmRefToString (tmp [0], refVar, true);
 		}
 
-		static string EscapeString (string str)
+		public static string GetUnixPath (string path)
+		{
+			if (Path.DirectorySeparatorChar != '/')
+				return path.Replace (Path.DirectorySeparatorChar, '/');
+			else
+				return path;
+		}
+
+		public static string FromUnixPath (string path)
+		{
+			if (Path.DirectorySeparatorChar != '/')
+				return path.Replace (Path.DirectorySeparatorChar, '/');
+			else
+				return path;
+		}
+
+		public static string ToMakefilePath (string str)
+		{
+			return EscapeString (GetUnixPath (str));
+		}
+
+		public static string FromMakefilePath (string str)
+		{
+			return FromUnixPath (UnescapeString (str));
+		}
+
+		public static string EscapeString (string str)
 		{
 			StringBuilder sb = new StringBuilder ();
 			int len = str.Length;
@@ -1601,7 +1648,7 @@ namespace MonoDevelop.Autotools
 
 	public class ConfiguredPackagesManager
 	{
-		Dictionary<string, List<string>> pkgVarNameToPkgName;
+		Dictionary<string, List<PackageContent>> pkgVarNameToPkgName;
 		Dictionary<string, string> pkgNameToPkgVarName;
 
 		//This dict has entries for all vars that have a
@@ -1625,13 +1672,29 @@ namespace MonoDevelop.Autotools
 		}
 
 		//Gets the pkg-config name from the makefile (or autoconf?) var name
+		// TODO: Remove me, if unused.
+		[Obsolete("Use 'GetPackageContentFromVarName' instead.")]
 		public List<string> GetNamesFromVarName (string varname)
 		{
 			if (!pkgVarNameToPkgName.ContainsKey (varname)) {
 				LoggingService.LogDebug ("pkg-config variable {0} not found in pkgVarNameToPkgName.", varname);
 				return null;
 			}
-
+			List<string> result = new List<string> ();
+			foreach (PackageContent content in pkgVarNameToPkgName [varname]) {
+				result.Add (content.Name);
+			}
+			return result;
+		}
+		
+		//Gets the pkg-config PackageContent from the makefile (or autoconf?) var name
+		public List<PackageContent> GetPackageContentFromVarName (string varname)
+		{
+			if (!pkgVarNameToPkgName.ContainsKey (varname)) {
+				LoggingService.LogDebug ("pkg-config variable {0} not found in pkgVarNameToPkgName.", varname);
+				return null;
+			}
+			
 			return pkgVarNameToPkgName [varname];
 		}
 
@@ -1650,22 +1713,24 @@ namespace MonoDevelop.Autotools
 		{
 			return varNameAcSubst.ContainsKey (varname);
 		}
-
+		
+		
 		void ReadPackagesList ()
 		{
-			pkgVarNameToPkgName = new Dictionary<string, List<string>> ();
+			pkgVarNameToPkgName = new Dictionary<string, List<PackageContent>> ();
 			pkgNameToPkgVarName = new Dictionary<string, string> ();
 			varNameAcSubst = new Dictionary<string, string> ();
 
 			string content = null;
 			using (StreamReader sr = new StreamReader (fullpath))
 				content = sr.ReadToEnd ();
+			var assemblyContext = MakefileData.GetMonoRuntimeContext ();
 
 			foreach (Match match in PkgCheckModulesRegex.Matches (content)) {
 				if (!match.Success)
 					continue;
 
-				List<string> pkgs = new List<string> ();
+				List<PackageContent> pkgs = new List<PackageContent> ();
 				string pkgId = match.Groups ["pkgId"].Value;
 				if (pkgId.Length > 2 && pkgId [0] == '[' && pkgId [pkgId.Length - 1] == ']')
 					// Remove [] used for quoting
@@ -1679,10 +1744,11 @@ namespace MonoDevelop.Autotools
 					if (s.Length > 2 && s [0] == '[' && s [s.Length - 1] == ']')
 						// Remove [] used for quoting
 						s = s.Substring (1, s.Length - 2);
-					pkgs.Add (s);
+					PackageContent packageContent = new PackageContent (s);
+					pkgs.Add (packageContent);
 					pkgNameToPkgVarName [s] = pkgId;
+					AddRequiredPackages (assemblyContext, pkgId, s, packageContent);
 				}
-
 				pkgVarNameToPkgName [pkgId] = pkgs;
 			}
 			
@@ -1695,10 +1761,10 @@ namespace MonoDevelop.Autotools
 					continue;
 
 				string pkgVarName = s.Replace ("_LIBS", String.Empty);
-				List<string> l = GetNamesFromVarName (pkgVarName);
-
-				if (l != null && l.Count == 1 &&
-					Runtime.SystemAssemblyService.GetPackage (l [0]) != null) {
+				List<PackageContent> l = GetPackageContentFromVarName (pkgVarName);
+				
+				IAssemblyContext r = MakefileData.GetMonoRuntimeContext ();
+				if (l != null && l.Count == 1 && r.GetPackage (l [0].Name) != null) {
 					//PKG_CHECK_MODULES for pkgVarName was found
 					//and it references only a single managed package
 					//
@@ -1710,6 +1776,20 @@ namespace MonoDevelop.Autotools
 			}
 		}
 
+		void AddRequiredPackages (IAssemblyContext assemblyContext, string variableName, string packageName, PackageContent packageContent)
+		{
+			SystemPackage package = assemblyContext.GetPackage (packageName);
+			if (package != null && !string.IsNullOrEmpty (package.Requires)) {
+				foreach (string requiredPackageName in package.Requires.Split (' ')) {
+					if (!pkgNameToPkgVarName.ContainsKey (requiredPackageName)) {
+						pkgNameToPkgVarName[requiredPackageName] = variableName;
+						packageContent.RequiredPackages.Add (requiredPackageName);
+					}
+					AddRequiredPackages (assemblyContext, variableName, requiredPackageName, packageContent);
+				}
+			}
+		}
+		
 		public DateTime LastWriteTime {
 			get { return lastWriteTime; }
 		}
@@ -1733,5 +1813,30 @@ namespace MonoDevelop.Autotools
 			}
 		}
 	}
-
+	
+	public class PackageContent
+	{
+		public string Name { get; set; }
+		
+		System.Collections.Generic.List<string> requiredPackages = new System.Collections.Generic.List<string> ();
+		public List<string> RequiredPackages { 
+			get {
+				return requiredPackages; 
+			}
+		}
+		
+		public IEnumerable<string> AllReferencedNames {
+			get {
+				yield return Name;
+				foreach (string name in requiredPackages) {
+					yield return name;
+				}
+			}
+		}
+		
+		public PackageContent (string name)
+		{
+			this.Name = name;
+		}
+	}
 }
