@@ -1,6 +1,8 @@
 using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Projects;
+using MonoDevelop.Core.Assemblies;
 
 using System;
 using System.Collections.Generic;
@@ -16,6 +18,7 @@ namespace MonoDevelop.Autotools
 		MakefileData data;
 		ComboBox [] combos = null;
 		bool isDotNetProject;
+		bool loading = true;
 		
 		public MakefileOptionPanelWidget (Project project, MakefileData tmpData)
 			: this ()
@@ -35,10 +38,7 @@ namespace MonoDevelop.Autotools
 
 				this.cbEnableMakefileIntegration.Active = false;
 
-				if (File.Exists (SPath.Combine (project.BaseDirectory, "Makefile.am")))
-					this.fileEntryMakefilePath.Path = SPath.Combine (project.BaseDirectory, "Makefile.am");
-				else if (File.Exists (SPath.Combine (project.BaseDirectory, "Makefile")))
-					this.fileEntryMakefilePath.Path = SPath.Combine (project.BaseDirectory, "Makefile");
+				FindExistingMakefile (project);
 
 				this.fileEntryMakefilePath.DefaultPath = project.BaseDirectory;
 
@@ -91,7 +91,7 @@ namespace MonoDevelop.Autotools
 
 			this.fileEntryConfigureInPath.Path = data.AbsoluteConfigureInPath;
 			if (String.IsNullOrEmpty (data.AbsoluteConfigureInPath))
-				this.fileEntryConfigureInPath.DefaultPath = project.RootCombine.BaseDirectory;
+				this.fileEntryConfigureInPath.DefaultPath = project.ParentSolution.BaseDirectory;
 			else
 				this.fileEntryConfigureInPath.DefaultPath = data.AbsoluteConfigureInPath;
 
@@ -112,6 +112,16 @@ namespace MonoDevelop.Autotools
 
 			((Gtk.Container) comboOutputDir.Parent).Remove (comboOutputDir);
 			((Gtk.Container) lblOutputDirVar.Parent).Remove (lblOutputDirVar);
+			
+			loading = false;
+		}
+
+		void FindExistingMakefile (Project project)
+		{
+			if (File.Exists (SPath.Combine (project.BaseDirectory, "Makefile.am")))
+				this.fileEntryMakefilePath.Path = SPath.Combine (project.BaseDirectory, "Makefile.am");
+			else if (File.Exists (SPath.Combine (project.BaseDirectory, "Makefile")))
+				this.fileEntryMakefilePath.Path = SPath.Combine (project.BaseDirectory, "Makefile");
 		}
 		
 		public void SetImportMode ()
@@ -132,7 +142,7 @@ namespace MonoDevelop.Autotools
 				//comboAssemblyName, comboOutputDir};
 		}
 		
-		public bool Store (Project project)
+		public bool ValidateChanges (Project project)
 		{
 			data.IntegrationEnabled = this.cbEnableMakefileIntegration.Active;
 			data.RelativeMakefileName = this.fileEntryMakefilePath.Path;
@@ -204,9 +214,7 @@ namespace MonoDevelop.Autotools
 					ShowMakefileNotFoundError (e);
 					return false;
 				} catch (Exception e) {
-					IdeApp.Services.MessageService.ShowError (e, GettextCatalog.GetString (
-						"Specified makefile is invalid: {0}", tmpData.AbsoluteMakefileName),
-						(Window) Toplevel, true);
+					MessageService.ShowException ((Window) Toplevel, e, GettextCatalog.GetString ("Specified makefile is invalid: {0}", tmpData.AbsoluteMakefileName));
 					return false;
 				}
 
@@ -214,9 +222,7 @@ namespace MonoDevelop.Autotools
 					!File.Exists (System.IO.Path.Combine (tmpData.AbsoluteConfigureInPath, "configure.in")) &&
 				    !File.Exists (System.IO.Path.Combine (tmpData.AbsoluteConfigureInPath, "configure.ac")))
 				{
-					IdeApp.Services.MessageService.ShowError (null, GettextCatalog.GetString (
-						"Path specified for configure.in is invalid: {0}", tmpData.RelativeConfigureInPath),
-						(Window) Toplevel, true);
+					MessageService.ShowError ((Window)Toplevel, GettextCatalog.GetString ("Path specified for configure.in is invalid: {0}", tmpData.RelativeConfigureInPath));
 					return false;
 				}
 
@@ -225,9 +231,7 @@ namespace MonoDevelop.Autotools
 					String.IsNullOrEmpty (tmpData.AsmRefVar.Name) ||
 					String.IsNullOrEmpty (tmpData.ProjectRefVar.Name))) {
 
-					IdeApp.Services.MessageService.ShowError (null, GettextCatalog.GetString (
-						"'Sync References' is enabled, but one of Reference variables is not set. Please correct this."),
-						(Window) Toplevel, true);
+					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString ("'Sync References' is enabled, but one of Reference variables is not set. Please correct this."));
 					return false;
 				}
 			
@@ -247,18 +251,15 @@ namespace MonoDevelop.Autotools
 				try {
 					tmpData.GetErrorRegex (true);
 				} catch (Exception e) {
-					IdeApp.Services.MessageService.ShowError (null, GettextCatalog.GetString (
-						"Invalid regex for Error messages: {0}", e.Message),
-						(Window) Toplevel, true);
+					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString ("Invalid regex for Error messages: {0}", e.Message));
 					return false;
 				}
 
 				try {
 					tmpData.GetWarningRegex (true);
 				} catch (Exception e) {
-					IdeApp.Services.MessageService.ShowError (null, GettextCatalog.GetString (
-						"Invalid regex for Warning messages: {0}", e.Message),
-						(Window) Toplevel, true);
+					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString (
+						"Invalid regex for Warning messages: {0}", e.Message));
 					return false;
 				}
 
@@ -277,13 +278,16 @@ namespace MonoDevelop.Autotools
  			return true;
 		}
 
+		public void Store (Project project)
+		{
+			// FIXME: Storing currently done in ValidateChanges. It should be done here.
+		}
+
 		bool CheckNonEmptyFileVar (MakefileVar var, string id)
 		{
 			if (var.Sync && String.IsNullOrEmpty (var.Name.Trim ())) {
-				IdeApp.Services.MessageService.ShowError (null, GettextCatalog.GetString (
-					"File variable ({0}) is set for sync'ing, but no valid variable is selected. " + 
-					"Either disable the sync'ing or select a variable name.", id),
-					(Window) Toplevel, true);
+				MessageService.ShowError ((Window) Toplevel,GettextCatalog.GetString (
+					"File variable ({0}) is set for sync'ing, but no valid variable is selected. Either disable the sync'ing or select a variable name.", id));
 
 				return false;
 			}
@@ -360,15 +364,32 @@ namespace MonoDevelop.Autotools
 		{
 			table1.Sensitive = active;
 			if (active) {
-				bool first_load = String.IsNullOrEmpty (data.RelativeMakefileName);
-				if (TryLoadMakefile (false)) {
-					if (first_load)
-						GuessVariables ();
-					else
-						LoadVariables ();
-				} else {
-					fileEntryMakefilePath.Path = fileEntryMakefilePath.DefaultPath;
-				}
+				bool tryAgain = false;
+				do {
+					bool first_load = String.IsNullOrEmpty (data.RelativeMakefileName);
+					if (TryLoadMakefile (false)) {
+						if (first_load)
+							GuessVariables ();
+						else
+							LoadVariables ();
+						tryAgain = false;
+					} else {
+						bool generate = false;
+						if (!tryAgain && !loading) {
+							generate = MessageService.Confirm (
+							      GettextCatalog.GetString ("No makefile was found in the project directory. Do you want to generate it now?"), 
+							      GettextCatalog.GetString ("Notice that generation of makefiles is not supported for single projects. A set of makefiles will have to be generated for the whole solution."), 
+							      new AlertButton (GettextCatalog.GetString ("Generate Makefile...")));
+						}
+						if (generate) {
+							AutotoolsCommandHandler.GenerateMakefiles (data.OwnerProject, data.OwnerProject.ParentSolution);
+							FindExistingMakefile (data.OwnerProject);
+						} else
+							fileEntryMakefilePath.Path = fileEntryMakefilePath.DefaultPath;
+						tryAgain = generate;
+					}
+				} while (tryAgain);
+				
 			} else {
 				SetActive (active);
 			}
@@ -720,17 +741,15 @@ namespace MonoDevelop.Autotools
 					ShowMakefileNotFoundError (e);
 			} catch (Exception e) {
 				if (showError)
-					IdeApp.Services.MessageService.ShowError (e,
-						GettextCatalog.GetString ("Error while trying to read the specified Makefile"),
-						(Window) this.Toplevel, true);
+					MessageService.ShowException ((Window) this.Toplevel,e,
+						GettextCatalog.GetString ("Error while trying to read the specified Makefile"));
 				return null;
 			}
 
 			if (vars != null && vars.Count == 0) {
 				if (showError)
-					IdeApp.Services.MessageService.ShowError (null, 
-						GettextCatalog.GetString ("No variables found in the selected Makefile"),
-						(Window) this.Toplevel, true);
+					MessageService.ShowError ((Window) this.Toplevel, 
+						GettextCatalog.GetString ("No variables found in the selected Makefile"));
 				return null;
 			}
 
@@ -841,7 +860,7 @@ namespace MonoDevelop.Autotools
 		{
 			if (cbBuildTarget.Sensitive && cbBuildTarget.Active) {
 				BuildTargetName.Sensitive = true;
-				BuildTargetName.Text = "all";
+				BuildTargetName.Text = string.IsNullOrEmpty(data.BuildTargetName)? "all": data.BuildTargetName;
 			} else {
 				BuildTargetName.Sensitive = false;
 				BuildTargetName.Text = "";
@@ -852,7 +871,7 @@ namespace MonoDevelop.Autotools
 		{
 			if (cbRunTarget.Sensitive && cbRunTarget.Active) {
 				ExecuteTargetName.Sensitive = true;
-				ExecuteTargetName.Text = "run";
+				ExecuteTargetName.Text = string.IsNullOrEmpty(data.ExecuteTargetName)? "run": data.ExecuteTargetName;
 			} else {
 				ExecuteTargetName.Sensitive = false;
 				ExecuteTargetName.Text = "";
@@ -863,7 +882,7 @@ namespace MonoDevelop.Autotools
 		{
 			if (cbCleanTarget.Sensitive && cbCleanTarget.Active) {
 				CleanTargetName.Sensitive = true;
-				CleanTargetName.Text = "clean";
+				CleanTargetName.Text = string.IsNullOrEmpty(data.CleanTargetName)? "clean": data.CleanTargetName;
 			} else {
 				CleanTargetName.Sensitive = false;
 				CleanTargetName.Text = "";
@@ -916,9 +935,9 @@ namespace MonoDevelop.Autotools
 
 		void ShowMakefileNotFoundError (Exception e)
 		{
-				IdeApp.Services.MessageService.ShowError (e,
-                    GettextCatalog.GetString ("Unable to find the specified Makefile. You need to specify the path to an existing Makefile for use with the 'Makefile Integration' feature."),
-                    (Window) this.Toplevel, true);
+				MessageService.ShowException ((Window) this.Toplevel, 
+			                                  e,
+			                                  GettextCatalog.GetString ("Unable to find the specified Makefile. You need to specify the path to an existing Makefile for use with the 'Makefile Integration' feature."));
 		}
 
 		// Returns true if either
@@ -933,20 +952,32 @@ namespace MonoDevelop.Autotools
 			if (prefix.Length > 0)
 				return true;
 
+			DotNetProject dnp = data.OwnerProject as DotNetProject;
+			if (dnp == null)
+				return false;
+			
 			// 'core' here simply means any assemblies in the gac
-			bool has_core_or_pkgref = false;
 			foreach (string file in list) {
-				if (MakefileData.CorePackageAssemblyNames.ContainsKey (file) || IsPkgRef (file)) {
-					has_core_or_pkgref = true;
-					continue;
+				try {
+					string fullName = dnp.AssemblyContext.GetAssemblyFullName (file, dnp.TargetFramework);
+					if (fullName != null) {
+						SystemAssembly asm = dnp.AssemblyContext.GetAssemblyFromFullName (fullName, null, dnp.TargetFramework);
+						if (asm != null && asm.Package.IsGacPackage)
+							return true;
+					}
+				} catch {
+					// Ignore
 				}
+					
+				if (IsPkgRef (file))
+					return true;
 
 				// invalid if any entry isn't one of core/variable/dll/pkgrefs
 				if (!IsVariable (file) && !IsDll (file))
 					return false;
 			}
 
-			return has_core_or_pkgref;
+			return false;
 		}
 
 		bool CheckRes (List<string> list, out string prefix)
