@@ -82,7 +82,7 @@ namespace MonoDevelop.Autotools
 			monitor.BeginTask (GettextCatalog.GetString ("Updating project from Makefile"), 1);
 			try { 
 				data.OwnerProject = project;
-				if (data.IntegrationEnabled)
+				if (data.SupportsIntegration)
 					data.UpdateProject (monitor, false);
 				monitor.Step (1);
 			} catch (Exception e) {
@@ -105,7 +105,7 @@ namespace MonoDevelop.Autotools
 				return;
 				
 			MakefileData data = project.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-			if (data == null || !data.IntegrationEnabled)
+			if (data == null || !data.SupportsIntegration)
 				return;
 
 			try {
@@ -117,13 +117,13 @@ namespace MonoDevelop.Autotools
 					"Error saving to Makefile ({0}) for project {1}", data.AbsoluteMakefileName, project.Name), e);
 			}
 		}
-		
-		public override List<string> GetItemFiles (SolutionEntityItem entry, bool includeReferencedFiles)
+
+		public override List<FilePath> GetItemFiles (SolutionEntityItem entry, bool includeReferencedFiles)
 		{
-			List<string> col = base.GetItemFiles (entry, includeReferencedFiles);
+			List<FilePath> col = base.GetItemFiles (entry, includeReferencedFiles);
 			
 			MakefileData data = entry.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-			if (data == null || !data.IntegrationEnabled || string.IsNullOrEmpty (data.AbsoluteMakefileName))
+			if (data == null || !data.SupportsIntegration || string.IsNullOrEmpty (data.AbsoluteMakefileName))
 				return col;
 			
 			col.Add (data.AbsoluteMakefileName);
@@ -139,20 +139,20 @@ namespace MonoDevelop.Autotools
 
 
 		//TODO
-		protected override bool GetNeedsBuilding (SolutionEntityItem entry, string configuration)
+		protected override bool GetNeedsBuilding (SolutionEntityItem entry, ConfigurationSelector configuration)
 		{
 			return base.GetNeedsBuilding (entry, configuration);
 		}
 
 		//FIXME: Check whether autogen.sh is required or not
-		protected override BuildResult Build (IProgressMonitor monitor, SolutionEntityItem entry, string configuration)
+		protected override BuildResult Build (IProgressMonitor monitor, SolutionEntityItem entry, ConfigurationSelector configuration)
 		{
 			Project project = entry as Project;
 			if (project == null)
 				return base.Build (monitor, entry, configuration);
 
 			MakefileData data = project.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-			if (data == null || !data.IntegrationEnabled || String.IsNullOrEmpty (data.BuildTargetName))
+			if (data == null || !data.SupportsIntegration || String.IsNullOrEmpty (data.BuildTargetName))
 				return base.Build (monitor, entry, configuration);
 
 			//FIXME: Gen autofoo ? autoreconf?
@@ -329,7 +329,7 @@ namespace MonoDevelop.Autotools
 				return null;
 		}
 
-		protected override void Clean (IProgressMonitor monitor, SolutionEntityItem entry, string configuration)
+		protected override void Clean (IProgressMonitor monitor, SolutionEntityItem entry, ConfigurationSelector configuration)
 		{
 			Project proj = entry as Project;
 			if (proj == null) {
@@ -338,7 +338,7 @@ namespace MonoDevelop.Autotools
 			}
 
 			MakefileData data = proj.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-			if (data == null || !data.IntegrationEnabled || String.IsNullOrEmpty (data.CleanTargetName)) {
+			if (data == null || !data.SupportsIntegration || String.IsNullOrEmpty (data.CleanTargetName)) {
 				base.Clean (monitor, entry, configuration); 
 				return;
 			}
@@ -373,19 +373,19 @@ namespace MonoDevelop.Autotools
 			monitor.ReportSuccess ( GettextCatalog.GetString ( "Project successfully cleaned"));
 		}
 
-		protected override bool CanExecute (SolutionEntityItem item, ExecutionContext context, string configuration)
+		protected override bool CanExecute (SolutionEntityItem item, ExecutionContext context, ConfigurationSelector configuration)
 		{
 			Project project = item as Project;
 			if (project != null) {
 				MakefileData data = project.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-				if (data != null && data.IntegrationEnabled && !String.IsNullOrEmpty (data.ExecuteTargetName))
+				if (data != null && data.SupportsIntegration && !String.IsNullOrEmpty (data.ExecuteTargetName))
 					return true;
 			}
 			return base.CanExecute (item, context, configuration);
 		}
 
 
-		protected override void Execute (IProgressMonitor monitor, SolutionEntityItem entry, ExecutionContext context, string configuration)
+		protected override void Execute (IProgressMonitor monitor, SolutionEntityItem entry, ExecutionContext context, ConfigurationSelector configuration)
 		{
 			Project project = entry as Project;
 			if (project == null) {
@@ -394,7 +394,7 @@ namespace MonoDevelop.Autotools
 			}
 
 			MakefileData data = project.ExtendedProperties ["MonoDevelop.Autotools.MakefileInfo"] as MakefileData;
-			if (data == null || !data.IntegrationEnabled || String.IsNullOrEmpty (data.ExecuteTargetName)) {
+			if (data == null || !data.SupportsIntegration || String.IsNullOrEmpty (data.ExecuteTargetName)) {
 				base.Execute (monitor, entry, context, configuration);
 				return;
 			}
