@@ -28,6 +28,7 @@ using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Collections;
+using MonoDevelop.Core.Assemblies;
 using Mono.Addins;
 using MonoDevelop.Deployment;
 
@@ -155,43 +156,43 @@ namespace MonoDevelop.Autotools
 				string pfpath = null;
 				foreach (ProjectFile projectFile in project.Files) 
 				{
-					pfpath = (PlatformID.Unix == Environment.OSVersion.Platform) ? projectFile.RelativePath : projectFile.RelativePath.Replace("\\","/");
-					pfpath = FileService.NormalizeRelativePath (pfpath);
+					pfpath = FileService.NormalizeRelativePath (projectFile.RelativePath);
 					switch ( projectFile.BuildAction )
 					{
-						case "Compile":
+						case BuildAction.Compile:
 							
 							if ( projectFile.Subtype != Subtype.Code ) continue;
-							files.AppendFormat ( "\\\n\t{0} ", EscapeSpace (pfpath));
+							files.AppendFormat ( "\\\n\t{0} ", MakefileData.ToMakefilePath (pfpath));
 							break;
 
-						case "None":
+						case BuildAction.Content:
+						case BuildAction.None:
 
-							extraFiles.Add (EscapeSpace(pfpath));
+							extraFiles.Add (MakefileData.ToMakefilePath (pfpath));
 							break;
 
-						case "EmbeddedResource":
+						case BuildAction.EmbeddedResource:
 
-							if ( !projectFile.FilePath.StartsWith ( ctx.BaseDirectory ) )
+							if ( !projectFile.FilePath.IsChildPathOf ( ctx.BaseDirectory ) )
 							{
 								// file is not within directory hierarchy, copy it in
 								string rdir = Path.Combine (Path.GetDirectoryName (project.FileName), resourcedir);
 								if ( !Directory.Exists ( rdir ) ) Directory.CreateDirectory ( rdir );
 								string newPath = Path.Combine (rdir, Path.GetFileName ( projectFile.FilePath ));
 								FileService.CopyFile ( projectFile.FilePath, newPath ) ;
-								pfpath = (PlatformID.Unix == Environment.OSVersion.Platform) ? project.GetRelativeChildPath (newPath) : project.GetRelativeChildPath (newPath).Replace("\\","/");
+								pfpath = project.GetRelativeChildPath (newPath);
 								pfpath = FileService.NormalizeRelativePath (pfpath);
 							}
 							if (!String.IsNullOrEmpty (projectFile.ResourceId) && projectFile.ResourceId != Path.GetFileName (pfpath))
-								res_files.AppendFormat ( "\\\n\t{0},{1} ", EscapeSpace (pfpath), EscapeSpace (projectFile.ResourceId));
+								res_files.AppendFormat ("\\\n\t{0},{1} ", MakefileData.ToMakefilePath (pfpath), MakefileData.EscapeString (projectFile.ResourceId));
 							else
-								res_files.AppendFormat ( "\\\n\t{0} ", EscapeSpace (pfpath));
+								res_files.AppendFormat ("\\\n\t{0} ", MakefileData.ToMakefilePath (pfpath));
  
 							break;
 
 						case "FileCopy":
-						
-							datafiles.AppendFormat ("\\\n\t{0} ", EscapeSpace (pfpath));
+
+							datafiles.AppendFormat ("\\\n\t{0} ", MakefileData.ToMakefilePath (pfpath));
 							break;
 					}
 				}
@@ -254,9 +255,7 @@ namespace MonoDevelop.Autotools
 
 					ConfigSection configSection = new ConfigSection (combineConfig.Id);
 
-					string assembly = (PlatformID.Unix == Environment.OSVersion.Platform) ?
-						project.GetRelativeChildPath ( config.CompiledOutputName ) :
-						project.GetRelativeChildPath ( config.CompiledOutputName ).Replace("\\","/");
+					string assembly = MakefileData.GetUnixPath (project.GetRelativeChildPath (config.CompiledOutputName));
 
 					configSection.BuildVariablesBuilder.AppendFormat ("ASSEMBLY_COMPILER_COMMAND = {0}\n",
 							setup.GetCompilerCommand ( project, config.Id ) );
@@ -310,18 +309,14 @@ namespace MonoDevelop.Autotools
 
 						projectReferences.WriteLine (" \\");
 						projectReferences.Write ("\t");
-						pref = (PlatformID.Unix == Environment.OSVersion.Platform) ?
-							project.GetRelativeChildPath ( dnpc.CompiledOutputName ) :
-							project.GetRelativeChildPath ( dnpc.CompiledOutputName ).Replace("\\","/");
+						pref = project.GetRelativeChildPath (dnpc.CompiledOutputName);
 
-						projectReferences.Write (EscapeSpace (pref));
+						projectReferences.Write (MakefileData.ToMakefilePath (pref));
 					}
 					configSection.BuildVariablesBuilder.AppendFormat ( "PROJECT_REFERENCES = {0}\n", projectReferences.ToString() );
 
-					string buildDir = (PlatformID.Unix == Environment.OSVersion.Platform) ?
-						project.GetRelativeChildPath (config.OutputDirectory) :
-						project.GetRelativeChildPath (config.OutputDirectory).Replace ("\\","/");
-					configSection.BuildVariablesBuilder.AppendFormat ("BUILD_DIR = {0}\n", buildDir);
+					string buildDir = project.GetRelativeChildPath (config.OutputDirectory);
+					configSection.BuildVariablesBuilder.AppendFormat ("BUILD_DIR = {0}\n", MakefileData.ToMakefilePath (buildDir));
 
 					// Register files built by this configuration.
 					// Built files won't be distributed.
@@ -329,7 +324,7 @@ namespace MonoDevelop.Autotools
 						ctx.AddBuiltFile (Path.Combine (config.OutputDirectory, bfile));
 
 					DeployFileCollection deployFiles = DeployService.GetDeployFiles (
-							ctx.DeployContext, new SolutionItem[] { project }, config.Id);
+							ctx.DeployContext, new SolutionItem[] { project }, config.Selector);
 
 					ProcessDeployFilesForConfig (deployFiles, project, configSection, ctx, config);
 					configSections.Add (configSection);
@@ -347,15 +342,14 @@ namespace MonoDevelop.Autotools
 					}
 
 					// Register files generated by the compiler
-					ctx.AddBuiltFile (project.GetOutputFileName (combineConfig.Id));
+					ctx.AddBuiltFile (project.GetOutputFileName (combineConfig.Selector));
 					if (config.DebugMode)
-						ctx.AddBuiltFile (project.GetOutputFileName (combineConfig.Id) + ".mdb");
+						ctx.AddBuiltFile (project.GetOutputFileName (combineConfig.Selector) + ".mdb");
 
 					if (config.SignAssembly) {
 						string spath = project.GetRelativeChildPath (config.AssemblyKeyFile);
-						spath = (PlatformID.Unix == Environment.OSVersion.Platform) ? spath : spath.Replace("\\","/");
 						spath = FileService.NormalizeRelativePath (spath);
-						extraFiles.Add (EscapeSpace (spath));
+						extraFiles.Add (MakefileData.ToMakefilePath (spath));
 					}
 
 					if (buildEnabled && pkgs.Count > 0)
@@ -364,7 +358,7 @@ namespace MonoDevelop.Autotools
 
 
 				foreach (string ef in extraFiles)
-					extras.AppendFormat ( "\\\n\t{0} ", EscapeSpace (ef));
+					extras.AppendFormat ("\\\n\t{0} ", MakefileData.ToMakefilePath (ef));
 
 				Dictionary<string, DeployFileData> commonDeployVars = new Dictionary<string, DeployFileData> (allDeployVars);
 				foreach (ConfigSection configSection in configSections) {
@@ -384,6 +378,14 @@ namespace MonoDevelop.Autotools
 
 					conf_vars.Append (configSection.BuildVariablesBuilder.ToString ());
 					conf_vars.Append ("\n");
+					
+					if (ctx.Switches != null) {
+						foreach (Switch s in ctx.Switches) {
+							conf_vars.AppendLine (string.Format (@"if ENABLE_{0}
+ASSEMBLY_COMPILER_FLAGS += -define:{1}
+endif", s.SwitchName.Replace ('-', '_').ToUpperInvariant (), s.Define));
+						}
+					}
 
 					foreach (KeyValuePair<string, DeployFileData> pair in allDeployVars) {
 						string targetDeployVar = pair.Key;
@@ -394,7 +396,7 @@ namespace MonoDevelop.Autotools
 						if (configSection.DeployFileVars.ContainsKey (targetDeployVar)) {
 							//use the dfile from the config section
 							DeployFile dfile = configSection.DeployFileVars [targetDeployVar];
-							string fname = EscapeSpace (
+							string fname = MakefileData.ToMakefilePath (
 									FileService.AbsoluteToRelativePath (
 										Path.GetFullPath (project.BaseDirectory),
 										Path.GetFullPath (dfile.SourcePath)));
@@ -403,10 +405,9 @@ namespace MonoDevelop.Autotools
 
 							if (!commonDeployVars.ContainsKey (targetDeployVar)) {
 								//FOO_DLL=$(BUILD_DIR)/foo.dll
-								conf_vars.AppendFormat ("{0}=$(BUILD_DIR){1}{2}\n",
+								conf_vars.AppendFormat ("{0}=$(BUILD_DIR)/{1}\n",
 										targetDeployVar,
-										Path.DirectorySeparatorChar,
-										EscapeSpace (dfile.RelativeTargetPath));
+										MakefileData.ToMakefilePath (dfile.RelativeTargetPath));
 							}
 						} else {
 							// not common and not part of @configSection
@@ -425,10 +426,9 @@ namespace MonoDevelop.Autotools
 
 					if (commonDeployVars.ContainsKey (pair.Key)) {
 						//FOO_DLL=$(BUILD_DIR)/foo.dll
-						deployFileCopyVars.AppendFormat ("{0} = $(BUILD_DIR){1}{2}\n",
+						deployFileCopyVars.AppendFormat ("{0} = $(BUILD_DIR)/{1}\n",
 									pair.Key,
-									Path.DirectorySeparatorChar,
-									EscapeSpace (pair.Value.File.RelativeTargetPath));
+									MakefileData.ToMakefilePath (pair.Value.File.RelativeTargetPath));
 					}
 				}
 				
@@ -519,7 +519,7 @@ namespace MonoDevelop.Autotools
 					else 
 					{
 						refWriter.WriteLine (" \\");			// store all refs for easy access
-						AssemblyName assembly = Runtime.SystemAssemblyService.ParseAssemblyName (reference.Reference);
+						AssemblyName assembly = SystemAssemblyService.ParseAssemblyName (reference.Reference);
 						refWriter.Write ("\t" + assembly.Name);
 						refWriter.Write ("");
 					}
@@ -531,9 +531,9 @@ namespace MonoDevelop.Autotools
 					dllRefWriter.WriteLine (" \\");
 					dllRefWriter.Write ("\t");
 
-					ctx.AddGlobalReferencedFile (EscapeSpace (FileService.AbsoluteToRelativePath (
+					ctx.AddGlobalReferencedFile (MakefileData.ToMakefilePath (FileService.AbsoluteToRelativePath (
 						Path.GetFullPath (ctx.BaseDirectory), assemblyPath)));
-					dllRefWriter.Write (EscapeSpace (FileService.AbsoluteToRelativePath (
+					dllRefWriter.Write (MakefileData.ToMakefilePath (FileService.AbsoluteToRelativePath (
 						project.BaseDirectory, assemblyPath)));
 
 				} 
@@ -554,7 +554,7 @@ namespace MonoDevelop.Autotools
 			//@deployFiles can have duplicates
 			Dictionary<string, DeployFile> uniqueDeployFiles = new Dictionary<string, DeployFile> ();
 			foreach (DeployFile dfile in deployFiles) {
-				if (dfile.SourcePath == project.GetOutputFileName (configSection.Name))
+				if (dfile.SourcePath == project.GetOutputFileName (configSection.Selector))
 					continue;
 
 				// DeployFileCollection can have duplicates, ignore them
@@ -565,7 +565,7 @@ namespace MonoDevelop.Autotools
 					continue;
 				uniqueDeployFiles [key] = dfile;
 
-				string targetDeployVar = GetDeployVar (deployFileVars, Path.GetFileName (dfile.RelativeTargetPath));
+				string targetDeployVar = GetDeployVar (deployFileVars, dfile.RelativeTargetPath);
 				configSection.DeployFileVars [targetDeployVar] = dfile;
 				DeployFileData data = new DeployFileData ();
 				data.File = dfile;
@@ -599,14 +599,14 @@ namespace MonoDevelop.Autotools
 				fname = FileService.NormalizeRelativePath (
 						FileService.AbsoluteToRelativePath (ctx.TargetSolution.BaseDirectory, full_fname));
 				infname = fname + ".in";
-				ctx.AddAutoconfFile (EscapeSpace (fname));
+				ctx.AddAutoconfFile (MakefileData.ToMakefilePath (fname));
 				ctx.AddGeneratedFile (full_fname + ".in");
 
 				//Path relative to project
 				fname = FileService.NormalizeRelativePath (
 						FileService.AbsoluteToRelativePath (project.BaseDirectory, full_fname));
 				infname = fname + ".in";
-				extras.AppendFormat ( "\\\n\t{0} ", EscapeSpace (infname));
+				extras.AppendFormat ("\\\n\t{0} ", MakefileData.ToMakefilePath (infname));
 
 				//dependencyDeployFile here should be filename relative to the project
 				dependencyDeployFile = fname;
@@ -619,7 +619,7 @@ namespace MonoDevelop.Autotools
 			if (dfile.ContainsPathReferences)
 				deployFileCopyTargets.AppendFormat ("$(eval $(call emit-deploy-wrapper,{0},{1}{2}))\n",
 					targetDeployVar,
-					EscapeSpace (dependencyDeployFile),
+					MakefileData.ToMakefilePath (dependencyDeployFile),
 					(dfile.FileAttributes & DeployFileAttributes.Executable) != 0 ? ",x" : String.Empty);
 			else {
 				// The emit-deploy-target macro copies the deployable file to the output directory.
@@ -635,7 +635,7 @@ namespace MonoDevelop.Autotools
 				default:
 					string var;
 					if (dfile.TargetDirectoryID != TargetDirectory.Binaries) {
-						string ddir = FileService.NormalizeRelativePath (Path.GetDirectoryName (dfile.RelativeTargetPath).Trim ('/',' '));
+						string ddir = FileService.NormalizeRelativePath (dfile.RelativeTargetPath.ParentDirectory.ToString().Trim ('/',' '));
 						if (ddir.Length > 0)
 							ddir = "/" + ddir;
 						var = ctx.GetDeployDirectoryVar (dfile.TargetDirectoryID + ddir);
@@ -752,15 +752,25 @@ namespace MonoDevelop.Autotools
 			return null;
 		}
 
-		string GetDeployVar (Dictionary<string, string> dict, string name)
+		string GetDeployVar (Dictionary<string, string> dict, string filePath)
 		{
+			filePath = MonoDevelop.Core.FileService.NormalizeRelativePath (filePath);
+			string name = Path.GetFileName (filePath);
 			name = name.Replace (Path.DirectorySeparatorChar, '_');
 			name = name.Replace ('.', '_');
 			name = name.ToUpper ();
 			name = AutotoolsContext.EscapeStringForAutoconf (name).Trim ('_');
+			
+			// Avoid var name colisions
+			int n = 2;
+			string extName = name;
+			string regPath;
+			while (dict.TryGetValue (extName, out regPath) && regPath != filePath) {
+				extName = name + (n++);
+			}
 
-			dict [name] = name;
-			return name;
+			dict [extName] = filePath;
+			return extName;
 		}
 
 		Project GetProjectFromName (string name, Solution targetSolution)
@@ -774,12 +784,6 @@ namespace MonoDevelop.Autotools
 			
 			return refp;
 		}
-
-		static string EscapeSpace (string str)
-		{
-			return str.Replace (" ", "\\ ");
-		}
-
 	}
 	
 	class DeployFileData
