@@ -4,7 +4,6 @@ using MonoDevelop.Core;
 using MonoDevelop.Projects;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Pads;
-using MonoDevelop.DesignerSupport.PropertyGrid;
 using MonoDevelop.Ide.Gui.Components;
 
 namespace MonoDevelop.DesignerSupport.Projects
@@ -25,7 +24,10 @@ namespace MonoDevelop.DesignerSupport.Projects
 	{
 		public object GetActiveComponent ()
 		{
-			return CurrentNode.DataItem;
+			if (CurrentNodes.Length == 1)
+				return CurrentNode.DataItem;
+			else
+				return null;
 		}
 		
 		public object GetProvider ()
