@@ -41,10 +41,12 @@ using System.Drawing.Design;
 
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
+using MonoDevelop.Core.Assemblies;
 using Mono.Addins;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Serialization;
+using MonoDevelop.Core.Serialization;
 
 using MonoDevelop.DesignerSupport.Toolbox;
 
@@ -56,6 +58,7 @@ namespace MonoDevelop.DesignerSupport
 		readonly static string toolboxProviderPath = "/MonoDevelop/DesignerSupport/ToolboxProviders";
 
 		ToolboxConfiguration config;
+		int initializing;
 		List<IToolboxLoader> loaders = new List<IToolboxLoader> ();
 		List<IToolboxDynamicProvider> dynamicProviders  = new List<IToolboxDynamicProvider> ();
 		
@@ -81,6 +84,10 @@ namespace MonoDevelop.DesignerSupport
 		
 		#region Extension loading
 		
+		public bool Initializing {
+			get { return initializing > 0; }
+		}
+		
 		void OnLoaderExtensionChanged (object s, ExtensionNodeEventArgs args)
 		{
 			if (args.Change == ExtensionChange.Add)
@@ -123,6 +130,7 @@ namespace MonoDevelop.DesignerSupport
 		{
 			ComponentSelectorDialog dlg = new ComponentSelectorDialog (currentConsumer);
 			try {
+				dlg.TransientFor = IdeApp.Workbench.RootWindow;
 				dlg.Fill ();
 				dlg.Run ();
 			}
@@ -131,24 +139,17 @@ namespace MonoDevelop.DesignerSupport
 			}
 		}
 		
-		void AddUserItems (string fileName)
+		void AddUserItems (IList<ItemToolboxNode> nodes)
 		{			
-			foreach (IToolboxLoader loader in loaders) {
-				if (!fileName.EndsWith (loader.FileTypes[0]))
-					continue;
-				
-				System.Collections.Generic.IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-				
-				//prevent user from loading the same items again
-				foreach (ItemToolboxNode node in loadedItems) {
-					bool found = false;
-					foreach (ItemToolboxNode n in Configuration.ItemList) {
-						if (node.Equals (n))
-							found = true;
-					}
-					if (!found)
-						Configuration.ItemList.Add (node);
+			//prevent user from loading the same items again
+			foreach (ItemToolboxNode node in nodes) {
+				bool found = false;
+				foreach (ItemToolboxNode n in Configuration.ItemList) {
+					if (node.Equals (n))
+						found = true;
 				}
+				if (!found)
+					Configuration.ItemList.Add (node);
 			}
 		}
 		
@@ -165,18 +166,39 @@ namespace MonoDevelop.DesignerSupport
 			
 			if (!Configuration.LoadedDefaultProviders.Contains (pname)) {
 				Configuration.LoadedDefaultProviders.Add (pname);
-				
-				IEnumerable<ItemToolboxNode> newItems = provider.GetDefaultItems ();
-				if (newItems != null)
-					Configuration.ItemList.AddRange (newItems);
-				
-				IEnumerable<string> files = provider.GetDefaultFiles ();
-				if (files != null) {
-					foreach (string f in files)
-						AddUserItems (f);
-				}
-				SaveConfiguration ();
+				initializing++;
 				OnToolboxContentsChanged ();
+
+				System.Threading.ThreadPool.QueueUserWorkItem (delegate {
+					List<ItemToolboxNode> nodes = new List<ItemToolboxNode> ();
+					try {
+						IEnumerable<ItemToolboxNode> newItems = provider.GetDefaultItems ();
+						if (newItems != null)
+							nodes.AddRange (newItems);
+					} catch (Exception ex) {
+						LoggingService.LogError ("Error getting default items from a IToolboxDefaultProvider", ex);
+					}
+
+					LoaderContext ctx = null;
+					try {
+						IEnumerable<string> files = provider.GetDefaultFiles ();
+						if (files != null) {
+							ctx = new LoaderContext ();
+							foreach (string f in files)
+								nodes.AddRange (GetFileItems (ctx, f));
+						}
+					} finally {
+						if (ctx != null)
+							ctx.Dispose ();
+					}
+
+					DispatchService.GuiDispatch (delegate {
+						AddUserItems (nodes);
+						initializing--;
+						SaveConfiguration ();
+						OnToolboxContentsChanged ();
+					});
+				});
 			}
 		}
 		
@@ -190,35 +212,29 @@ namespace MonoDevelop.DesignerSupport
 			}
 		}
 		
-		internal IList<ItemToolboxNode> GetFileItems (string fileName)
+		internal IList<ItemToolboxNode> GetFileItems (LoaderContext ctx, string fileName)
 		{
 			// Gets the list of items provided by a file.
-			// It checks every toolbox loader, and loads the file in an external
-			// process if required.
-			
 			List<ItemToolboxNode> items = new List<ItemToolboxNode> ();
-			bool isolatedLoaded = false;
+			
 			foreach (IToolboxLoader loader in loaders) {
+				//check whether the loader can handle this file extension
+				bool match = false;
+				foreach (string ext in loader.FileTypes) {
+					if (fileName.EndsWith (ext)) {
+						match = true;
+						break;
+					}
+				}
+				if (!match)
+					continue;
+				
 				if (!fileName.EndsWith (loader.FileTypes[0]))
 					continue;
 				
 				try {
-					if (loader.ShouldIsolate) {
-						// Load the widgets in an external process. ExternalFileLoader.Load() class will
-						// load the file using all loaders which require isolation.
-						if (isolatedLoaded)
-							continue;
-						isolatedLoaded = true;
-						ExternalFileLoader eloader = (ExternalFileLoader) Runtime.ProcessService.CreateExternalProcessObject (typeof(ExternalFileLoader),true);
-						string s = eloader.LoadFile (fileName);
-						XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
-						ToolboxList list = (ToolboxList) ser.Deserialize (new StringReader (s), typeof(ToolboxList));
-						items.AddRange (list);
-					}
-					else {
-						IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-						items.AddRange (loadedItems);
-					}
+					IList<ItemToolboxNode> loadedItems = loader.Load (ctx, fileName);
+					items.AddRange (loadedItems);
 				}
 				catch (Exception ex) {
 					// Ignore
@@ -228,30 +244,6 @@ namespace MonoDevelop.DesignerSupport
 			return items;
 		}
 		
-		internal IList<ItemToolboxNode> InternalGetFileItems (string fileName)
-		{
-			// Method called by from the external process to get the items of
-			// a file. Only loaders requiring isolation are used here, since
-			// the other loaders are handled in the main process.
-			
-			List<ItemToolboxNode> items = new List<ItemToolboxNode> ();
-			foreach (IToolboxLoader loader in loaders) {
-				if (!fileName.EndsWith (loader.FileTypes[0]))
-					continue;
-				
-				try {
-					if (loader.ShouldIsolate) {
-						IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-						items.AddRange (loadedItems);
-					}
-				}
-				catch {
-					// Ignore
-				}
-			}
-			return items;
-		}
-		
 		~ToolboxService ()
 		{
 			AddinManager.RemoveExtensionNodeHandler (toolboxLoaderPath, OnLoaderExtensionChanged);
@@ -286,7 +278,7 @@ namespace MonoDevelop.DesignerSupport
 				try {
 					config.SaveContents (ToolboxConfigFile);
 				} catch (Exception ex) {
-					LoggingService.LogError (ex.ToString ());
+					LoggingService.LogError ("Error saving toolbox configuration.", ex);
 				}
 			}
 		}
@@ -295,7 +287,7 @@ namespace MonoDevelop.DesignerSupport
 		
 		#region Actions and properties
 		
-		public IList GetCurrentToolboxItems ()
+		public IList<ItemToolboxNode> GetCurrentToolboxItems ()
 		{
 			return GetToolboxItems (CurrentConsumer);
 		}
@@ -309,7 +301,7 @@ namespace MonoDevelop.DesignerSupport
 		}
 		
 		
-		public IList GetToolboxItems (IToolboxConsumer consumer)
+		public IList<ItemToolboxNode> GetToolboxItems (IToolboxConsumer consumer)
 		{
 			List<ItemToolboxNode> arr = new List<ItemToolboxNode> ();
 			Hashtable nodes = new Hashtable ();
@@ -326,7 +318,7 @@ namespace MonoDevelop.DesignerSupport
 				
 				//merge the list of dynamic items from each provider
 				foreach (IToolboxDynamicProvider prov in dynamicProviders) {
-					IList<ItemToolboxNode> dynItems = prov.GetDynamicItems (consumer);
+					IEnumerable<ItemToolboxNode> dynItems = prov.GetDynamicItems (consumer);
 					if (dynItems != null) {
 						foreach (ItemToolboxNode node in dynItems) {
 							if (!nodes.Contains (node)) {
@@ -337,8 +329,8 @@ namespace MonoDevelop.DesignerSupport
 					}
 				}
 			}
+			//arr.Sort ();
 			
-			arr.Sort ();
 			return arr;
 		}
 		
@@ -386,8 +378,17 @@ namespace MonoDevelop.DesignerSupport
 			if ((CurrentConsumer == null) || (selectedItem == null))
 				return;
 			
-			CurrentConsumer.DragItem (selectedItem, source, ctx);
-			OnToolboxUsed (CurrentConsumer, selectedItem);
+			try {
+				CurrentConsumer.DragItem (selectedItem, source, ctx);
+				OnToolboxUsed (CurrentConsumer, selectedItem);
+			} catch (Exception ex) {
+				MonoDevelop.Core.LoggingService.LogError ("Error dragging toolbox item.", ex);
+				//run this dialog on a timeout so it doesn't block the drag completing
+				GLib.Timeout.Add (100, delegate {
+					MonoDevelop.Core.Gui.MessageService.ShowException (ex, "Error dragging toolbox item.");
+					return false;
+				});
+			}
 		}
 		
 		#endregion
@@ -395,24 +396,64 @@ namespace MonoDevelop.DesignerSupport
 		#region Change notification
 		
 		Document oldActiveDoc =  null;
+		Project oldProject = null;
+		bool configChanged;
+		IToolboxDynamicProvider viewProvider = null;
+		
 		void onActiveDocChanged (object o, EventArgs e)
 		{
 			if (oldActiveDoc != null)
 				oldActiveDoc.ViewChanged -= OnViewChanged;
-			if (IdeApp.Workbench.ActiveDocument != null)
-				IdeApp.Workbench.ActiveDocument.ViewChanged += OnViewChanged;
+			if (oldProject != null)
+				oldProject.Modified -= onProjectConfigChanged;
+			
 			oldActiveDoc = IdeApp.Workbench.ActiveDocument;
+			oldProject = oldActiveDoc != null?
+				oldActiveDoc.Project : null;
+			
+			if (oldActiveDoc != null)
+				oldActiveDoc.ViewChanged += OnViewChanged;					
+			if (oldProject != null)
+				oldProject.Modified += onProjectConfigChanged;
 			
 			OnViewChanged (null, null);
 		}
 		
 		void OnViewChanged (object sender, EventArgs args)
 		{
+			if (viewProvider != null) {
+				this.dynamicProviders.Remove (viewProvider);
+				viewProvider.ItemsChanged -= OnProviderItemsChanged;
+			}
+			
 			//only treat active ViewContent as a Toolbox consumer if it implements IToolboxConsumer
-			if (IdeApp.Workbench.ActiveDocument != null)
+			if (IdeApp.Workbench.ActiveDocument != null) {
 				CurrentConsumer = IdeApp.Workbench.ActiveDocument.ActiveView as IToolboxConsumer;
-			else
+				viewProvider    = IdeApp.Workbench.ActiveDocument.ActiveView as IToolboxDynamicProvider;
+				if (viewProvider != null)  {
+					this.dynamicProviders.Add (viewProvider);
+					viewProvider.ItemsChanged += OnProviderItemsChanged;
+					OnToolboxContentsChanged ();
+				}
+			} else {
 				CurrentConsumer = null;
+				viewProvider = null;
+			}
+		}
+		
+		//changing project settings could cause the toolbox contents to change
+		void onProjectConfigChanged (object sender, EventArgs args)
+		{
+			if (configChanged)
+				return;
+			
+			//project usually broadcasts many changes at once, so try to group them
+			configChanged = true;
+			GLib.Timeout.Add (500, delegate {
+				configChanged = false;
+				OnToolboxContentsChanged ();
+				return false;
+			});
 		}
 		
 		protected virtual void OnToolboxContentsChanged ()
@@ -448,8 +489,11 @@ namespace MonoDevelop.DesignerSupport
 		
 		#region Filtering
 		
-		public virtual bool IsSupported (ItemToolboxNode node, IToolboxConsumer consumer)
+		public bool IsSupported (ItemToolboxNode node, IToolboxConsumer consumer)
 		{
+			if (consumer is ICustomFilteringToolboxConsumer)
+				return ((ICustomFilteringToolboxConsumer)consumer).SupportsItem (node);
+			
 			//if something has no filters it is more useful and efficient
 			//to show it for everything than to show it for nothing
 			if (node.ItemFilters == null || node.ItemFilters.Count == 0)
@@ -474,7 +518,8 @@ namespace MonoDevelop.DesignerSupport
 		}
 		
 		//evaluate a filter attribute against a list, and check whether permitted
-		private bool FilterPermitted (ItemToolboxNode node, ToolboxItemFilterAttribute desFa, ICollection filterAgainst, IToolboxConsumer consumer)
+		private bool FilterPermitted (ItemToolboxNode node, ToolboxItemFilterAttribute desFa, 
+		    ICollection<ToolboxItemFilterAttribute> filterAgainst, IToolboxConsumer consumer)
 		{
 			switch (desFa.FilterType) {
 				case ToolboxItemFilterType.Allow:
@@ -509,24 +554,30 @@ namespace MonoDevelop.DesignerSupport
 			
 			// Get the list of assemblies that need to be updated
 			
-			Hashtable files = new Hashtable ();
+			HashSet<string> files = new HashSet<string> ();
 			List<ComponentIndexFile> toupdate = new List<ComponentIndexFile> ();
 			List<ComponentIndexFile> todelete = new List<ComponentIndexFile> ();
 			foreach (ComponentIndexFile ia in index.Files) {
-				files [ia.FileName] = ia.FileName;
+				files.Add (ia.FileName);
 				if (!File.Exists (ia.FileName))
 					todelete.Add (ia);
 				if (ia.NeedsUpdate)
 					toupdate.Add (ia);
+				if (monitor.IsCancelRequested)
+					return index;
 			}
 			
 			// Look for new assemblies
 			
-			foreach (string filePath in Runtime.SystemAssemblyService.GetAssemblyPaths (ClrVersion.Default)) {
-				if (!files.Contains (filePath)) {
-					ComponentIndexFile c = new ComponentIndexFile (filePath);
-					index.Files.Add (c);
-					toupdate.Add (c);
+			foreach (TargetRuntime runtime in Runtime.SystemAssemblyService.GetTargetRuntimes ()) {
+				foreach (SystemAssembly asm in runtime.AssemblyContext.GetAssemblies ()) {
+					if (files.Add (asm.Location)) {
+						ComponentIndexFile c = new ComponentIndexFile (asm.Location);
+						index.Files.Add (c);
+						toupdate.Add (c);
+					}
+					if (monitor.IsCancelRequested)
+						return index;
 				}
 			}
 			
@@ -536,11 +587,18 @@ namespace MonoDevelop.DesignerSupport
 			
 			if (toupdate.Count > 0) {
 				monitor.BeginTask (GettextCatalog.GetString ("Looking for components..."), toupdate.Count);
-				foreach (ComponentIndexFile ia in toupdate) {
-					ia.Update ();
-					monitor.Step (1);
+				LoaderContext ctx = new LoaderContext ();
+				try {
+					foreach (ComponentIndexFile ia in toupdate) {
+						ia.Update (ctx);
+						monitor.Step (1);
+						if (monitor.IsCancelRequested)
+							return index;
+					}
+				} finally {
+					ctx.Dispose ();
+					monitor.EndTask ();
 				}
-				monitor.EndTask ();
 			}
 			
 			if (toupdate.Count > 0 || todelete.Count > 0)
@@ -566,7 +624,7 @@ namespace MonoDevelop.DesignerSupport
 			if (!File.Exists (ToolboxIndexFile))
 				return new ComponentIndex ();
 			
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
+			XmlDataSerializer ser = new XmlDataSerializer (IdeApp.Services.ProjectService.DataContext);
 			try {
 				using (StreamReader sr = new StreamReader (ToolboxIndexFile)) {
 					return (ComponentIndex) ser.Deserialize (sr, typeof(ComponentIndex));
@@ -581,7 +639,7 @@ namespace MonoDevelop.DesignerSupport
 		
 		public void Save ()
 		{
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
+			XmlDataSerializer ser = new XmlDataSerializer (IdeApp.Services.ProjectService.DataContext);
 			try {
 				using (StreamWriter sw = new StreamWriter (ToolboxIndexFile)) {
 					ser.Serialize (sw, this, typeof(ComponentIndex));
@@ -596,7 +654,12 @@ namespace MonoDevelop.DesignerSupport
 		public ComponentIndexFile AddFile (string file)
 		{
 			ComponentIndexFile cf = new ComponentIndexFile (file);
-			cf.Update ();
+			LoaderContext ctx = new LoaderContext ();
+			try {
+				cf.Update (ctx);
+			} finally {
+				ctx.Dispose ();
+			}
 			if (cf.Components.Count == 0)
 				return null;
 			else {
@@ -668,9 +731,9 @@ namespace MonoDevelop.DesignerSupport
 			}
 		}
 
-		public void Update ()
+		public void Update (LoaderContext ctx)
 		{
-			IList<ItemToolboxNode> items = DesignerSupport.Service.ToolboxService.GetFileItems (fileName);
+			IList<ItemToolboxNode> items = DesignerSupport.Service.ToolboxService.GetFileItems (ctx, fileName);
 			
 			location = null;
 			this.timestamp = GetFileTimestamp (fileName);
@@ -678,7 +741,7 @@ namespace MonoDevelop.DesignerSupport
 			
 			// Set the location field only if this is a widget library
 			if (entries.Count > 0 && Runtime.SystemAssemblyService.GetPackageFromPath (fileName) != null)
-				location = Runtime.SystemAssemblyService.GetAssemblyFullName (fileName);
+				location = Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (fileName, TargetFramework.Default);
 		}
 		
 		string GetFileTimestamp (string file)
@@ -688,23 +751,6 @@ namespace MonoDevelop.DesignerSupport
 		}
 	}
 	
-	[MonoDevelop.Core.Execution.AddinDependency ("MonoDevelop.DesignerSupport")] 
-	class ExternalFileLoader: RemoteProcessObject
-	{
-		public string LoadFile (string file)
-		{
-			Gtk.Application.Init ();
-			ToolboxService service = new ToolboxService ();
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
-			ToolboxList tl = new ToolboxList ();
-			IList<ItemToolboxNode> list = service.InternalGetFileItems (file);
-			tl.AddRange (list);
-			StringWriter sw = new StringWriter ();
-			ser.Serialize (sw, tl);
-			return sw.ToString ();
-		}
-	}
-	
 	public delegate void ToolboxConsumerChangedHandler (object sender, ToolboxConsumerChangedEventArgs e);
 	
 	public class ToolboxConsumerChangedEventArgs
@@ -758,4 +804,5 @@ namespace MonoDevelop.DesignerSupport
 			get { return consumer; }
 		}
 	}
+
 }
