@@ -35,23 +35,18 @@ using MonoDevelop.Ide.Codons;
 
 namespace MonoDevelop.Gettext
 {	
-	public class GettextEditorDisplayBinding : IDisplayBinding
+	public class GettextEditorDisplayBinding : DefaultDisplayBinding
 	{
-		public string DisplayName 
-		{
+		public override string Name {
 			get { return GettextCatalog.GetString ("Gettext Editor"); }
 		}
 		
-		public bool CanCreateContentForFile (string fileName)
+		public override bool CanCreateContentForUri (string fileName)
 		{
 			return Path.GetExtension (fileName).Equals (".po", StringComparison.OrdinalIgnoreCase);
 		}
-		public bool CanCreateContentForMimeType (string mimeType)
-		{
-			return mimeType == "text/x-gettext-translation";
-		}
 		
-		public IViewContent CreateContentForFile (string fileName)
+		public override IViewContent CreateContentForUri (string fileName)
 		{
 			foreach (TranslationProject tp in IdeApp.Workspace.GetAllSolutionItems<TranslationProject>  ())
 				if (tp.BaseDirectory == Path.GetDirectoryName (fileName))
@@ -59,16 +54,5 @@ namespace MonoDevelop.Gettext
 			
 			return new Editor.CatalogEditorView (null, fileName);
 		}
-		
-		public IViewContent CreateContentForMimeType (string mimeType, Stream content)
-		{
-//			StreamReader sr = new StreamReader (content);
-//			string text = sr.ReadToEnd ();
-//			sr.Close ();
-			
-			// text/x-gettext-translation
-			// TODO: implement such loading
-			return null;
-		}
 	}
 }
