@@ -33,6 +33,7 @@ using Gdk;
 using Glade;
 using MonoDevelop.Projects.Dom;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Core.Gui;
 
 namespace MonoDevelop.GtkCore.Dialogs
 {
@@ -63,13 +64,14 @@ namespace MonoDevelop.GtkCore.Dialogs
 			treeClasses.AppendColumn (column);
 			
 			foreach (IType cls in classes) {
-				Pixbuf pic = IdeApp.Services.Resources.GetIcon (cls.StockIcon);
+				Pixbuf pic = ImageService.GetPixbuf (cls.StockIcon);
 				store.AppendValues (pic, cls.FullName);
 			}
 		}
 		
 		public bool Run ()
 		{
+			dialog.TransientFor = MonoDevelop.Ide.Gui.IdeApp.Workbench.RootWindow;
 			return dialog.Run () == (int) ResponseType.Ok;
 		}
 		
