@@ -43,15 +43,15 @@ using MonoDevelop.GtkCore.Dialogs;
 
 namespace MonoDevelop.GtkCore.GuiBuilder
 {
-	public class ActionGroupDisplayBinding: IDisplayBinding
+	public class ActionGroupDisplayBinding : DefaultDisplayBinding
 	{
 		bool excludeThis = false;
 		
-		public string DisplayName {
+		public override string Name {
 			get { return "Action Group Editor"; }
 		}
 		
-		public virtual bool CanCreateContentForFile (string fileName)
+		public override bool CanCreateContentForUri (string fileName)
 		{
 			if (excludeThis)
 				return false;
@@ -62,34 +62,23 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				return false;
 			
 			excludeThis = true;
-			IDisplayBinding db = IdeApp.Workbench.DisplayBindings.GetBindingPerFileName (fileName);
+			IDisplayBinding db = DisplayBindingService.GetBindingForUri (fileName);
 			excludeThis = false;
 			return db != null;
 		}
-
-		public virtual bool CanCreateContentForMimeType (string mimetype)
-		{
-			return false;
-		}
 		
-		public virtual IViewContent CreateContentForFile (string fileName)
+		public override IViewContent CreateContentForUri (string fileName)
 		{
 			excludeThis = true;
-			IDisplayBinding db = IdeApp.Workbench.DisplayBindings.GetBindingPerFileName (fileName);
+			IDisplayBinding db = DisplayBindingService.GetBindingForUri (fileName);
 			
 			Project project = IdeApp.Workspace.GetProjectContainingFile (fileName);
 			GtkDesignInfo info = GtkDesignInfo.FromProject ((DotNetProject) project);
 			
-			ActionGroupView view = new ActionGroupView (db.CreateContentForFile (fileName), GetActionGroup (fileName), info.GuiBuilderProject);
+			ActionGroupView view = new ActionGroupView (db.CreateContentForUri (fileName), GetActionGroup (fileName), info.GuiBuilderProject);
 			excludeThis = false;
 			return view;
 		}
-		
-		public virtual IViewContent CreateContentForMimeType (string mimeType, System.IO.Stream content)
-		{
-			return null;
-		}
-		
 		Stetic.ActionGroupInfo GetActionGroup (string file)
 		{
 			Project project = IdeApp.Workspace.GetProjectContainingFile (file);
