@@ -50,7 +50,9 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 {
 	public class GuiBuilderProject
 	{
-		ArrayList formInfos;
+		internal object MemoryProbe = Counters.GuiProjectsInMemory.CreateMemoryProbe ();
+		
+		List<GuiBuilderWindow> formInfos;
 		Stetic.Project gproject;
 		DotNetProject project;
 		string fileName;
@@ -73,6 +75,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			this.fileName = fileName;
 			this.project = project;
+			Counters.GuiProjectsLoaded++;
 		}
 		
 		void Load ()
@@ -81,7 +84,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				return;
 			
 			gproject = GuiBuilderService.SteticApp.CreateProject ();
-			formInfos = new ArrayList ();
+			formInfos = new List<GuiBuilderWindow> ();
 			
 			if (!System.IO.File.Exists (fileName)) {
 				// Regenerate the gtk-gui folder if the stetic project
@@ -96,6 +99,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				hasError = true;
 			}
 
+			Counters.SteticProjectsLoaded++;
 			gproject.ResourceProvider = GtkDesignInfo.FromProject (project).ResourceProvider;
 			gproject.WidgetAdded += OnAddWidget;
 			gproject.WidgetRemoved += OnRemoveWidget;
@@ -124,6 +128,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			if (gproject == null)
 				return;
 
+			Counters.SteticProjectsLoaded--;
+			
 			if (Unloaded != null)
 				Unloaded (this, EventArgs.Empty);
 			if (formInfos != null) {
@@ -217,7 +223,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 		}
 		
-		public ICollection Windows {
+		public ICollection<GuiBuilderWindow> Windows {
 			get {
 				Load ();
 				return formInfos; 
@@ -230,6 +236,9 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public void Dispose ()
 		{
+			if (disposed)
+				return;
+			Counters.GuiProjectsLoaded--;
 			disposed = true;
 			if (watcher != null)
 				watcher.Dispose ();
@@ -316,7 +325,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		void OnFileAdded (object sender, ProjectFileEventArgs args)
 		{
-			ParsedDocument doc = ProjectDomService.GetParsedDocument (args.ProjectFile.Name);
+			
+			ParsedDocument doc = ProjectDomService.GetParsedDocument (ProjectDomService.GetProjectDom (args.Project), args.ProjectFile.Name);
 			if (doc == null || doc.CompilationUnit == null)
 				return;
 
@@ -339,7 +349,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			ArrayList toDelete = new ArrayList ();
 
-			ParsedDocument doc = ProjectDomService.GetParsedDocument (args.ProjectFile.Name);
+			ParsedDocument doc = ProjectDomService.GetParsedDocument (ProjectDomService.GetProjectDom (args.Project), args.ProjectFile.Name);
 			if (doc == null || doc.CompilationUnit == null)
 				return;
 
@@ -406,6 +416,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				new Gtk.FileChooserDialog ("Open Glade File", null, Gtk.FileChooserAction.Open,
 						       Gtk.Stock.Cancel, Gtk.ResponseType.Cancel,
 						       Gtk.Stock.Open, Gtk.ResponseType.Ok);
+			dialog.TransientFor = MonoDevelop.Ide.Gui.IdeApp.Workbench.RootWindow;
 			int response = dialog.Run ();
 			if (response == (int)Gtk.ResponseType.Ok) {
 				SteticProject.ImportGlade (dialog.Filename);
@@ -424,8 +435,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			return null;
 		}
-		
-		public GuiBuilderWindow GetWindowForFile (string fileName)
+
+		public GuiBuilderWindow GetWindowForFile (FilePath fileName)
 		{
 			if (Windows != null) {
 				foreach (GuiBuilderWindow win in Windows) {
@@ -446,8 +457,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			return null;
 		}
-		
-		public Stetic.ActionGroupInfo GetActionGroupForFile (string fileName)
+
+		public Stetic.ActionGroupInfo GetActionGroupForFile (FilePath fileName)
 		{
 			foreach (Stetic.ActionGroupInfo group in SteticProject.ActionGroups) {
 				if (fileName == GetSourceCodeFile (group, true))
@@ -460,13 +471,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			return SteticProject.GetActionGroup (name);
 		}
-		
-		public string GetSourceCodeFile (Stetic.ProjectItemInfo obj)
+
+		public FilePath GetSourceCodeFile (Stetic.ProjectItemInfo obj)
 		{
 			return GetSourceCodeFile (obj, true);
 		}
-		
-		public string GetSourceCodeFile (Stetic.ProjectItemInfo obj, bool getUserClass)
+
+		public FilePath GetSourceCodeFile (Stetic.ProjectItemInfo obj, bool getUserClass)
 		{
 			IType cls = GetClass (obj, getUserClass);
 			if (cls != null && cls.CompilationUnit != null)
@@ -487,7 +498,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public IType FindClass (string className, bool getUserClass)
 		{
-			string gui_folder = GtkDesignInfo.FromProject (project).GtkGuiFolder;
+			FilePath gui_folder = GtkDesignInfo.FromProject (project).GtkGuiFolder;
 			ProjectDom ctx = GetParserContext ();
 			if (ctx == null)
 				return null;
@@ -500,13 +511,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 						// Return this class only if it is declared outside the gtk-gui
 						// folder. Generated partial classes will be ignored.
 						foreach (IType part in cls.Parts) {
-							if (part.CompilationUnit != null && !part.CompilationUnit.FileName.StartsWith (gui_folder)) {
+							if (part.CompilationUnit != null && !part.CompilationUnit.FileName.IsChildPathOf (gui_folder)) {
 								return part;
 							}
 						}
 						continue;
 					}
-					if (getUserClass && cls.CompilationUnit != null && !string.IsNullOrEmpty (cls.CompilationUnit.FileName) && cls.CompilationUnit.FileName.StartsWith (gui_folder))
+					if (getUserClass && cls.CompilationUnit != null && !string.IsNullOrEmpty (cls.CompilationUnit.FileName) && cls.CompilationUnit.FileName.IsChildPathOf (gui_folder))
 						continue;
 					return cls;
 				}
@@ -551,6 +562,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			ReferenceManager refmgr = new ReferenceManager (project);
 			string target_version = refmgr.TargetGtkVersion;
+			refmgr.Dispose ();
 			
 			// Make sure the target gtk version is properly set
 			if (gproject.TargetGtkVersion != target_version) {
@@ -607,7 +619,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			if (hasError)
 				return files;
 
-			IDotNetLanguageBinding binding = MonoDevelop.Projects.Services.Languages.GetBindingPerLanguageName (project.LanguageName) as IDotNetLanguageBinding;
+			IDotNetLanguageBinding binding = LanguageBindingService.GetBindingPerLanguageName (project.LanguageName) as IDotNetLanguageBinding;
 			string path = Path.Combine (guiFolder, binding.GetFileName ("generated"));
 			if (!System.IO.File.Exists (path)) {
 				// Generate an empty build class
