@@ -36,7 +36,8 @@ using System.CodeDom.Compiler;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Parser;
+using MonoDevelop.Projects.Dom;
+using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Projects.CodeGeneration;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Text;
@@ -49,7 +50,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 {
 	class GuiBuilderService
 	{
-		static GuiBuilderProjectPad widgetTreePad;
 		static string GuiBuilderLayout = "GUI Builder";
 		
 #if DUMMY_STRINGS_FOR_TRANSLATION_DO_NOT_COMPILE
@@ -76,9 +76,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			if (IdeApp.Workbench == null)
 				return;
 			IdeApp.Workbench.ActiveDocumentChanged += new EventHandler (OnActiveDocumentChanged);
-			IdeApp.ProjectOperations.StartBuild += OnBeforeCompile;
 			IdeApp.ProjectOperations.EndBuild += OnProjectCompiled;
-			IdeApp.ProjectOperations.ParserDatabase.AssemblyInformationChanged += (AssemblyInformationEventHandler) DispatchService.GuiDispatch (new AssemblyInformationEventHandler (OnAssemblyInfoChanged));
+//			IdeApp.Workspace.ParserDatabase.AssemblyInformationChanged += (AssemblyInformationEventHandler) DispatchService.GuiDispatch (new AssemblyInformationEventHandler (OnAssemblyInfoChanged));
 			
 			IdeApp.Exited += delegate {
 				if (steticApp != null) {
@@ -88,17 +87,14 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			};
 		}
 		
-		internal static GuiBuilderProjectPad WidgetTreePad {
-			get { return widgetTreePad; }
-			set { widgetTreePad = value; }
-		}
-		
 		public static Stetic.Application SteticApp {
 			get {
 				if (steticApp == null) {
-					steticApp = new Stetic.Application (IsolationMode);
+					steticApp = Stetic.ApplicationFactory.CreateApplication (Stetic.IsolationMode.None);
 					steticApp.AllowInProcLibraries = false;
 					steticApp.ShowNonContainerWarning = PropertyService.Get ("MonoDevelop.GtkCore.ShowNonContainerWarning", true);
+					steticApp.MimeResolver = OnMimeResolve;
+					steticApp.ShowUrl = OnShowUrl;
 					steticApp.WidgetLibraryResolver = OnAssemblyResolve;
 				}
 				return steticApp;
@@ -107,7 +103,17 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		static string OnAssemblyResolve (string assemblyName)
 		{
-			return Runtime.SystemAssemblyService.GetAssemblyLocation (assemblyName);
+			return Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyLocation (assemblyName, null);
+		}
+		
+		static string OnMimeResolve (string url)
+		{
+			return MonoDevelop.Core.Gui.DesktopService.GetMimeTypeForUri (url);
+		}
+		
+		static void OnShowUrl (string url)
+		{
+			MonoDevelop.Core.Gui.DesktopService.ShowUrl (url);
 		}
 		
 		internal static void StoreConfiguration ()
@@ -115,20 +121,19 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			PropertyService.Set ("MonoDevelop.GtkCore.ShowNonContainerWarning", steticApp.ShowNonContainerWarning);
 			PropertyService.SaveProperties ();
 		}
-
 		
-		public static GuiBuilderProject GetGuiBuilderProject (Project project)
-		{
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
-			if (info != null)
-				return info.GuiBuilderProject;
-			else
-				return null;
+		public static bool AutoSwitchGuiLayout {
+			get {
+				return PropertyService.Get ("MonoDevelop.GtkCore.AutoSwitchGuiLayout", false);
+			}
+			set {
+				PropertyService.Set ("MonoDevelop.GtkCore.AutoSwitchGuiLayout", value);
+			}
 		}
 		
 		public static ActionGroupView OpenActionGroup (Project project, Stetic.ActionGroupInfo group)
 		{
-			GuiBuilderProject p = GetGuiBuilderProject (project);
+			GuiBuilderProject p = GtkDesignInfo.FromProject (project).GuiBuilderProject ;
 			string file = p != null ? p.GetSourceCodeFile (group) : null;
 			if (file == null) {
 				file = ActionGroupDisplayBinding.BindToClass (project, group);
@@ -169,7 +174,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		static void SetDesignerLayout ()
 		{
-			if (IdeApp.Workbench.CurrentLayout != GuiBuilderLayout) {
+			if (AutoSwitchGuiLayout && IdeApp.Workbench.CurrentLayout != GuiBuilderLayout) {
 				bool exists = Array.IndexOf (IdeApp.Workbench.Layouts, GuiBuilderLayout) != -1;
 				defaultLayout = IdeApp.Workbench.CurrentLayout;
 				IdeApp.Workbench.CurrentLayout = GuiBuilderLayout;
@@ -184,30 +189,21 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		static void RestoreLayout ()
 		{
-			if (defaultLayout != null) {
+			if (AutoSwitchGuiLayout && defaultLayout != null) {
 				IdeApp.Workbench.CurrentLayout = defaultLayout;
 				defaultLayout = null;
 			}
 		}
 		
-		static void OnBeforeCompile (object s, BuildEventArgs args)
-		{
-			if (IdeApp.ProjectOperations.CurrentOpenCombine == null)
-				return;
-
-			// Generate stetic files for all modified projects
-			GtkProjectServiceExtension.GenerateSteticCode = true;
-		}
-
 		static void OnProjectCompiled (object s, BuildEventArgs args)
 		{
 			if (args.Success) {
 				// Unload stetic projects which are not currently
 				// being used by the IDE. This will avoid unnecessary updates.
-				if (IdeApp.ProjectOperations.CurrentOpenCombine != null) {
-					foreach (Project prj in IdeApp.ProjectOperations.CurrentOpenCombine.GetAllProjects ()) {
-						GtkDesignInfo info = GtkCoreService.GetGtkInfo (prj);
-						if (info != null && !HasOpenDesigners (prj, false)) {
+				if (IdeApp.Workspace.IsOpen) {
+					foreach (Project prj in IdeApp.Workspace.GetAllProjects ()) {
+						GtkDesignInfo info = GtkDesignInfo.FromProject (prj);
+						if (!HasOpenDesigners (prj, false)) {
 							info.ReloadGuiBuilderProject ();
 						}
 					}
@@ -217,7 +213,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			else {
 				// Some gtk# packages don't include the .pc file unless you install gtk-sharp-devel
-				if (Runtime.SystemAssemblyService.GetPackage ("gtk-sharp-2.0") == null) {
+				if (Runtime.SystemAssemblyService.DefaultAssemblyContext.GetPackage ("gtk-sharp-2.0") == null) {
 					string msg = GettextCatalog.GetString ("ERROR: MonoDevelop could not find the Gtk# 2.0 development package. Compilation of projects depending on Gtk# libraries will fail. You may need to install development packages for gtk-sharp-2.0.");
 					args.ProgressMonitor.Log.WriteLine ();
 					args.ProgressMonitor.Log.WriteLine (msg);
@@ -234,10 +230,10 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			return false;
 		}
 		
-		static void OnAssemblyInfoChanged (object s, AssemblyInformationEventArgs args)
-		{
+		//static void OnAssemblyInfoChanged (object s, AssemblyInformationEventArgs args)
+//		{
 			//SteticApp.UpdateWidgetLibraries (false);
-		}
+//		}
 
 		internal static void AddCurrentWidgetToClass ()
 		{
@@ -259,14 +255,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public static void ImportGladeFile (Project project)
 		{
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
-			if (info == null) info = GtkCoreService.EnableGtkSupport (project);
+			GtkDesignInfo info = GtkDesignInfo.FromProject (project);
 			info.GuiBuilderProject.ImportGladeFile ();
 		}
 		
 		public static string GetBuildCodeFileName (Project project, string componentName)
 		{
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
+			GtkDesignInfo info = GtkDesignInfo.FromProject (project);
 			return Path.Combine (info.GtkGuiFolder, componentName + Path.GetExtension (info.SteticGeneratedFile));
 		}
 		
@@ -294,8 +289,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				name = name.Substring (i+1);
 			}
 			
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
-			
 			if (saveToFile && !overwrite && File.Exists (fileName))
 				return fileName;
 			
@@ -304,7 +297,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			CodeCompileUnit cu = new CodeCompileUnit ();
 			
-			if (info.GeneratePartialClasses) {
+			if (project.UsePartialTypes) {
 				CodeNamespace cns = new CodeNamespace (ns);
 				cu.Namespaces.Add (cns);
 				
@@ -351,37 +344,46 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				fileStream.Close ();
 			}
 
-			if (IdeApp.ProjectOperations.ParserDatabase.IsLoaded (project)) {
+			if (ProjectDomService.HasDom (project)) {
 				// Only update the parser database if the project is actually loaded in the IDE.
-				if (saveToFile)
-					IdeApp.ProjectOperations.ParserDatabase.GetProjectParserContext (project).UpdateDatabase ();
+				if (saveToFile) {
+					ProjectDomService.Parse (project, fileName, "");
+					FileService.NotifyFileChanged (fileName);
+				}
 				else
-					IdeApp.ProjectOperations.ParserDatabase.UpdateFile (project, fileName, ((StringWriter)fileStream).ToString ());
+					ProjectDomService.Parse (project, fileName, "", ((StringWriter)fileStream).ToString ());
 			}
 
-			FileService.NotifyFileChanged (fileName);
 			return fileName;
 		}
 		
 		
-		public static Stetic.CodeGenerationResult GenerateSteticCode (IProgressMonitor monitor, Project prj)
+		public static Stetic.CodeGenerationResult GenerateSteticCode (IProgressMonitor monitor, DotNetProject project, ConfigurationSelector configuration)
 		{
-			if (generating)
+			if (generating || !GtkDesignInfo.HasDesignedObjects (project))
 				return null;
 
-			DotNetProject project = prj as DotNetProject;
-			if (project == null)
-				return null;
-				
-			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
-			if (info == null)
-				return null;
+			GtkDesignInfo info = GtkDesignInfo.FromProject (project);
+
+			DateTime last_gen_time = File.Exists (info.SteticGeneratedFile) ? File.GetLastWriteTime (info.SteticGeneratedFile) : DateTime.MinValue;
 			
-			// Check if the stetic file has been modified since last generation
-			if (File.Exists (info.SteticGeneratedFile) && File.Exists (info.SteticFile)) {
-				if (File.GetLastWriteTime (info.SteticGeneratedFile) > File.GetLastWriteTime (info.SteticFile))
-					return null;
+			bool ref_changed = false;
+			foreach (ProjectReference pref in project.References) {
+				if (!pref.IsValid)
+					continue;
+				foreach (string filename in pref.GetReferencedFileNames (configuration)) {
+					if (File.GetLastWriteTime (filename) > last_gen_time) {
+						ref_changed = true;
+						break;
+					}
+				}
+				if (ref_changed)
+					break;
 			}
+
+			// Check if generated code is already up to date.
+			if (!ref_changed && last_gen_time >= File.GetLastWriteTime (info.SteticFile))
+				return null;
 			
 			if (info.GuiBuilderProject.HasError) {
 				monitor.ReportError (GettextCatalog.GetString ("GUI code generation failed for project '{0}'. The file '{1}' could not be loaded.", project.Name, info.SteticFile), null);
@@ -396,15 +398,10 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			// Make sure the referenced assemblies are up to date. It is necessary to do
 			// it now since they may contain widget libraries.
-			prj.CopyReferencesToOutputPath (false);
+			project.CopySupportFiles (monitor, configuration);
 			
 			info.GuiBuilderProject.UpdateLibraries ();
 			
-			if (info.IsWidgetLibrary) {
-				// Make sure the widget export file is up to date.
-				GtkCoreService.UpdateObjectsFile (project);
-			}
-
 			ArrayList projects = new ArrayList ();
 			projects.Add (info.GuiBuilderProject.File);
 			
@@ -421,7 +418,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 						// Generate the code in another process if stetic is not isolated
 						CodeGeneratorProcess cob = (CodeGeneratorProcess) Runtime.ProcessService.CreateExternalProcessObject (typeof (CodeGeneratorProcess), false);
 						using (cob) {
-							generationResult = cob.GenerateCode (projects, info.GenerateGettext, info.GettextClass, info.GeneratePartialClasses);
+							generationResult = cob.GenerateCode (projects, info.GenerateGettext, info.GettextClass, project.UsePartialTypes);
 						}
 					} catch (Exception ex) {
 						generatedException = ex;
@@ -441,7 +438,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 					Stetic.GenerationOptions options = new Stetic.GenerationOptions ();
 					options.UseGettext = info.GenerateGettext;
 					options.GettextClass = info.GettextClass;
-					options.UsePartialClasses = info.GeneratePartialClasses;
+					options.UsePartialClasses = project.UsePartialTypes;
 					options.GenerateSingleFile = false;
 					generationResult = SteticApp.GenerateProjectCode (options, info.GuiBuilderProject.SteticProject);
 				} catch (Exception ex) {
@@ -471,11 +468,15 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 					fname = info.SteticGeneratedFile;
 				else
 					fname = Path.Combine (basePath, unit.Name) + ext;
-				StreamWriter fileStream = new StreamWriter (fname);
+				StringWriter sw = new StringWriter ();
 				try {
-					provider.GenerateCodeFromCompileUnit (unit, fileStream, new CodeGeneratorOptions ());
+					provider.GenerateCodeFromCompileUnit (unit, sw, new CodeGeneratorOptions ());
+					// Remove the runtime version number from the file. It may generate unnecessary
+					// version control changes when upgrading the mono version.
+					string content = sw.ToString ();
+					content = content.Replace ("Mono Runtime Version: " + Environment.Version, "");
+					File.WriteAllText (fname, content);
 				} finally {
-					fileStream.Close ();
 					FileService.NotifyFileChanged (fname);
 				}
 			}
@@ -483,7 +484,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			// Make sure the generated files are added to the project
 			if (info.UpdateGtkFolder ()) {
 				Gtk.Application.Invoke (delegate {
-					IdeApp.ProjectOperations.SaveProject (project);
+					IdeApp.ProjectOperations.Save (project);
 				});
 			}
 			
@@ -492,20 +493,21 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		internal static string ImportFile (Project prj, string file)
 		{
-			ProjectFile pfile = prj.ProjectFiles.GetFile (file);
+			ProjectFile pfile = prj.Files.GetFile (file);
 			if (pfile == null) {
 				string[] files = IdeApp.ProjectOperations.AddFilesToProject (prj, new string[] { file }, prj.BaseDirectory);
 				if (files.Length == 0)
 					return null;
 				if (files [0] == null)
 					return null;
-				pfile = prj.ProjectFiles.GetFile (files[0]);
+				pfile = prj.Files.GetFile (files[0]);
 			}
-			if (pfile.BuildAction == BuildAction.EmbedAsResource) {
-				if (!IdeApp.Services.MessageService.AskQuestion (GettextCatalog.GetString ("You are requesting the file '{0}' to be used as source for an image. However, this file is already added to the project as a resource. Are you sure you want to continue (the file will have to be removed from the resource list)?")))
+			if (pfile.BuildAction == BuildAction.EmbeddedResource) {
+				AlertButton embedButton = new AlertButton (GettextCatalog.GetString ("_Use as Source"));
+				if (MessageService.AskQuestion (GettextCatalog.GetString ("You are requesting the file '{0}' to be used as source for an image. However, this file is already added to the project as a resource. Are you sure you want to continue (the file will have to be removed from the resource list)?"), AlertButton.Cancel, embedButton) == embedButton)
 					return null;
 			}
-			pfile.BuildAction = BuildAction.FileCopy;
+			pfile.BuildAction = BuildAction.Content;
 			DeployProperties props = DeployService.GetDeployProperties (pfile);
 			props.UseProjectRelativePath = true;
 			return pfile.FilePath;
@@ -520,7 +522,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			Gtk.Application.Init ();
 			
-			Stetic.Application app = new Stetic.Application (Stetic.IsolationMode.None);
+			Stetic.Application app = Stetic.ApplicationFactory.CreateApplication (Stetic.IsolationMode.None);
 			
 			Stetic.Project[] projects = new Stetic.Project [projectFiles.Count];
 			for (int n=0; n < projectFiles.Count; n++) {
