@@ -78,7 +78,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			get { return rootWidget.Name; }
 		}
 		
-		public string SourceCodeFile {
+		public FilePath SourceCodeFile {
 			get { return fproject.GetSourceCodeFile (rootWidget); }
 		}
 		
@@ -107,7 +107,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public bool BindToClass ()
 		{
-			if (SourceCodeFile != null)
+			if (SourceCodeFile != FilePath.Null)
 				return true;
 			
 			// Find the classes that could be bound to this design
@@ -246,7 +246,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 	
 	class OpenDocumentFileProvider: ITextFileProvider
 	{
-		public IEditableTextFile GetEditableTextFile (string filePath)
+		public IEditableTextFile GetEditableTextFile (FilePath filePath)
 		{
 			foreach (Document doc in IdeApp.Workbench.Documents) {
 				if (doc.FileName == filePath) {
