@@ -76,11 +76,11 @@ namespace MonoDevelop.GtkCore.NodeBuilders
 			label = win.Name;
 			
 //			if (win.RootWidget.IsWindow)
-//				icon = IdeApp.Services.Resources.GetIcon ("md-gtkcore-window");
+//				icon = ImageService.GetPixbuf ("md-gtkcore-window");
 			if (win.RootWidget.IsWindow)
-				icon = IdeApp.Services.Resources.GetIcon ("md-gtkcore-dialog", Gtk.IconSize.Menu);
+				icon = ImageService.GetPixbuf ("md-gtkcore-dialog", Gtk.IconSize.Menu);
 			else
-				icon = IdeApp.Services.Resources.GetIcon ("md-gtkcore-widget", Gtk.IconSize.Menu);
+				icon = ImageService.GetPixbuf ("md-gtkcore-widget", Gtk.IconSize.Menu);
 		}
 		
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -122,7 +122,7 @@ namespace MonoDevelop.GtkCore.NodeBuilders
 		public override void ActivateItem ()
 		{
 			GuiBuilderWindow w = (GuiBuilderWindow) CurrentNode.DataItem;
-			if (w.SourceCodeFile == null && !w.BindToClass ())
+			if (w.SourceCodeFile == FilePath.Null && !w.BindToClass ())
 				return;
 			
 			Document doc = IdeApp.Workbench.OpenDocument (w.SourceCodeFile, true);
