@@ -87,7 +87,7 @@ namespace Stetic.Wrapper
 			// Make sure children's IDs don't conflict with other widgets
 			// in the parent container.
 			if (!Loading)
-				ValidateChildNames (Wrapped);
+				ValidateChildNames (child);
 
 			Widget w = Widget.Lookup (child);
 			if (w != null) {
@@ -302,10 +302,15 @@ namespace Stetic.Wrapper
 				}
 			}
 			
-			ReadActionGroups (reader, elem);
-			ReadProperties (reader, elem);
-			ReadChildren (reader, elem);
-			DoSync ();
+			try {
+				Loading = true;
+				ReadActionGroups (reader, elem);
+				ReadProperties (reader, elem);
+				ReadChildren (reader, elem);
+				DoSync ();
+			} finally {
+				Loading = false;
+			}
 		}
 		
 		protected virtual void ReadChildren (ObjectReader reader, XmlElement elem)
@@ -1066,7 +1071,7 @@ namespace Stetic.Wrapper
 				// to be. (Eg, if you select a widget in a hidden window, the window
 				// should map. If you select a widget on a non-current notebook
 				// page, the notebook should switch pages, etc.)
-				if (selection.IsDrawable && Visible) {
+				if (selection.IsDrawable && selection.Visible) {
 					ShowSelectionBox (selection, dragHandles);
 				}
 				
