@@ -235,6 +235,11 @@ namespace Stetic.Wrapper {
 			}
 		}
 		
+		public string Name {
+			get { return Wrapped.Name; }
+			set { Wrapped.Name = value; EmitNotify ("Name"); }
+		}
+
 		public string MemberName {
 			get { return member != null ? member : ""; }
 			set { member = value; EmitNotify ("MemberName"); }
@@ -820,7 +825,7 @@ namespace Stetic.Wrapper {
 		
 		public bool InWindow {
 			get {
-				return this.GetTopLevel ().Wrapped is Gtk.Window;
+				return this.GetTopLevel ().Wrapped is TopLevelWindow;
 			}
 		}
 		
@@ -878,7 +883,8 @@ namespace Stetic.Wrapper {
 						
 					string on = oldName;
 					oldName = Wrapped.Name;
-					OnNameChanged (new WidgetNameChangedArgs (this, on, Wrapped.Name));
+					if (!Loading)
+						OnNameChanged (new WidgetNameChangedArgs (this, on, Wrapped.Name));
 					
 					// Keep the member name in sync with the widget name
 					if (on == MemberName)
@@ -890,7 +896,8 @@ namespace Stetic.Wrapper {
 					base.EmitNotify (propertyName);
 					string on = oldMemberName;
 					oldMemberName = MemberName;
-					OnMemberNameChanged (new WidgetNameChangedArgs (this, on, MemberName));
+					if (!Loading)
+						OnMemberNameChanged (new WidgetNameChangedArgs (this, on, MemberName));
 				}
 			}
 			else {
