@@ -39,9 +39,7 @@ namespace Stetic
 	{
 		ObjectWrapper wrapper;
 		Gtk.Widget preview;
-		Metacity.Preview metacityPreview;
 		ResizableFixed resizableFixed;
-		static Metacity.Theme theme;
 		
 		static IObjectViewer defaultObjectViewer;
 		
@@ -52,20 +50,6 @@ namespace Stetic
 			set { defaultObjectViewer = value; }
 		}
 		
-		internal Metacity.Theme Theme {
-			set { 
-				if (metacityPreview != null)
-					metacityPreview.Theme = value;
-			}
-		}
-				
-		internal string Title {
-			set { 
-				if (metacityPreview != null)
-					metacityPreview.Title = value;
-			}
-		}
-		
 		public object Selection {
 			get { 
 				IObjectSelection sel = resizableFixed.GetSelection ();
@@ -90,21 +74,18 @@ namespace Stetic
 			resizableFixed.ObjectViewer = defaultObjectViewer;
 			
 			wrapper = ObjectWrapper.Lookup (container);
-			Gtk.Window window = container as Gtk.Window;
+			TopLevelWindow window = container as TopLevelWindow;
 			
 			if (window != null) {
-				try {
-					metacityPreview = CreateMetacityPreview (window);
-					preview = metacityPreview;
-					if (wrapper != null)
-						wrapper.Notify += OnWindowPropChange;
-				} catch {
-					// If metacity is not available, use a regular box.
+				preview = Stetic.Metacity.Preview.Create (window);
+				if (preview == null)
+					preview = Stetic.Windows.Preview.Create (window);
+				if (preview == null) {
+					// Use a regular box.
 					EventBox eventBox = new EventBox ();
 					eventBox.Add (container);
 					preview = eventBox;
 				}
-
 			} else {
 				EventBox eventBox = new EventBox ();
 				eventBox.Add (container);
@@ -131,19 +112,18 @@ namespace Stetic
 			resizableFixed.SelectionChanged += OnSelectionChanged;
 		}
 		
-		public override void Dispose ()
+		protected override void OnDestroyed ()
 		{
 			if (preview != null) {
 				if (wrapper != null)
 					wrapper.DetachDesigner (resizableFixed);
 				preview.SizeAllocated -= new Gtk.SizeAllocatedHandler (OnResized);
 				resizableFixed.SelectionChanged -= OnSelectionChanged;
-//				resizableFixed.Destroy ();
 				resizableFixed = null;
 				preview = null;
 				wrapper = null;
 			}
-			base.Dispose ();
+			base.OnDestroyed ();
 		}
 		
 		public IObjectViewer ObjectViewer {
@@ -159,85 +139,6 @@ namespace Stetic
 			resizableFixed.UpdateObjectViewers ();
 		}
 		
-		void OnWindowPropChange (object ob, string name)
-		{
-			if (name == "Title") {
-				Title = ((Gtk.Window)((ObjectWrapper)ob).Wrapped).Title;
-			}
-		}
-		
-		Metacity.Preview CreateMetacityPreview (Gtk.Window window)
-		{
-			Metacity.Preview metacityPreview = new Metacity.Preview ();
-			
-			switch (window.TypeHint) {
-			case Gdk.WindowTypeHint.Normal:
-				metacityPreview.FrameType = Metacity.FrameType.Normal;
-				break;
-			case Gdk.WindowTypeHint.Dialog:
-				metacityPreview.FrameType = window.Modal ? Metacity.FrameType.ModalDialog : Metacity.FrameType.Dialog;	
-				break;
-			case Gdk.WindowTypeHint.Menu:
-				metacityPreview.FrameType = Metacity.FrameType.Menu;
-				break;
-			case Gdk.WindowTypeHint.Splashscreen:
-				metacityPreview.FrameType = Metacity.FrameType.Border;
-				break;
-			case Gdk.WindowTypeHint.Utility:
-				metacityPreview.FrameType = Metacity.FrameType.Utility;
-				break;
-			default:
-				metacityPreview.FrameType = Metacity.FrameType.Normal;
-				break;
-			}
-
-			Metacity.FrameFlags flags =
-				Metacity.FrameFlags.AllowsDelete |
-				Metacity.FrameFlags.AllowsVerticalResize |
-				Metacity.FrameFlags.AllowsHorizontalResize |
-				Metacity.FrameFlags.AllowsMove |
-				Metacity.FrameFlags.AllowsShade |
-				Metacity.FrameFlags.HasFocus;
-				
-			if (window.Resizable)
-				flags = flags | Metacity.FrameFlags.AllowsMaximize;
-				
-			metacityPreview.FrameFlags = flags;
-			metacityPreview.ShowAll ();
-			metacityPreview.Add (window);
-			
-			metacityPreview.Theme = GetTheme ();
-			
-			return metacityPreview;
-		}
-
-		static Metacity.Theme GetTheme ()
-		{
-			if (theme == null) {
-				try {
-					Assembly assm = Assembly.Load ("gconf-sharp, Version=2.8.0.0, Culture=neutral, PublicKeyToken=35e10195dab3c99f");
-					Type client_type = assm.GetType ("GConf.Client");
-					MethodInfo method = client_type.GetMethod ("Get", BindingFlags.Instance | BindingFlags.Public);
-					object client = Activator.CreateInstance (client_type, new object[] {});
-					string themeName = (string) method.Invoke (client, new object[] {"/apps/metacity/general/theme"});
-					theme = Metacity.Theme.Load (themeName);
-				} catch {
-					// Don't crash if metacity is not available
-					return null;
-				}
-			}
-			return theme;
-		}
-
-/*		static void GConfNotify (object obj, GConf.NotifyEventArgs args)
-		{
-			if (args.Key == "/apps/metacity/general/theme") {
-				theme = Metacity.Theme.Load ((string)args.Value);
-				foreach (Metacity.Preview prev in wrappers.Values)
-					prev.Theme = Theme;
-			}
-		}
-*/		
 		void OnSelectionChanged (object ob, EventArgs a)
 		{
 			if (SelectionChanged != null)
@@ -333,6 +234,10 @@ namespace Stetic
 			selectionBox.Show ();
 		}
 		
+		public ResizableFixed (IntPtr p): base (p)
+		{
+		}
+		
 		protected override void OnDestroyed ()
 		{
 			if (cursorX != null) {
@@ -585,6 +490,11 @@ namespace Stetic
 		
 		protected override void OnSizeRequested (ref Requisition req)
 		{
+			// The Toplevel check is done to ensure that this widget is anchored,
+			// otherwise the Realize call will fail.
+			if (!child.IsRealized && Toplevel is Gtk.Window)
+				child.Realize ();
+			
 			req = child.SizeRequest ();
 			// Make some room for the border
 			req.Width += padding * 2 + selectionBorder * 2;
@@ -727,14 +637,33 @@ namespace Stetic
 		protected override bool OnExposeEvent (Gdk.EventExpose ev)
 		{
 			bool r = base.OnExposeEvent (ev);
-			
-			//this.Style.DarkGC (Gtk.StateType.Normal)
-//			GdkWindow.DrawRectangle (Style.MidGC (Gtk.StateType.Normal), true, 0, 0, 1000, 1000);
-			
+			//FIXME Disabled checkerboard background because it's very inefficient and makes the control *very* slow to resize
+			// It should take the EventExpose area into account, invalidate more selectively during resizes (GTK viewport 
+			// code would probably be a good start), and take advantage of the flat block color the parent is rendering.
+			/*
+			int size = 8;
+			bool squareColor = true;
+			bool startsquareColor = true;
+			int x1 = 0;
+			int x2 = Allocation.Width;
+			int y1 = 0;
+			int y2 = Allocation.Height;
+			for (int y = y1; y < y2; y += size) {
+				squareColor = startsquareColor;
+				startsquareColor = !startsquareColor;
+				for (int x = x1; x < x2; x += size) {
+					GdkWindow.DrawRectangle (squareColor ? Style.DarkGC (StateType.Normal) : Style.DarkGC (StateType.Active), true, x, y, size, size);
+					squareColor = !squareColor;
+				}
+			}
+
+			foreach (Widget cw in Children)
+				PropagateExpose (cw, ev);*/
+
 			Gdk.Rectangle rect = child.Allocation;
-/*			rect.Inflate (selectionBorder, selectionBorder);
-			GdkWindow.DrawRectangle (Style.BlackGC, false, rect.X, rect.Y, rect.Width, rect.Height);
-*/			
+			if (Stetic.Metacity.Preview.ThemeError) 
+				GdkWindow.DrawRectangle (this.Style.BackgroundGC (StateType.Normal), true, rect.X, rect.Y, rect.Width, rect.Height);
+			
 			Pixbuf sh = Shadow.AddShadow (rect.Width, rect.Height);
 			GdkWindow.DrawPixbuf (this.Style.BackgroundGC (StateType.Normal), sh, 0, 0, rect.X - 5, rect.Y - 5, sh.Width, sh.Height, RgbDither.None, 0, 0); 
 			return r;
