@@ -33,6 +33,7 @@ using MonoDevelop.Core.Gui;
 using Mono.TextEditor.Highlighting;
 using MonoDevelop.Core;
 using MonoDevelop.Components;
+using System.Text;
 
 namespace MonoDevelop.SourceEditor.OptionPanels
 {
@@ -77,8 +78,8 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 		{
 			styleStore.Clear ();
 			TreeIter selectedIter = styleStore.AppendValues (GetMarkup (GettextCatalog.GetString ("Default"), GettextCatalog.GetString ("The default color scheme.")), "Default");
-			foreach (string styleName in SyntaxModeService.Styles) {
-				Mono.TextEditor.Highlighting.Style style = SyntaxModeService.GetColorStyle (null, styleName);
+			foreach (string styleName in Mono.TextEditor.Highlighting.SyntaxModeService.Styles) {
+				Mono.TextEditor.Highlighting.Style style = Mono.TextEditor.Highlighting.SyntaxModeService.GetColorStyle (null, styleName);
 				TreeIter iter = styleStore.AppendValues (GetMarkup (GettextCatalog.GetString (style.Name), GettextCatalog.GetString (style.Description)), style.Name);
 				if (style.Name == DefaultSourceEditorOptions.Instance.ColorScheme)
 					selectedIter = iter;
@@ -92,11 +93,11 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 			TreeIter selectedIter;
 			if (styleTreeview.Selection.GetSelected (out selectedIter)) 
 				styleName = (string)this.styleStore.GetValue (selectedIter, 1);
-			Mono.TextEditor.Highlighting.Style style = SyntaxModeService.GetColorStyle (this, styleName);
-			UrlXmlProvider provider = SyntaxModeService.GetProvider (style) as UrlXmlProvider;
+			var style = Mono.TextEditor.Highlighting.SyntaxModeService.GetColorStyle (this.Style, styleName);
+			UrlXmlProvider provider = Mono.TextEditor.Highlighting.SyntaxModeService.GetProvider (style) as UrlXmlProvider;
 			if (provider != null) {
 				if (provider.Url.StartsWith (SourceEditorDisplayBinding.SyntaxModePath)) {
-					SyntaxModeService.Remove (style);
+					Mono.TextEditor.Highlighting.SyntaxModeService.Remove (style);
 					File.Delete (provider.Url);
 					SourceEditorDisplayBinding.LoadCustomStylesAndModes ();
 					ShowStyles ();
@@ -108,24 +109,49 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 		void AddColorScheme (object sender, EventArgs args)
 		{
 			FileSelector fd = new FileSelector ();
-			int response = fd.Run ();
-			if (response == (int)ResponseType.Ok) {
-				if (SyntaxModeService.IsValidStyle (fd.Filename)) {
-					string newFileName = System.IO.Path.Combine (SourceEditorDisplayBinding.SyntaxModePath, System.IO.Path.GetFileName (fd.Filename));
-					bool success = true;
+			fd.TransientFor = this.Toplevel as Gtk.Window;
+			try {
+				int response = fd.Run ();
+				if (response == (int)ResponseType.Ok) {
+					System.Collections.Generic.List<System.Xml.Schema.ValidationEventArgs> validationResult;
 					try {
-						File.Copy (fd.Filename, newFileName);
-					} catch (Exception e) {
-						success = false;
-						LoggingService.LogError ("Can't copy syntax mode file.", e);
+						validationResult = Mono.TextEditor.Highlighting.SyntaxModeService.ValidateStyleFile (fd.Filename);
+					} catch (Exception) {
+						MessageService.ShowError (GettextCatalog.GetString ("Validation of style file failed."));
+						return;
 					}
-					if (success) {
-						SourceEditorDisplayBinding.LoadCustomStylesAndModes ();
-						ShowStyles ();
+					if (validationResult.Count == 0) {
+						string newFileName = System.IO.Path.Combine (SourceEditorDisplayBinding.SyntaxModePath, System.IO.Path.GetFileName (fd.Filename));
+						if (!newFileName.EndsWith ("Style.xml"))
+							newFileName = System.IO.Path.Combine (SourceEditorDisplayBinding.SyntaxModePath, System.IO.Path.GetFileNameWithoutExtension (fd.Filename) + "Style.xml");
+						bool success = true;
+						try {
+							File.Copy (fd.Filename, newFileName);
+						} catch (Exception e) {
+							success = false;
+							LoggingService.LogError ("Can't copy syntax mode file.", e);
+						}
+						if (success) {
+							SourceEditorDisplayBinding.LoadCustomStylesAndModes ();
+							ShowStyles ();
+						}
+					} else {
+						StringBuilder errorMessage = new StringBuilder ();
+						errorMessage.AppendLine (GettextCatalog.GetString ("Validation of style file failed."));
+						int count = 0;
+						foreach (System.Xml.Schema.ValidationEventArgs vArg in validationResult) {
+							errorMessage.AppendLine (vArg.Message);
+							if (count++ > 5) {
+								errorMessage.AppendLine ("...");
+								break;
+							}
+						}
+						MessageService.ShowError (errorMessage.ToString ());
 					}
 				}
+			} finally {
+				fd.Destroy ();
 			}
-			fd.Destroy ();
 		}
 		
 		void EnableHighlightingCheckbuttonToggled (object sender, EventArgs e)
