@@ -38,43 +38,43 @@ using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Projects.Gui.Completion;
+using MonoDevelop.Projects.CodeGeneration;
 using MonoDevelop.Components.Commands;
 using Mono.TextEditor.Highlighting;
 using MonoDevelop.Ide.CodeTemplates;
 using Mono.Addins;
+using MonoDevelop.Projects.Text;
 
 namespace MonoDevelop.SourceEditor
 {
 	public class ExtensibleTextEditor : Mono.TextEditor.TextEditor
 	{
-		ITextEditorExtension extension = null;
+		internal object MemoryProbe = Counters.EditorsInMemory.CreateMemoryProbe ();
+		
 		SourceEditorView view;
 		Dictionary<int, ErrorMarker> errors;
 		
 		Gdk.Point menuPopupLocation;
 		
 		public ITextEditorExtension Extension {
-			get {
-				return extension;
-			}
-			set {
-				extension = value;
-			}
+			get;
+			set;
 		}
 		
 		public new ISourceEditorOptions Options {
 			get { return (ISourceEditorOptions)base.Options; }
 		}
 		
-		public ExtensibleTextEditor (SourceEditorView view, ISourceEditorOptions options, Mono.TextEditor.Document doc) : base (doc, null)
+		public ExtensibleTextEditor (SourceEditorView view, ISourceEditorOptions options, Mono.TextEditor.Document doc) : base(doc, null)
 		{
 			base.Options = options;
 			Initialize (view);
 		}
+
 		
 		public ExtensibleTextEditor (SourceEditorView view)
 		{
-			base.Options = new StyledSourceEditorOptions (view.Project);
+			base.Options = new StyledSourceEditorOptions (view.Project, null);
 			Initialize (view);
 		}
 		
@@ -91,19 +91,21 @@ namespace MonoDevelop.SourceEditor
 		{
 			this.view = view;
 			Caret.PositionChanged += delegate {
-				if (extension != null) {
+				if (Extension != null) {
 					try {
-						extension.CursorPositionChanged ();
+						Extension.CursorPositionChanged ();
 					} catch (Exception ex) {
 						ReportExtensionError (ex);
 					}
 				}
 			};
-			Document.TextReplaced += delegate (object sender, ReplaceEventArgs args) {
-				if (extension != null) {
+			
+			Document.TextReplaced += HandleSkipCharsOnReplace;
+			
+			Document.TextReplaced += delegate(object sender, ReplaceEventArgs args) {
+				if (Extension != null) {
 					try {
-						extension.TextChanged (args.Offset, 
-						    args.Offset + Math.Max (args.Count, args.Value != null ? args.Value.Length : 0));
+						Extension.TextChanged (args.Offset, args.Offset + Math.Max (args.Count, args.Value != null ? args.Value.Length : 0));
 					} catch (Exception ex) {
 						ReportExtensionError (ex);
 					}
@@ -111,12 +113,31 @@ namespace MonoDevelop.SourceEditor
 			};
 			
 			UpdateEditMode ();
+			this.GetTextEditorData ().Paste += HandleTextPaste;
 			
 			this.ButtonPressEvent += OnPopupMenu;
 
 			AddinManager.AddExtensionNodeHandler ("MonoDevelop/SourceEditor2/TooltipProviders", OnTooltipProviderChanged);
 		}
 		
+		void HandleSkipCharsOnReplace (object sender, ReplaceEventArgs args)
+		{
+			for (int i = 0; i < skipChars.Count; i++) {
+				SkipChar sc = skipChars[i];
+				if (args.Offset < sc.Start || args.Offset > sc.Offset) {
+					skipChars.RemoveAt (i);
+					i--;
+					continue;
+				}
+				if (args.Offset <= sc.Offset) {
+					sc.Offset -= args.Count;
+					if (!string.IsNullOrEmpty (args.Value)) {
+						sc.Offset += args.Value.Length;
+					}
+				}
+			}
+		}
+		
 		void UpdateEditMode ()
 		{
 			if (Options.UseViModes) {
@@ -134,7 +155,7 @@ namespace MonoDevelop.SourceEditor
 
 		protected override void OnDestroyed ()
 		{
-			extension = null;
+			Extension = null;
 			view = null;
 			this.ButtonPressEvent -= OnPopupMenu;
 			AddinManager.RemoveExtensionNodeHandler  ("MonoDevelop/SourceEditor2/TooltipProviders", OnTooltipProviderChanged);
@@ -207,7 +228,7 @@ namespace MonoDevelop.SourceEditor
 		bool ExtensionKeyPress (Gdk.Key key, uint ch, Gdk.ModifierType state)
 		{
 			try {
-				return extension.KeyPress (key, (char)ch, state);
+				return Extension.KeyPress (key, (char)ch, state);
 			} catch (Exception ex) {
 				ReportExtensionError (ex);
 			}
@@ -260,15 +281,35 @@ namespace MonoDevelop.SourceEditor
 				}
 			}
 		}
+		
+		class SkipChar {
+			public int Start { get; set; }
+			public int Offset { get; set; }
+			public char Char  { get; set; }
 			
+			public override string ToString ()
+			{
+				return string.Format ("[SkipChar: Start={0}, Offset={1}, Char={2}]", Start, Offset, Char);
+			}
+		}
+		
+		List<SkipChar> skipChars = new List<SkipChar> ();
+		void SetInsertionChar (int offset, char ch)
+		{
+			skipChars.Add (new SkipChar () {
+				Start = offset - 1,
+				Offset = offset,
+				Char = ch
+			});
+		}
+		
 		protected override bool OnIMProcessedKeyPressEvent (Gdk.Key key, uint ch, Gdk.ModifierType state)
 		{
 			bool result = true;
 			if (key == Gdk.Key.Escape) {
 				bool b;
-				if (extension != null)
-					b = ExtensionKeyPress (key, ch, state);
-				else
+				if (Extension != null)
+					b = ExtensionKeyPress (key, ch, state); else
 					b = base.OnIMProcessedKeyPressEvent (key, ch, state);
 				if (b) {
 					view.SourceEditorWidget.RemoveSearchWidget ();
@@ -276,78 +317,98 @@ namespace MonoDevelop.SourceEditor
 				}
 				return false;
 			}
-			
-			
+
 			if (Document == null)
 				return true;
-			
+
 			bool inStringOrComment = false;
 			LineSegment line = Document.GetLine (Caret.Line);
 			if (line == null)
 				return true;
+			bool inChar = false;
+//			string escape = "\"";
 			Stack<Span> stack = line.StartSpan != null ? new Stack<Span> (line.StartSpan) : new Stack<Span> ();
-			SyntaxModeService.ScanSpans (Document, Document.SyntaxMode, stack, line.Offset, Caret.Offset);
+			Mono.TextEditor.Highlighting.SyntaxModeService.ScanSpans (Document, Document.SyntaxMode, Document.SyntaxMode, stack, line.Offset, Caret.Offset);
 			foreach (Span span in stack) {
-				if (span.Color == "comment" || span.Color == "literal") {
+				if (span.Color == "string.single" || span.Color == "string.double") {
 					inStringOrComment = true;
+					inChar |= span.Color == "string.single";
+					//escape = span.Escape;
 					break;
 				}
 			}
-			
+
 			Document.BeginAtomicUndo ();
-			
+
 			// insert template when space is typed (currently disabled - it's annoying).
-			bool templateInserted = false; //!inStringOrComment && (key == Gdk.Key.space) && DoInsertTemplate ();
-			bool returnBetweenBraces =
-				key == Gdk.Key.Return
-				&& (state & (Gdk.ModifierType.ControlMask | Gdk.ModifierType.ShiftMask)) == Gdk.ModifierType.None
-				&& Caret.Offset > 0
-				&& Caret.Offset < Document.Length
-				&& Document.GetCharAt (Caret.Offset - 1) == '{'
-				&& Document.GetCharAt (Caret.Offset)     == '}'
-				&& !inStringOrComment;
+			bool templateInserted = false;
+			//!inStringOrComment && (key == Gdk.Key.space) && DoInsertTemplate ();
+			bool returnBetweenBraces = key == Gdk.Key.Return && (state & (Gdk.ModifierType.ControlMask | Gdk.ModifierType.ShiftMask)) == Gdk.ModifierType.None && Caret.Offset > 0 && Caret.Offset < Document.Length && Document.GetCharAt (Caret.Offset - 1) == '{' && Document.GetCharAt (Caret.Offset) == '}' && !inStringOrComment;
 			int initialOffset = Caret.Offset;
-			
-			if (Options.AutoInsertMatchingBracket && !inStringOrComment && MonoDevelop.Ide.Gui.TextEditor.IsOpenBrace ((char)ch)) {
-				char closingBrace = MonoDevelop.Ide.Gui.TextEditor.GetMatchingBrace ((char)ch);
-				int count = 0;
-				foreach (char curCh in TextWithoutCommentsAndStrings) {
-					if (curCh == ch) {
-						count++;
-					} else if (curCh == closingBrace) {
-						count--;
+			const string openBrackets = "{[('\"";
+			const string closingBrackets = "}])'\"";
+			int braceIndex = openBrackets.IndexOf ((char)ch);
+			SkipChar skipChar = skipChars.Find (sc => sc.Char == (char)ch && sc.Offset == Caret.Offset);
+
+			// special handling for escape chars inside ' and "
+			if (Caret.Offset > 0) {
+				char charBefore = Document.GetCharAt (Caret.Offset - 1);
+				if (inStringOrComment && (ch == '"' || (inChar && ch == '\'')) && charBefore == '\\')
+					skipChar = null;
+			}
+			char insertionChar = '\0';
+			if (skipChar == null && Options.AutoInsertMatchingBracket && braceIndex >= 0) {
+				if (!inStringOrComment) {
+					char closingBrace = closingBrackets[braceIndex];
+					char openingBrace = openBrackets[braceIndex];
+
+					int count = 0;
+					foreach (char curCh in TextWithoutCommentsAndStrings) {
+						if (curCh == openingBrace) {
+							count++;
+						} else if (curCh == closingBrace) {
+							count--;
+						}
+					}
+
+					if (count >= 0) {
+						GetTextEditorData ().EnsureCaretIsNotVirtual ();
+						insertionChar = closingBrace;
+						Insert (Caret.Offset, closingBrace.ToString ());
+					}
+				} else {
+					char charBefore = Document.GetCharAt (Caret.Offset - 1);
+					if (!inChar && ch == '"' && charBefore != '\\') {
+						GetTextEditorData ().EnsureCaretIsNotVirtual ();
+						insertionChar = '"';
+						Insert (Caret.Offset, "\"");
 					}
 				}
-				if (count >= 0) {
-					string openBrackets = "{[('\"";
-					if (openBrackets.Contains (ch.ToString()))
-						Insert (Caret.Offset, closingBrace.ToString());
-				}
 			}
 			
-			if (extension != null) {
-				if (ExtensionKeyPress (key, ch, state)) 
-					result = base.OnIMProcessedKeyPressEvent (key, ch, state);
-				if (returnBetweenBraces) {
-					Caret.Offset = initialOffset;
-					ExtensionKeyPress (Gdk.Key.Return, (char)0, Gdk.ModifierType.None);
-				}
+			//Console.WriteLine (Caret.Offset + "/" + insOff);
+			if (skipChar != null) {
+				Caret.Offset++;
+				skipChars.Remove (skipChar);
 			} else {
-				result = base.OnIMProcessedKeyPressEvent (key, ch, state);
-				if (returnBetweenBraces) {
-					Caret.Offset = initialOffset;
-					base.SimulateKeyPress (Gdk.Key.Return, 0, Gdk.ModifierType.None);
+				if (Extension != null) {
+					if (ExtensionKeyPress (key, ch, state)) 
+						result = base.OnIMProcessedKeyPressEvent (key, ch, state);
+					if (returnBetweenBraces) {
+						Caret.Offset = initialOffset;
+						ExtensionKeyPress (Gdk.Key.Return, (char)0, Gdk.ModifierType.None);
+					}
+				} else {
+					result = base.OnIMProcessedKeyPressEvent (key, ch, state);
+					if (returnBetweenBraces) {
+						Caret.Offset = initialOffset;
+						base.SimulateKeyPress (Gdk.Key.Return, 0, Gdk.ModifierType.None);
+					}
 				}
 			}
-/* auto insert templates IS annoying !!!
-			// auto insert templates
-			if (!templateInserted &&
-			    Options.AutoInsertTemplates && 
-			    ch != '\0' && // don't auto insert templates for control keys
-			    IsTemplateKnown () // only insert templates when there is a 100% match (ex.: scw vs. scwl)
-			    ) {
-				templateInserted = !inStringOrComment && DoInsertTemplate ();
-			}*/
+			if (insertionChar != '\0')
+				SetInsertionChar (Caret.Offset, insertionChar);
+			
 			if (templateInserted) {
 				Document.EndAtomicUndo ();
 				return true;
@@ -421,6 +482,32 @@ namespace MonoDevelop.SourceEditor
 				return null;
 			DocumentLocation loc = Document.OffsetToLocation (offset);
 			string savedExpression = null;
+			// special handling for 'var' "keyword"
+			if (expressionResult.ExpressionContext == ExpressionContext.IdentifierExpected && expressionResult.Expression != null && expressionResult.Expression.Trim () == "var") {
+				int endOffset = Document.LocationToOffset (expressionResult.Region.End.Line - 1, expressionResult.Region.End.Column - 1);
+				StringBuilder identifer = new StringBuilder ();
+				for (int i = endOffset; i >= 0 && i < Document.Length; i++) {
+					char ch = Document.GetCharAt (i);
+					if (Char.IsWhiteSpace (ch))
+						continue;
+					if (ch == '=')
+						break;
+					if (Char.IsLetterOrDigit (ch) || ch =='_') {
+						identifer.Append (ch);
+						continue;
+					}
+					identifer.Length = 0;
+					break;
+				}
+				if (identifer.Length > 0) {
+					expressionResult.Expression = identifer.ToString ();
+					this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
+					if (this.resolveResult != null) {
+						this.resolveResult = new MemberResolveResult (dom.GetType (resolveResult.ResolvedType));
+						return this.resolveResult;
+					}
+				}
+			}
 			
 			if (expressionResult.ExpressionContext == ExpressionContext.Attribute) {
 				savedExpression = expressionResult.Expression;
@@ -429,6 +516,110 @@ namespace MonoDevelop.SourceEditor
 			} 
 			this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
 			
+			if (savedExpression != null && this.resolveResult == null) {
+				expressionResult.Expression = savedExpression;
+				this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
+			}
+			// Search for possible generic parameters.
+//			if (this.resolveResult == null || this.resolveResult.ResolvedType == null || String.IsNullOrEmpty (this.resolveResult.ResolvedType.Name)) {
+			if (!expressionResult.Region.IsEmpty) {
+				int j = Document.LocationToOffset (expressionResult.Region.End.Line - 1, expressionResult.Region.End.Column - 1);
+				int bracket = 0;
+				for (int i = j; i >= 0 && i < Document.Length; i++) {
+					char ch = Document.GetCharAt (i);
+					if (Char.IsWhiteSpace (ch))
+						continue;
+					if (ch == '<') {
+						bracket++;
+					} else if (ch == '>') {
+						bracket--;
+						if (bracket == 0) {
+							expressionResult.Expression += Document.GetTextBetween (j, i + 1);
+							expressionResult.ExpressionContext = ExpressionContext.ObjectCreation;
+							this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
+							break;
+						}
+					} else {
+						if (bracket == 0)
+							break;
+					}
+				}
+			}
+			
+			// To resolve method overloads the full expression must be parsed.
+			// ex.: Overload (1)/ Overload("one") - parsing "Overload" gives just a MethodResolveResult
+			// and for constructor initializers it's tried too to to resolve constructor overloads.
+			if (this.resolveResult is ThisResolveResult || 
+			    this.resolveResult is BaseResolveResult || 
+			    this.resolveResult is MethodResolveResult && ((MethodResolveResult)resolveResult).Methods.Count > 1) {
+				// put the search offset at the end of the invocation to be able to find the full expression
+				// the resolver finds it itself if spaces are between the method name and the argument opening parentheses.
+				if (txt[wordEnd] == '(') {
+					int matchingBracket = Document.GetMatchingBracketOffset (wordEnd);
+					if (matchingBracket > 0)
+						wordEnd = matchingBracket;
+				}
+				ResolveResult possibleResult = resolver.Resolve (expressionFinder.FindFullExpression (txt, wordEnd), new DomLocation (loc.Line + 1, loc.Column + 1)) ?? this.resolveResult;
+				if (possibleResult is MethodResolveResult)
+					this.resolveResult = possibleResult;
+			}
+			
+			return this.resolveResult;
+		}
+		
+		public CodeTemplateContext GetTemplateContext ()
+		{
+			if (IsSomethingSelected) {
+				string fileName = view.ContentName ?? view.UntitledName;
+				IParser parser = ProjectDomService.GetParser (fileName, Document.MimeType);
+				if (parser == null)
+					return CodeTemplateContext.Standard;
+
+				IExpressionFinder expressionFinder = parser.CreateExpressionFinder (ProjectDom);
+				if (expressionFinder == null) 
+					return CodeTemplateContext.Standard;
+				if (expressionFinder.IsExpression (Document.GetTextAt (SelectionRange)))
+					return CodeTemplateContext.InExpression;
+			}
+			return CodeTemplateContext.Standard;
+		}
+		
+		public ResolveResult GetLanguageItem (int offset, string expression)
+		{
+			string txt = this.Document.Text;
+			string fileName = view.ContentName ?? view.UntitledName;
+			
+			oldOffset = offset;
+			
+			this.resolveResult = null;
+			MonoDevelop.Ide.Gui.Document doc = IdeApp.Workbench.ActiveDocument;
+			if (doc == null)
+				return null;
+			
+			IParser parser = ProjectDomService.GetParser (fileName, Document.MimeType);
+			if (parser == null)
+				return null;
+
+			ProjectDom        dom      = this.ProjectDom;
+			IResolver         resolver = parser.CreateResolver (dom, doc, fileName);
+			IExpressionFinder expressionFinder = parser.CreateExpressionFinder (dom);
+			if (resolver == null || expressionFinder == null) 
+				return null;
+			int wordEnd = offset;
+			while (wordEnd < txt.Length && (Char.IsLetterOrDigit (txt[wordEnd]) || txt[wordEnd] == '_'))
+				wordEnd++;
+			ExpressionResult expressionResult = new ExpressionResult (expression);
+			expressionResult.ExpressionContext = ExpressionContext.MethodBody;
+			
+			DocumentLocation loc = Document.OffsetToLocation (offset);
+			string savedExpression = null;
+			
+			if (expressionResult.ExpressionContext == ExpressionContext.Attribute) {
+				savedExpression = expressionResult.Expression;
+				expressionResult.Expression += "Attribute";
+				expressionResult.ExpressionContext = ExpressionContext.ObjectCreation;
+			} 
+			this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
 			if (savedExpression != null && this.resolveResult == null) {
 				expressionResult.Expression = savedExpression;
 				this.resolveResult = resolver.Resolve (expressionResult, new DomLocation (loc.Line + 1, loc.Column + 1));
@@ -521,7 +712,6 @@ namespace MonoDevelop.SourceEditor
 			Gtk.Menu menu = IdeApp.CommandService.CreateMenu (cset);
 			menu.Append (new SeparatorMenuItem ());
 			menu.Append (CreateInputMethodMenuItem (GettextCatalog.GetString ("_Input Methods")));
-			
 			menu.Destroyed += delegate {
 				this.QueueDraw ();
 			};
@@ -529,26 +719,17 @@ namespace MonoDevelop.SourceEditor
 			menu.Popup (null, null, new Gtk.MenuPositionFunc (PositionPopupMenu), 0, Gtk.Global.CurrentEventTime);
 		}
 		
-		void PositionPopupMenu (Menu menu, out int x, out int y, out bool pushIn) 
+		void PositionPopupMenu (Menu menu, out int x, out int y, out bool pushIn)
 		{
 			this.GdkWindow.GetOrigin (out x, out y);
 			x += this.menuPopupLocation.X;
 			y += this.menuPopupLocation.Y;
+			Requisition request = menu.SizeRequest ();
+			y = Math.Min (y, GdkWindow.Screen.Height - request.Height);
+			x = Math.Min (x, GdkWindow.Screen.Width - request.Width);
 			pushIn = true;
 		}
 		
-//		protected override void OnPopulatePopup (Menu menu)
-//		{
-//			
-//			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet ("");
-//			if (cset.Count > 0) {
-//				cset.AddItem (Command.Separator);
-//				IdeApp.CommandService.InsertOptions (menu, cset, 0);
-//			}
-//			base.OnPopulatePopup (menu);
-//		}
-//		
-		
 #region Templates
 		int FindPrevWordStart (int offset)
 		{
@@ -567,13 +748,8 @@ namespace MonoDevelop.SourceEditor
 		public bool IsTemplateKnown ()
 		{
 			string word = GetWordBeforeCaret ();
-			
-			CodeTemplateGroup templateGroup = CodeTemplateService.GetTemplateGroupPerFilename (this.view.ContentName);
-			if (String.IsNullOrEmpty (word) || templateGroup == null) 
-				return false;
-			
 			bool result = false;
-			foreach (CodeTemplate template in templateGroup.Templates) {
+			foreach (CodeTemplate template in CodeTemplateService.GetCodeTemplates (Document.MimeType)) {
 				if (template.Shortcut == word) {
 					result = true;
 				} else if (template.Shortcut.StartsWith (word)) {
@@ -587,23 +763,91 @@ namespace MonoDevelop.SourceEditor
 		public bool DoInsertTemplate ()
 		{
 			string word = GetWordBeforeCaret ();
-			CodeTemplateGroup templateGroup = CodeTemplateService.GetTemplateGroupPerFilename (this.view.ContentName);
-			if (String.IsNullOrEmpty (word) || templateGroup == null) 
-				return false;
-			
-			foreach (CodeTemplate template in templateGroup.Templates) {
+			foreach (CodeTemplate template in CodeTemplateService.GetCodeTemplates (Document.MimeType)) {
 				if (template.Shortcut == word) {
-					InsertTemplate (template);
+					InsertTemplate (template, view.WorkbenchWindow.Document);
 					return true;
 				}
 			}
 			return false;
 		}
 		
-		public void InsertTemplate (CodeTemplate template)
+		void HandleTextPaste (int insertionOffset, string text)
 		{
-			template.InsertTemplate (MonoDevelop.Ide.Gui.TextEditor.GetTextEditor (this.view));
-		}		
+			if (PropertyService.Get ("OnTheFlyFormatting", false)) {
+				IPrettyPrinter prettyPrinter = TextFileService.GetPrettyPrinter (Document.MimeType);
+				if (prettyPrinter != null && prettyPrinter.SupportsOnTheFlyFormatting && ProjectDom != null && text != null) {
+					try {
+						string newText = prettyPrinter.FormatText (ProjectDom.Project, Document.MimeType, Document.Text, insertionOffset, insertionOffset + text.Length);
+						if (!string.IsNullOrEmpty (newText)) {
+							Replace (insertionOffset, text.Length, newText);
+							Caret.Offset = insertionOffset + newText.Length;
+						}
+					} catch (Exception e) {
+						Console.WriteLine (e);
+					}
+				}
+			}
+		}
+		
+		public void InsertTemplate (CodeTemplate template, MonoDevelop.Ide.Gui.Document document)
+		{
+			Document.BeginAtomicUndo ();
+			CodeTemplate.TemplateResult result = template.InsertTemplate (document);
+			TextLinkEditMode tle = new TextLinkEditMode (this, 
+			                                             result.InsertPosition,
+			                                             result.TextLinks);
+			
+			if (PropertyService.Get ("OnTheFlyFormatting", false)) {
+				IPrettyPrinter prettyPrinter = TextFileService.GetPrettyPrinter (Document.MimeType);
+				if (prettyPrinter != null && prettyPrinter.SupportsOnTheFlyFormatting) {
+					int endOffset = result.InsertPosition + result.Code.Length;
+					string text = prettyPrinter.FormatText (document.Project, Document.MimeType, Document.Text, result.InsertPosition, endOffset);
+					string oldText = Document.GetTextAt (result.InsertPosition, result.Code.Length);
+					//					Console.WriteLine (result.InsertPosition);
+					//					Console.WriteLine ("old:" + oldText);
+					//					Console.WriteLine ("new:" + text);
+					Replace (result.InsertPosition, result.Code.Length, text);
+					Caret.Offset = result.InsertPosition + TranslateOffset (oldText, text, Caret.Offset - result.InsertPosition);
+					foreach (TextLink textLink in tle.Links) {
+						foreach (ISegment segment in textLink.Links) {
+							segment.Offset = TranslateOffset (oldText, text, segment.Offset);
+						}
+					}
+				}
+			}
+			
+			if (tle.ShouldStartTextLinkMode) {
+				tle.OldMode = CurrentMode;
+				tle.StartMode ();
+				CurrentMode = tle;
+			}
+			Document.EndAtomicUndo ();
+		}
+		
+		static int TranslateOffset (string baseInput, string formattedInput, int offset)
+		{
+			int i = 0;
+			int j = 0;
+			while (i < baseInput.Length && j < formattedInput.Length && i < offset) {
+				char ch1 = baseInput[i];
+				char ch2 = formattedInput[j];
+				bool ch1IsWs = Char.IsWhiteSpace (ch1);
+				bool ch2IsWs = Char.IsWhiteSpace (ch2);
+				if (ch1 == ch2 || ch1IsWs && ch2IsWs) {
+					i++;
+					j++;
+				} else if (!ch1IsWs && ch2IsWs) {
+					j++;
+				} else if (ch1IsWs && !ch2IsWs) {
+					i++;
+				} else {
+					return -1;
+				}
+			}
+			return j;
+		}
+		
 #endregion
 		
 #region Key bindings
@@ -626,7 +870,6 @@ namespace MonoDevelop.SourceEditor
 			RunAction (DeleteActions.Backspace);
 		}
 		
-		[CommandHandler (MonoDevelop.Ide.Commands.EditCommands.Delete)]
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeleteRightChar)]
 		internal void OnDeleteRightChar ()
 		{
@@ -704,6 +947,18 @@ namespace MonoDevelop.SourceEditor
 		{
 			RunAction (ScrollActions.Down);
 		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.ScrollPageUp)]
+		internal void OnScrollPageUp ()
+		{
+			RunAction (ScrollActions.PageUp);
+		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.ScrollPageDown)]
+		internal void OnScrollPageDown ()
+		{
+			RunAction (ScrollActions.PageDown);
+		}
 
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.GotoMatchingBrace)]
 		internal void OnGotoMatchingBrace ()
@@ -746,6 +1001,30 @@ namespace MonoDevelop.SourceEditor
 		{
 			RunAction (SelectionActions.MoveNextWord);
 		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.MovePrevSubword)]
+		internal void OnMovePrevSubword ()
+		{
+			RunAction (CaretMoveActions.PreviousSubword);
+		}
+
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.MoveNextSubword)]
+		internal void OnMoveNextSubword ()
+		{
+			RunAction (CaretMoveActions.NextSubword);
+		}
+
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.SelectionMovePrevSubword)]
+		internal void OnSelectionMovePrevSubword ()
+		{
+			RunAction (SelectionActions.MovePreviousSubword);
+		}
+
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.SelectionMoveNextSubword)]
+		internal void OnSelectionMoveNextSubword ()
+		{
+			RunAction (SelectionActions.MoveNextSubword);
+		}
 
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.SelectionMoveUp)]
 		internal void OnSelectionMoveUp ()
@@ -806,7 +1085,30 @@ namespace MonoDevelop.SourceEditor
 		{
 			RunAction (MiscActions.InsertNewLine);
 		}
-
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.InsertNewLineAtEnd)]
+		internal void OnInsertNewLineAtEnd ()
+		{
+			RunAction (MiscActions.InsertNewLineAtEnd);
+		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.InsertNewLinePreserveCaretPosition)]
+		internal void OnInsertNewLinePreserveCaretPosition ()
+		{
+			RunAction (MiscActions.InsertNewLinePreserveCaretPosition);
+		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.CompleteStatement)]
+		internal void OnCompleteStatement ()
+		{
+			CodeRefactorer refactorer = IdeApp.Workspace.GetCodeRefactorer (IdeApp.ProjectOperations.CurrentSelectedSolution);
+			DomLocation caretLocation = refactorer.CompleteStatement (ProjectDom, Document.FileName, new DomLocation (Caret.Line, Caret.Column));
+			Caret.Line   = caretLocation.Line;
+			Caret.Column = caretLocation.Column;
+		}
+		
+		
+		
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeletePrevWord)]
 		internal void OnDeletePrevWord ()
 		{
@@ -818,6 +1120,18 @@ namespace MonoDevelop.SourceEditor
 		{
 			RunAction (DeleteActions.NextWord);
 		}
+		
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeletePrevSubword)]
+		internal void OnDeletePrevSubword ()
+		{
+			RunAction (DeleteActions.PreviousSubword);
+		}
+
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeleteNextSubword)]
+		internal void OnDeleteNextSubword ()
+		{
+			RunAction (DeleteActions.NextSubword);
+		}
 
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.SelectionPageDownAction)]
 		internal void OnSelectionPageDownAction ()
@@ -831,6 +1145,12 @@ namespace MonoDevelop.SourceEditor
 			RunAction (SelectionActions.MovePageUp);
 		}
 		
+		[CommandHandler (MonoDevelop.SourceEditor.SourceEditorCommands.PulseCaret)]
+		internal void OnPulseCaretCommand ()
+		{
+			StartCaretPulseAnimation ();
+		}
+		
 #endregion
 		
 	}
