@@ -32,6 +32,8 @@ namespace MonoDevelop.SourceEditor
 {
 	public enum SourceEditorCommands
 	{
-		MarkerOperations
+		MarkerOperations,
+		DynamicAbbrev,
+		PulseCaret
 	}
 }
