@@ -186,7 +186,7 @@ namespace MonoDevelop.XmlEditor.Gui
 
 		#region Code completion
 
-		public override ICompletionDataList CodeCompletionCommand (ICodeCompletionContext completionContext)
+		public override ICompletionDataList CodeCompletionCommand (CodeCompletionContext completionContext)
 		{
 			int pos = completionContext.TriggerOffset;
 			string txt = Editor.GetText (pos - 1, pos);
@@ -200,7 +200,7 @@ namespace MonoDevelop.XmlEditor.Gui
 		}
 
 		public override ICompletionDataList HandleCodeCompletion (
-		    ICodeCompletionContext completionContext, char completionChar, ref int triggerWordLength)
+		    CodeCompletionContext completionContext, char completionChar, ref int triggerWordLength)
 		{
 			int pos = completionContext.TriggerOffset;
 			if (pos > 0 && Editor.GetCharAt (pos - 1) == completionChar) {
@@ -245,6 +245,13 @@ namespace MonoDevelop.XmlEditor.Gui
 				}
 				return null;
 			}
+			// Auto insert '>' when '/' is typed inside tag state (for quick tag closing)
+			if (tracker.Engine.CurrentState is XmlTagState && currentChar == '/') {
+				buf.BeginAtomicUndo ();
+				buf.InsertText (buf.CursorPosition, ">");
+				buf.EndAtomicUndo ();
+				return null;
+			}
 			
 			//element completion
 			if (currentChar == '<' && tracker.Engine.CurrentState is XmlFreeState) {
@@ -308,16 +315,13 @@ namespace MonoDevelop.XmlEditor.Gui
 					if (!forced)
 						triggerWordLength = 1;
 					
-					Dictionary<string, string> existingAtts = new Dictionary<string,string>
-						(StringComparer.InvariantCultureIgnoreCase);
+					var existingAtts = new Dictionary<string,string> (StringComparer.OrdinalIgnoreCase);
 					
 					foreach (XAttribute att in attributedOb.Attributes) {
 						existingAtts [att.Name.FullName] = att.Value ?? string.Empty;
 					}
 					
-					CompletionDataList list = new CompletionDataList ();
-					GetAttributeCompletions (list, attributedOb, existingAtts);
-					return list.Count > 0? list : null;
+					return GetAttributeCompletions (attributedOb, existingAtts);
 				}
 			}
 			
@@ -351,9 +355,7 @@ namespace MonoDevelop.XmlEditor.Gui
 						if (Tracker.Engine.CurrentStateLength == 1)
 							triggerWordLength = 1;
 						
-						CompletionDataList list = new CompletionDataList ();
-						GetAttributeValueCompletions (list, attributedOb, att);	
-						return list.Count > 0? list : null;
+						return GetAttributeValueCompletions (attributedOb, att);
 					}
 				}
 				
@@ -377,15 +379,15 @@ namespace MonoDevelop.XmlEditor.Gui
 		{
 		}
 		
-		protected virtual void GetAttributeCompletions (CompletionDataList list, IAttributedXObject attributedOb,
-		                                                Dictionary<string, string> existingAtts)
+		protected virtual CompletionDataList GetAttributeCompletions (IAttributedXObject attributedOb,
+			Dictionary<string, string> existingAtts)
 		{
+			return null;
 		}
 		
-		protected virtual void GetAttributeValueCompletions (CompletionDataList list,
-		                                                     IAttributedXObject attributedOb,
-		                                                     XAttribute att)
+		protected virtual CompletionDataList GetAttributeValueCompletions (IAttributedXObject attributedOb, XAttribute att)
 		{
+			return null;
 		}
 		
 		protected virtual void GetEntityCompletions (CompletionDataList list)
