@@ -33,17 +33,21 @@ using Gtk;
 using Gdk;
 
 using MonoDevelop.Core;
+using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Pads;
+using MonoDevelop.Projects;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.NUnit.Commands;
+using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide.Execution;
 
 namespace MonoDevelop.NUnit
 {
-	public class TestPad: TreeViewPad
+	public class TestPad : TreeViewPad
 	{
-		NUnitService testService = (NUnitService) ServiceManager.GetService (typeof(NUnitService));
+		NUnitService testService = NUnitService.Instance;
 		
 		IAsyncOperation runningTestOperation;
 		VPaned paned;
@@ -72,16 +76,55 @@ namespace MonoDevelop.NUnit
 		VBox detailsPad;
 		
 		ArrayList testNavigationHistory = new ArrayList ();
-		
-		public override void Initialize (NodeBuilder[] builders, TreePadOption[] options)
+
+		ToolButton buttonRunAll, buttonRun, buttonStop;
+		public override void Initialize (NodeBuilder[] builders, TreePadOption[] options, string menuPath)
 		{
-			base.Initialize (builders, options);
+			base.Initialize (builders, options, menuPath);
 			
 			testChangedHandler = (EventHandler) DispatchService.GuiDispatch (new EventHandler (OnDetailsTestChanged));
 			testService.TestSuiteChanged += (EventHandler) DispatchService.GuiDispatch (new EventHandler (OnTestSuiteChanged));
-			
 			paned = new VPaned ();
-			paned.Pack1 (base.Control, true, true);
+			
+			VBox vbox = new VBox ();
+			Toolbar topToolbar = new Toolbar ();
+			topToolbar.IconSize = IconSize.Menu;
+			topToolbar.ToolbarStyle = ToolbarStyle.Icons;
+			vbox.PackStart (topToolbar, false, false, 0);
+			
+			Gdk.Pixbuf pixbuf  = Gtk.IconTheme.Default.LoadIcon (Gtk.Stock.GoUp, 16, (Gtk.IconLookupFlags) 0);
+//			Gdk.Pixbuf pixbuf2 = Gtk.IconTheme.Default.LoadIcon (Gtk.Stock.Execute, 16, (Gtk.IconLookupFlags) 0);
+//			pixbuf2.CopyArea (0, 0, 15, 16, pixbuf, 1, 0);
+//			Gtk.Image image = new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu);
+//			Gdk.Pixmap pixbuf, mask;
+//			image.GetPixmap (out pixbuf, out mask);
+//			
+//			pixbuf.DrawDrawable (vbox.Style.BaseGC (StateType.Normal), pixbuf, 5, 5 , 0, 0, image.WidthRequest, image.HeightRequest);
+			
+			buttonRunAll = new ToolButton (new Gtk.Image (pixbuf), null);
+			buttonRunAll.IsImportant = true;
+			buttonRunAll.Clicked += new EventHandler (OnRunAllClicked);
+			buttonRunAll.Sensitive = true;
+			buttonRunAll.TooltipText = GettextCatalog.GetString ("Run all tests");
+			topToolbar.Insert (buttonRunAll, -1);
+			
+			buttonRun = new ToolButton (new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu), null);
+			buttonRun.IsImportant = true;
+			buttonRun.Clicked += new EventHandler (OnRunClicked);
+			buttonRun.Sensitive = true;
+			buttonRun.TooltipText = GettextCatalog.GetString ("Run test");
+			topToolbar.Insert (buttonRun, -1);
+			
+			buttonStop = new ToolButton (Gtk.Stock.Stop);
+			buttonStop.Clicked += new EventHandler (OnStopClicked);
+			buttonStop.Sensitive = false;
+			buttonStop.TooltipText = GettextCatalog.GetString ("Cancel running test");
+			topToolbar.Insert (buttonStop, -1);
+			
+			vbox.PackEnd (base.Control, true, true, 0);
+			vbox.FocusChain = new Gtk.Widget [] { base.Control };
+			
+			paned.Pack1 (vbox, true, true);
 			
 			detailsPad = new VBox ();
 			
@@ -307,16 +350,19 @@ namespace MonoDevelop.NUnit
 			regressionTree.RowActivated += new Gtk.RowActivatedHandler (OnRegressionTestActivated);
 			failedTree.RowActivated += new Gtk.RowActivatedHandler (OnFailedTestActivated);
 			
-			if (testService.RootTest != null)
-				LoadTree (testService.RootTest);
+			foreach (UnitTest t in testService.RootTests)
+				TreeView.AddChild (t);
 		}
 		
 		void OnTestSuiteChanged (object sender, EventArgs e)
 		{
-			if (testService.RootTest != null)
-				LoadTree (testService.RootTest);
+			if (testService.RootTests.Length > 0) {
+				TreeView.Clear ();
+				foreach (UnitTest t in testService.RootTests)
+					TreeView.AddChild (t);
+			}
 			else {
-				Clear ();
+				TreeView.Clear ();
 				ClearDetails ();
 			}
 		}
@@ -332,7 +378,7 @@ namespace MonoDevelop.NUnit
 		
 		ITreeNavigator FindTestNode (UnitTest t)
 		{
-			ITreeNavigator nav = GetNodeAtObject (t);
+			ITreeNavigator nav = TreeView.GetNodeAtObject (t);
 			if (nav != null)
 				return nav;
 			if (t.Parent == null)
@@ -344,7 +390,7 @@ namespace MonoDevelop.NUnit
 				return null;
 				
 			nav.MoveToFirstChild ();	// Make sure the children are created
-			return GetNodeAtObject (t);
+			return TreeView.GetNodeAtObject (t);
 		}
 		
 		public override Gtk.Widget Control {
@@ -356,7 +402,7 @@ namespace MonoDevelop.NUnit
 		[CommandHandler (TestCommands.RunTest)]
 		protected void OnRunTest ()
 		{
-			RunSelectedTest ();
+			RunSelectedTest (null);
 		}
 		
 		[CommandUpdateHandler (TestCommands.RunTest)]
@@ -365,31 +411,94 @@ namespace MonoDevelop.NUnit
 			info.Enabled = runningTestOperation == null;
 		}
 		
-		public override void ActivateCurrentItem ()
+		[CommandHandler (TestCommands.RunTestWith)]
+		protected void OnRunTest (object data)
 		{
-			RunSelectedTest ();
+			IExecutionHandler h = ExecutionModeCommandService.GetExecutionModeForCommand (data);
+			if (h != null)
+				RunSelectedTest (h);
 		}
 		
-		void RunSelectedTest ()
+		[CommandUpdateHandler (TestCommands.RunTestWith)]
+		protected void OnUpdateRunTest (CommandArrayInfo info)
 		{
-			ITreeNavigator nav = GetSelectedNode ();
-			UnitTest test = (UnitTest) nav.DataItem;
+			UnitTest test = GetSelectedTest ();
+			if (test != null && IdeApp.ProjectOperations.CurrentSelectedProject != null) {
+				ExecutionModeCommandService.GenerateExecutionModeCommands (
+				    IdeApp.ProjectOperations.CurrentSelectedProject,
+				    test.CanRun,
+				    info);
+			}
+		}
+		
+		public TestPad ()
+		{
+			base.TreeView.CurrentItemActivated += delegate {
+				RunSelectedTest (null);
+			};
+		}
+		
+		void OnStopClicked (object sender, EventArgs args)
+		{
+			if (runningTestOperation != null)
+				runningTestOperation.Cancel ();
+		}
+		
+		void OnRunClicked (object sender, EventArgs args)
+		{
+			RunSelectedTest (null);
+		}
 			
-			runningTestOperation = testService.RunTest (test);
+		UnitTest GetSelectedTest ()
+		{
+			ITreeNavigator nav = TreeView.GetSelectedNode ();
+			if (nav == null)
+				return null;
+			return nav.DataItem as UnitTest;
+		}
+		
+		void RunTest (ITreeNavigator nav, IExecutionHandler mode)
+		{
+			if (nav == null)
+				return;
+			UnitTest test = nav.DataItem as UnitTest;
+			if (test == null)
+				return;
+			TestSession.ResetResult (test.RootTest);
+			
+			this.buttonRun.Sensitive = false;
+			this.buttonRunAll.Sensitive = false;
+			this.buttonStop.Sensitive = true;
+			
+			IdeApp.Workbench.GetPad<TestPad> ().BringToFront ();
+			runningTestOperation = testService.RunTest (test, mode);
 			runningTestOperation.Completed += (OperationHandler) DispatchService.GuiDispatch (new OperationHandler (TestSessionCompleted));
 		}
 		
+		void OnRunAllClicked (object sender, EventArgs args)
+		{
+			RunTest (TreeView.GetRootNode (), null);
+		}
+		
+		void RunSelectedTest (IExecutionHandler mode)
+		{
+			RunTest (TreeView.GetSelectedNode (), mode);
+		}
+		
 		void TestSessionCompleted (IAsyncOperation op)
 		{
 			if (op.Success)
 				RefreshDetails ();
 			runningTestOperation = null;
+			this.buttonRun.Sensitive = true;
+			this.buttonRunAll.Sensitive = true;
+			this.buttonStop.Sensitive = false;
 		}
 		
 		protected override void OnSelectionChanged (object sender, EventArgs args)
 		{
 			base.OnSelectionChanged (sender, args);
-			ITreeNavigator nav = GetSelectedNode ();
+			ITreeNavigator nav = TreeView.GetSelectedNode ();
 			if (nav != null) {
 				UnitTest test = (UnitTest) nav.DataItem;
 				if (test != null)
@@ -419,7 +528,7 @@ namespace MonoDevelop.NUnit
 			if (!detailsPad.Visible) {
 				detailsPad.Show ();
 				
-				ITreeNavigator nav = GetSelectedNode ();
+				ITreeNavigator nav = TreeView.GetSelectedNode ();
 				if (nav != null) {
 					UnitTest test = (UnitTest) nav.DataItem;
 					FillDetails (test, false);
