@@ -28,7 +28,9 @@
 
 using System;
 using System.Text;
+using System.Text.RegularExpressions;
 using System.Collections;
+using System.Collections.Generic;
 using System.Threading;
 using Gtk;
 using Gdk;
@@ -44,12 +46,11 @@ namespace MonoDevelop.NUnit
 {
 	class TestResultsPad: IPadContent, ITestProgressMonitor
 	{
-		NUnitService testService = (NUnitService) ServiceManager.GetService (typeof(NUnitService));
+		NUnitService testService = NUnitService.Instance;
 		
 		IPadWindow window;
 		VBox panel;
 		HPaned book;
-		Gtk.Tooltips tips = new Gtk.Tooltips ();
 		
 		Label infoFailed = new Label (GettextCatalog.GetString ("<b>Failed</b>: {0}", 0));
 		Label infoIgnored = new Label (GettextCatalog.GetString ("<b>Ignored</b>: {0}", 0));
@@ -62,6 +63,8 @@ namespace MonoDevelop.NUnit
 		TreeView failuresTreeView;
 		TreeStore failuresStore;
 		TextView outputView;
+		TextTag bold;
+		Dictionary<UnitTest,int> outIters = new Dictionary<UnitTest,int> ();
 		Widget outputViewScrolled;
 		VSeparator infoSep;
 		
@@ -109,7 +112,7 @@ namespace MonoDevelop.NUnit
 			buttonSuccess.IconWidget = new Gtk.Image (CircleImage.Success);
 			buttonSuccess.IsImportant = true;
 			buttonSuccess.Toggled += new EventHandler (OnShowSuccessfulToggled);
-			buttonSuccess.SetTooltip (tips, GettextCatalog.GetString ("Show Successful Tests"), GettextCatalog.GetString ("Show Successful Tests"));
+			buttonSuccess.TooltipText = GettextCatalog.GetString ("Show Successful Tests");
 			toolbar.Insert (buttonSuccess, -1);
 			
 			buttonFailures = new ToggleToolButton ();
@@ -118,7 +121,7 @@ namespace MonoDevelop.NUnit
 			buttonFailures.IconWidget = new Gtk.Image (CircleImage.Failure);
 			buttonFailures.IsImportant = true;
 			buttonFailures.Toggled += new EventHandler (OnShowFailuresToggled);
-			buttonFailures.SetTooltip (tips, GettextCatalog.GetString ("Show Failed Tests"), GettextCatalog.GetString ("Show Failed Tests"));
+			buttonFailures.TooltipText = GettextCatalog.GetString ("Show Failed Tests");
 			toolbar.Insert (buttonFailures, -1);
 			
 			buttonIgnored = new ToggleToolButton ();
@@ -127,22 +130,23 @@ namespace MonoDevelop.NUnit
 			buttonIgnored.IconWidget = new Gtk.Image (CircleImage.NotRun);
 			buttonIgnored.Toggled += new EventHandler (OnShowIgnoredToggled);
 			buttonIgnored.IsImportant = true;
-			buttonIgnored.SetTooltip (tips, GettextCatalog.GetString( "Show Ignored Tests"), GettextCatalog.GetString ("Show Ignored Tests"));
+			buttonIgnored.TooltipText = GettextCatalog.GetString( "Show Ignored Tests");
 			toolbar.Insert (buttonIgnored, -1);
 			
 			buttonOutput = new ToggleToolButton ();
 			buttonOutput.Label = GettextCatalog.GetString ("Output");
 			buttonOutput.Active = false;
-			buttonOutput.IconWidget = Services.Resources.GetImage (MonoDevelop.Core.Gui.Stock.OutputIcon, IconSize.Menu);
+			buttonOutput.IconWidget = ImageService.GetImage (MonoDevelop.Core.Gui.Stock.OutputIcon, IconSize.Menu);
 			buttonOutput.Toggled += new EventHandler (OnShowOutputToggled);
 			buttonOutput.IsImportant = true;
-			buttonOutput.SetTooltip (tips, GettextCatalog.GetString ("Show Output"), GettextCatalog.GetString ("Show Output"));
+			buttonOutput.TooltipText = GettextCatalog.GetString ("Show Output");
 			toolbar.Insert (buttonOutput, -1);
 			
 			toolbar.Insert (new SeparatorToolItem (), -1);
 			
 			buttonRun = new ToolButton (new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu), GettextCatalog.GetString ("Run Test"));
 			buttonRun.IsImportant = true;
+			buttonRun.Sensitive = false;
 			toolbar.Insert (buttonRun, -1);
 			
 			buttonStop = new ToolButton (Gtk.Stock.Stop);
@@ -155,6 +159,7 @@ namespace MonoDevelop.NUnit
 			
 			book = new HPaned ();
 			panel.PackStart (book, true, true, 0);
+			panel.FocusChain = new Gtk.Widget [] { book };
 			
 			// Failures tree
 			failuresTreeView = new TreeView ();
@@ -178,6 +183,9 @@ namespace MonoDevelop.NUnit
 			
 			outputView = new TextView();
 			outputView.Editable = false;
+			bold = new TextTag ("bold");
+			bold.Weight = Pango.Weight.Bold;
+			outputView.Buffer.TagTable.Add (bold);
 			sw = new Gtk.ScrolledWindow ();
 			sw.Add(outputView);
 			frame = new Frame ();
@@ -216,6 +224,8 @@ namespace MonoDevelop.NUnit
 			buttonStop.Clicked += new EventHandler (OnStopClicked);
 			buttonRun.Clicked += new EventHandler (OnRunClicked);
 			failuresTreeView.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler (OnPopupMenu);
+			failuresTreeView.RowActivated += OnRowActivated;
+			failuresTreeView.Selection.Changed += OnRowSelected;
 			
 			Control.ShowAll ();
 			
@@ -236,12 +246,18 @@ namespace MonoDevelop.NUnit
 			results.Clear ();
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
+			outIters.Clear ();
 			progressBar.Fraction = 0;
 			progressBar.Text = "";
 			testsRun = 0;
 			testsFailed = 0;
 			testsIgnored = 0;
 			UpdateCounters ();
+			if (rootTest != null) {
+				rootTest = testService.SearchTest (rootTest.FullName);
+				if (rootTest == null)
+					buttonRun.Sensitive = false;
+			}
 		}
 		
 		bool Running {
@@ -296,19 +312,22 @@ namespace MonoDevelop.NUnit
 			
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
+			outIters.Clear ();
 			cancel = false;
 			Running = true;
 			
-			configuration = rootTest.ActiveConfiguration;
+			configuration = IdeApp.Workspace.ActiveConfigurationId;
 			
 			AddStartMessage ();
 		}
 		
 		public void AddStartMessage ()
 		{
-			Gdk.Pixbuf infoIcon = failuresTreeView.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.Menu, "");
-			string msg = string.Format (GettextCatalog.GetString ("Running tests for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
-			failuresStore.AppendValues (infoIcon, msg, rootTest);
+			if (rootTest != null) {
+				Gdk.Pixbuf infoIcon = failuresTreeView.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.Menu, "");
+				string msg = string.Format (GettextCatalog.GetString ("Running tests for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
+				failuresStore.AppendValues (infoIcon, msg, rootTest);
+			}
 		}
 
 		public void ReportRuntimeError (string message, Exception exception)
@@ -356,14 +375,15 @@ namespace MonoDevelop.NUnit
 		
 		void OnStopClicked (object sender, EventArgs args)
 		{
-			Cancel ();
-			failuresStore.AppendValues (CircleImage.Failure, GettextCatalog.GetString ("Test execution cancelled."), null);
+			if (running)
+				Cancel ();
 		}
 		
 		void OnRunClicked (object sender, EventArgs args)
 		{
-			NUnitService testService = (NUnitService) ServiceManager.GetService (typeof(NUnitService));
-			testService.RunTest (rootTest);
+			if (rootTest == null)
+				return;
+			NUnitService.Instance.RunTest (rootTest, null);
 		}
 		
 		void OnPopupMenu (object o, Gtk.ButtonReleaseEventArgs args)
@@ -372,11 +392,31 @@ namespace MonoDevelop.NUnit
 				IdeApp.CommandService.ShowContextMenu ("/MonoDevelop/NUnit/ContextMenu/TestResultsPad");
 			}
 		}
+
+		void OnRowActivated (object s, EventArgs a)
+		{
+			OnShowTest ();
+		}
+		
+		void OnRowSelected (object s, EventArgs a)
+		{
+			UnitTest test = GetSelectedTest ();
+			if (test != null) {
+				int offset;
+				if (outIters.TryGetValue (test, out offset)) {
+					TextIter it = outputView.Buffer.GetIterAtOffset (offset);
+					outputView.Buffer.MoveMark (outputView.Buffer.InsertMark, it);
+					outputView.Buffer.MoveMark (outputView.Buffer.SelectionBound, it);
+					outputView.ScrollToMark (outputView.Buffer.InsertMark, 0.0, true, 0.0, 0.0);
+				}
+			}
+		}
 		
 		[CommandHandler (TestCommands.SelectTestInTree)]
 		protected void OnSelectTestInTree ()
 		{
 			Pad pad = IdeApp.Workbench.GetPad<TestPad> ();
+			pad.BringToFront ();
 			TestPad content = (TestPad) pad.Content;
 			content.SelectTest (GetSelectedTest ());
 		}
@@ -394,7 +434,12 @@ namespace MonoDevelop.NUnit
 			UnitTest test = GetSelectedTest ();
 			if (test == null)
 				return;
-			SourceCodeLocation loc = test.SourceCodeLocation;
+			SourceCodeLocation loc = null;
+			UnitTestResult res = test.GetLastResult ();
+			if (res != null && res.IsFailure)
+				loc = res.GetFailureLocation ();
+			if (loc == null)
+				loc = test.SourceCodeLocation;
 			if (loc != null)
 				IdeApp.Workbench.OpenDocument (loc.FileName, loc.Line, loc.Column, true);
 		}
@@ -441,6 +486,7 @@ namespace MonoDevelop.NUnit
 		{
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
+			outIters.Clear ();
 			AddStartMessage ();
 				
 			foreach (ResultRecord res in results) {
@@ -482,10 +528,16 @@ namespace MonoDevelop.NUnit
 					failuresStore.AppendValues (testRow, null, Escape (result.Message), test);
 				failuresTreeView.ScrollToCell (failuresStore.GetPath (testRow), null, false, 0, 0);
 			}
+			
+			string msg = GettextCatalog.GetString ("Running {0} ...", test.FullName);
+			TextIter it = outputView.Buffer.EndIter;
+			outIters [test] = it.Offset;
+			outputView.Buffer.InsertWithTags (ref it, msg, bold);
+			outputView.Buffer.Insert (ref it, "\n");
 			if (result.ConsoleOutput != null)
-				outputView.Buffer.InsertAtCursor (result.ConsoleOutput);
+				outputView.Buffer.Insert (ref it, result.ConsoleOutput);
 			if (result.ConsoleError != null)
-				outputView.Buffer.InsertAtCursor (result.ConsoleError);
+				outputView.Buffer.Insert (ref it, result.ConsoleError);
 			outputView.ScrollMarkOnscreen (outputView.Buffer.InsertMark);
 		}
 		
@@ -515,7 +567,13 @@ namespace MonoDevelop.NUnit
 			ShowTestResult (test, result);
 			
 			UpdateCounters ();
-			progressBar.Fraction = ((double)testsRun / (double)testsToRun);
+
+			double frac;
+			if (testsToRun != 0)
+				frac = ((double)testsRun / (double)testsToRun);
+			else
+				frac = 1;
+			progressBar.Fraction = frac;
 			progressBar.Text = testsRun + " / " + testsToRun;
 		}
 		
@@ -527,7 +585,12 @@ namespace MonoDevelop.NUnit
 		
 		public void Cancel ()
 		{
+			if (cancel)
+				return;
 			cancel = true;
+			Gtk.Application.Invoke (delegate {
+				failuresStore.AppendValues (CircleImage.Failure, GettextCatalog.GetString ("Test execution cancelled."), null);
+			});
 			if (CancelRequested != null)
 				CancelRequested ();
 		}
