@@ -66,25 +66,25 @@ namespace MonoDevelop.NUnit
 			return conf;
 		}
 		
-		protected override void OnClean (IProgressMonitor monitor, string configuration)
+		protected override void OnClean (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 		}
 		
-		protected override BuildResult OnBuild (IProgressMonitor monitor, string configuration)
+		protected override BuildResult OnBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			return null;
 		}
 		
-		protected override void OnExecute (IProgressMonitor monitor, ExecutionContext context, string configuration)
+		protected override void OnExecute (IProgressMonitor monitor, ExecutionContext context, ConfigurationSelector configuration)
 		{
 		}
 		
-		protected override bool OnGetNeedsBuilding (string configuration)
+		protected override bool OnGetNeedsBuilding (ConfigurationSelector configuration)
 		{
 			return false;
 		}
 		
-		protected override void OnSetNeedsBuilding (bool value, string configuration)
+		protected override void OnSetNeedsBuilding (bool value, ConfigurationSelector configuration)
 		{
 		}
 	}
@@ -174,7 +174,7 @@ namespace MonoDevelop.NUnit
 		
 		protected override void OnCreateTests ()
 		{
-			NUnitAssemblyGroupProjectConfiguration conf = (NUnitAssemblyGroupProjectConfiguration) project.GetConfiguration (ActiveConfiguration);
+			NUnitAssemblyGroupProjectConfiguration conf = (NUnitAssemblyGroupProjectConfiguration) project.GetConfiguration ((ItemConfigurationSelector) ActiveConfiguration);
 			if (conf != null) {
 				foreach (TestAssembly t in conf.Assemblies)
 					Tests.Add (t);
