@@ -1,6 +1,7 @@
 //  VBBindingCompilerServices.cs
 //
-//  This file was derived from a file from #Develop.
+//  This file was derived from a file from #Develop, and relicensed
+//  by Markus Palme to MIT/X11
 //
 //  Authors:
 //    Markus Palme <MarkusPalme@gmx.de>
@@ -9,24 +10,30 @@
 //  Copyright (C) 2001-2007 Markus Palme <MarkusPalme@gmx.de>
 //  Copyright (C) 2008 Novell, Inc (http://www.novell.com)
 // 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
 // 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 using System;
 using System.Text;
 using System.Text.RegularExpressions;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Diagnostics;
 using System.CodeDom.Compiler;
@@ -50,7 +57,7 @@ namespace MonoDevelop.VBNetBinding {
 		//matches "/home/path/Default.aspx.vb (40,31) : Error VBNC30205: Expected end of statement."
 		//and "Error : VBNC99999: vbnc crashed nearby this location in the source code."
 		//and "Error : VBNC99999: Unexpected error: Object reference not set to an instance of an object" 
-		static Regex regexError = new Regex (@"^\s*((?<file>.*) \((?<line>\d*),(?<column>\d*)\) : )?(?<level>\w+) :? ?(?<number>[^:]*): (?<message>.*)$",
+		static Regex regexError = new Regex (@"^\s*((?<file>.*)\s?\((?<line>\d*)(,(?<column>\d*))?\) : )?(?<level>\w+) :? ?(?<number>[^:]*): (?<message>.*)$",
 		                                     RegexOptions.Compiled | RegexOptions.ExplicitCapture);
 		
 		string GenerateOptions (DotNetProjectConfiguration configuration, VBCompilerParameters compilerparameters, VBProjectParameters projectparameters, string outputFileName)
@@ -63,6 +70,7 @@ namespace MonoDevelop.VBNetBinding {
 			
 			sb.AppendLine ("-nologo");
 			sb.AppendLine ("-utf8output");
+			sb.AppendLine ("-quiet");
 
 			sb.AppendFormat ("-debug:{0}", compilerparameters.DebugType);
 			sb.AppendLine ();
@@ -145,7 +153,7 @@ namespace MonoDevelop.VBNetBinding {
 			}
 
 			if (!string.IsNullOrEmpty (compilerparameters.DocumentationFile)) {
-				sb.AppendFormat ("-doc:{0}", compilerparameters.DocumentationFile);
+				sb.AppendFormat ("\"-doc:{0}\"", compilerparameters.DocumentationFile);
 				sb.AppendLine ();
 			}
 
@@ -183,7 +191,7 @@ namespace MonoDevelop.VBNetBinding {
 			return sb.ToString();
 		}
 		
-		public BuildResult Compile (ProjectItemCollection items, DotNetProjectConfiguration configuration, IProgressMonitor monitor)
+		public BuildResult Compile (ProjectItemCollection items, DotNetProjectConfiguration configuration, ConfigurationSelector configSelector, IProgressMonitor monitor)
 		{
 			VBCompilerParameters compilerparameters = (VBCompilerParameters) configuration.CompilationParameters;
 			if (compilerparameters == null)
@@ -201,7 +209,7 @@ namespace MonoDevelop.VBNetBinding {
 
 			// Write references
 			foreach (ProjectReference lib in items.GetAll<ProjectReference> ()) {
-				foreach (string fileName in lib.GetReferencedFileNames(configuration.Id)) {
+				foreach (string fileName in lib.GetReferencedFileNames (configSelector)) {
 					writer.Write ("\"-r:");
 					writer.Write (fileName);
 					writer.WriteLine ("\"");
@@ -239,7 +247,13 @@ namespace MonoDevelop.VBNetBinding {
 			writer.Close();
 			
 			string output = "";
-			string compilerName = "vbnc";
+			string compilerName = configuration.TargetRuntime.GetToolPath (configuration.TargetFramework, "vbc");
+			if (compilerName == null) {
+				BuildResult res = new BuildResult ();
+				res.AddError (string.Format ("Visual Basict .NET compiler not found ({0})", configuration.TargetRuntime.DisplayName));
+				return res;
+			}
+			
 			string outstr = String.Concat (compilerName, " @", responseFileName);
 			
 			
@@ -248,8 +262,10 @@ namespace MonoDevelop.VBNetBinding {
 				workingDir = configuration.ParentItem.BaseDirectory;
 			int exitCode;
 			
-			monitor.Log.WriteLine (compilerName + " " + string.Join (" ", File.ReadAllLines (responseFileName)));
-			exitCode = DoCompilation (outstr, tf, workingDir, ref output);
+			Dictionary<string,string> envVars = configuration.TargetRuntime.GetToolsEnvironmentVariables (configuration.TargetFramework);
+			
+			monitor.Log.WriteLine (Path.GetFileName (compilerName) + " " + string.Join (" ", File.ReadAllLines (responseFileName)));
+			exitCode = DoCompilation (outstr, tf, workingDir, envVars, ref output);
 			
 			monitor.Log.WriteLine (output);			                                                          
 			BuildResult result = ParseOutput (tf, output);
@@ -333,7 +349,7 @@ namespace MonoDevelop.VBNetBinding {
 				error.IsWarning = match.Result ("${level}").ToLowerInvariant () == "warning";
 				error.ErrorNumber = match.Result("${number}");
 				error.ErrorText = match.Result("${message}");
-				error.FileName = match.Result ("${file}");
+				error.FileName = match.Result ("${file}").Trim ();
 				if (int.TryParse (match.Result ("${line}"), out i))
 					error.Line = i;
 				if (int.TryParse (match.Result ("${column}"), out i))
@@ -349,13 +365,28 @@ namespace MonoDevelop.VBNetBinding {
 			return null;
 		}
 		
-		private int DoCompilation (string outstr, TempFileCollection tf, string working_dir, ref string output)
+		private int DoCompilation (string outstr, TempFileCollection tf, string working_dir, Dictionary<string,string> envVars, ref string output)
 		{
 			StringWriter outwr = new StringWriter ();
 			string[] tokens = outstr.Split (' ');			
 			try {
 				outstr = outstr.Substring (tokens[0].Length+1);
-				ProcessWrapper pw = Runtime.ProcessService.StartProcess (tokens[0], "\"" + outstr + "\"", working_dir, outwr, outwr, null);
+				
+				ProcessStartInfo pinfo = new ProcessStartInfo (tokens[0], "\"" + outstr + "\"");
+				pinfo.WorkingDirectory = working_dir;
+				
+				foreach (KeyValuePair<string,string> ev in envVars) {
+					if (ev.Value == null)
+						pinfo.EnvironmentVariables.Remove (ev.Key);
+					else
+						pinfo.EnvironmentVariables [ev.Key] = ev.Value;
+				}
+			
+				pinfo.UseShellExecute = false;
+				pinfo.RedirectStandardOutput = true;
+				pinfo.RedirectStandardError = true;
+				
+				ProcessWrapper pw = Runtime.ProcessService.StartProcess (pinfo, outwr, outwr, null);
 				pw.WaitForOutput ();
 				output = outwr.ToString ();
 				
