@@ -23,21 +23,23 @@ namespace MonoDevelop.VersionControl.Subversion.Gui
 		public bool Save {
 			get { return checkSave.Active; }
 		}
-		
-		internal static bool Show (string realm, int may_save, out LibSvnClient.svn_auth_cred_ssl_client_cert_t retData)
+
+		internal static bool Show (string realm, bool may_save, out string cert_file, out bool save)
 		{
-			LibSvnClient.svn_auth_cred_ssl_client_cert_t data = new LibSvnClient.svn_auth_cred_ssl_client_cert_t ();
+			string local_cert_file = null;
+			bool local_save = false;
 			
 			bool res = false;
 			object monitor = new Object ();
 			
 			EventHandler del = delegate {
-					ClientCertificateDialog dlg = new ClientCertificateDialog (realm, may_save != 0);
+					ClientCertificateDialog dlg = new ClientCertificateDialog (realm, may_save);
+					dlg.TransientFor = MonoDevelop.Ide.Gui.IdeApp.Workbench.RootWindow;
 					try {
 						res = (dlg.Run () == (int) Gtk.ResponseType.Ok);
 						if (res) {
-							data.may_save = dlg.Save ? 1 : 0;
-							data.cert_file = dlg.File;
+							local_save = dlg.Save;
+							local_cert_file = dlg.File;
 						}
 					} finally {
 						dlg.Destroy ();
@@ -57,7 +59,8 @@ namespace MonoDevelop.VersionControl.Subversion.Gui
 					System.Threading.Monitor.Wait (monitor);
 				}
 			}
-			retData = data;
+			cert_file = local_cert_file;
+			save = local_save;
 			return res;
 		}
 	}
