@@ -12,7 +12,7 @@ using MonoDevelop.Core;
 using MonoDevelop.VersionControl.Subversion.Gui;
 
 
-namespace MonoDevelop.VersionControl.Subversion {
+namespace MonoDevelop.VersionControl.Subversion.Unix {
 	public abstract class LibSvnClient {
 		public LibSvnClient ()
 		{
@@ -135,6 +135,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		public abstract IntPtr client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
 		
+		public abstract IntPtr client_blame (string path, ref Rev rev_start, ref Rev rev_end, svn_client_blame_receiver_t receiver, IntPtr baton, IntPtr ctx, IntPtr pool);
+		
 		public class DirEnt {
 			public readonly string Name;
 			public readonly bool IsDirectory;
@@ -254,44 +256,6 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
-		public class LogEnt {
-			public readonly int Revision;
-			public readonly string Author;
-			public readonly DateTime Time;
-			public readonly string Message;
-			public readonly LogEntChangedPath[] ChangedPaths;
-			
-			internal LogEnt (int rev, string author, DateTime time, string msg, LogEntChangedPath[] changes)
-			{
-				Revision = rev;
-				Author = author;
-				Time = time;
-				Message = msg;
-				ChangedPaths = changes;
-			}
-		}
-		
-		public class LogEntChangedPath {
-			public readonly string Path;
-			public readonly RevisionAction Action;
-			public readonly string ActionDesc;
-			public readonly string CopyFromPath;
-			public readonly int CopyFromRevision;
-			
-			internal LogEntChangedPath(string path, svn_log_changed_path_t info) {
-				Path = path;
-				CopyFromPath = info.copy_from_path;
-				CopyFromRevision = info.copy_from_rev;
-				
-				switch (info.action) {
-				case 'A': Action = RevisionAction.Add; break;
-				case 'D': Action = RevisionAction.Delete; break;
-				case 'R': Action = RevisionAction.Replace; break;
-				default: Action = RevisionAction.Modify; break; // should be an 'M'
-				}
-			}
-		}
-		
 		// Native Interop
 		[StructLayout(LayoutKind.Sequential)]
 		public struct svn_client_ctx_t {
@@ -462,6 +426,20 @@ namespace MonoDevelop.VersionControl.Subversion {
 				return r;
 			}
 			
+			public static explicit operator Rev (SvnRevision rev)
+			{
+				Rev r = new Rev ();
+				r.kind = (IntPtr) rev.Kind;
+				if (rev.Kind == 1)
+					r.value.number = (IntPtr) rev.Rev;
+				return r;
+			}
+			
+			public static explicit operator Rev (Revision rev)
+			{
+				return (Rev) (SvnRevision) rev;
+			}
+			
 			public readonly static Rev Blank = new Rev(0);
 			public readonly static Rev First = new Rev(1);
 			public readonly static Rev Committed = new Rev(3);
@@ -596,6 +574,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		public delegate IntPtr svn_client_get_commit_log_t (ref IntPtr log_msg, ref IntPtr tmp_file, IntPtr commit_items,
 		                                                    IntPtr baton, IntPtr pool);
+		
+		public delegate IntPtr svn_client_blame_receiver_t (IntPtr baton, long line_no, int revision, string author, string date, string line, IntPtr pool);
 	}
 	
 	public class LibSvnClient0 : LibSvnClient {
@@ -843,6 +823,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return svn_client_prop_get (out value, name, target, ref revision, recurse, ctx, pool);
 		}
 		
+		public override IntPtr client_blame (string path, ref Rev rev_start, ref Rev rev_end, svn_client_blame_receiver_t receiver, System.IntPtr baton, System.IntPtr ctx, System.IntPtr pool)
+		{
+			return svn_client_blame (path, ref rev_start, ref rev_end, receiver, baton, ctx, pool);
+		}
+
+		
 		[DllImport(svnclientlib)] static extern void svn_config_ensure (string config_dir, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_auth_open (out IntPtr auth_baton, IntPtr providers, IntPtr pool);
@@ -948,6 +934,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool);
 		
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
+
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_blame (string path, ref Rev rev_start, ref Rev rev_end, svn_client_blame_receiver_t receiver, IntPtr baton, IntPtr ctx, IntPtr pool);
 	}
 	
 	public class LibSvnClient1 : LibSvnClient {
@@ -1194,6 +1182,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 		{
 			return svn_client_prop_get (out value, name, target, ref revision, recurse, ctx, pool);
 		}
+		
+		public override IntPtr client_blame (string path, ref Rev rev_start, ref Rev rev_end, svn_client_blame_receiver_t receiver, System.IntPtr baton, System.IntPtr ctx, System.IntPtr pool)
+		{
+			return svn_client_blame (path, ref rev_start, ref rev_end, receiver, baton, ctx, pool);
+		}
+
 
 		[DllImport(svnclientlib)] static extern void svn_config_ensure (string config_dir, IntPtr pool);
 		[DllImport(svnclientlib)] static extern void svn_config_get_config (ref IntPtr cfg_hash, string config_dir, IntPtr pool);
@@ -1301,5 +1295,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_unlock (IntPtr apr_array_header_t_targets, int break_lock, IntPtr ctx, IntPtr pool);
 		
 		[DllImport(svnclientlib)] static extern IntPtr svn_client_prop_get (out IntPtr value, string name, string target, ref Rev revision, int recurse, IntPtr ctx, IntPtr pool);
+		
+		[DllImport(svnclientlib)] static extern IntPtr svn_client_blame (string path, ref Rev rev_start, ref Rev rev_end, svn_client_blame_receiver_t receiver, IntPtr baton, IntPtr ctx, IntPtr pool);
 	}
 }
