@@ -1,5 +1,6 @@
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Threading;
 
@@ -7,8 +8,10 @@ using System.Runtime.InteropServices;
 using MonoDevelop.Core;
 using MonoDevelop.VersionControl.Subversion.Gui;
 
-namespace MonoDevelop.VersionControl.Subversion {
-	public class SvnClient : IDisposable {
+namespace MonoDevelop.VersionControl.Subversion.Unix
+{
+	class SvnClient : SubversionVersionControl
+	{
 		static LibSvnClient svn;
 		static LibApr apr;
 		static bool isInstallChecked;
@@ -53,7 +56,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
-		internal static bool IsInstalled {
+		public override bool IsInstalled {
 			get {
 				if (!isInstallChecked)
 					CheckInstalled ();
@@ -90,8 +93,18 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return p;
 		}
 		
+		public override string Id {
+			get {
+				return "MonoDevelop.VersionControl.Subversion.SubversionVersionControl";
+			}
+		}
+
+		
 		public SvnClient ()
 		{
+			if (!IsInstalled)
+				return;
+			
 			// Allocate the APR pool and the SVN client context.
 			pool = newpool (IntPtr.Zero);
 
@@ -178,7 +191,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 		public void Dispose ()
 		{
 			if (!disposed) {
-				apr.pool_destroy(pool);
+				if (apr != null)
+					apr.pool_destroy(pool);
 				disposed = true;
 			}
 		}
@@ -203,8 +217,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		static IntPtr OnAuthSimplePrompt (ref IntPtr cred, IntPtr baton, [MarshalAs (UnmanagedType.LPStr)] string realm, [MarshalAs (UnmanagedType.LPStr)] string user_name, [MarshalAs (UnmanagedType.SysInt)] int may_save, IntPtr pool)
 		{
-			LibSvnClient.svn_auth_cred_simple_t data; 
-			if (UserPasswordDialog.Show (user_name, realm, may_save != 0, out data)) {
+			LibSvnClient.svn_auth_cred_simple_t data = new LibSvnClient.svn_auth_cred_simple_t ();
+			bool ms;
+			if (SimpleAuthenticationPrompt (realm, may_save != 0, ref data.username, out data.password, out ms)) {
+				data.may_save = ms ? 1 : 0;
 				cred = apr.pcalloc (pool, data);
 				return IntPtr.Zero;
 			} else {
@@ -218,8 +234,11 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		static IntPtr OnAuthUsernamePrompt (ref IntPtr cred, IntPtr baton, [MarshalAs (UnmanagedType.LPStr)] string realm, [MarshalAs (UnmanagedType.SysInt)] int may_save, IntPtr pool)
 		{
-			LibSvnClient.svn_auth_cred_username_t data;
-			if (UserPasswordDialog.Show (realm, may_save != 0, out data)) {
+			LibSvnClient.svn_auth_cred_username_t data = new LibSvnClient.svn_auth_cred_username_t ();
+			data.username = "";
+			bool ms;
+			if (UserNameAuthenticationPrompt (realm, may_save != 0, ref data.username, out ms)) {
+				data.may_save = ms ? 1 : 0;
 				cred = apr.pcalloc (pool, data);
 				return IntPtr.Zero;
 			} else {
@@ -232,8 +251,21 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		static IntPtr OnAuthSslServerTrustPrompt (ref IntPtr cred, IntPtr baton, [MarshalAs (UnmanagedType.LPStr)] string realm, uint failures, ref LibSvnClient.svn_auth_ssl_server_cert_info_t cert_info, [MarshalAs (UnmanagedType.SysInt)] int may_save, IntPtr pool)
 		{
-			LibSvnClient.svn_auth_cred_ssl_server_trust_t data;
-			if (SslServerTrustDialog.Show (realm, failures, may_save, cert_info, out data) && data.accepted_failures != 0) {
+			LibSvnClient.svn_auth_cred_ssl_server_trust_t data = new LibSvnClient.svn_auth_cred_ssl_server_trust_t ();
+			
+			CertficateInfo ci = new CertficateInfo ();
+			ci.AsciiCert = cert_info.ascii_cert;
+			ci.Fingerprint = cert_info.fingerprint;
+			ci.HostName = cert_info.hostname;
+			ci.IssuerName = cert_info.issuer_dname;
+			ci.ValidFrom = cert_info.valid_from;
+			ci.ValidUntil = cert_info.valid_until;
+
+			SslFailure accepted_failures;
+			bool ms;
+			if (SslServerTrustAuthenticationPrompt (realm, (SslFailure) failures, may_save != 0, ci, out accepted_failures, out ms) && accepted_failures != SslFailure.None) {
+				data.may_save = ms ? 1 : 0;
+				data.accepted_failures = (uint) accepted_failures;
 				cred = apr.pcalloc (pool, data);
 				return IntPtr.Zero;
 			} else {
@@ -246,8 +278,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		static IntPtr OnAuthSslClientCertPrompt (ref IntPtr cred, IntPtr baton, [MarshalAs (UnmanagedType.LPStr)] string realm, [MarshalAs (UnmanagedType.SysInt)] int may_save, IntPtr pool)
 		{
-			LibSvnClient.svn_auth_cred_ssl_client_cert_t data;
-			if (ClientCertificateDialog.Show (realm, may_save, out data)) {
+			LibSvnClient.svn_auth_cred_ssl_client_cert_t data = new LibSvnClient.svn_auth_cred_ssl_client_cert_t ();
+			bool ms;
+			if (SslClientCertAuthenticationPrompt (realm, may_save != 0, out data.cert_file, out ms)) {
+				data.may_save = ms ? 1 : 0;
 				cred = apr.pcalloc (pool, data);
 				return IntPtr.Zero;
 			} else {
@@ -261,7 +295,9 @@ namespace MonoDevelop.VersionControl.Subversion {
 		static IntPtr OnAuthSslClientCertPwPrompt (ref IntPtr cred, IntPtr baton, [MarshalAs (UnmanagedType.LPStr)] string realm, [MarshalAs (UnmanagedType.SysInt)] int may_save, IntPtr pool)
 		{
 			LibSvnClient.svn_auth_cred_ssl_client_cert_pw_t data;
-			if (ClientCertificatePasswordDialog.Show (realm, may_save, out data)) {
+			bool ms;
+			if (SslClientCertPwAuthenticationPrompt (realm, may_save != 0, out data.password, out ms)) {
+				data.may_save = ms ? 1 : 0;
 				cred = apr.pcalloc (pool, data);
 				return IntPtr.Zero;
 			} else {
@@ -274,21 +310,21 @@ namespace MonoDevelop.VersionControl.Subversion {
 		
 		// Wrappers for native interop
 		
-		public string Version {
-			get {
-				IntPtr ptr = svn.client_version ();
-				LibSvnClient.svn_version_t ver = (LibSvnClient.svn_version_t) Marshal.PtrToStructure (ptr, typeof (LibSvnClient.svn_version_t));				
-				return ver.major + "." + ver.minor + "." + ver.patch;
-			}
-		}
-		
-		public IList List (string pathorurl, bool recurse, LibSvnClient.Rev revision)
+		public override string GetVersion ()
 		{
-			if (pathorurl == null)
+			IntPtr ptr = svn.client_version ();
+			LibSvnClient.svn_version_t ver = (LibSvnClient.svn_version_t) Marshal.PtrToStructure (ptr, typeof (LibSvnClient.svn_version_t));				
+			return ver.major + "." + ver.minor + "." + ver.patch;
+		}
+
+		public override IEnumerable<DirectoryEntry> List (FilePath pathorurl, bool recurse, SvnRevision rev)
+		{
+			if (pathorurl == FilePath.Null)
 				throw new ArgumentNullException ();
 			
+			LibSvnClient.Rev revision = (LibSvnClient.Rev) rev;
 			IntPtr localpool = newpool (pool);
-			ArrayList items = new ArrayList ();
+			List<DirectoryEntry> items = new List<DirectoryEntry> ();
 			try {
 				IntPtr hash;
 				
@@ -305,7 +341,15 @@ namespace MonoDevelop.VersionControl.Subversion {
 					LibSvnClient.svn_dirent_t ent = (LibSvnClient.svn_dirent_t) Marshal.PtrToStructure (val, typeof (LibSvnClient.svn_dirent_t));				
 					item = apr.hash_next (item);
 					
-					items.Add (new LibSvnClient.DirEnt (name, ent));
+					DirectoryEntry dent = new DirectoryEntry ();
+					dent.Name = name;
+					dent.IsDirectory = (ent.kind.ToInt32 () == (int) LibSvnClient.NodeKind.Dir);
+					dent.Size = ent.size;
+					dent.HasProps = ent.has_props != 0;
+					dent.CreatedRevision = ent.created_rev;
+					dent.Time = new DateTime (1970, 1, 1).AddTicks(ent.time * 10);
+					dent.LastAuthor = ent.last_author;
+					items.Add (dent);
 				}
 			} finally {
 				apr.pool_destroy (localpool);
@@ -314,16 +358,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return items;
 		}
 
-		public IList Status (string path, LibSvnClient.Rev revision)
+		public override IEnumerable<VersionInfo> Status (Repository repo, FilePath path, SvnRevision rev, bool descendDirs, bool changedItemsOnly, bool remoteStatus)
 		{
-			return Status (path, revision, false, false, false);
-		}
-		
-		public IList Status (string path, LibSvnClient.Rev revision, bool descendDirs, bool changedItemsOnly, bool remoteStatus)
-		{
-			if (path == null)
+			if (path == FilePath.Null)
 				throw new ArgumentNullException ();
-			
+		
+			LibSvnClient.Rev revision = (LibSvnClient.Rev) rev;
 			ArrayList ret = new ArrayList ();
 			
 			StatusCollector collector = new StatusCollector (ret);
@@ -340,15 +380,19 @@ namespace MonoDevelop.VersionControl.Subversion {
 				apr.pool_destroy (localpool);
 			}
 			
-			return ret;
+			foreach (LibSvnClient.StatusEnt ent in ret)
+				yield return CreateNode (ent, repo);
 		}
-		
-		public IList Log (string path, LibSvnClient.Rev revisionStart, LibSvnClient.Rev revisionEnd)
+
+		public override IEnumerable<SvnRevision> Log (Repository repo, FilePath path, SvnRevision revStart, SvnRevision revEnd)
 		{
-			if (path == null)
+			if (path == FilePath.Null)
 				throw new ArgumentNullException ();
 			
-			ArrayList ret = new ArrayList ();
+			LibSvnClient.Rev revisionStart = (LibSvnClient.Rev) revStart;
+			LibSvnClient.Rev revisionEnd = (LibSvnClient.Rev) revEnd;
+			
+			List<SvnRevision> ret = new List<SvnRevision> ();
 			IntPtr localpool = newpool (pool);
 			IntPtr strptr = IntPtr.Zero;
 			try {
@@ -371,9 +415,36 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return ret;
 		}
 		
-		public string GetPathUrl (string path)
+		public override List<string> GetAnnotations (Repository repo, FilePath file, SvnRevision revStart, SvnRevision revEnd)
 		{
-			if (path == null)
+			if (file == FilePath.Null)
+				throw new ArgumentNullException ();
+				
+			LibSvnClient.Rev revisionStart = (LibSvnClient.Rev) revStart;
+			LibSvnClient.Rev revisionEnd = (LibSvnClient.Rev) revEnd;
+			
+			int numAnnotations = File.ReadAllLines (repo.GetPathToBaseText(file)).Length;
+			List<string> annotations = new List<string> (numAnnotations);
+			for (int i=0; i<numAnnotations; ++i) {
+				annotations.Insert (0, string.Empty);
+			}
+			
+			AnnotationCollector collector = new AnnotationCollector (annotations);
+			
+			IntPtr localpool = newpool (pool);
+			
+			try {
+				CheckError (svn.client_blame (file.FullPath, ref revisionStart, ref revisionEnd, new LibSvnClient.svn_client_blame_receiver_t (collector.Func), IntPtr.Zero, ctx, localpool));
+			} finally {
+				apr.pool_destroy (localpool);
+			}
+			
+			return annotations;
+		}
+
+		public override string GetPathUrl (FilePath path)
+		{
+			if (path == FilePath.Null)
 				throw new ArgumentNullException();
 			
 			IntPtr ret = IntPtr.Zero;
@@ -390,10 +461,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return Marshal.PtrToStringAnsi (ret);
 		}
 		
-		public string Cat (string pathorurl, LibSvnClient.Rev revision)
+		public override string GetTextAtRevision (string pathorurl, Revision revision)
 		{
 			MemoryStream memstream = new MemoryStream ();
-			Cat (pathorurl, revision, memstream);
+			Cat (pathorurl, (SvnRevision) revision, memstream);
 			
 			try {
 				return System.Text.Encoding.UTF8.GetString (memstream.GetBuffer ());
@@ -403,10 +474,12 @@ namespace MonoDevelop.VersionControl.Subversion {
 			return System.Text.Encoding.ASCII.GetString (memstream.GetBuffer ());
 		}
 		
-		public void Cat (string pathorurl, LibSvnClient.Rev revision, Stream stream)
+		public void Cat (string pathorurl, SvnRevision rev, Stream stream)
 		{
 			if (pathorurl == null || stream == null)
 				throw new ArgumentNullException ();
+		
+			LibSvnClient.Rev revision = (LibSvnClient.Rev) rev;
 			
 			IntPtr localpool = newpool (pool);
 			try {
@@ -419,10 +492,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 				apr.pool_destroy (localpool);
 			}
 		}
-		
-		public void Update (string path, bool recurse, IProgressMonitor monitor)
+
+		public override void Update (FilePath path, bool recurse, IProgressMonitor monitor)
 		{
-			if (path == null || monitor == null)
+			if (path == FilePath.Null || monitor == null)
 				throw new ArgumentNullException();
 			
 			lock (sync) {
@@ -448,8 +521,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Revert (string[] paths, bool recurse, IProgressMonitor monitor)
+
+		public override void Revert (FilePath[] paths, bool recurse, IProgressMonitor monitor)
 		{
 			if (paths == null || monitor == null)
 				throw new ArgumentNullException();
@@ -478,10 +551,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Resolve (string path, bool recurse, IProgressMonitor monitor)
+
+		public override void Resolve (FilePath path, bool recurse, IProgressMonitor monitor)
 		{
-			if (path == null || monitor == null)
+			if (path == FilePath.Null || monitor == null)
 				throw new ArgumentNullException();
 			
 			lock (sync) {
@@ -501,10 +574,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Add (string path, bool recurse, IProgressMonitor monitor)
+
+		public override void Add (FilePath path, bool recurse, IProgressMonitor monitor)
 		{
-			if (path == null || monitor == null)
+			if (path == FilePath.Null || monitor == null)
 				throw new ArgumentNullException ();
 			
 			lock (sync) {
@@ -512,7 +585,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 					throw new SubversionException ("Another Subversion operation is already in progress.");
 				inProgress = true;
 			}
-			
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			IntPtr localpool = newpool (pool);
 			try {
@@ -523,18 +596,22 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Checkout (string url, string path, LibSvnClient.Rev rev, bool recurse, IProgressMonitor monitor)
+
+		public override void Checkout (string url, FilePath path, Revision revision, bool recurse, IProgressMonitor monitor)
 		{
 			if (url == null || monitor == null)
 				throw new ArgumentNullException ();
 			
+			if (revision == null)
+				revision = SvnRevision.Head;
+			LibSvnClient.Rev rev = (LibSvnClient.Rev) revision;
+			
 			lock (sync) {
 				if (inProgress)
 					throw new SubversionException ("Another Subversion operation is already in progress.");
 				inProgress = true;
 			}
-			
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			IntPtr result_rev = IntPtr.Zero;
 			IntPtr localpool = newpool (pool);
@@ -546,8 +623,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Commit (string[] paths, string message, IProgressMonitor monitor)
+
+		public override void Commit (FilePath[] paths, string message, IProgressMonitor monitor)
 		{
 			if (paths == null || message == null || monitor == null)
 				throw new ArgumentNullException();
@@ -558,6 +635,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = true;
 			}
 			
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			
 			IntPtr localpool = newpool (pool);
@@ -575,8 +653,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 				
 				CheckError (svn.client_commit (ref commit_info, array, 0, ctx, localpool));
 				unsafe {
-					if (commit_info != IntPtr.Zero)
+					if (commit_info != IntPtr.Zero) {
+						monitor.Log.WriteLine ();
 						monitor.Log.WriteLine (GettextCatalog.GetString ("Revision: {0}", ((LibSvnClient.svn_client_commit_info_t *) commit_info.ToPointer())->revision));
+					}
 				}
 			} finally {
 				commitmessage = null;
@@ -585,8 +665,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Mkdir (string[] paths, string message, IProgressMonitor monitor) 
+
+		public override void Mkdir (string[] paths, string message, IProgressMonitor monitor) 
 		{
 			if (paths == null || monitor == null)
 				throw new ArgumentNullException ();
@@ -597,6 +677,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = true;
 			}
 
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			
 			IntPtr localpool = newpool(pool);
@@ -619,10 +700,10 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Delete (string path, bool force, IProgressMonitor monitor)
+
+		public override void Delete (FilePath path, bool force, IProgressMonitor monitor)
 		{
-			if (path == null || monitor == null)
+			if (path == FilePath.Null || monitor == null)
 				throw new ArgumentNullException ();
 			
 			lock (sync) {
@@ -631,6 +712,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = true;
 			}
 			
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			
 			IntPtr localpool = newpool (pool);
@@ -650,18 +732,21 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Move (string srcPath, string destPath, LibSvnClient.Rev revision, bool force, IProgressMonitor monitor)
+
+		public override void Move (FilePath srcPath, FilePath destPath, SvnRevision rev, bool force, IProgressMonitor monitor)
 		{
-			if (srcPath == null || destPath == null || monitor == null)
+			if (srcPath == FilePath.Null || destPath == FilePath.Null || monitor == null)
 				throw new ArgumentNullException ();
 			
+			LibSvnClient.Rev revision = (LibSvnClient.Rev) rev;
+			
 			lock (sync) {
 				if (inProgress)
 					throw new SubversionException ("Another Subversion operation is already in progress.");
 				inProgress = true;
 			}
 			
+			nb = new notify_baton ();
 			updatemonitor = monitor;
 			IntPtr commit_info = IntPtr.Zero;
 			IntPtr localpool = newpool (pool);
@@ -674,8 +759,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public void Lock (IProgressMonitor monitor, string comment, bool stealLock, params string[] paths)
+
+		public override void Lock (IProgressMonitor monitor, string comment, bool stealLock, params FilePath[] paths)
 		{
 			lock (sync) {
 				if (inProgress)
@@ -686,6 +771,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 			IntPtr localpool = newpool (pool);
 			IntPtr array = apr.array_make (localpool, 0, IntPtr.Size);
 			updatemonitor = monitor;
+			nb = new notify_baton ();
 
 			try {
 				foreach (string path in paths) {
@@ -706,7 +792,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
-		public void Unlock (IProgressMonitor monitor, bool breakLock, params string[] paths)
+		public override void Unlock (IProgressMonitor monitor, bool breakLock, params FilePath[] paths)
 		{
 			lock (sync) {
 				if (inProgress)
@@ -717,6 +803,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 			IntPtr localpool = newpool (pool);
 			IntPtr array = apr.array_make (localpool, 0, IntPtr.Size);
 			updatemonitor = monitor;
+			nb = new notify_baton ();
 			
 			try {
 				foreach (string path in paths) {
@@ -736,8 +823,8 @@ namespace MonoDevelop.VersionControl.Subversion {
 				inProgress = false;
 			}
 		}
-		
-		public string PathDiff (string path1, LibSvnClient.Rev revision1, string path2, LibSvnClient.Rev revision2, bool recursive)
+
+		public override string GetUnifiedDiff (FilePath path1, SvnRevision rev1, FilePath path2, SvnRevision rev2, bool recursive)
 		{
 			IntPtr localpool = newpool (pool);
 			IntPtr outfile = IntPtr.Zero;
@@ -745,6 +832,9 @@ namespace MonoDevelop.VersionControl.Subversion {
 			string fout = null;
 			string ferr = null;
 			
+			LibSvnClient.Rev revision1 = (LibSvnClient.Rev) rev1;
+			LibSvnClient.Rev revision2 = (LibSvnClient.Rev) rev2;
+			
 			try {
 				IntPtr options = apr.array_make (localpool, 0, IntPtr.Size);
 				
@@ -755,7 +845,9 @@ namespace MonoDevelop.VersionControl.Subversion {
 				
 				if (res1 == 0 && res2 == 0) {
 					CheckError (svn.client_diff (options, path1, ref revision1, path2, ref revision2, (recursive ? 1 : 0), 0, 1, outfile, errfile, ctx, localpool));
-					return fout;
+					using (StreamReader sr = new StreamReader (fout)) {
+						return sr.ReadToEnd ();
+					}
 				} else {
 					throw new Exception ("Could not get diff information");
 				}
@@ -763,10 +855,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				try {
 					if (outfile != IntPtr.Zero)
 						apr.file_close (outfile);
-					if (fout != null)
-						FileService.DeleteFile (fout);
 					outfile = IntPtr.Zero;
-					fout = null;
 				} catch {}
 				throw;
 			} finally {
@@ -779,18 +868,21 @@ namespace MonoDevelop.VersionControl.Subversion {
 						apr.file_close (errfile);
 					if (ferr != null)
 						FileService.DeleteFile (ferr);
+					if (fout != null)
+						FileService.DeleteFile (fout);
 				} catch {}
 			}
 		}
 
-		public void RevertToRevision (string path, int revision)
+		public override void RevertToRevision (FilePath path, Revision revision, IProgressMonitor monitor)
 		{
-			Merge (path, LibSvnClient.Rev.Head, LibSvnClient.Rev.Number(revision));
+			Merge (path, LibSvnClient.Rev.Head, (LibSvnClient.Rev) revision);
 		}
-		
-		public void RevertRevision (string path, int revision)
+
+		public override void RevertRevision (FilePath path, Revision revision, IProgressMonitor monitor)
 		{
-			Merge (path, LibSvnClient.Rev.Number(revision), LibSvnClient.Rev.Number (revision - 1));
+			SvnRevision srev = (SvnRevision) revision;
+			Merge (path, (LibSvnClient.Rev) srev, LibSvnClient.Rev.Number (srev.Rev - 1));
 		}
 		
 		private void Merge (string path, LibSvnClient.Rev revision1, LibSvnClient.Rev revision2)
@@ -832,50 +924,246 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 			
 			if (msg == null)
-				msg = "Unknown error";
+				msg = GettextCatalog.GetString ("Unknown error");
 			
 			throw new SubversionException (msg);
 		}
 		
+		private VersionInfo CreateNode (LibSvnClient.StatusEnt ent, Repository repo) 
+		{
+			VersionStatus rs = VersionStatus.Unversioned;
+			Revision rr = null;
+			
+			if (ent.RemoteTextStatus != LibSvnClient.VersionStatus.EMPTY) {
+				rs = ConvertStatus (LibSvnClient.NodeSchedule.Normal, ent.RemoteTextStatus);
+				rr = new SvnRevision (repo, ent.LastCommitRevision, ent.LastCommitDate,
+				                      ent.LastCommitAuthor, "(unavailable)", null);
+			}
+
+			VersionStatus status = ConvertStatus (ent.Schedule, ent.TextStatus);
+			
+			bool readOnly = File.Exists (ent.LocalFilePath) && (File.GetAttributes (ent.LocalFilePath) & FileAttributes.ReadOnly) != 0;
+			
+			if (ent.RepoLocked) {
+				status |= VersionStatus.LockRequired;
+				if (ent.LockOwned)
+					status |= VersionStatus.LockOwned;
+				else
+					status |= VersionStatus.Locked;
+			} else if (readOnly)
+				status |= VersionStatus.LockRequired;
+
+			VersionInfo ret = new VersionInfo (ent.LocalFilePath, ent.Url, ent.IsDirectory,
+			                                   status, new SvnRevision (repo, ent.Revision),
+			                                   rs, rr);
+			return ret;
+		}
+		
+		private VersionStatus ConvertStatus (LibSvnClient.NodeSchedule schedule, LibSvnClient.VersionStatus status) {
+			switch (schedule) {
+				case LibSvnClient.NodeSchedule.Add: return VersionStatus.Versioned | VersionStatus.ScheduledAdd;
+				case LibSvnClient.NodeSchedule.Delete: return VersionStatus.Versioned | VersionStatus.ScheduledDelete;
+				case LibSvnClient.NodeSchedule.Replace: return VersionStatus.Versioned | VersionStatus.ScheduledReplace;
+			}
+			
+			switch (status) {
+				case LibSvnClient.VersionStatus.None: return VersionStatus.Versioned;
+				case LibSvnClient.VersionStatus.Normal: return VersionStatus.Versioned;
+				case LibSvnClient.VersionStatus.Unversioned: return VersionStatus.Unversioned;
+				case LibSvnClient.VersionStatus.Modified: return VersionStatus.Versioned | VersionStatus.Modified;
+				case LibSvnClient.VersionStatus.Merged: return VersionStatus.Versioned | VersionStatus.Modified;
+				case LibSvnClient.VersionStatus.Conflicted: return VersionStatus.Versioned | VersionStatus.Conflicted;
+				case LibSvnClient.VersionStatus.Ignored: return VersionStatus.Unversioned | VersionStatus.Ignored;
+				case LibSvnClient.VersionStatus.Obstructed: return VersionStatus.Versioned;
+				case LibSvnClient.VersionStatus.Added: return VersionStatus.Versioned | VersionStatus.ScheduledAdd;
+				case LibSvnClient.VersionStatus.Deleted: return VersionStatus.Versioned | VersionStatus.ScheduledDelete;
+				case LibSvnClient.VersionStatus.Replaced: return VersionStatus.Versioned | VersionStatus.ScheduledReplace;
+			}
+			
+			return VersionStatus.Unversioned;
+		}
+		
+		// same logic as svn_mime_type_is_binary, but I don't see a reason for a native call
+		static bool MimeTypeIsBinary (string mimeType)
+		{
+			if (string.IsNullOrEmpty (mimeType))
+				return false;
+			return !(mimeType.StartsWith ("text/") || 
+			         mimeType == "image/x-xbitmap" || 
+			         mimeType == "image/x-xpixmap");
+		}
+		
+		struct notify_baton {
+			public bool received_some_change;
+			public bool is_checkout;
+			public bool is_export;
+			public bool suppress_final_line;
+			public bool sent_first_txdelta;
+			public bool in_external;
+			public bool had_print_error;
+		}
+		notify_baton nb;
 		void svn_wc_notify_func_t_impl (IntPtr baton, ref LibSvnClient.svn_wc_notify_t data, IntPtr pool)
 		{
-			string actiondesc = data.action.ToString ();
+			string actiondesc;
 			string file = Marshal.PtrToStringAnsi (data.path);
 			bool notifyChange = false;
-			
+			bool skipEol = false;
+//			System.Console.WriteLine(data.action);
 			switch (data.action) {
-			case LibSvnClient.NotifyAction.UpdateAdd: actiondesc = GettextCatalog.GetString ("Added"); break;
-			case LibSvnClient.NotifyAction.UpdateDelete: actiondesc = GettextCatalog.GetString ("Deleted"); break;
-			case LibSvnClient.NotifyAction.UpdateUpdate: actiondesc = GettextCatalog.GetString ("Updating"); notifyChange = true; break;
-			case LibSvnClient.NotifyAction.UpdateExternal: actiondesc = GettextCatalog.GetString ("External Updated"); break;
-			case LibSvnClient.NotifyAction.UpdateCompleted: actiondesc = GettextCatalog.GetString ("Finished"); break;
+			case LibSvnClient.NotifyAction.Skip: 
+				if (data.content_state == LibSvnClient.NotifyState.Missing) {
+					actiondesc = string.Format (GettextCatalog.GetString ("Skipped missing target: '{0}'"), file); 
+				} else {
+					actiondesc = string.Format (GettextCatalog.GetString ("Skipped '{0}'"), file); 
+				}
+				break;
+			case LibSvnClient.NotifyAction.UpdateDelete: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Deleted   '{0}'"), file);
+				break;
+//			case LibSvnClient.NotifyAction.UpdateReplace: 
+//				actiondesc = string.Format (GettextCatalog.GetString ("Replaced  '{0}'"), file);
+//				break;
 				
-			case LibSvnClient.NotifyAction.CommitAdded: actiondesc = GettextCatalog.GetString ("Added"); break;
-			case LibSvnClient.NotifyAction.CommitDeleted: actiondesc = GettextCatalog.GetString ("Deleted"); break;
-			case LibSvnClient.NotifyAction.CommitModified: actiondesc = GettextCatalog.GetString ("Modified"); notifyChange = true; break;
-			case LibSvnClient.NotifyAction.CommitReplaced: actiondesc = GettextCatalog.GetString ("Replaced"); notifyChange = true; break;
-			case LibSvnClient.NotifyAction.CommitPostfixTxDelta: actiondesc = GettextCatalog.GetString ("Sending Content"); break;
+			case LibSvnClient.NotifyAction.UpdateAdd: 
+				if (data.content_state == LibSvnClient.NotifyState.Conflicted) {
+					actiondesc = string.Format (GettextCatalog.GetString ("Conflict {0}"), file); 
+				} else {
+					actiondesc = string.Format (GettextCatalog.GetString ("Added   {0}"), file); 
+				}
+				break;
+//			case LibSvnClient.NotifyAction.Exists:
+//				// original is untranslated, we'll make it a bit shorter
+///*				actiondesc = data.content_state == LibSvnClient.NotifyState.Conflicted ? "C" : "E";
+//				if (data.prop_state == LibSvnClient.NotifyState.Conflicted) {
+//					actiondesc += "C";
+//				} else if (data.prop_state == LibSvnClient.NotifyState.Merged) {
+//					actiondesc += "G";
+//				}
+//				actiondesc += " {0}";
+//				actiondesc = string.Format (actiondesc, file); */
+//				actiondesc = string.Format (GettextCatalog.GetString ("Exists   {0}"), file);
+//				break;
+			case LibSvnClient.NotifyAction.Restore: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Restored '{0}'"), file); 
+				break;
+			case LibSvnClient.NotifyAction.Revert: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Reverted '{0}'"), file); 
+				break;
+			case LibSvnClient.NotifyAction.FailedRevert: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Failed to revert '{0}' -- try updating instead."), file);
+				break;
+			case LibSvnClient.NotifyAction.Resolved:
+				actiondesc = string.Format (GettextCatalog.GetString ("Resolved conflict state of '{0}'"), file);
+				break;
+			case LibSvnClient.NotifyAction.Add: 
+				if (MimeTypeIsBinary (Marshal.PtrToStringAuto (data.mime_type))) {
+					actiondesc = string.Format (GettextCatalog.GetString ("Add (bin) '{0}'"), file); 
+				} else {
+					actiondesc = string.Format (GettextCatalog.GetString ("Add       '{0}'"), file); 
+				}
+				break;
+			case LibSvnClient.NotifyAction.Delete: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Delete    '{0}'"), file);
+				break;
+				
+			case LibSvnClient.NotifyAction.UpdateUpdate: 
+				// original is untranslated, we'll make it a bit shorter
+			/*	actiondesc = "";
+				if (data.content_state == LibSvnClient.NotifyState.Conflicted) {
+					actiondesc += "C";
+				} else if (data.content_state == LibSvnClient.NotifyState.Merged) {
+					actiondesc += "G";
+				} else if (data.content_state == LibSvnClient.NotifyState.Changed) {
+					actiondesc += "U";
+				}
+				
+				if (data.prop_state == LibSvnClient.NotifyState.Conflicted) {
+					actiondesc += "C";
+				} else if (data.prop_state == LibSvnClient.NotifyState.Merged) {
+					actiondesc += "G";
+				} else if (data.prop_state == LibSvnClient.NotifyState.Changed) {
+					actiondesc += "U";
+				}
+				if (data.lock_state == LibSvnClient.NotifyLockState.Unlocked)
+					actiondesc += "B";
+				
+				actiondesc += " '{0}'"; 
+				actiondesc = string.Format (actiondesc, file); */
+				actiondesc = string.Format (GettextCatalog.GetString ("Update '{0}'"), file);
+				notifyChange = true;
+				break;
+			case LibSvnClient.NotifyAction.UpdateExternal: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Fetching external item into '{0}'"), file); 
+				break;
+			case LibSvnClient.NotifyAction.UpdateCompleted:  // TODO
+				actiondesc = GettextCatalog.GetString ("Finished"); 
+				break;
+			case LibSvnClient.NotifyAction.StatusExternal: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Performing status on external item at '{0}'"), file);
+				break;
+			case LibSvnClient.NotifyAction.StatusCompleted: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Status against revision: '{0}'"), data.revision);
+				break;
+				
+			case LibSvnClient.NotifyAction.CommitDeleted: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Deleting       {0}"), file); 
+				break;
+			case LibSvnClient.NotifyAction.CommitModified: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Sending        {0}"), file);
+				notifyChange = true; 
+				break;
+			case LibSvnClient.NotifyAction.CommitAdded: 
+				if (MimeTypeIsBinary (Marshal.PtrToStringAuto (data.mime_type))) {
+					actiondesc = string.Format (GettextCatalog.GetString ("Adding  (bin)  '{0}'"), file); 
+				} else {
+					actiondesc = string.Format (GettextCatalog.GetString ("Adding         '{0}'"), file); 
+				}
+				break;
+			case LibSvnClient.NotifyAction.CommitReplaced: 
+				actiondesc = string.Format (GettextCatalog.GetString ("Replacing      {0}"), file);
+				notifyChange = true; 
+				break;
+			case LibSvnClient.NotifyAction.CommitPostfixTxDelta: 
+				if (!nb.sent_first_txdelta) {
+					actiondesc = GettextCatalog.GetString ("Transmitting file data");
+					nb.sent_first_txdelta = true;
+				} else {
+					actiondesc = ".";
+					skipEol = true;
+				}
+				break;
 					
-			case LibSvnClient.NotifyAction.Locked: actiondesc = GettextCatalog.GetString ("Locked"); break;
-			case LibSvnClient.NotifyAction.Unlocked: actiondesc = GettextCatalog.GetString ("Unlocked"); break;
-			case LibSvnClient.NotifyAction.FailedLock: actiondesc = GettextCatalog.GetString ("Lock Failed"); break;
-			case LibSvnClient.NotifyAction.FailedUnlock: actiondesc = GettextCatalog.GetString ("Unlock Failed"); break;
-					
-				/*Add,
-				 Copy,
-				 Delete,
-				 Restore,
-				 Revert,
-				 FailedRevert,
-				 Resolved,
-				 Skip,
+			case LibSvnClient.NotifyAction.Locked: 
+				LibSvnClient.svn_lock_t repoLock = (LibSvnClient.svn_lock_t) Marshal.PtrToStructure (data.repo_lock, typeof (LibSvnClient.svn_lock_t));
+				actiondesc = string.Format (GettextCatalog.GetString ("'{0}' locked by user '{1}'."), file, repoLock.owner);
+				break;
+			case LibSvnClient.NotifyAction.Unlocked: 
+				actiondesc = string.Format (GettextCatalog.GetString ("'{0}' unlocked."), file);
+				break;
+//			case LibSvnClient.NotifyAction.ChangeListSet: 
+//				actiondesc = string.Format (GettextCatalog.GetString ("Path '{0}' is now a member of changelist '{1}'."), file, Marshal.PtrToStringAuto (data.changelist_name));
+//				break;
+//			case LibSvnClient.NotifyAction.ChangeListClear: 
+//				actiondesc = string.Format (GettextCatalog.GetString ("Path '{0}' is no longer a member of a changelist."), file);
+//				break;
+			default:
+				System.Console.WriteLine("untranslated action:" + data.action);
+				actiondesc = data.action.ToString () + " " + file;
+				break;
+				/*
 				 StatusCompleted,
 				 StatusExternal,
 				 BlameRevision*/
 			}
 			
-			if (updatemonitor != null)
-				updatemonitor.Log.WriteLine (actiondesc + " " + file);
+			if (updatemonitor != null && !string.IsNullOrEmpty (actiondesc)) {
+				if (skipEol) {
+					updatemonitor.Log.Write (actiondesc);
+				} else {
+					updatemonitor.Log.WriteLine (actiondesc);
+				}
+			}
 			if (updateFileList != null && notifyChange && File.Exists (file))
 				updateFileList.Add (file);
 			
@@ -898,12 +1186,13 @@ namespace MonoDevelop.VersionControl.Subversion {
 			}
 		}
 		
-		private class LogCollector {
+		private class LogCollector
+		{
 			static readonly DateTime Epoch = new DateTime (1970, 1, 1);
 			
-			ArrayList logs;
+			List<SvnRevision> logs;
 			
-			public LogCollector (ArrayList logs) { this.logs = logs; }
+			public LogCollector (List<SvnRevision> logs) { this.logs = logs; }
 			
 			public IntPtr Func (IntPtr baton, IntPtr apr_hash_changed_paths, int revision, IntPtr author, IntPtr date, IntPtr message, IntPtr pool)
 			{
@@ -914,7 +1203,7 @@ namespace MonoDevelop.VersionControl.Subversion {
 				if (message != IntPtr.Zero)
 					smessage = Marshal.PtrToStringAnsi (message).Trim ();
 				
-				ArrayList items = new ArrayList();
+				List<RevisionPath> items = new List<RevisionPath>();
 				
 				IntPtr item = apr.hash_first (pool, apr_hash_changed_paths);
 				while (item != IntPtr.Zero) {
@@ -926,11 +1215,18 @@ namespace MonoDevelop.VersionControl.Subversion {
 					LibSvnClient.svn_log_changed_path_t ch = (LibSvnClient.svn_log_changed_path_t) Marshal.PtrToStructure (val, typeof (LibSvnClient.svn_log_changed_path_t));
 					item = apr.hash_next (item);
 					
-					items.Add (new LibSvnClient.LogEntChangedPath (name, ch));
+					RevisionAction ac;
+					switch (ch.action) {
+						case 'A': ac = RevisionAction.Add; break;
+						case 'D': ac = RevisionAction.Delete; break;
+						case 'R': ac = RevisionAction.Replace; break;
+						default: ac = RevisionAction.Modify; break; // should be an 'M'
+					}
+					items.Add (new RevisionPath (name, ac, ""));
 				}
 				
-				logs.Add (new LibSvnClient.LogEnt (revision, Marshal.PtrToStringAnsi (author), Epoch.AddTicks (time * 10),
-				                                   smessage, (LibSvnClient.LogEntChangedPath[]) items.ToArray (typeof (LibSvnClient.LogEntChangedPath))));
+				SvnRevision ent = new SvnRevision (null, revision, Epoch.AddTicks (time * 10), Marshal.PtrToStringAnsi (author), smessage, items.ToArray ());
+				logs.Add (ent);
 				
 				return IntPtr.Zero;
 			}
@@ -954,5 +1250,31 @@ namespace MonoDevelop.VersionControl.Subversion {
 				return IntPtr.Zero;
 			}
 		}
+		
+		/// <summary>
+		/// Class for collecting annotations from libsvn_client
+		/// </summary>
+		private class AnnotationCollector
+		{
+			List<string> annotations;
+			
+			/// <summary>
+			/// A svn_client_blame_receiver_t implementation.
+			/// </summary>
+			public IntPtr Func (IntPtr baton, long line_no, int revision, string author, string date, string line, IntPtr pool)
+			{
+				if (line_no < annotations.Count) {
+					annotations[(int)line_no] = string.Format ("{0} {1} {2}", revision, author, date).Substring (0, 15);
+				}
+				
+				return IntPtr.Zero;
+			}
+			
+			public AnnotationCollector (List<string> annotations)
+			{
+				this.annotations = annotations;
+			}
+		}
+		
 	}
 }
