@@ -22,7 +22,7 @@ namespace MonoDevelop.VersionControl
 {
 	class VersionControlNodeExtension : NodeBuilderExtension
 	{
-		Hashtable filePaths = new Hashtable();
+		Dictionary<string,object> filePaths = new Dictionary<string, object> ();
 	
 		public override bool CanBuildNode (Type dataType)
 		{
@@ -53,15 +53,15 @@ namespace MonoDevelop.VersionControl
 				return;
 			} else if (dataObject is ProjectFolder) {
 				ProjectFolder ce = (ProjectFolder) dataObject;
-				if (ce.Project != null) {
-					Repository rep = VersionControlService.GetRepository (ce.Project);
+				if (ce.ParentWorkspaceObject != null) {
+					Repository rep = VersionControlService.GetRepository (ce.ParentWorkspaceObject);
 					if (rep != null)
 						AddFolderOverlay (rep, ce.Path, ref icon, ref closedIcon);
 				}
 				return;
 			}
 			
-			Project prj;
+			IWorkspaceObject prj;
 			string file;
 			
 			if (dataObject is ProjectFile) {
@@ -70,7 +70,7 @@ namespace MonoDevelop.VersionControl
 				file = pfile.FilePath;
 			} else {
 				SystemFile pfile = (SystemFile) dataObject;
-				prj = pfile.Project;
+				prj = pfile.ParentWorkspaceObject;
 				file = pfile.Path;
 			}
 			
@@ -140,8 +140,8 @@ namespace MonoDevelop.VersionControl
 		
 		void Monitor (object sender, FileUpdateEventArgs args)
 		{
-			object obj = filePaths [args.FilePath];
-			if (obj != null) {
+			object obj;
+			if (filePaths.TryGetValue (args.FilePath, out obj)) {
 				ITreeBuilder builder = Context.GetTreeBuilder (obj);
 				if (builder != null)
 					builder.UpdateAll();
@@ -150,15 +150,15 @@ namespace MonoDevelop.VersionControl
 		
 		public override void OnNodeAdded (object dataObject)
 		{
-			string path = GetPath (dataObject);
-			if (path != null)
+			FilePath path = GetPath (dataObject);
+			if (path != FilePath.Null)
 				filePaths [path] = dataObject;
 		}
 		
 		public override void OnNodeRemoved (object dataObject)
 		{
-			string path = GetPath (dataObject);
-			if (path != null)
+			FilePath path = GetPath (dataObject);
+			if (path != FilePath.Null)
 				filePaths.Remove (path);
 		}
 		
@@ -173,7 +173,7 @@ namespace MonoDevelop.VersionControl
 			} else if (dataObject is ProjectFolder) {
 				return ((ProjectFolder)dataObject).Path;
 			}
-			return null;
+			return FilePath.Null;
 		}
 		
 		public override Type CommandHandlerType {
@@ -304,6 +304,28 @@ namespace MonoDevelop.VersionControl
 		protected void UpdateUnlock(CommandInfo item) {
 			TestCommand(Commands.Unlock, item);
 		}
+		
+		[AllowMultiSelection]
+		[CommandHandler (Commands.Annotate)]
+		protected void OnAnnotate() {
+			RunCommand(Commands.Annotate, false);
+		}
+		
+		[CommandUpdateHandler (Commands.Annotate)]
+		protected void UpdateAnnotate(CommandInfo item) {
+			TestCommand(Commands.Annotate, item);
+		}
+		
+		[AllowMultiSelection]
+		[CommandHandler (Commands.CreatePatch)]
+		protected void OnCreatePatch() {
+			RunCommand(Commands.CreatePatch, false);
+		}
+		
+		[CommandUpdateHandler (Commands.CreatePatch)]
+		protected void UpdateCreatePatch(CommandInfo item) {
+			TestCommand(Commands.CreatePatch, item);
+		}
 			
 		private void TestCommand(Commands cmd, CommandInfo item) {
 			TestResult res = RunCommand(cmd, true);
@@ -368,6 +390,12 @@ namespace MonoDevelop.VersionControl
 						if (items.Count == 1 && it.IsDirectory && it.WorkspaceObject != null)
 							res = PublishCommand.Publish (it.WorkspaceObject, it.Path, test);
 						break;
+					case Commands.Annotate:
+						res = AnnotateView.Show (items[0].Repository, items[0].Path, test);
+						break;
+					case Commands.CreatePatch:
+						res = CreatePatchCommand.CreatePatch (items, test);
+						break;
 				}
 			}
 			catch (Exception ex) {
