@@ -11,7 +11,8 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Serialization;
+using MonoDevelop.Projects.Policies;
+using MonoDevelop.Core.Serialization;
 using Mono.Addins;
 
 namespace MonoDevelop.VersionControl
@@ -25,6 +26,8 @@ namespace MonoDevelop.VersionControl
 		internal static Gdk.Pixbuf overlay_controled;
 		static Gdk.Pixbuf overlay_unversioned;
 		static Gdk.Pixbuf overlay_protected;
+		static Gdk.Pixbuf overlay_locked;
+		static Gdk.Pixbuf overlay_unlocked;
 //		static Gdk.Pixbuf overlay_normal;
 
 		static Gdk.Pixbuf icon_modified;
@@ -48,27 +51,32 @@ namespace MonoDevelop.VersionControl
 		
 		static VersionControlService ()
 		{
-			overlay_modified = Gdk.Pixbuf.LoadFromResource("overlay_modified.png");
-			overlay_removed = Gdk.Pixbuf.LoadFromResource("overlay_removed.png");
-			overlay_conflicted = Gdk.Pixbuf.LoadFromResource("overlay_conflicted.png");
-			overlay_added = Gdk.Pixbuf.LoadFromResource("overlay_added.png");
-			overlay_controled = Gdk.Pixbuf.LoadFromResource("overlay_controled.png");
-			overlay_unversioned = Gdk.Pixbuf.LoadFromResource("overlay_unversioned.png");
-			overlay_protected = Gdk.Pixbuf.LoadFromResource("overlay_locked.png");
-//			overlay_normal = Gdk.Pixbuf.LoadFromResource("overlay_normal.png");
+			try {
+				overlay_modified = Gdk.Pixbuf.LoadFromResource("overlay_modified.png");
+				overlay_removed = Gdk.Pixbuf.LoadFromResource("overlay_removed.png");
+				overlay_conflicted = Gdk.Pixbuf.LoadFromResource("overlay_conflicted.png");
+				overlay_added = Gdk.Pixbuf.LoadFromResource("overlay_added.png");
+				overlay_controled = Gdk.Pixbuf.LoadFromResource("overlay_controled.png");
+				overlay_unversioned = Gdk.Pixbuf.LoadFromResource("overlay_unversioned.png");
+				overlay_protected = Gdk.Pixbuf.LoadFromResource("overlay_lock_required.png");
+				overlay_unlocked = Gdk.Pixbuf.LoadFromResource("overlay_unlocked.png");
+				overlay_locked = Gdk.Pixbuf.LoadFromResource("overlay_locked.png");
+	//			overlay_normal = Gdk.Pixbuf.LoadFromResource("overlay_normal.png");
 			
-			icon_modified = MonoDevelop.Core.Gui.Services.Resources.GetIcon ("gtk-edit", Gtk.IconSize.Menu);
-			icon_removed = MonoDevelop.Core.Gui.Services.Resources.GetIcon (Gtk.Stock.Remove, Gtk.IconSize.Menu);
-			icon_conflicted = MonoDevelop.Core.Gui.Services.Resources.GetIcon (Gtk.Stock.DialogWarning, Gtk.IconSize.Menu);
-			icon_added = MonoDevelop.Core.Gui.Services.Resources.GetIcon (Gtk.Stock.Add, Gtk.IconSize.Menu);
-			icon_controled = Gdk.Pixbuf.LoadFromResource("overlay_controled.png");
+				icon_modified = ImageService.GetPixbuf ("gtk-edit", Gtk.IconSize.Menu);
+				icon_removed = ImageService.GetPixbuf (Gtk.Stock.Remove, Gtk.IconSize.Menu);
+				icon_conflicted = ImageService.GetPixbuf (Gtk.Stock.DialogWarning, Gtk.IconSize.Menu);
+				icon_added = ImageService.GetPixbuf (Gtk.Stock.Add, Gtk.IconSize.Menu);
+				icon_controled = Gdk.Pixbuf.LoadFromResource("overlay_controled.png");
+			} catch (Exception e) {
+				LoggingService.LogError ("Error while loading icons.", e);
+			}
+			IdeApp.Workspace.FileAddedToProject += OnFileAdded;
+			//IdeApp.Workspace.FileChangedInProject += OnFileChanged;
+			//IdeApp.Workspace.FileRemovedFromProject += OnFileRemoved;
+			//IdeApp.Workspace.FileRenamedInProject += OnFileRenamed;
 			
-			IdeApp.ProjectOperations.FileAddedToProject += OnFileAdded;
-			//IdeApp.ProjectOperations.FileChangedInProject += OnFileChanged;
-			//IdeApp.ProjectOperations.FileRemovedFromProject += OnFileRemoved;
-			//IdeApp.ProjectOperations.FileRenamedInProject += OnFileRenamed;
-			
-			IdeApp.ProjectOperations.EntryAddedToCombine += OnEntryAdded;
+			IdeApp.Workspace.ItemAddedToSolution += OnEntryAdded;
 			IdeApp.Exiting += delegate {
 				DelayedSaveComments (null);
 			};
@@ -92,53 +100,60 @@ namespace MonoDevelop.VersionControl
 		
 		public static Gdk.Pixbuf LoadOverlayIconForStatus(VersionStatus status)
 		{
-			switch (status) {
-				case VersionStatus.Unchanged:
-					return null;
-				case VersionStatus.Protected:
-					return overlay_protected;
+			if ((status & VersionStatus.Versioned) == 0)
+				return overlay_unversioned;
+			
+			switch (status & VersionStatus.LocalChangesMask) {
 				case VersionStatus.Modified:
+				case VersionStatus.ScheduledReplace:
+				case VersionStatus.ScheduledIgnore:
 					return overlay_modified;
 				case VersionStatus.Conflicted:
 					return overlay_conflicted;
 				case VersionStatus.ScheduledAdd:
 					return overlay_added;
+				case VersionStatus.Missing:
 				case VersionStatus.ScheduledDelete:
 					return overlay_removed;
-				case VersionStatus.UnversionedIgnored:
-				case VersionStatus.Unversioned:
-					return overlay_unversioned;
 			}
+			
+			if ((status & VersionStatus.LockOwned) != 0)
+				return overlay_unlocked;
+			
+			if ((status & VersionStatus.Locked) != 0)
+				return overlay_locked;
+			
+			if ((status & VersionStatus.LockRequired) != 0)
+				return overlay_protected;
+				
 			return null;
 		}
 		
-		public static Gdk.Pixbuf LoadIconForStatus(VersionStatus status)
+		public static Gdk.Pixbuf LoadIconForStatus (VersionStatus status)
 		{
-			switch (status) {
-				case VersionStatus.Unchanged:
-					return null;
+			switch (status & VersionStatus.LocalChangesMask) {
 				case VersionStatus.Modified:
+				case VersionStatus.ScheduledReplace:
 					return icon_modified;
 				case VersionStatus.Conflicted:
 					return icon_conflicted;
 				case VersionStatus.ScheduledAdd:
 					return icon_added;
+				case VersionStatus.Missing:
 				case VersionStatus.ScheduledDelete:
 					return icon_removed;
-				default:
-					if (status != VersionStatus.Unversioned && status != VersionStatus.UnversionedIgnored)
-						return icon_controled;
-					break;
 			}
 			return null;
 		}
 
 		public static string GetStatusLabel (VersionStatus status)
 		{
-			switch (status) {
-				case VersionStatus.Unchanged:
-					return "";
+			if ((status & VersionStatus.Versioned) == 0)
+				return GettextCatalog.GetString ("Unversioned");
+			
+			switch (status & VersionStatus.LocalChangesMask) {
 				case VersionStatus.Modified:
+				case VersionStatus.ScheduledReplace:
 					return GettextCatalog.GetString ("Modified");
 				case VersionStatus.Conflicted:
 					return GettextCatalog.GetString ("Conflict");
@@ -146,15 +161,13 @@ namespace MonoDevelop.VersionControl
 					return GettextCatalog.GetString ("Add");
 				case VersionStatus.ScheduledDelete:
 					return GettextCatalog.GetString ("Delete");
-				default:
-					if (status != VersionStatus.Unversioned && status != VersionStatus.UnversionedIgnored)
-						return "";
-					break;
+				case VersionStatus.Missing:
+					return GettextCatalog.GetString ("Missing");
 			}
-			return GettextCatalog.GetString ("Unversioned");
+			return String.Empty;
 		}
 		
-		public static Repository GetRepository (CombineEntry entry)
+		public static Repository GetRepository (IWorkspaceObject entry)
 		{
 			Repository repo = (Repository) entry.ExtendedProperties [typeof(Repository)];
 			if (repo != null)
@@ -203,11 +216,21 @@ namespace MonoDevelop.VersionControl
 		{
 			if (comments != null)
 				return comments;
-			
+
+			ResolveEventHandler localResolve = delegate (object s, ResolveEventArgs args) {
+				foreach (System.Reflection.Assembly asm in AppDomain.CurrentDomain.GetAssemblies ()) {
+					if (asm.GetName ().FullName == args.Name)
+						return asm;
+				}
+				return null;
+			};
+
 			string file = Path.Combine (PropertyService.ConfigPath, "version-control-commit-msg");
 			if (File.Exists (file)) {
 				FileStream stream = null;
 				try {
+					AppDomain.CurrentDomain.AssemblyResolve += localResolve;
+
 					stream = File.OpenRead (file);
 					BinaryFormatter formatter = new BinaryFormatter ();
 					comments = (Hashtable) formatter.Deserialize (stream);
@@ -230,6 +253,7 @@ namespace MonoDevelop.VersionControl
 					LoggingService.LogError (ex.ToString ());
 					comments = new Hashtable ();
 				} finally {
+					AppDomain.CurrentDomain.AssemblyResolve -= localResolve;
 					if (stream != null)
 						stream.Close ();
 				}
@@ -294,7 +318,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					PrepareCommit (null, new CommitEventArgs (repo, changeSet, false));
 				} catch (Exception ex) {
-					IdeApp.Services.MessageService.ShowError (ex);
+					MessageService.ShowException (ex);
 					return false;
 				}
 			}
@@ -307,7 +331,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					BeginCommit (null, new CommitEventArgs (repo, changeSet, false));
 				} catch (Exception ex) {
-					IdeApp.Services.MessageService.ShowError (ex);
+					MessageService.ShowException (ex);
 					return false;
 				}
 			}
@@ -320,7 +344,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					EndCommit (null, new CommitEventArgs (repo, changeSet, success));
 				} catch (Exception ex) {
-					IdeApp.Services.MessageService.ShowError (ex);
+					MessageService.ShowException (ex);
 					return false;
 				}
 			}
@@ -331,8 +355,8 @@ namespace MonoDevelop.VersionControl
 			}
 			return true;
 		}
-		
-		internal static void NotifyFileStatusChanged (Repository repo, string localPath, bool isDirectory) 
+
+		public static void NotifyFileStatusChanged (Repository repo, FilePath localPath, bool isDirectory) 
 		{
 			if (!DispatchService.IsGuiThread)
 				Gtk.Application.Invoke (delegate {
@@ -385,25 +409,46 @@ namespace MonoDevelop.VersionControl
 			}
 		}
 */
-		static void CombineEntryAddFiles (CombineEntry entry, ArrayList files)
+		static void SolutionItemAddFiles (string rootPath, SolutionItem entry, ArrayList files)
 		{
-			files.Add (entry.FileName);
+			if (entry is SolutionEntityItem) {
+				string file = ((SolutionEntityItem)entry).FileName;
+				SolutionItemAddFile (rootPath, files, file);
+			}
 			
 			if (entry is Project) {
-				foreach (ProjectFile file in ((Project) entry).ProjectFiles) {
+				foreach (ProjectFile file in ((Project) entry).Files) {
 					if (file.Subtype != Subtype.Directory)
-						files.Add (file.FilePath);
+						SolutionItemAddFile (rootPath, files, file.FilePath);
 				}
-			} else if (entry is Combine) {
-				foreach (CombineEntry ent in ((Combine) entry).Entries)
-					CombineEntryAddFiles (ent, files);
+			} else if (entry is SolutionFolder) {
+				foreach (SolutionItem ent in ((SolutionFolder) entry).Items)
+					SolutionItemAddFiles (rootPath, ent, files);
 			}
 		}
 		
-		static void OnEntryAdded (object o, CombineEntryEventArgs args)
+		static void SolutionItemAddFile (string rootPath, ArrayList files, string file)
+		{
+			if (!file.StartsWith (rootPath + Path.DirectorySeparatorChar))
+			    return;
+			if (!File.Exists (file))
+				return;
+			if (files.Contains (file))
+				return;
+			string dir = Path.GetDirectoryName (file);
+			while (dir != rootPath) {
+				if (files.Contains (dir))
+					break;
+				files.Add (dir);
+				dir = Path.GetDirectoryName (dir);
+			}
+			files.Add (file);
+		}
+		
+		static void OnEntryAdded (object o, SolutionItemEventArgs args)
 		{
 			// handles addition of solutions and projects
-			CombineEntry parent = (CombineEntry) args.CombineEntry.ParentCombine;
+			SolutionItem parent = (SolutionItem) args.SolutionItem.ParentFolder;
 			
 			if (parent == null)
 				return;
@@ -413,7 +458,7 @@ namespace MonoDevelop.VersionControl
 			if (repo == null)
 				return;
 			
-			CombineEntry entry = args.CombineEntry;
+			SolutionItem entry = args.SolutionItem;
 			string path = entry.BaseDirectory;
 			
 			if (!repo.CanAdd (path))
@@ -426,7 +471,8 @@ namespace MonoDevelop.VersionControl
 			ArrayList files = new ArrayList ();
 			
 			files.Add (path);
-			CombineEntryAddFiles (entry, files);
+			SolutionItemAddFiles (path, entry, files);
+			files.Sort ();
 			
 			using (IProgressMonitor monitor = GetStatusMonitor ()) {
 				string[] paths = (string[]) files.ToArray (typeof (string));
@@ -478,7 +524,7 @@ namespace MonoDevelop.VersionControl
 					XmlDataSerializer ser = new XmlDataSerializer (dataContext);
 					XmlTextReader reader = new XmlTextReader (new StreamReader (ConfigFile));
 					try {
-						configuration = (VersionControlConfiguration) ser.Deserialize (reader, typeof(VersionControlConfiguration));
+						configuration = (VersionControlConfiguration) ser.Deserialize (reader, typeof (VersionControlConfiguration));
 					} finally {
 						reader.Close ();
 					}
@@ -495,7 +541,7 @@ namespace MonoDevelop.VersionControl
 				XmlDataSerializer ser = new XmlDataSerializer (dataContext);
 				XmlTextWriter tw = new XmlTextWriter (new StreamWriter (ConfigFile));
 				try {
-					ser.Serialize (tw, configuration, typeof(VersionControlConfiguration));
+					ser.Serialize (tw, configuration, typeof (VersionControlConfiguration));
 				} finally {
 					tw.Close ();
 				}
@@ -529,7 +575,7 @@ namespace MonoDevelop.VersionControl
 					return true;
 			}
 			
-			IdeApp.Services.MessageService.ShowError (GettextCatalog.GetString ("There isn't any supported version control system installed. You may need to install additional add-ins or packages."));
+			MessageService.ShowError (GettextCatalog.GetString ("There isn't any supported version control system installed. You may need to install additional add-ins or packages."));
 			return false;
 		}
 		
@@ -560,6 +606,51 @@ namespace MonoDevelop.VersionControl
 				tw.Close ();
 			}
 		}
+		
+		public static CommitMessageFormat GetCommitMessageFormat (SolutionItem item)
+		{
+			CommitMessageFormat format = new CommitMessageFormat ();
+			format.Style = item.Policies.Get<VersionControlPolicy> ().CommitMessageStyle;
+			return format;
+		}
+		
+		public static CommitMessageFormat GetCommitMessageFormat (ChangeSet cset, out AuthorInformation authorInfo)
+		{
+			// If all files belong to a project, use that project's policy. If not, use the solution policy
+			Project project = null;
+			bool sameProject = true;
+			foreach (ChangeSetItem item in cset.Items) {
+				if (project != null) {
+					if (project.Files.GetFile (item.LocalPath) == null) {
+						// Not all files belong to the same project
+						sameProject = false;
+						break;
+					}
+				} else {
+					project = IdeApp.Workspace.GetProjectContainingFile (item.LocalPath);
+				}
+			}
+			CommitMessageStyle style;
+			
+			if (project != null) {
+				VersionControlPolicy policy;
+				if (sameProject)
+					policy = project.Policies.Get<VersionControlPolicy> ();
+				else
+					policy = project.ParentSolution.Policies.Get<VersionControlPolicy> ();
+				style = policy.CommitMessageStyle;
+			}
+			else {
+				style = PolicyService.GetDefaultPolicy<CommitMessageStyle> ();
+			}
+			
+			authorInfo = IdeApp.Workspace.GetAuthorInformation (project);
+			
+			CommitMessageFormat format = new CommitMessageFormat ();
+			format.Style = style;
+			
+			return format;
+		}
 	}
 	
 	[Serializable]
