@@ -39,21 +39,20 @@ using MonoDevelop.Components.Commands;
 using System.Net;
 using System.Xml;
 using System.IO;
+using System.Collections.Generic;
+
 
 namespace MonoDevelop.WelcomePage
 {	
 	public abstract class WelcomePageView : AbstractViewContent
 	{
-		string datadir;
 		bool loadingProject;
 		EventHandler recentChangesHandler;
 		
 		// netNewsXml is where online the news.xml file can be found
 		static string netNewsXml {
 			get {
-				return "http://mono.ximian.com/monodevelop-news/"
-					+ Mono.Addins.AddinManager.CurrentAddin.Version 
-					+ "/news.xml";
+				return "http://www.monodevelop.com/files/news/news.xml";
 			}
 		}
 		
@@ -75,11 +74,6 @@ namespace MonoDevelop.WelcomePage
 			this.ContentName = GettextCatalog.GetString ("Welcome");
 			this.IsViewOnly = true;
 			
-			datadir = Path.GetDirectoryName (typeof(ShowWelcomePageHandler).Assembly.Location) + "/";
-
-			if (PlatformID.Unix != Environment.OSVersion.Platform)
-				datadir = datadir.Replace("\\","/");
-
 			recentChangesHandler = (EventHandler) DispatchService.GuiDispatch (new EventHandler (RecentChangesHandler));
 			IdeApp.Workbench.RecentOpen.RecentProjectChanged += recentChangesHandler;
 			NewsUpdated += (EventHandler) DispatchService.GuiDispatch (new EventHandler (HandleNewsUpdate));
@@ -174,6 +168,7 @@ namespace MonoDevelop.WelcomePage
 			}
 			System.Threading.ThreadStart ts = new System.Threading.ThreadStart (updateNewsXmlThread);
 			System.Threading.Thread updateThread = new System.Threading.Thread (ts);
+			updateThread.IsBackground = true;
 			updateThread.Start ();
 		}
 		
@@ -220,12 +215,13 @@ namespace MonoDevelop.WelcomePage
 			{
 				//Launch the Uri externally
 				try {
-					IdeApp.Services.PlatformService.ShowUrl (uri);
+					DesktopService.ShowUrl (uri);
 				} catch (Exception) {
 					string msg = String.Format (GettextCatalog.GetString ("Could not open the url {0}"), uri);
 					Gtk.MessageDialog md = new Gtk.MessageDialog
 						(null, Gtk.DialogFlags.Modal| Gtk.DialogFlags.DestroyWithParent,
 						 Gtk.MessageType.Error, Gtk.ButtonsType.Ok, msg);
+					md.TransientFor = IdeApp.Workbench.RootWindow;
 					try {
 						md.Run ();
 						md.Hide ();
@@ -281,13 +277,15 @@ namespace MonoDevelop.WelcomePage
 			return GettextCatalog.GetString ("Less than a minute");
 		}
 		
-		public virtual string DataDirectory {
-			get { return datadir; }
+		public IEnumerable<RecentItem> RecentProjects {
+			get {
+				return IdeApp.Workbench.RecentOpen.RecentProjects;
+			}
 		}
 		
-		public RecentItem[] RecentProjects {
+		public int RecentProjectsCount {
 			get {
-				return IdeApp.Workbench.RecentOpen.RecentProject;
+				return IdeApp.Workbench.RecentOpen.RecentProjectsCount;
 			}
 		}
 	}
