@@ -108,7 +108,7 @@ namespace MonoDevelop.Components.Commands
 				mi = new AutoHideMenuItem (name != null ? name : "");
 			else
 				mi = new Gtk.MenuItem (name != null ? name : "");
-			mi.Submenu = manager.CreateMenu (this);
+			mi.Submenu = new CommandMenu (manager, this);
 			return mi;
 		}
 
@@ -128,6 +128,8 @@ namespace MonoDevelop.Components.Commands
 		
 		public bool HasVisibleChildren {
 			get {
+				if (!Submenu.IsRealized)
+					Submenu.Realize ();
 				foreach (Gtk.Widget item in ((Gtk.Menu)Submenu).Children) {
 					if (item.Visible)
 						return true;
