@@ -47,16 +47,28 @@ namespace MonoDevelop.Components.Commands
 		Dictionary<object,Command> cmds = new Dictionary<object,Command> ();
 		Hashtable handlerInfo = new Hashtable ();
 		ArrayList toolbars = new ArrayList ();
-		ArrayList globalHandlers = new ArrayList ();
+		CommandTargetChain globalHandlerChain;
 		ArrayList commandUpdateErrors = new ArrayList ();
 		ArrayList visitors = new ArrayList ();
 		Dictionary<Gtk.Window,Gtk.Window> topLevelWindows = new Dictionary<Gtk.Window,Gtk.Window> ();
 		Stack delegatorStack = new Stack ();
+		
+		HashSet<object> visitedTargets = new HashSet<object> ();
+		
 		bool disposed;
 		bool toolbarUpdaterRunning;
 		bool enableToolbarUpdate;
 		int guiLock;
 		
+		// Fields used to keep track of the application focus
+		bool appHasFocus;
+		Gtk.Window lastFocused;
+		DateTime focusCheckDelayTimeout = DateTime.MinValue;
+		
+		internal static readonly object CommandRouteTerminator = new object ();
+		
+		internal bool handlerFoundInMulticast;
+		
 		public CommandManager (): this (null)
 		{
 		}
@@ -79,7 +91,7 @@ namespace MonoDevelop.Components.Commands
 		{
 			KeyBindingService.LoadBindingsFromExtensionPath (addinPath);
 		}
-		
+
 		void OnExtensionChange (object s, ExtensionNodeEventArgs args)
 		{
 			if (args.Change == ExtensionChange.Add) {
@@ -133,6 +145,15 @@ namespace MonoDevelop.Components.Commands
 			ShowContextMenu (CreateCommandEntrySet (addinPath));
 		}
 		
+		public CommandEntrySet CreateCommandEntrySet (ExtensionContext ctx, string addinPath)
+		{
+			CommandEntrySet cset = new CommandEntrySet ();
+			object[] items = ctx.GetExtensionObjects (addinPath, false);
+			foreach (CommandEntry e in items)
+				cset.Add (e);
+			return cset;
+		}
+		
 		public CommandEntrySet CreateCommandEntrySet (string addinPath)
 		{
 			CommandEntrySet cset = new CommandEntrySet ();
@@ -158,9 +179,10 @@ namespace MonoDevelop.Components.Commands
 		[GLib.ConnectBefore]
 		void OnKeyPressed (object o, Gtk.KeyPressEventArgs e)
 		{
-			string accel = KeyBindingManager.AccelFromKey (e.Event.Key, e.Event.State);
+			bool complete;
+			string accel = KeyBindingManager.AccelFromKey (e.Event, out complete);
 			
-			if (accel == null) {
+			if (!complete) {
 				// incomplete accel
 				e.RetVal = true;
 				return;
@@ -191,8 +213,11 @@ namespace MonoDevelop.Components.Commands
 			for (int i = 0; i < commands.Count; i++) {
 				CommandInfo cinfo = GetCommandInfo (commands[i].Id, null);
 				if (cinfo.Enabled && cinfo.Visible && DispatchCommand (commands[i].Id))
-					break;
+					return;
 			}
+			
+			e.RetVal = false;
+			mode = null;
 		}
 		
 		public void SetRootWindow (Gtk.Window root)
@@ -220,11 +245,15 @@ namespace MonoDevelop.Components.Commands
 			w.Destroyed -= TopLevelDestroyed;
 			w.KeyPressEvent -= OnKeyPressed;
 			topLevelWindows.Remove (w);
+			if (w == lastFocused)
+				lastFocused = null;
 		}
 		
 		public void Dispose ()
 		{
 			disposed = true;
+			bindings.Dispose ();
+			lastFocused = null;
 		}
 		
 		public void LockAll ()
@@ -268,6 +297,7 @@ namespace MonoDevelop.Components.Commands
 		
 		public void RegisterCommand (Command cmd)
 		{
+			KeyBindingService.StoreDefaultBinding (cmd);
 			KeyBindingService.LoadBinding (cmd);
 			
 			cmds[cmd.Id] = cmd;
@@ -279,16 +309,21 @@ namespace MonoDevelop.Components.Commands
 			bindings.UnregisterCommand (cmd);
 			cmds.Remove (cmd.Id);
 		}
+
+		public void LoadUserBindings ()
+		{
+			foreach (Command cmd in cmds.Values)
+				KeyBindingService.LoadBinding (cmd);
+		}
 		
 		public void RegisterGlobalHandler (object handler)
 		{
-			if (!globalHandlers.Contains (handler))
-				globalHandlers.Add (handler);
+			globalHandlerChain = CommandTargetChain.AddTarget (globalHandlerChain, handler);
 		}
 		
 		public void UnregisterGlobalHandler (object handler)
 		{
-			globalHandlers.Remove (handler);
+			globalHandlerChain = CommandTargetChain.RemoveTarget (globalHandlerChain, handler);
 		}
 		
 		public void RegisterCommandTargetVisitor (ICommandTargetVisitor visitor)
@@ -346,9 +381,9 @@ namespace MonoDevelop.Components.Commands
 			return CreateToolbar ("", entrySet);
 		}
 		
-*/		public Gtk.Menu CreateMenu (CommandEntrySet entrySet)
+*/	
+		public Gtk.Menu CreateMenu (CommandEntrySet entrySet, CommandMenu menu)
 		{
-			CommandMenu menu = new CommandMenu (this);
 			foreach (CommandEntry entry in entrySet) {
 				Gtk.MenuItem mi = entry.CreateMenuItem (this);
 				CustomItem ci = mi.Child as CustomItem;
@@ -359,6 +394,11 @@ namespace MonoDevelop.Components.Commands
 			return menu;
 		}
 		
+		public Gtk.Menu CreateMenu (CommandEntrySet entrySet)
+		{
+			return CreateMenu (entrySet, new CommandMenu (this));
+		}
+		
 		public void InsertOptions (Gtk.Menu menu, CommandEntrySet entrySet, int index)
 		{
 			foreach (CommandEntry entry in entrySet) {
@@ -390,11 +430,6 @@ namespace MonoDevelop.Components.Commands
 		public void ShowContextMenu (Gtk.Menu menu)
 		{
 			menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
-			menu.Hidden += delegate {
-				// Needed to make sure the menu entries are destroyed and
-				// can't handle accelerators
-				menu.Destroy ();
-			};
 		}
 		
 		public void ShowContextMenu (Gtk.Menu menu, object initialCommandTarget)
@@ -405,9 +440,26 @@ namespace MonoDevelop.Components.Commands
 			ShowContextMenu (menu);
 		}
 		
+		public Gtk.Toolbar CreateToolbar (CommandEntrySet entrySet)
+		{
+			return CreateToolbar ("", entrySet, null);
+		}
+		
+		public Gtk.Toolbar CreateToolbar (CommandEntrySet entrySet, object initialTarget)
+		{
+			return CreateToolbar ("", entrySet, initialTarget);
+		}
+		
 		public Gtk.Toolbar CreateToolbar (string id, CommandEntrySet entrySet)
+		{
+			return CreateToolbar (id, entrySet, null);
+		}
+		
+		public Gtk.Toolbar CreateToolbar (string id, CommandEntrySet entrySet, object initialTarget)
 		{
 			CommandToolbar toolbar = new CommandToolbar (this, id, entrySet.Name);
+			toolbar.InitialCommandTarget = initialTarget;
+			
 			foreach (CommandEntry entry in entrySet) {
 				Gtk.ToolItem ti = entry.CreateToolItem (this);
 				CustomItem ci = ti.Child as CustomItem;
@@ -434,17 +486,18 @@ namespace MonoDevelop.Components.Commands
 		{
 			if (guiLock > 0)
 				return false;
-			
+
+			NotifyCommandTargetScanStarted ();
+			List<HandlerCallback> handlers = new List<HandlerCallback> ();
 			ActionCommand cmd = null;
 			try {
 				cmd = GetActionCommand (commandId);
 				if (cmd == null)
 					return false;
 				
-				int globalPos = -1;
-				object cmdTarget = initialTarget != null ? initialTarget : GetFirstCommandTarget (out globalPos);
+				object cmdTarget = GetFirstCommandTarget (initialTarget);
 				CommandInfo info = new CommandInfo (cmd);
-				
+
 				while (cmdTarget != null)
 				{
 					HandlerTypeInfo typeInfo = GetTypeHandlerInfo (cmdTarget);
@@ -460,7 +513,7 @@ namespace MonoDevelop.Components.Commands
 							if (!ainfo.Bypass) {
 								bool found = false;
 								foreach (CommandInfo ci in ainfo) {
-									if (Object.Equals (dataItem, ci.DataItem)) {
+									if (dataItem == ci.DataItem || Object.Equals (dataItem, ci.DataItem)) {
 										found = true;
 										break;
 									}
@@ -483,38 +536,64 @@ namespace MonoDevelop.Components.Commands
 					if (!bypass) {
 						CommandHandlerInfo chi = typeInfo.GetCommandHandler (commandId);
 						if (chi != null) {
-							if (cmd.CommandArray)
-								chi.Run (cmdTarget, dataItem);
+							object localTarget = cmdTarget;
+							if (cmd.CommandArray) {
+								handlers.Add (delegate {
+									chi.Run (localTarget, cmd, dataItem);
+								});
+							}
+							else {
+								handlers.Add (delegate {
+									chi.Run (localTarget, cmd);
+								});
+							}
+							handlerFoundInMulticast = true;
+							cmdTarget = NextMulticastTarget ();
+							if (cmdTarget == null)
+								break;
 							else
-								chi.Run (cmdTarget);
-							UpdateToolbars ();
-							return true;
+								continue;
 						}
 					}
-					cmdTarget = GetNextCommandTarget (cmdTarget, ref globalPos);
+					cmdTarget = GetNextCommandTarget (cmdTarget);
+				}
+
+				if (handlers.Count > 0) {
+					foreach (HandlerCallback c in handlers)
+						c ();
+					UpdateToolbars ();
+					return true;
 				}
 	
-				return cmd.DispatchCommand (dataItem);
+				if (cmd.DispatchCommand (dataItem)) {
+					UpdateToolbars ();
+					return true;
+				}
 			}
 			catch (Exception ex) {
 				string name = (cmd != null && cmd.Text != null && cmd.Text.Length > 0) ? cmd.Text : commandId.ToString ();
 				name = name.Replace ("_","");
 				ReportError (commandId, "Error while executing command: " + name, ex);
-				return false;
 			}
+			finally {
+				NotifyCommandTargetScanFinished ();
+			}
+			return false;
 		}
 		
-		internal CommandInfo GetCommandInfo (object commandId, object initialTarget)
+		public CommandInfo GetCommandInfo (object commandId, object initialTarget)
 		{
 			ActionCommand cmd = GetActionCommand (commandId);
 			if (cmd == null)
 				throw new InvalidOperationException ("Invalid action command id: " + commandId);
-			
+
+			NotifyCommandTargetScanStarted ();
 			CommandInfo info = new CommandInfo (cmd);
 			
 			try {
-				int globalPos = -1;
-				object cmdTarget = initialTarget != null ? initialTarget : GetFirstCommandTarget (out globalPos);
+				object cmdTarget = GetFirstCommandTarget (initialTarget);
+				bool multiCastEnabled = true;
+				bool multiCastVisible = false;
 				
 				while (cmdTarget != null)
 				{
@@ -522,6 +601,8 @@ namespace MonoDevelop.Components.Commands
 					CommandUpdaterInfo cui = typeInfo.GetCommandUpdater (commandId);
 					
 					bool bypass = false;
+					bool handlerFound = false;
+					
 					if (cui != null) {
 						if (cmd.CommandArray) {
 							info.ArrayInfo = new CommandArrayInfo (info);
@@ -529,7 +610,7 @@ namespace MonoDevelop.Components.Commands
 							if (!info.ArrayInfo.Bypass) {
 								if (guiLock > 0)
 									info.Enabled = false;
-								return info;
+								handlerFound = true;
 							}
 						}
 						else {
@@ -538,19 +619,36 @@ namespace MonoDevelop.Components.Commands
 							if (!info.Bypass) {
 								if (guiLock > 0)
 									info.Enabled = false;
-								return info;
+								handlerFound = true;
 							}
 						}
-						bypass = true;
+						if (!handlerFound)
+							bypass = true;
 					}
-					
-					if (!bypass && typeInfo.GetCommandHandler (commandId) != null) {
+
+					if (handlerFound) {
+						handlerFoundInMulticast = true;
+						if (!info.Enabled || !info.Visible)
+							multiCastEnabled = false;
+						if (info.Visible)
+							multiCastVisible = true;
+						cmdTarget = NextMulticastTarget ();
+						if (cmdTarget == null) {
+							if (!multiCastEnabled)
+								info.Enabled = false;
+							if (multiCastVisible)
+								info.Visible = true;
+							return info;
+						}
+						continue;
+					}
+					else if (!bypass && typeInfo.GetCommandHandler (commandId) != null) {
 						info.Enabled = guiLock == 0;
 						info.Visible = true;
 						return info;
 					}
 					
-					cmdTarget = GetNextCommandTarget (cmdTarget, ref globalPos);
+					cmdTarget = GetNextCommandTarget (cmdTarget);
 				}
 				
 				cmd.UpdateCommandInfo (info);
@@ -562,7 +660,10 @@ namespace MonoDevelop.Components.Commands
 				}
 				info.Enabled = false;
 				info.Visible = true;
+			} finally {
+				NotifyCommandTargetScanFinished ();
 			}
+
 			if (guiLock > 0)
 				info.Enabled = false;
 			return info;
@@ -570,15 +671,14 @@ namespace MonoDevelop.Components.Commands
 		
 		public object VisitCommandTargets (ICommandTargetVisitor visitor, object initialTarget)
 		{
-			int globalPos = -1;
-			object cmdTarget = initialTarget != null ? initialTarget : GetFirstCommandTarget (out globalPos);
+			object cmdTarget = GetFirstCommandTarget (initialTarget);
 			
 			while (cmdTarget != null)
 			{
 				if (visitor.Visit (cmdTarget))
 					return cmdTarget;
 
-				cmdTarget = GetNextCommandTarget (cmdTarget, ref globalPos);
+				cmdTarget = GetNextCommandTarget (cmdTarget);
 			}
 			
 			visitor.Visit (null);
@@ -644,6 +744,20 @@ namespace MonoDevelop.Components.Commands
 			}
 		}
 		
+		internal void NotifySelected (CommandInfo cmdInfo)
+		{
+			if (CommandSelected != null) {
+				CommandSelectedEventArgs args = new CommandSelectedEventArgs (cmdInfo);
+				CommandSelected (this, args);
+			}
+		}
+		
+		internal void NotifyDeselected ()
+		{
+			if (CommandDeselected != null)
+				CommandDeselected (this, EventArgs.Empty);
+		}
+		
 		HandlerTypeInfo GetTypeHandlerInfo (object cmdTarget)
 		{
 			HandlerTypeInfo typeInfo = (HandlerTypeInfo) handlerInfo [cmdTarget.GetType ()];
@@ -652,57 +766,155 @@ namespace MonoDevelop.Components.Commands
 			Type type = cmdTarget.GetType ();
 			typeInfo = new HandlerTypeInfo ();
 			
-			ArrayList handlers = new ArrayList ();
-			ArrayList updaters = new ArrayList ();
+			List<CommandHandlerInfo> handlers = new List<CommandHandlerInfo> ();
+			List<CommandUpdaterInfo> updaters = new List<CommandUpdaterInfo> ();
 			
 			Type curType = type;
 			while (curType != null && curType.Assembly != typeof(Gtk.Widget).Assembly && curType.Assembly != typeof(object).Assembly) {
 				MethodInfo[] methods = curType.GetMethods (BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.DeclaredOnly);
 				foreach (MethodInfo method in methods) {
-					foreach (CommandHandlerAttribute attr in method.GetCustomAttributes (typeof(CommandHandlerAttribute), true))
-						handlers.Add (new CommandHandlerInfo (method, (CommandHandlerAttribute) attr));
 
-					foreach (CommandUpdateHandlerAttribute attr in method.GetCustomAttributes (typeof(CommandUpdateHandlerAttribute), true))
-						updaters.Add (new CommandUpdaterInfo (method, (CommandUpdateHandlerAttribute) attr));
+					ICommandUpdateHandler customHandlerChain = null;
+					ICommandArrayUpdateHandler customArrayHandlerChain = null;
+					ICommandTargetHandler customTargetHandlerChain = null;
+					ICommandArrayTargetHandler customArrayTargetHandlerChain = null;
+					List<CommandHandlerInfo> methodHandlers = new List<CommandHandlerInfo> ();
+					
+					foreach (object attr in method.GetCustomAttributes (true)) {
+						if (attr is CommandHandlerAttribute)
+							methodHandlers.Add (new CommandHandlerInfo (method, (CommandHandlerAttribute) attr));
+						else if (attr is CommandUpdateHandlerAttribute)
+							AddUpdater (updaters, method, (CommandUpdateHandlerAttribute) attr);
+						else {
+							customHandlerChain = ChainHandler (customHandlerChain, attr);
+							customArrayHandlerChain = ChainHandler (customArrayHandlerChain, attr);
+							customTargetHandlerChain = ChainHandler (customTargetHandlerChain, attr);
+							customArrayTargetHandlerChain = ChainHandler (customArrayTargetHandlerChain, attr);
+						}
+					}
+
+					foreach (object attr in type.GetCustomAttributes (true)) {
+						customHandlerChain = ChainHandler (customHandlerChain, attr);
+						customArrayHandlerChain = ChainHandler (customArrayHandlerChain, attr);
+						customTargetHandlerChain = ChainHandler (customTargetHandlerChain, attr);
+						customArrayTargetHandlerChain = ChainHandler (customArrayTargetHandlerChain, attr);
+					}
+					
+					if (methodHandlers.Count > 0) {
+						if (customHandlerChain != null || customArrayHandlerChain != null) {
+							// There are custom handlers. Create update handlers for all commands
+							// that the method handles so the custom update handlers can be chained
+							foreach (CommandHandlerInfo ci in methodHandlers) {
+								CommandUpdaterInfo c = AddUpdateHandler (updaters, ci.CommandId);
+								c.AddCustomHandlers (customHandlerChain, customArrayHandlerChain);
+							}
+						}
+						if (customTargetHandlerChain != null || customArrayTargetHandlerChain != null) {
+							foreach (CommandHandlerInfo ci in methodHandlers)
+								ci.AddCustomHandlers (customTargetHandlerChain, customArrayTargetHandlerChain);
+						}
+					}
+					handlers.AddRange (methodHandlers);
 				}
 				curType = curType.BaseType;
 			}
 			
 			if (handlers.Count > 0)
-				typeInfo.CommandHandlers = (CommandHandlerInfo[]) handlers.ToArray (typeof(CommandHandlerInfo)); 
+				typeInfo.CommandHandlers = handlers.ToArray (); 
 			if (updaters.Count > 0)
-				typeInfo.CommandUpdaters = (CommandUpdaterInfo[]) updaters.ToArray (typeof(CommandUpdaterInfo));
+				typeInfo.CommandUpdaters = updaters.ToArray ();
 				 
 			handlerInfo [type] = typeInfo;
 			return typeInfo;
 		}
-		
-		object GetFirstCommandTarget (out int globalPos)
+
+		CommandUpdaterInfo AddUpdateHandler (List<CommandUpdaterInfo> methodUpdaters, object cmdId)
 		{
-			object cmdTarget = GetActiveWidget (rootWidget);
-			if (cmdTarget == null) {
-				globalPos = 0;
-				if (globalHandlers.Count == 0)
-					return null;
-				return globalHandlers [0];
-			} else {
-				globalPos = -1;
-				return cmdTarget;
+			foreach (CommandUpdaterInfo ci in methodUpdaters) {
+				if (ci.CommandId.Equals (cmdId))
+					return ci;
 			}
+			// Not found, it needs to be added
+			CommandUpdaterInfo cinfo = new CommandUpdaterInfo (cmdId);
+			methodUpdaters.Add (cinfo);
+			return cinfo;
+		}
+
+		void AddUpdater (List<CommandUpdaterInfo> methodUpdaters, MethodInfo method, CommandUpdateHandlerAttribute attr)
+		{
+			foreach (CommandUpdaterInfo ci in methodUpdaters) {
+				if (ci.CommandId.Equals (attr.CommandId)) {
+					ci.Init (method, attr);
+					return;
+				}
+			}
+			// Not found, it needs to be added
+			CommandUpdaterInfo cinfo = new CommandUpdaterInfo (method, attr);
+			methodUpdaters.Add (cinfo);
+		}
+
+		ICommandArrayUpdateHandler ChainHandler (ICommandArrayUpdateHandler chain, object attr)
+		{
+			ICommandArrayUpdateHandler h = attr as ICommandArrayUpdateHandler;
+			if (h == null) return chain;
+			h.Next = chain ?? DefaultCommandHandler.Instance;
+			return h;
+		}
+
+		ICommandUpdateHandler ChainHandler (ICommandUpdateHandler chain, object attr)
+		{
+			ICommandUpdateHandler h = attr as ICommandUpdateHandler;
+			if (h == null) return chain;
+			h.Next = chain ?? DefaultCommandHandler.Instance;
+			return h;
+		}
+
+		ICommandTargetHandler ChainHandler (ICommandTargetHandler chain, object attr)
+		{
+			ICommandTargetHandler h = attr as ICommandTargetHandler;
+			if (h == null) return chain;
+			h.Next = chain ?? DefaultCommandHandler.Instance;
+			return h;
+		}
+
+		ICommandArrayTargetHandler ChainHandler (ICommandArrayTargetHandler chain, object attr)
+		{
+			ICommandArrayTargetHandler h = attr as ICommandArrayTargetHandler;
+			if (h == null) return chain;
+			h.Next = chain ?? DefaultCommandHandler.Instance;
+			return h;
 		}
 		
-		object GetNextCommandTarget (object cmdTarget, ref int globalPos)
+		object GetFirstCommandTarget (object initialTarget)
 		{
-			if (globalPos != -1) {
-				if (++globalPos < globalHandlers.Count)
-					return globalHandlers [globalPos];
-				else
-					return null;
+			delegatorStack.Clear ();
+			visitedTargets.Clear ();
+			handlerFoundInMulticast = false;
+			object cmdTarget;
+			if (initialTarget != null)
+				cmdTarget = initialTarget;
+			else {
+				cmdTarget = GetActiveWidget (rootWidget);
+				if (cmdTarget == null) {
+					cmdTarget = globalHandlerChain;
+				}
 			}
+			visitedTargets.Add (cmdTarget);
+			return cmdTarget;
+		}
+		
+		object GetNextCommandTarget (object cmdTarget)
+		{
+			if (cmdTarget is IMultiCastCommandRouter) 
+				cmdTarget = new MultiCastDelegator (this, (IMultiCastCommandRouter)cmdTarget);
 			
 			if (cmdTarget is ICommandDelegatorRouter) {
-				delegatorStack.Push (cmdTarget);
-				cmdTarget = ((ICommandDelegatorRouter)cmdTarget).GetDelegatedCommandTarget ();
+				object oldCmdTarget = cmdTarget;
+				cmdTarget = ((ICommandDelegatorRouter)oldCmdTarget).GetDelegatedCommandTarget ();
+				if (cmdTarget != null)
+					delegatorStack.Push (oldCmdTarget);
+				else
+					cmdTarget = ((ICommandDelegatorRouter)oldCmdTarget).GetNextCommandTarget ();
 			}
 			else if (cmdTarget is ICommandRouter)
 				cmdTarget = ((ICommandRouter)cmdTarget).GetNextCommandTarget ();
@@ -711,32 +923,58 @@ namespace MonoDevelop.Components.Commands
 			else
 				cmdTarget = null;
 			
-			if (cmdTarget == null) {
+			if (cmdTarget == null || !visitedTargets.Add (cmdTarget)) {
 				if (delegatorStack.Count > 0) {
 					ICommandDelegatorRouter del = (ICommandDelegatorRouter) delegatorStack.Pop ();
 					cmdTarget = del.GetNextCommandTarget ();
+					if (cmdTarget == CommandManager.CommandRouteTerminator)
+						return null;
 					if (cmdTarget != null)
 						return cmdTarget;
 				}
-				if (globalHandlers.Count == 0)
-					return null;
-				globalPos = 0;
-				return globalHandlers [0];
+				return globalHandlerChain;
 			} else
 				return cmdTarget;
 		}
+
+		internal object NextMulticastTarget ()
+		{
+			while (delegatorStack.Count > 0) {
+				MultiCastDelegator del = delegatorStack.Pop () as MultiCastDelegator;
+				if (del != null) {
+					object cmdTarget = GetNextCommandTarget (del);
+					return cmdTarget == globalHandlerChain ? null : cmdTarget;
+				}
+			}
+			return null;
+		}
 		
 		Gtk.Widget GetActiveWidget (Gtk.Window win)
 		{
 			Gtk.Window[] wins = Gtk.Window.ListToplevels ();
 			
+			bool hasFocus = false;
+			bool lastFocusedExists = lastFocused == null;
+			Gtk.Window newFocused = null;
 			foreach (Gtk.Window w in wins) {
-				if (w.IsActive && w.Visible && w.Type == Gtk.WindowType.Toplevel && !(w is Gtk.Dialog)) {
-					win = w;
-					break;
+				if (w.Visible) {
+					if (w.HasToplevelFocus) {
+						hasFocus = true;
+						newFocused = w;
+					}
+					if (w.IsActive && w.Type == Gtk.WindowType.Toplevel && !(w is Gtk.Dialog)) {
+						win = w;
+						break;
+					}
+					if (lastFocused == w) {
+						lastFocusedExists = true;
+					}
 				}
 			}
 			
+			lastFocused = newFocused;
+			UpdateAppFocusStatus (hasFocus, lastFocusedExists);
+			
 			if (win != null) {
 				RegisterTopWindow (win);
 				Gtk.Widget widget = win;
@@ -775,6 +1013,11 @@ namespace MonoDevelop.Components.Commands
 		
 		void UpdateToolbars ()
 		{
+			// This might get called after the app has exited, e.g. after executing the quit command
+			// It then queries widgets, which resurrects widget wrappers, which breaks on managed widgets
+			if (this.disposed)
+				return;
+			
 			object activeWidget = GetActiveWidget (rootWidget);
 			foreach (CommandToolbar toolbar in toolbars) {
 				if (toolbar.Visible)
@@ -782,7 +1025,31 @@ namespace MonoDevelop.Components.Commands
 			}
 			foreach (ICommandTargetVisitor v in visitors)
 				VisitCommandTargets (v, null);
+		}
+		
+		void UpdateAppFocusStatus (bool hasFocus, bool lastFocusedExists)
+		{
+			if (hasFocus != appHasFocus) {
+				// The last focused window has been destroyed. Wait a few ms since another app's window
+				// may gain focus again
+				DateTime now = DateTime.Now;
+				if (now < focusCheckDelayTimeout)
+					return;
+				if (!hasFocus && !lastFocusedExists) {
+					focusCheckDelayTimeout = now.AddMilliseconds (100);
+					return;
+				}
+				focusCheckDelayTimeout = DateTime.MinValue;
 				
+				appHasFocus = hasFocus;
+				if (appHasFocus) {
+					if (ApplicationFocusIn != null)
+						ApplicationFocusIn (this, EventArgs.Empty);
+				} else {
+					if (ApplicationFocusOut != null)
+						ApplicationFocusOut (this, EventArgs.Empty);
+				}
+			}
 		}
 		
 		public void ReportError (object commandId, string message, Exception ex)
@@ -793,7 +1060,25 @@ namespace MonoDevelop.Components.Commands
 			}
 		}
 		
+		void NotifyCommandTargetScanStarted ()
+		{
+			if (CommandTargetScanStarted != null)
+				CommandTargetScanStarted (this, EventArgs.Empty);
+		}
+		
+		void NotifyCommandTargetScanFinished ()
+		{
+			if (CommandTargetScanFinished != null)
+				CommandTargetScanFinished (this, EventArgs.Empty);
+		}
+		
 		public event CommandErrorHandler CommandError;
+		public event EventHandler<CommandSelectedEventArgs> CommandSelected;
+		public event EventHandler CommandDeselected;
+		public event EventHandler ApplicationFocusIn; // Fired when the application gets the focus
+		public event EventHandler ApplicationFocusOut;  // Fired when the application loses the focus
+		public event EventHandler CommandTargetScanStarted;
+		public event EventHandler CommandTargetScanFinished;
 	}
 	
 	internal class HandlerTypeInfo
@@ -828,17 +1113,82 @@ namespace MonoDevelop.Components.Commands
 		
 		public CommandMethodInfo (MethodInfo method, CommandMethodAttribute attr)
 		{
-			this.Method = method;
-			CommandId = attr.CommandId;
+			Init (method, attr);
+		}
+		
+		protected void Init (MethodInfo method, CommandMethodAttribute attr)
+		{
+			// Don't assign the method if there is already one assigned (maybe from a subclass)
+			if (this.Method == null) {
+				this.Method = method;
+				CommandId = attr.CommandId;
+			}
+		}
+		
+		public CommandMethodInfo (object commandId)
+		{
+			CommandId = commandId;
+		}
+	}
+	
+	internal class CommandHandlerInfo: CommandMethodInfo
+	{
+		ICommandTargetHandler  customHandlerChain;
+		ICommandArrayTargetHandler  customArrayHandlerChain;
+		
+		public CommandHandlerInfo (MethodInfo method, CommandHandlerAttribute attr): base (method, attr)
+		{
+			ParameterInfo[] pars = method.GetParameters ();
+			if (pars.Length > 1)
+				throw new InvalidOperationException ("Invalid signature for command handler: " + method.DeclaringType + "." + method.Name + "()");
+		}
+		
+		public void Run (object cmdTarget, Command cmd)
+		{
+			if (customHandlerChain != null) {
+				cmd.HandlerData = Method;
+				customHandlerChain.Run (cmdTarget, cmd);
+			}
+			else
+				Method.Invoke (cmdTarget, null);
+		}
+		
+		public void Run (object cmdTarget, Command cmd, object dataItem)
+		{
+			if (customArrayHandlerChain != null) {
+				cmd.HandlerData = Method;
+				customArrayHandlerChain.Run (cmdTarget, cmd, dataItem);
+			}
+			else
+				Method.Invoke (cmdTarget, new object[] {dataItem});
+		}
+		
+		public void AddCustomHandlers (ICommandTargetHandler handlerChain, ICommandArrayTargetHandler arrayHandlerChain)
+		{
+			this.customHandlerChain = handlerChain;
+			this.customArrayHandlerChain = arrayHandlerChain;
 		}
 	}
 		
 	internal class CommandUpdaterInfo: CommandMethodInfo
 	{
+		ICommandUpdateHandler customHandlerChain;
+		ICommandArrayUpdateHandler customArrayHandlerChain;
+		
 		bool isArray;
 		
+		public CommandUpdaterInfo (object commandId): base (commandId)
+		{
+		}
+		
 		public CommandUpdaterInfo (MethodInfo method, CommandUpdateHandlerAttribute attr): base (method, attr)
 		{
+			Init (method, attr);
+		}
+
+		public void Init (MethodInfo method, CommandUpdateHandlerAttribute attr)
+		{
+			base.Init (method, attr);
 			ParameterInfo[] pars = method.GetParameters ();
 			if (pars.Length == 1) {
 				Type t = pars[0].ParameterType;
@@ -851,42 +1201,109 @@ namespace MonoDevelop.Components.Commands
 			}
 			throw new InvalidOperationException ("Invalid signature for command update handler: " + method.DeclaringType + "." + method.Name + "()");
 		}
+
+		public void AddCustomHandlers (ICommandUpdateHandler handlerChain, ICommandArrayUpdateHandler arrayHandlerChain)
+		{
+			this.customHandlerChain = handlerChain;
+			this.customArrayHandlerChain = arrayHandlerChain;
+		}
 		
 		public void Run (object cmdTarget, CommandInfo info)
 		{
-			if (isArray)
-				throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
-			Method.Invoke (cmdTarget, new object[] {info} );
+			if (customHandlerChain != null) {
+				info.UpdateHandlerData = Method;
+				customHandlerChain.CommandUpdate (cmdTarget, info);
+			} else {
+				if (Method == null)
+					throw new InvalidOperationException ("Invalid custom update handler. An implementation of ICommandUpdateHandler was expected.");
+				if (isArray)
+					throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
+				Method.Invoke (cmdTarget, new object[] {info} );
+			}
 		}
 		
 		public void Run (object cmdTarget, CommandArrayInfo info)
 		{
-			if (!isArray)
-				throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
-			Method.Invoke (cmdTarget, new object[] {info} );
+			if (customArrayHandlerChain != null) {
+				info.UpdateHandlerData = Method;
+				customArrayHandlerChain.CommandUpdate (cmdTarget, info);
+			} else {
+				if (Method == null)
+					throw new InvalidOperationException ("Invalid custom update handler. An implementation of ICommandArrayUpdateHandler was expected.");
+				if (!isArray)
+					throw new InvalidOperationException ("Invalid signature for command update handler: " + Method.DeclaringType + "." + Method.Name + "()");
+				Method.Invoke (cmdTarget, new object[] {info} );
+			}
 		}
 	}
 	
-	internal class CommandHandlerInfo: CommandMethodInfo
+	class DefaultCommandHandler: ICommandUpdateHandler, ICommandArrayUpdateHandler, ICommandTargetHandler, ICommandArrayTargetHandler
 	{
-		public CommandHandlerInfo (MethodInfo method, CommandHandlerAttribute attr): base (method, attr)
+		public static DefaultCommandHandler Instance = new DefaultCommandHandler ();
+		
+		public void CommandUpdate (object target, CommandInfo info)
 		{
-			ParameterInfo[] pars = method.GetParameters ();
-			if (pars.Length > 1)
-				throw new InvalidOperationException ("Invalid signature for command handler: " + method.DeclaringType + "." + method.Name + "()");
+			MethodInfo mi = (MethodInfo) info.UpdateHandlerData;
+			if (mi != null)
+				mi.Invoke (target, new object[] {info} );
 		}
 		
-		public void Run (object cmdTarget)
+		public void CommandUpdate (object target, CommandArrayInfo info)
 		{
-			Method.Invoke (cmdTarget, null);
+			MethodInfo mi = (MethodInfo) info.UpdateHandlerData;
+			if (mi != null)
+				mi.Invoke (target, new object[] {info} );
+		}
+
+		public void Run (object target, Command cmd)
+		{
+			MethodInfo mi = (MethodInfo) cmd.HandlerData;
+			if (mi != null)
+				mi.Invoke (target, new object[0] );
 		}
 		
-		public void Run (object cmdTarget, object dataItem)
+		public void Run (object target, Command cmd, object data)
 		{
-			Method.Invoke (cmdTarget, new object[] {dataItem});
+			MethodInfo mi = (MethodInfo) cmd.HandlerData;
+			if (mi != null)
+				mi.Invoke (target, new object[] {data} );
+		}
+
+		ICommandArrayTargetHandler ICommandArrayTargetHandler.Next {
+			get {
+				return null;
+			}
+			set {
+			}
+		}
+		
+		ICommandTargetHandler ICommandTargetHandler.Next {
+			get {
+				return null;
+			}
+			set {
+			}
+		}
+		
+		ICommandArrayUpdateHandler ICommandArrayUpdateHandler.Next {
+			get {
+				// Last one in the chain
+				return null;
+			}
+			set {
+			}
+		}
+		
+		public ICommandUpdateHandler Next {
+			get {
+				// Last one in the chain
+				return null;
+			}
+			set {
+			}
 		}
 	}
-	
+
 	internal class ToolbarTracker
 	{
 		Gtk.IconSize lastSize;
@@ -914,5 +1331,103 @@ namespace MonoDevelop.Components.Commands
 			}
 		}
 	}
+
+	class MultiCastDelegator: ICommandDelegatorRouter
+	{
+		IEnumerator enumerator;
+		object nextTarget;
+		CommandManager manager;
+		bool done;
+		
+		public MultiCastDelegator (CommandManager manager, IMultiCastCommandRouter mcr)
+		{
+			this.manager = manager;
+			enumerator = mcr.GetCommandTargets ().GetEnumerator ();
+		}
+		
+		public object GetNextCommandTarget ()
+		{
+			if (nextTarget != null)
+				return this;
+			else {
+				if (manager.handlerFoundInMulticast)
+					return manager.NextMulticastTarget ();
+				else
+					return null;
+			}
+		}
+		
+		public object GetDelegatedCommandTarget ()
+		{
+			object currentTarget;
+			if (done)
+				return null;
+			if (nextTarget != null) {
+				currentTarget = nextTarget;
+				nextTarget = null;
+			} else {
+				if (enumerator.MoveNext ())
+					currentTarget = enumerator.Current;
+				else
+					return null;
+			}
+			
+			if (enumerator.MoveNext ())
+				nextTarget = enumerator.Current;
+			else {
+				done = true;
+				nextTarget = null;
+			}
+
+			return currentTarget;
+		}
+	}
+
+	class CommandTargetChain: ICommandDelegatorRouter
+	{
+		object target;
+		internal CommandTargetChain Next;
+
+		public CommandTargetChain (object target)
+		{
+			this.target = target;
+		}
+		
+		public object GetNextCommandTarget ()
+		{
+			if (Next == null)
+				return CommandManager.CommandRouteTerminator;
+			else
+				return Next;
+		}
+		
+		public object GetDelegatedCommandTarget ()
+		{
+			return target;
+		}
+
+		public static CommandTargetChain RemoveTarget (CommandTargetChain chain, object target)
+		{
+			if (chain == null)
+				return null;
+			if (chain.target == target)
+				return chain.Next;
+			else if (chain.Next != null)
+				chain.Next = CommandTargetChain.RemoveTarget (chain.Next, target);
+			return chain;
+		}
+
+		public static CommandTargetChain AddTarget (CommandTargetChain chain, object target)
+		{
+			if (chain == null)
+				return new CommandTargetChain (target);
+			else {
+				chain.Next = AddTarget (chain.Next, target);
+				return chain;
+			}
+		}
+	}
+
+	delegate void HandlerCallback ();
 }
 
