@@ -48,10 +48,31 @@ namespace MonoDevelop.Components.Commands
 {
 	public class KeyBindingManager : IDisposable
 	{
+		static bool isX11 = false;
+		
 		Dictionary<string, List<Command>> bindings = new Dictionary<string, List<Command>> ();
 		Dictionary<string, int> modes = new Dictionary<string, int> ();
 		List<Command> commands = new List<Command> ();
 		
+		static KeyBindingManager ()
+		{
+			isX11 = !isMac && System.Environment.OSVersion.Platform == PlatformID.Unix;
+			
+			if (isMac) {
+				SelectionModifierAlt = Gdk.ModifierType.Mod5Mask;
+				SelectionModifierControl = Gdk.ModifierType.Mod1Mask | Gdk.ModifierType.MetaMask;
+				SelectionModifierSuper = Gdk.ModifierType.ControlMask;
+			} else {
+				SelectionModifierAlt = Gdk.ModifierType.Mod1Mask;
+				SelectionModifierControl = Gdk.ModifierType.ControlMask;
+				SelectionModifierSuper = Gdk.ModifierType.SuperMask;
+			}
+		}
+		
+		static bool isMac {
+			get { return MonoDevelop.Core.PropertyService.IsMac; }
+		}
+		
 		public void Dispose ()
 		{
 			if (commands == null)
@@ -62,12 +83,30 @@ namespace MonoDevelop.Components.Commands
 			commands = null;
 		}
 		
-#region Static Public Helpers For Building Key-Binding Strings
+		#region Platform-dependent selection modifiers
+		//FIXME: these should be named for what they do, not what they are
+		
+		public static Gdk.ModifierType SelectionModifierAlt {
+			get; private set;
+		}
+		
+		public static Gdk.ModifierType SelectionModifierControl {
+			get; private set;
+		}
+		
+		public static Gdk.ModifierType SelectionModifierSuper {
+			get; private set;
+		}
+		
+		#endregion
+		
+		#region Static Public Helpers For Building Key-Binding Strings
+		
 		static string KeyToString (Gdk.Key key)
 		{
-			char c;
+			char c = (char) Gdk.Keyval.ToUnicode ((uint) key);
 			
-			if ((c = (char) Gdk.Keyval.ToUnicode ((uint) key)) != 0) {
+			if (c != 0) {
 				if (c == ' ')
 					return "Space";
 				
@@ -92,50 +131,35 @@ namespace MonoDevelop.Components.Commands
 			return key.ToString ();
 		}
 		
-		public static string AccelFromKey (Gdk.Key key, Gdk.ModifierType modifier)
+		//USED INTERNALLY ONLY
+		static string AccelFromKey (Gdk.Key key, Gdk.ModifierType modifier)
 		{
-			string accel = String.Empty;
-			
-			switch (key) {
-			case Gdk.Key.Alt_L:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Alt_R:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Meta_L:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Meta_R:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Shift_L:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Shift_R:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Control_L:
-				// incomplete accel
-				return null;
-			case Gdk.Key.Control_R:
-				// incomplete accel
-				return null;
-			}
-			
-			if ((modifier & Gdk.ModifierType.ControlMask) != 0)
-				accel += "Control+";
-			if ((modifier & Gdk.ModifierType.Mod1Mask) != 0)
-				accel += "Alt+";
-			if ((modifier & Gdk.ModifierType.ShiftMask) != 0)
-				accel += "Shift+";
-			
-			accel += KeyToString (key);
-			
-			return accel;
+			bool complete;
+			return AccelFromKey (key, modifier, out complete);
 		}
 		
-		static bool AccelToKey (string accel, ref int i, out uint key, out Gdk.ModifierType modifier)
+		public static string AccelFromKey (Gdk.EventKey raw, out bool complete)
+		{
+			Gdk.Key key;
+			Gdk.ModifierType modifier;
+			MapRawKeys (raw, out key, out modifier);
+			return AccelFromKey (key, modifier, out complete);
+		}
+		
+		static string AccelFromKey (Gdk.Key key, Gdk.ModifierType modifier, out bool complete)
+		{
+			bool keyIsModifier;
+			string modifierlabel = ModifierToPartialAccel (modifier, key, out keyIsModifier);
+			complete = !keyIsModifier;
+			
+			if (keyIsModifier) {
+				return modifierlabel;
+			} else {
+				return modifierlabel + KeyToString (key);
+			}
+		}
+		
+		static bool AccelToKeyPartial (string accel, ref int i, out uint key, out Gdk.ModifierType modifier)
 		{
 			Gdk.ModifierType mod, mask = 0;
 			string str;
@@ -172,16 +196,13 @@ namespace MonoDevelop.Components.Commands
 			
 			i = j;
 			
-			if (k == 0)
-				return false;
-			
 			modifier = mask;
 			key = k;
 			
-			return true;
+			return k != 0;
 		}
 		
-		public static bool AccelToKey (string accel, out uint key, out Gdk.ModifierType modifier)
+		static bool AccelToKeyPartial (string accel, out uint key, out Gdk.ModifierType modifier)
 		{
 			int i = 0;
 			
@@ -191,16 +212,22 @@ namespace MonoDevelop.Components.Commands
 			if (accel == null || accel == String.Empty)
 				return false;
 			
-			if (AccelToKey (accel, ref i, out key, out modifier) && i == accel.Length)
+			if (AccelToKeyPartial (accel, ref i, out key, out modifier) && i == accel.Length)
 				return true;
 			
-			modifier = Gdk.ModifierType.None;
-			key = 0;
-			
 			return false;
 		}
 		
-		public static bool BindingToKeys (string binding, out uint modeKey, out Gdk.ModifierType modeModifier, out uint key, out Gdk.ModifierType modifier)
+		public static bool AccelToKey (string accel, out uint key, out Gdk.ModifierType modifier)
+		{
+			if (AccelToKeyPartial (accel, out key, out modifier))
+				return true;
+			modifier = Gdk.ModifierType.None;
+			key = 0;
+			return false;
+		}
+		
+		static bool BindingToKeysPartial (string binding, out uint modeKey, out Gdk.ModifierType modeModifier, out uint key, out Gdk.ModifierType modifier)
 		{
 			int i = 0;
 			
@@ -212,7 +239,7 @@ namespace MonoDevelop.Components.Commands
 			if (binding == null || binding == String.Empty)
 				return false;
 			
-			if (!AccelToKey (binding, ref i, out key, out modifier))
+			if (!AccelToKeyPartial (binding, ref i, out key, out modifier))
 				return false;
 			
 			if (i == binding.Length) {
@@ -229,14 +256,19 @@ namespace MonoDevelop.Components.Commands
 			modeKey = key;
 			i++;
 			
-			return AccelToKey (binding, ref i, out key, out modifier) && i == binding.Length;
+			return AccelToKeyPartial (binding, ref i, out key, out modifier) && i == binding.Length;
 		}
 		
-		public static string BindingFromKey (string mode, Gdk.Key key, Gdk.ModifierType modifier)
+		public static bool BindingToKeys (string binding, out uint modeKey, out Gdk.ModifierType modeModifier, out uint key, out Gdk.ModifierType modifier)
 		{
-			return Binding (mode, AccelFromKey (key, modifier));
+			if (BindingToKeysPartial (binding, out modeKey, out modeModifier, out key, out modifier))
+				return true;
+			
+			modeModifier = modifier = Gdk.ModifierType.None;
+			modeKey = key = 0;
+			return false;
 		}
-		
+				
 		public static string Binding (string mode, string accel)
 		{
 			if (mode == null) {
@@ -252,33 +284,184 @@ namespace MonoDevelop.Components.Commands
 			return mode + "|" + accel;
 		}
 		
-		static string ModifierToLabel (Gdk.ModifierType mod)
+		
+		static void MapRawKeys (Gdk.EventKey evt, out Gdk.Key key, out Gdk.ModifierType mod)
+		{
+			key = evt.Key;
+			mod = evt.State;
+			
+			if (isX11) {
+				//this is a workaround for a common X mapping issue
+				//where the alt key is mapped to the meta key when the shift modifier is active
+				if (key.Equals (Gdk.Key.Meta_L) || key.Equals (Gdk.Key.Meta_R))
+					key = Gdk.Key.Alt_L;
+			}
+			
+			//HACK: the MAC GTK+ port currently does some horrible, un-GTK-ish key mappings
+			// so we work around them by playing some tricks to remap and decompose modifiers.
+			// We also decompose keys to the root physical key so that the Mac command 
+			// combinations appear as expected, e.g. shift-{ is treated as shift-[.
+			if (isMac && !isX11) {
+				// Mac GTK+ maps the command key to the Mod1 modifier, which usually means alt/
+				// We map this instead to meta, because the Mac GTK+ has mapped the cmd key
+				// to the meta key (yay inconsistency!). IMO super would have been saner.
+				if ((mod & Gdk.ModifierType.Mod1Mask) != 0) {
+					mod ^= Gdk.ModifierType.Mod1Mask;
+					mod |= Gdk.ModifierType.MetaMask;
+				}
+				
+				// If Mod5 is active it *might* mean that opt/alt is active,
+				// so we can unset this and map it back to the normal modifier.
+				if ((mod & Gdk.ModifierType.Mod5Mask) != 0) {
+					mod ^= Gdk.ModifierType.Mod5Mask;
+					mod |= Gdk.ModifierType.Mod1Mask;
+				}
+				
+				// When shift or opt modifiers are active, we need to decompose keys down to the 
+				// "root" physical key to make the command appear correct for Mac.
+				if (evt.Group == (byte) 1 || (mod & Gdk.ModifierType.ShiftMask) != 0)
+					key = GetRootKey (evt);
+				
+				// In addition, we can only inspect whether the opt/alt key is pressed by examining
+				// the key's "group", because the Mac GTK+ treats opt as a group modifier and does
+				// not expose it as an actual GDK modifier.
+				if (evt.Group == (byte) 1)
+					mod |= Gdk.ModifierType.Mod1Mask;
+			}
+		}
+		
+		static Dictionary<Gdk.Key,Gdk.Key> hardwareMappings;
+		static Gdk.Keymap keymap;
+		
+		static void InitKeymaps ()
+		{
+			if (keymap != null) 
+				return;
+			
+			keymap = Gdk.Keymap.Default;
+			keymap.KeysChanged += delegate {
+				hardwareMappings.Clear ();
+			};
+			hardwareMappings = new Dictionary<Gdk.Key,Gdk.Key> ();
+		}
+		
+		static Gdk.Key GetRootKey (Gdk.EventKey evt)
+		{
+			InitKeymaps ();
+			Gdk.Key ret;
+			if (hardwareMappings.TryGetValue (evt.Key, out ret))
+				return ret;
+			
+			//FIXME: LookupKey isn't implemented on Mac, so we have to use this workaround
+			uint[] keyvals;
+			Gdk.KeymapKey [] keys;
+			keymap.GetEntriesForKeycode (evt.HardwareKeycode, out keys, out keyvals);
+			
+			for (uint i = 0; i < keys.Length; i++) {
+				if (keys[i].Group == 0 && keys[i].Level == 0) {
+					ret = (Gdk.Key)keyvals[i];
+					foreach (Gdk.Key key in keyvals)
+						hardwareMappings[key] = ret;
+					return ret;
+				}
+			}
+			
+			//failed, but avoid looking it up again
+			return hardwareMappings[evt.Key] = evt.Key;
+		}
+		
+		static string ModifierToPartialAccel (Gdk.ModifierType mod, Gdk.Key key, out bool keyIsModifier)
 		{
 			string label = String.Empty;
 			
 			if ((mod & Gdk.ModifierType.ControlMask) != 0)
-				label += "Ctrl+";
+				label += "Control+";
 			if ((mod & Gdk.ModifierType.Mod1Mask) != 0)
 				label += "Alt+";
 			if ((mod & Gdk.ModifierType.ShiftMask) != 0)
 				label += "Shift+";
+			if ((mod & Gdk.ModifierType.MetaMask) != 0)
+				label += "Meta+";
+			if ((mod & Gdk.ModifierType.SuperMask) != 0)
+				label += "Super+";
+			
+			keyIsModifier = true;
+			if (key.Equals (Gdk.Key.Control_L) || key.Equals (Gdk.Key.Control_R))
+				label += "Control+";
+			else if (key.Equals (Gdk.Key.Alt_L) || key.Equals (Gdk.Key.Alt_R))
+				label += "Alt+";
+			else if (key.Equals (Gdk.Key.Shift_L) || key.Equals (Gdk.Key.Shift_R))
+				label += "Shift+";
+			else if (key.Equals (Gdk.Key.Meta_L) || key.Equals (Gdk.Key.Meta_R))
+				label += "Meta+";
+			else if (key.Equals (Gdk.Key.Super_L) || key.Equals (Gdk.Key.Super_L))
+				label += "Super+";
+			else
+				keyIsModifier = false;
 			
 			return label;
 		}
 		
-		public static string BindingToLabel (string binding)
+		#endregion
+		
+		#region Display label formatting
+		
+		public static string BindingToDisplayLabel (string binding, bool concise)
+		{
+			return BindingToDisplayLabel (binding, concise, false);
+		}
+		
+		public static string BindingToDisplayLabel (string binding, bool concise, bool includeIncomplete)
 		{
 			Gdk.ModifierType modeMod, mod;
 			string label = String.Empty;
 			uint modeKey, key;
 			
-			if (!BindingToKeys (binding, out modeKey, out modeMod, out key, out mod))
+			if (includeIncomplete) {
+				BindingToKeysPartial (binding, out modeKey, out modeMod, out key, out mod);
+			} else if (!BindingToKeys (binding, out modeKey, out modeMod, out key, out mod)) {
 				return null;
+			}
 			
-			if (modeKey != 0)
-				label += ModifierToLabel (modeMod) + KeyToString ((Gdk.Key) modeKey) + " ";
+			if (modeKey != 0) {
+				label += ModifierToDisplayLabel (modeMod, concise) + KeyToDisplayLabel ((Gdk.Key)modeKey) + (isMac? " " : "|");
+			}
+			label += ModifierToDisplayLabel (mod, concise);
+			if (key != 0)
+				label += KeyToDisplayLabel ((Gdk.Key)key);
 			
-			label += ModifierToLabel (mod) + KeyToString ((Gdk.Key) key);
+			return label;
+		}
+		
+		static string KeyToDisplayLabel (Gdk.Key key)
+		{
+			if (isMac) {
+				char appl = AppleMapKeyToSymbol (key);
+				if (appl != '\0') {
+					return new string (appl, 1);
+				}
+			}
+			return KeyToString (key);
+		}
+		
+		static string ModifierToDisplayLabel (Gdk.ModifierType mod, bool concise)
+		{
+			if (isMac) {
+				return AppleMapModifierToSymbols (mod);
+			}
+			
+			string label = String.Empty;
+			
+			if ((mod & Gdk.ModifierType.ControlMask) != 0)
+				label += concise? "Ctrl+" : "Control+";
+			if ((mod & Gdk.ModifierType.Mod1Mask) != 0)
+				label += "Alt+";
+			if ((mod & Gdk.ModifierType.ShiftMask) != 0)
+				label += "Shift+";
+			if ((mod & Gdk.ModifierType.MetaMask) != 0)
+				label += "Meta+";
+			if ((mod & Gdk.ModifierType.SuperMask) != 0)
+				label += "Super+";
 			
 			return label;
 		}
@@ -291,7 +474,12 @@ namespace MonoDevelop.Components.Commands
 			case "Shift":
 				return Gdk.ModifierType.ShiftMask;
 			case "Control":
+			case "Ctrl":
 				return Gdk.ModifierType.ControlMask;
+			case "Meta":
+				return Gdk.ModifierType.MetaMask;
+			case "Super":
+				return Gdk.ModifierType.SuperMask;
 			default:
 				return Gdk.ModifierType.None;
 			}
@@ -383,9 +571,11 @@ namespace MonoDevelop.Components.Commands
 			
 			return Binding (mode, accel);
 		}
-#endregion
 		
-#region Public Methods
+		#endregion
+		
+		#region Public Methods
+		
 		public bool BindingExists (string binding)
 		{
 			return bindings.ContainsKey (binding);
@@ -548,7 +738,151 @@ namespace MonoDevelop.Components.Commands
 			
 			return bindings[binding];
 		}
-#endregion
+		
+		#endregion
+		
+		#region Apple symbol mappings
+		
+		static string AppleMapModifierToSymbols (Gdk.ModifierType mod)
+		{
+			string ret = "";
+			
+			if ((mod & Gdk.ModifierType.ControlMask) != 0) {
+				ret += "⌃";
+				mod ^= Gdk.ModifierType.ControlMask;
+			}
+			if ((mod & Gdk.ModifierType.Mod1Mask) != 0) {
+				ret += "⌥";
+				mod ^= Gdk.ModifierType.Mod1Mask;
+			}
+			if ((mod & Gdk.ModifierType.ShiftMask) != 0) {
+				ret += "⇧";
+				mod ^= Gdk.ModifierType.ShiftMask;
+			}
+			if ((mod & Gdk.ModifierType.MetaMask) != 0) {
+				ret += "⌘";
+				mod ^= Gdk.ModifierType.MetaMask;
+			}
+			if (mod != 0)
+				throw new InvalidOperationException ("Unexpected modifiers: " + mod.ToString ());
+			return ret;
+		}
+		
+		static char AppleMapKeyToSymbol (Gdk.Key key)
+		{
+			// unicode codes from http://macbiblioblog.blogspot.com/2005/05/special-key-symbols.html
+			// unmapped:
+			// ⇤ tab back
+			// ⌤ enter
+			
+			switch (key) {
+			case Gdk.Key.Escape:
+				return '⎋';
+			case Gdk.Key.Tab:
+				return '⇥';
+			case Gdk.Key.Caps_Lock:
+				return '⇪';
+			case Gdk.Key.Shift_L:
+			case Gdk.Key.Shift_R:
+				return '⇧';
+			case Gdk.Key.Control_L:
+			case Gdk.Key.Control_R:
+				return '⌃';
+			case Gdk.Key.BackSpace:
+				return '⌫';
+			case Gdk.Key.Delete:
+				return '⌦';
+			case Gdk.Key.Home:
+				return '⇱';
+			case Gdk.Key.End:
+				return '⇲';
+			case Gdk.Key.Page_Up:
+				return '⇞';
+			case Gdk.Key.Page_Down:
+				return '⇟';
+			case Gdk.Key.Up:
+				return '↑';
+			case Gdk.Key.Down:
+				return '↓';
+			case Gdk.Key.Left:
+				return '←';
+			case Gdk.Key.Right:
+				return '→';
+			case Gdk.Key.Clear:
+				return '⌧';
+			case Gdk.Key.Num_Lock:
+				return '⇭';
+			case Gdk.Key.Return:
+				return '⏎';
+			case Gdk.Key.space:
+				return '␣';
+			case Gdk.Key.Meta_L:
+			case Gdk.Key.Meta_R:
+				return '⌘';
+			case Gdk.Key.Alt_L:
+			case Gdk.Key.Alt_R:
+				return '⌥';
+			}
+			return (char)0;
+		}
+		
+/*		static Gdk.Key AppleMapSymbolToKey (char ch)
+		{
+			// unicode codes from http://macbiblioblog.blogspot.com/2005/05/special-key-symbols.html
+			// unmapped:
+			// ⇤ tab back
+			// ⌥ option (alt, alternative)
+			// ⌘ command
+			// ⌤ enter
+			
+			switch (ch) {
+			case '⎋':
+				return Gdk.Key.Escape;
+			case '⇥':
+				return Gdk.Key.Tab;
+			case '⇪':
+				return Gdk.Key.Caps_Lock;
+			case '⇧':
+				return Gdk.Key.Shift_L;
+			case '⌃':
+				return Gdk.Key.Control_L;
+			case '⌫':
+				return Gdk.Key.BackSpace;
+			case '⌦':
+				return Gdk.Key.Delete;
+			case '⇱':
+				return Gdk.Key.Home;
+			case '⇲':
+				return Gdk.Key.End;
+			case '⇞':
+				return Gdk.Key.Page_Up;
+			case '⇟':
+				return Gdk.Key.Page_Down;
+			case '↑':
+				return Gdk.Key.Up;
+			case '↓':
+				return Gdk.Key.Down;
+			case '←':
+				return Gdk.Key.Left;
+			case '→':
+				return Gdk.Key.Right;
+			case '⌧':
+				return Gdk.Key.Clear;
+			case '⇭':
+				return Gdk.Key.Num_Lock;
+			case '⏎':
+				return Gdk.Key.Return;
+			case '␣':
+				return Gdk.Key.space;
+			case '⌘':
+				return Gdk.Key.Meta_L;
+			case '⌥':
+				return Gdk.Key.Alt_L;
+			}
+			return (Gdk.Key)0;
+		}
+		*/
+		#endregion
 	}
 	
 //#region KeyBindingException
