@@ -70,6 +70,8 @@ namespace MonoDevelop.Components.Commands
 			} catch (Exception) {
 				string msg = AddinManager.CurrentLocalizer.GetString ("Could not open the url {0}", url);
 				Gtk.MessageDialog md = new Gtk.MessageDialog (null, Gtk.DialogFlags.Modal | Gtk.DialogFlags.DestroyWithParent, Gtk.MessageType.Error, Gtk.ButtonsType.Ok, msg);
+				if (sender != null && sender is Gtk.Widget)
+					md.TransientFor = (sender as Gtk.Widget).Toplevel as Gtk.Window;
 				md.Run ();
 				md.Hide ();
 			}
