@@ -113,7 +113,6 @@ namespace MonoDevelop.Components.DockToolbars
 			bar.AnchorOffset = offset;
 			SortBars ();
 			UpdateRowSizes (bar.DockRow);
-			PackBars ();
 			EnableAnimation (ea);
 		}
 		
@@ -139,16 +138,16 @@ namespace MonoDevelop.Components.DockToolbars
 		protected override void OnRealized ()
 		{
 			base.OnRealized ();
-			bool ea = EnableAnimation (false);
-			ResetBarPositions ();
-			EnableAnimation (ea);
+			ResetBarPositions (false);
 		}
 
-		public void ResetBarPositions ()
+		public void ResetBarPositions (bool animate)
 		{
 			int x=0, row=0;
 			int width = PanelWidth;
 			
+			bool ea = EnableAnimation (animate);
+			
 			foreach (DockToolbar b in bars) {
 				int barw = GetChildWidth (b);
 				if (x + barw < width)
@@ -161,7 +160,7 @@ namespace MonoDevelop.Components.DockToolbars
 				x += barw;
 			}
 			SortBars ();
-
+			EnableAnimation (ea);
 		}
 		
 		void SetPlaceholder (DockToolbar bar, int offset, int row)
@@ -623,7 +622,8 @@ namespace MonoDevelop.Components.DockToolbars
 		
 		void OnBarSizeChanged (object s, EventArgs e)
 		{
-			UpdateRowSizes (((DockToolbar)s).DockRow);
+			if (IsRealized)
+				UpdateRowSizes (((DockToolbar)s).DockRow);
 		}
 		
 		void UpdateRowSizes (int row)
@@ -660,11 +660,16 @@ namespace MonoDevelop.Components.DockToolbars
 		
 		int PackRow (int sn)
 		{
+			// The 'sn' parameter is the index if the first toolbar of the row.
+			// It returns the index of the first toolbar of the next row
+			
 			int n = sn;
 			int row = ((DockToolbar)bars[n]).DockRow;
 			int lastx = 0;
 			int gaps = 0;
-
+			
+			// Calculates the free space in the row
+			
 			while (n < bars.Count) {
 				DockToolbar bar = (DockToolbar) bars [n];
 				if (bar.DockRow != row) break;
@@ -785,6 +790,13 @@ namespace MonoDevelop.Components.DockToolbars
 			return t;
 		}
 		
+		int MaxRow {
+			get {
+				var lastBar = (DockToolbar)bars[bars.Count -1];
+				return lastBar.DockRow;
+			}
+		}
+		
 		void SortBars ()
 		{
 			bars.Sort (DocBarComparer.Instance);
@@ -811,6 +823,34 @@ namespace MonoDevelop.Components.DockToolbars
 			enableAnimations = enable;
 			return r;
 		}
+		
+		protected override bool OnExposeEvent (EventExpose evnt)
+		{
+			//leave this plain unless there are horizontal children, so that the panel matches the toolbars
+			//see DockToolbar.OnExposeEvent
+			if (bars.Count == 0 || this.Orientation != Orientation.Horizontal)
+				return base.OnExposeEvent (evnt);
+			
+			// get a proper GTK+ toolbar style by using one of the children, i.e. a DockToolbar, a subclass of GtkToolbar
+			var styleProvider = (Widget)bars[0];
+			var shadowType = (ShadowType) styleProvider.StyleGetProperty ("shadow-type");
+			
+			//render each row separately, so the theme treats each as a row, and they match the individual toolbars' themed painting
+			//FIXME: Mac only seems to display the last-painted row, even though we make paint calls for all of them
+			int row = MaxRow;
+			for (int i = bars.Count - 1; i >= 0; i--) {
+				var bar = (DockToolbar) bars[i];
+				if (bar.DockRow == row) {
+					row--;
+					Style.PaintBox (bar.Style, evnt.Window, State, shadowType, evnt.Area, bar, "toolbar", 
+					                Allocation.X, bar.Allocation.Y, Allocation.Width, bar.Allocation.Height);
+				}
+			}
+			
+			foreach (DockToolbar bar in bars)
+				this.PropagateExpose (bar, evnt);
+			return true;
+		}
 	}
 
 
