@@ -31,24 +31,24 @@ using System;
 using System.IO;
 using System.Collections.Generic;
 using System.Threading;
+using System.Collections;
 using System.Diagnostics;
-using Mono.Remoting.Channels.Unix;
-using System.Runtime.Remoting;
-using System.Runtime.Remoting.Channels;
-using System.Runtime.Remoting.Channels.Tcp;
 
 using MonoDevelop.Core.AddIns;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Assemblies;
+using MonoDevelop.Core.Instrumentation;
 using Mono.Addins;
 
 namespace MonoDevelop.Core.Execution
 {
-	public class ProcessService : AbstractService
+	public class ProcessService
 	{
 		ProcessHostController externalProcess;
 		List<ExtensionNode> executionHandlers;
-		string remotingChannel = "unix";
-		string unixRemotingFile;
+		DefaultExecutionModeSet defaultExecutionModeSet = new DefaultExecutionModeSet ();
+		IExecutionHandler defaultExecutionHandler = new DefaultExecutionHandler ();
+		IExecutionMode defaultExecutionMode = new DefaultExecutionMode ();
 		
 		Dictionary<string, string> environmentVariableOverrides = null;
 		
@@ -72,11 +72,8 @@ namespace MonoDevelop.Core.Execution
 			}
 		}
 		
-		public override void InitializeService ()
+		internal ProcessService ()
 		{
-			if (PlatformID.Unix != Environment.OSVersion.Platform) {
-				remotingChannel = "tcp";
-			}
 		}
 		
 		public ProcessWrapper StartProcess (string command, string arguments, string workingDirectory, EventHandler exited) 
@@ -135,11 +132,17 @@ namespace MonoDevelop.Core.Execution
 			
 			if (exited != null)
 				p.Exited += exited;
-				
+
+			startInfo.CreateNoWindow = true;
 			p.StartInfo = startInfo;
 			ProcessEnvironmentVariableOverrides (p.StartInfo);
-			p.EnableRaisingEvents = true;
 			
+			// WORKAROUND for "Bug 410743 - wapi leak in System.Diagnostic.Process"
+			// Process leaks when an exit event is registered
+			// instead we use another thread to monitor I/O and wait for exit
+			// p.EnableRaisingEvents = true;
+			
+			Counters.ProcessesStarted++;
 			p.Start ();
 			return p;
 		}
@@ -177,36 +180,18 @@ namespace MonoDevelop.Core.Execution
 		public ProcessWrapper StartConsoleProcess (string command, string arguments, string workingDirectory, IDictionary<string, string> environmentVariables, IConsole console, EventHandler exited)
 		{
 			if (console == null || (console is ExternalConsole)) {
-				string additionalCommands = "";
-				if (!console.CloseOnDispose)
-					additionalCommands = @"echo; read -p 'Press any key to continue...' -n1;";
-				string xtermCommand = String.Format (
-					@" -title ""{4}"" -e ""cd {3} ; '{0}' {1} ; {2}""",
-					command,
-					arguments.Replace ("\\", "\\\\").Replace ("\"", "\\\""),
-					additionalCommands,
-					workingDirectory.Replace (" ", "\\ "),
-				    GettextCatalog.GetString ("MonoDevelop External Console")
-				);
-				ProcessStartInfo psi = new ProcessStartInfo("xterm",xtermCommand);
-				psi.UseShellExecute = false;
-				
-				if (workingDirectory != null)
-					psi.WorkingDirectory = workingDirectory;
+				ProcessStartInfo psi = ExternalConsoleLocator.GetConsoleProcess (command, arguments, workingDirectory, environmentVariables,
+				    GettextCatalog.GetString ("MonoDevelop External Console"), !console.CloseOnDispose);
 
-				psi.UseShellExecute  =  false;
-				
-				if (environmentVariables != null)
-					foreach (KeyValuePair<string, string> kvp in environmentVariables)
-						psi.EnvironmentVariables [kvp.Key] = kvp.Value;
-				
 				ProcessWrapper p = new ProcessWrapper();
 				
 				if (exited != null)
 					p.Exited += exited;
-				
+
+				psi.CreateNoWindow = true;
 				p.StartInfo = psi;
 				ProcessEnvironmentVariableOverrides (p.StartInfo);
+				Counters.ProcessesStarted++;
 				p.Start();
 				return p;
 			} else {
@@ -220,18 +205,50 @@ namespace MonoDevelop.Core.Execution
 			}
 		}
 		
-		public IExecutionHandler GetDefaultExecutionHandler (string platformId)
+		public IExecutionHandler GetDefaultExecutionHandler (ExecutionCommand command)
 		{
 			if (executionHandlers == null) {
 				executionHandlers = new List<ExtensionNode> ();
 				AddinManager.AddExtensionNodeHandler ("/MonoDevelop/Core/ExecutionHandlers", OnExtensionChange);
 			}
 			
-			foreach (ExecutionHandlerCodon codon in executionHandlers)
-				if (codon.Platform == platformId) return codon.ExecutionHandler;
+			foreach (TypeExtensionNode codon in executionHandlers) {
+				IExecutionHandler handler = (IExecutionHandler) codon.GetInstance (typeof(IExecutionHandler));
+				if (handler.CanExecute (command)) return handler;
+			}
 			return null;
 		}
 		
+		public ExecutionCommand CreateCommand (string file)
+		{
+			string f = file.ToLower ();
+			if (f.EndsWith (".exe") || f.EndsWith (".dll"))
+				return new DotNetExecutionCommand (file);
+			else
+				return new NativeExecutionCommand (file);
+		}
+		
+		public IEnumerable<IExecutionModeSet> GetExecutionModes ()
+		{
+			yield return defaultExecutionModeSet;
+			foreach (ExtensionNode node in AddinManager.GetExtensionNodes ("/MonoDevelop/Core/ExecutionModes")) {
+				if (node is ExecutionModeSetNode)
+					yield return (ExecutionModeSetNode) node;
+				else if (!(node is ExecutionModeNode))
+					yield return (IExecutionModeSet) ((TypeExtensionNode)node).GetInstance (typeof (IExecutionModeSet));
+			}
+		}
+		
+		public IExecutionHandler DefaultExecutionHandler {
+			get {
+				return defaultExecutionHandler;
+			}
+		}
+		
+		public IExecutionMode DefaultExecutionMode {
+			get { return defaultExecutionMode; }
+		}
+		
 		void OnExtensionChange (object s, ExtensionNodeEventArgs args)
 		{
 			if (args.Change == ExtensionChange.Add)
@@ -240,32 +257,63 @@ namespace MonoDevelop.Core.Execution
 				executionHandlers.Remove (args.ExtensionNode);
 		}
 		
-		ProcessHostController GetHost (string id, bool shared)
+		ProcessHostController GetHost (string id, bool shared, IExecutionHandler executionHandler)
 		{
 			if (!shared)
-				return new ProcessHostController (id, 0);
+				return new ProcessHostController (id, 0, executionHandler);
 			
 			lock (this) {
 				if (externalProcess == null)
-					externalProcess = new ProcessHostController ("SharedHostProcess", 10000);
+					externalProcess = new ProcessHostController ("SharedHostProcess", 10000, null);
 	
 				return externalProcess;
 			}
 		}
 		
-		public RemoteProcessObject CreateExternalProcessObject (Type type)
+		public IDisposable CreateExternalProcessObject (Type type)
 		{
 			return CreateExternalProcessObject (type, true);
 		}
 		
-		public RemoteProcessObject CreateExternalProcessObject (Type type, bool shared)
+		void CheckRemoteType (Type type)
 		{
-			return GetHost (type.ToString(), shared).CreateInstance (type.Assembly.Location, type.FullName, GetRequiredAddins (type));
+			if (!typeof(IDisposable).IsAssignableFrom (type))
+				throw new ArgumentException ("The remote object type must implement IDisposable", "type");
 		}
 		
-		public RemoteProcessObject CreateExternalProcessObject (string assemblyPath, string typeName, bool shared, params string[] requiredAddins)
+		public IDisposable CreateExternalProcessObject (Type type, bool shared)
 		{
-			return GetHost (typeName, shared).CreateInstance (assemblyPath, typeName, requiredAddins);
+			CheckRemoteType (type);
+			ProcessHostController hc = GetHost (type.ToString(), shared, null);
+			return (IDisposable) hc.CreateInstance (type.Assembly.Location, type.FullName, GetRequiredAddins (type));
+		}
+		
+		public IDisposable CreateExternalProcessObject (Type type, TargetRuntime runtime)
+		{
+			return CreateExternalProcessObject (type, runtime.GetExecutionHandler ());
+		}
+		
+		public IDisposable CreateExternalProcessObject (Type type, IExecutionHandler executionHandler)
+		{
+			CheckRemoteType (type);
+			return (IDisposable) GetHost (type.ToString(), false, executionHandler).CreateInstance (type.Assembly.Location, type.FullName, GetRequiredAddins (type));
+		}
+		
+		public IDisposable CreateExternalProcessObject (string assemblyPath, string typeName, bool shared, params string[] requiredAddins)
+		{
+			return (IDisposable) GetHost (typeName, shared, null).CreateInstance (assemblyPath, typeName, requiredAddins);
+		}
+		
+		public IDisposable CreateExternalProcessObject (string assemblyPath, string typeName, IExecutionHandler executionHandler, params string[] requiredAddins)
+		{
+			return (IDisposable) GetHost (typeName, false, executionHandler).CreateInstance (assemblyPath, typeName, requiredAddins);
+		}
+		
+		public bool IsValidForRemoteHosting (IExecutionHandler handler)
+		{
+			string location = Path.GetDirectoryName (System.Reflection.Assembly.GetExecutingAssembly ().Location);
+			location = Path.Combine (location, "mdhost.exe");
+			return handler.CanExecute (new DotNetExecutionCommand (location));
 		}
 		
 		string[] GetRequiredAddins (Type type)
@@ -280,35 +328,44 @@ namespace MonoDevelop.Core.Execution
 				return null;
 		}
 		
-		public string ExternalProcessRemotingChannel {
-			get { return remotingChannel; }
-			set { 
-				if (value != "tcp" && value != "unix")
-					throw new InvalidOperationException ("Channel not supported: " + value);
-				remotingChannel = value; 
-			}
+		internal void Dispose ()
+		{
+			RemotingService.Dispose ();
 		}
 		
-		internal string RegisterRemotingChannel ()
+		public class ExecutionModeReference
 		{
-			if (remotingChannel == "tcp") {
-				IChannel ch = ChannelServices.GetChannel ("tcp");
-				if (ch == null)
-					ChannelServices.RegisterChannel (new TcpChannel (0), false);
-			} else {
-				IChannel ch = ChannelServices.GetChannel ("unix");
-				if (ch == null) {
-					unixRemotingFile = Path.GetTempFileName ();
-					ChannelServices.RegisterChannel (new UnixChannel (unixRemotingFile), false);
-				}
+			// This class can be used to hold a reference to the execution mode of an
+			// execution set, and be able to compare it with other references.
+			// It's useful for comparing references to IExecutionMode objects
+			// obtained from different GetExecutionModes calls (which may return
+			// new instances of IExecutionMode).
+			
+			IExecutionModeSet mset;
+			IExecutionMode mode;
+			
+			public ExecutionModeReference (IExecutionModeSet mset, IExecutionMode mode)
+			{
+				this.mset = mset;
+				this.mode = mode;
+			}
+			
+			public override bool Equals (object obj)
+			{
+				ExecutionModeReference mref = obj as ExecutionModeReference;
+				if (mref == null)
+					return false;
+				return mref.mset == mset && mref.mode.Name == mode.Name;
+			}
+			
+			public override int GetHashCode ()
+			{
+				return mset.GetHashCode () + mode.Name.GetHashCode ();
+			}
+			
+			public IExecutionMode ExecutionMode {
+				get { return mode; }
 			}
-			return remotingChannel;
-		}
-		
-		public override void UnloadService ()
-		{
-			if (unixRemotingFile != null)
-				File.Delete (unixRemotingFile);
 		}
 	}
 	
@@ -316,9 +373,9 @@ namespace MonoDevelop.Core.Execution
 	{
 		public IConsole console;
 		EventHandler exited;
-		IAsyncOperation operation;
+		IProcessAsyncOperation operation;
 
-		public ProcessMonitor (IConsole console, IAsyncOperation operation, EventHandler exited)
+		public ProcessMonitor (IConsole console, IProcessAsyncOperation operation, EventHandler exited)
 		{
 			this.exited = exited;
 			this.operation = operation;
@@ -332,6 +389,11 @@ namespace MonoDevelop.Core.Execution
 			try {
 				if (exited != null)
 					exited (op, null);
+				
+				if (!PropertyService.IsWindows && Mono.Unix.Native.Syscall.WIFSIGNALED (operation.ExitCode))
+					console.Log.WriteLine (GettextCatalog.GetString ("The application was terminated by a signal: {0}"), Mono.Unix.Native.Syscall.WTERMSIG (operation.ExitCode));
+				else if (operation.ExitCode != 0)
+					console.Log.WriteLine (GettextCatalog.GetString ("The application exited with code: {0}"), operation.ExitCode);
 			} finally {
 				console.Dispose ();
 			}
@@ -357,7 +419,7 @@ namespace MonoDevelop.Core.Execution
 		
 		public void WriteOut (object sender, string s)
 		{
-			writer.WriteLine (s);
+			writer.Write (s);
 		}
 		
 		public static ProcessEventHandler GetWriteHandler (TextWriter tw)
