@@ -37,6 +37,7 @@ namespace MonoDevelop.Core
 	public static class LoggingService
 	{
 		static List<ILogger> loggers = new List<ILogger> ();
+		static RemoteLogger remoteLogger;
 		
 		static LoggingService ()
 		{
@@ -70,6 +71,14 @@ namespace MonoDevelop.Core
 			}
 		}
 		
+		internal static RemoteLogger RemoteLogger {
+			get {
+				if (remoteLogger == null)
+					remoteLogger = new RemoteLogger ();
+				return remoteLogger;
+			}
+		}
+		
 #region the core service
 		
 		public static bool IsLevelEnabled (LogLevel level)
