@@ -120,30 +120,40 @@ namespace MonoDevelop.Components.Docking
 			get {
 				if (widget == null) {
 					widget = new DockItemContainer (frame, this);
+					widget.Visible = false; // Required to ensure that the Shown event is fired
 					widget.Label = label;
-					if (ContentRequired != null) {
-						gettingContent = true;
-						try {
-							ContentRequired (this, EventArgs.Empty);
-						} finally {
-							gettingContent = false;
-						}
-					}
-					widget.UpdateContent ();
-					widget.Shown += delegate {
-						UpdateContentVisibleStatus ();
-					};
-					widget.Hidden += delegate {
-						UpdateContentVisibleStatus ();
-					};
-					widget.ParentSet += delegate {
-						UpdateContentVisibleStatus ();
-					};
+					widget.Shown += SetupContent;
 				}
 				return widget;
 			}
 		}
 		
+		void SetupContent (object ob, EventArgs args)
+		{
+			widget.Shown -= SetupContent;
+			
+			if (ContentRequired != null) {
+				gettingContent = true;
+				try {
+					ContentRequired (this, EventArgs.Empty);
+				} finally {
+					gettingContent = false;
+				}
+			}
+
+			widget.UpdateContent ();
+			widget.Shown += delegate {
+				UpdateContentVisibleStatus ();
+			};
+			widget.Hidden += delegate {
+				UpdateContentVisibleStatus ();
+			};
+			widget.ParentSet += delegate {
+				UpdateContentVisibleStatus ();
+			};
+			UpdateContentVisibleStatus ();
+		}
+		
 		public Widget Content {
 			get {
 				return content;
@@ -228,9 +238,9 @@ namespace MonoDevelop.Components.Docking
 		public void Present (bool giveFocus)
 		{
 			if (dockBarItem != null)
-				dockBarItem.Present (giveFocus);
+				dockBarItem.Present (Status == DockItemStatus.AutoHide || giveFocus);
 			else
-				frame.Present (this, giveFocus);
+				frame.Present (this, Status == DockItemStatus.AutoHide || giveFocus);
 		}
 
 		public bool ContentVisible {
@@ -243,9 +253,14 @@ namespace MonoDevelop.Components.Docking
 
 		internal void SetFocus ()
 		{
-			Widget.ChildFocus (DirectionType.TabForward);
+			SetFocus (Widget);
+		}
+		
+		internal static void SetFocus (Widget w)
+		{
+			w.ChildFocus (DirectionType.TabForward);
 
-			Window win = Widget.Toplevel as Gtk.Window;
+			Window win = w.Toplevel as Gtk.Window;
 			if (win == null)
 				return;
 
@@ -354,8 +369,10 @@ namespace MonoDevelop.Components.Docking
 		internal void SetAutoHideMode (Gtk.PositionType pos, int size)
 		{
 			ResetMode ();
-			if (widget != null)
+			if (widget != null) {
+				widget.Hide (); // Avoids size allocation warning
 				widget.Unparent ();
+			}
 			dockBarItem = frame.BarDock (pos, this, size);
 			if (widget != null)
 				widget.UpdateBehavior ();
