@@ -135,6 +135,8 @@ namespace MonoDevelop.Components.Docking
 		void OnTabPress (object s, Gtk.ButtonPressEventArgs args)
 		{
 			CurrentTab = Array.IndexOf (box.Children, s);
+			Tab t = (Tab) s;
+			DockItem.SetFocus (t.Page);
 			QueueDraw ();
 		}
 
@@ -160,9 +162,41 @@ namespace MonoDevelop.Components.Docking
 				ellipsized = ellipsize;
 			}
 		}
+
+		public Gdk.Rectangle GetTabArea (int ntab)
+		{
+			Gtk.Widget[] tabs = box.Children;
+			Tab tab = (Tab) tabs[ntab];
+			Gdk.Rectangle rect = GetTabArea (tab, ntab);
+			int x, y;
+			tab.GdkWindow.GetRootOrigin (out x, out y);
+			rect.X += x;
+			rect.Y += y;
+			return rect;
+		}
 		
 		protected override bool OnExposeEvent (Gdk.EventExpose evnt)
 		{
+			Gdk.Rectangle rect = Allocation;
+			
+			using (Cairo.Context cr = Gdk.CairoHelper.Create (evnt.Window)) {
+				cr.NewPath ();
+				cr.MoveTo (rect.X, rect.Y);
+				cr.RelLineTo (rect.Width, 0);
+				cr.RelLineTo (0, rect.Height);
+				cr.RelLineTo (-rect.Width, 0);
+				cr.RelLineTo (0, -rect.Height);
+				cr.ClosePath ();
+				Cairo.Gradient pat = new Cairo.LinearGradient (rect.X, rect.Y, rect.X, rect.Y + rect.Height/3);
+				Cairo.Color color1 = DockFrame.ToCairoColor (Style.Mid (StateType.Normal));
+				color1.A = 0.6;
+				pat.AddColorStop (0, color1);
+				color1.A = 0;
+				pat.AddColorStop (1, color1);
+				cr.Pattern = pat;
+				cr.FillPreserve ();
+			}
+			
 			Gtk.Widget[] tabs = box.Children;
 			for (int n=tabs.Length - 1; n>=0; n--) {
 				Tab tab = (Tab) tabs [n];
@@ -171,32 +205,89 @@ namespace MonoDevelop.Components.Docking
 			}
 			if (currentTab != -1) {
 				Tab ctab = (Tab) tabs [currentTab];
-				//GdkWindow.DrawLine (Style.DarkGC (Gtk.StateType.Normal), Allocation.X, Allocation.Y, Allocation.Right, Allocation.Y);
+//				GdkWindow.DrawLine (Style.DarkGC (Gtk.StateType.Normal), Allocation.X, Allocation.Y, Allocation.Right, Allocation.Y);
 				DrawTab (evnt, ctab, currentTab);
 			}
 			return base.OnExposeEvent (evnt);
 		}
-		
-		void DrawTab (Gdk.EventExpose evnt, Tab tab, int pos)
+
+		public Gdk.Rectangle GetTabArea (Tab tab, int pos)
 		{
+			Gdk.Rectangle rect = tab.Allocation;
+
 			int xdif = 0;
 			if (pos > 0)
 				xdif = 2;
-			
+
 			int reqh;
-			StateType st;
-			
+//			StateType st;
+
 			if (tab.Active) {
-				st = StateType.Normal;
+//				st = StateType.Normal;
 				reqh = tab.Allocation.Height;
-			} else {
-				reqh = tab.Allocation.Height - 3;
-				st = StateType.Active;
 			}
-			
-			Gtk.Style.PaintExtension (Style, GdkWindow, st, ShadowType.Out, evnt.Area, this, "tab", tab.Allocation.X - xdif, tab.Allocation.Y, tab.Allocation.Width + xdif, reqh, Gtk.PositionType.Top); 
+			else {
+				reqh = tab.Allocation.Height - 3;
+//				st = StateType.Active;
+			}
+
+			if (DockFrame.IsWindows) {
+				rect.Height = reqh - 1;
+				rect.Width--;
+				if (pos > 0) {
+					rect.X--;
+					rect.Width++;
+				}
+				return rect;
+			}
+			else {
+				rect.X -= xdif;
+				rect.Width += xdif;
+				rect.Height = reqh;
+				return rect;
+			}
 		}
 
+		void DrawTab (Gdk.EventExpose evnt, Tab tab, int pos)
+		{
+			Gdk.Rectangle rect = GetTabArea (tab, pos);
+			StateType st;
+			if (tab.Active)
+				st = StateType.Normal;
+			else
+				st = StateType.Active;
+
+			if (DockFrame.IsWindows) {
+				GdkWindow.DrawRectangle (Style.DarkGC (Gtk.StateType.Normal), false, rect);
+				rect.X++;
+				rect.Width--;
+				if (tab.Active) {
+					GdkWindow.DrawRectangle (Style.LightGC (Gtk.StateType.Normal), true, rect);
+				}
+				else {
+					using (Cairo.Context cr = Gdk.CairoHelper.Create (evnt.Window)) {
+						cr.NewPath ();
+						cr.MoveTo (rect.X, rect.Y);
+						cr.RelLineTo (rect.Width, 0);
+						cr.RelLineTo (0, rect.Height);
+						cr.RelLineTo (-rect.Width, 0);
+						cr.RelLineTo (0, -rect.Height);
+						cr.ClosePath ();
+						Cairo.Gradient pat = new Cairo.LinearGradient (rect.X, rect.Y, rect.X, rect.Y + rect.Height);
+						Cairo.Color color1 = DockFrame.ToCairoColor (Style.Mid (Gtk.StateType.Normal));
+						pat.AddColorStop (0, color1);
+						color1.R *= 1.2;
+						color1.G *= 1.2;
+						color1.B *= 1.2;
+						pat.AddColorStop (1, color1);
+						cr.Pattern = pat;
+						cr.FillPreserve ();
+					}
+				}
+			}
+			else
+				Gtk.Style.PaintExtension (Style, GdkWindow, st, ShadowType.Out, evnt.Area, this, "tab", rect.X, rect.Y, rect.Width, rect.Height, Gtk.PositionType.Top); 
+		}
 	}
 	
 	class Tab: Gtk.EventBox
@@ -206,11 +297,11 @@ namespace MonoDevelop.Components.Docking
 		Gtk.Label labelWidget;
 		int labelWidth;
 		
-		const int TopPadding = 4;
-		const int BottomPadding = 6;
-		const int TopPaddingActive = 5;
-		const int BottomPaddingActive = 8;
-		const int HorzPadding = 7;
+		const int TopPadding = 2;
+		const int BottomPadding = 4;
+		const int TopPaddingActive = 3;
+		const int BottomPaddingActive = 5;
+		const int HorzPadding = 5;
 		
 		public Tab ()
 		{
