@@ -60,12 +60,13 @@ namespace MonoDevelop.Ide.Gui.Components
 		internal const int FilledColumn       = 5;
 		
 		NodeBuilder[] builders;
-		Hashtable builderChains = new Hashtable ();
+		Dictionary<Type, NodeBuilder[]> builderChains = new Dictionary<Type, NodeBuilder[]> ();
 		Hashtable nodeHash = new Hashtable ();
 		
 		Gtk.TreeView tree = new Gtk.TreeView ();
 		Gtk.TreeStore store;
 		internal Gtk.TreeViewColumn complete_column;
+		internal Gtk.CellRendererPixbuf pix_render;
 		internal Gtk.CellRendererText text_render;
 		TreeBuilderContext builderContext;
 		Hashtable callbacks = new Hashtable ();
@@ -73,7 +74,7 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		TreePadOption[] options;
 		TreeOptions globalOptions;
-		Hashtable nodeOptions = new Hashtable ();
+		Dictionary<Gtk.TreeIter, TreeOptions> nodeOptions;
 		
 		TreeNodeNavigator workNode;
 		TreeNodeNavigator compareNode1;
@@ -155,7 +156,7 @@ namespace MonoDevelop.Ide.Gui.Components
 		{
 			this.contextMenuPath = contextMenuPath;
 			builderContext = new TreeBuilderContext (this);
-		
+
 			SetBuilders (builders, options);
 			/*
 			0 -- Text
@@ -165,9 +166,10 @@ namespace MonoDevelop.Ide.Gui.Components
 			4 -- Builder chain
 			5 -- Expanded
 			*/
-			store = new Gtk.TreeStore (typeof (string), typeof (Gdk.Pixbuf), typeof (Gdk.Pixbuf), typeof (object), typeof (object), typeof(bool));
+			store = new Gtk.TreeStore (typeof(string), typeof(Gdk.Pixbuf), typeof(Gdk.Pixbuf), typeof(object), typeof(object), typeof(bool));
 			tree.Model = store;
 			tree.Selection.Mode = Gtk.SelectionMode.Multiple;
+			nodeOptions = new Dictionary<Gtk.TreeIter, TreeOptions> (new IterComparer (store));
 
 			tree.EnableModelDragDest (target_table, Gdk.DragAction.Copy | Gdk.DragAction.Move);
 			Gtk.Drag.SourceSet (tree, Gdk.ModifierType.Button1Mask, target_table, Gdk.DragAction.Copy | Gdk.DragAction.Move);
@@ -181,7 +183,7 @@ namespace MonoDevelop.Ide.Gui.Components
 			complete_column = new Gtk.TreeViewColumn ();
 			complete_column.Title = "column";
 
-			Gtk.CellRendererPixbuf pix_render = new Gtk.CellRendererPixbuf ();
+			pix_render = new Gtk.CellRendererPixbuf ();
 			complete_column.PackStart (pix_render, false);
 			complete_column.AddAttribute (pix_render, "pixbuf", OpenIconColumn);
 			complete_column.AddAttribute (pix_render, "pixbuf-expander-open", OpenIconColumn);
@@ -195,32 +197,32 @@ namespace MonoDevelop.Ide.Gui.Components
 			}
 			text_render.Ypad = 1;
 			PropertyService.PropertyChanged += PropertyChanged;
-			text_render.Edited += new Gtk.EditedHandler (HandleOnEdit);
-			text_render.EditingCanceled += new EventHandler (HandleOnEditCancelled);
+			text_render.Edited += HandleOnEdit;
+			text_render.EditingCanceled += HandleOnEditCancelled;
 			
 			complete_column.PackStart (text_render, true);
 			complete_column.AddAttribute (text_render, "markup", TextColumn);
 			
 			tree.AppendColumn (complete_column);
 			
-			tree.TestExpandRow += new Gtk.TestExpandRowHandler (OnTestExpandRow);
-			tree.RowActivated += new Gtk.RowActivatedHandler(OnNodeActivated);
+			tree.TestExpandRow += OnTestExpandRow;
+			tree.RowActivated += OnNodeActivated;
 			
-			tree.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler (OnButtonRelease);
-			tree.ButtonPressEvent += new Gtk.ButtonPressEventHandler (OnButtonPress);
+			tree.ButtonReleaseEvent += OnButtonRelease;
+			tree.ButtonPressEvent += OnButtonPress;
 			
-			tree.PopupMenu += new Gtk.PopupMenuHandler (OnPopupMenu);
+			tree.PopupMenu += OnPopupMenu;
 			workNode = new TreeNodeNavigator (this);
 			compareNode1 = new TreeNodeNavigator (this);
 			compareNode2 = new TreeNodeNavigator (this);
 			
-			tree.DragBegin += new Gtk.DragBeginHandler (OnDragBegin);
-			tree.DragDataReceived += new Gtk.DragDataReceivedHandler (OnDragDataReceived);
-			tree.DragDrop += new Gtk.DragDropHandler (OnDragDrop);
-			tree.DragEnd += new Gtk.DragEndHandler (OnDragEnd);
-			tree.DragMotion += new Gtk.DragMotionHandler (OnDragMotion);
+			tree.DragBegin += OnDragBegin;
+			tree.DragDataReceived += OnDragDataReceived;
+			tree.DragDrop += OnDragDrop;
+			tree.DragEnd += OnDragEnd;
+			tree.DragMotion += OnDragMotion;
 			
-			tree.CursorChanged += new EventHandler (OnSelectionChanged);
+			tree.CursorChanged += OnSelectionChanged;
 			tree.KeyPressEvent += OnKeyPress;
 				
 			this.Add (tree);
@@ -379,19 +381,6 @@ namespace MonoDevelop.Ide.Gui.Components
 			return foundHandler;
 		}
 
-
-		public override void Dispose ()
-		{
-			Clear ();
-			foreach (NodeBuilder nb in builders) {
-				try {
-					nb.Dispose ();
-				} catch (Exception ex) {
-					LoggingService.LogError (ex.ToString ());
-				}
-			}
-		}
-
 		internal void LockUpdates ()
 		{
 			if (++updateLockCount == 1)
@@ -404,6 +393,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				TransactedNodeStore store = transactionStore;
 				transactionStore = null;
 				store.CommitChanges ();
+				CleanupNodeOptions ();
 			}
 		}
 
@@ -508,7 +498,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				NotifyNodeRemoved (dataObject, null);
 			
 			nodeHash = new Hashtable ();
-			nodeOptions = new Hashtable ();
+			nodeOptions.Clear ();
 			store.Clear ();
 		}
 		
@@ -1095,10 +1085,10 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		public NodeBuilder[] GetBuilderChain (Type type)
 		{
-			NodeBuilder[] chain = builderChains [type] as NodeBuilder[];
-			if (chain == null)
-			{
-				ArrayList list = new ArrayList ();
+			NodeBuilder[] chain;
+			builderChains.TryGetValue (type, out chain);
+			if (chain == null) {
+				List<NodeBuilder> list = new List<NodeBuilder> ();
 				
 				// Find the most specific node builder type.
 				TypeNodeBuilder bestTypeNodeBuilder = null;
@@ -1113,8 +1103,7 @@ namespace MonoDevelop.Ide.Gui.Components
 								bestTypeNodeBuilder = tnb;
 							}
 						}
-					}
-					else {
+					} else {
 						try {
 							if (((NodeBuilderExtension)nb).CanBuildNode (type))
 								list.Add (nb);
@@ -1126,7 +1115,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				
 				if (bestTypeNodeBuilder != null) {
 					list.Insert (0, bestTypeNodeBuilder);
-					chain = (NodeBuilder[]) list.ToArray (typeof(NodeBuilder));
+					chain = list.ToArray ();
 				} else
 					chain = null;
 				
@@ -1143,10 +1132,9 @@ namespace MonoDevelop.Ide.Gui.Components
 			return null;
 		}
 		
-		int CompareNodes (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
+		internal int CompareNodes (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
 		{
 			sorting = true;
-			
 			try {
 				NodeBuilder[] chain1 = (NodeBuilder[]) store.GetValue (a, BuilderChainColumn);
 				if (chain1 == null) return -1;
@@ -1199,8 +1187,9 @@ namespace MonoDevelop.Ide.Gui.Components
 				return false; // There is only one node, GetFirstNode returned it
 			else {
 				Gtk.TreeIter[] its = (Gtk.TreeIter[]) it;
+				Gtk.TreePath iterPath = store.GetPath (iter);
 				for (int n=0; n<its.Length; n++) {
-					if (its [n].Equals (iter)) {
+					if (store.GetPath (its[n]).Equals (iterPath)) {
 						if (n < its.Length - 1) {
 							iter = its [n+1];
 							return true;
@@ -1267,26 +1256,55 @@ namespace MonoDevelop.Ide.Gui.Components
 				}
 			}
 			
-			nodeOptions.Remove (iter);
+			if (store.IterIsValid (iter))
+				nodeOptions.Remove (iter);
 			object currentIt = nodeHash [dataObject];
 			if (currentIt is Gtk.TreeIter[]) {
 				Gtk.TreeIter[] arr = (Gtk.TreeIter[]) currentIt;
-				int i = Array.IndexOf (arr, iter);
-				if (arr.Length > 2) {
-					Gtk.TreeIter[] newArr = new Gtk.TreeIter[arr.Length - 1];
-					Array.Copy (arr, 0, newArr, 0, i);
-					if (i < newArr.Length)
-						Array.Copy (arr, i+1, newArr, i, arr.Length - i - 1);
-					nodeHash [dataObject] = newArr;
-				} else {
-					if (i == 0) nodeHash [dataObject] = arr[1];
-					else nodeHash [dataObject] = arr[0];
+				Gtk.TreePath path = null;
+				List<Gtk.TreeIter> iters = new List<Gtk.TreeIter> ();
+				if (store.IterIsValid (iter))
+					path = store.GetPath (iter);
+				
+				// Iters can't be directly compared (TreeIter.Equals is broken), so we have
+				// to compare paths.
+				foreach (Gtk.TreeIter it in arr) {
+					if (store.IterIsValid (it) && (path == null || !path.Equals (store.GetPath (it))))
+						iters.Add (it);
 				}
+				if (iters.Count > 1)
+					nodeHash [dataObject] = iters.ToArray ();
+				else if (iters.Count == 1)
+					nodeHash [dataObject] = iters[0];
+				else
+					nodeHash.Remove (dataObject);
 			} else {
 				nodeHash.Remove (dataObject);
 				NotifyNodeRemoved (dataObject, chain);
 			}
 		}
+		
+		internal void RemoveChildren (Gtk.TreeIter it)
+		{
+			Gtk.TreeIter child;
+			while (store.IterChildren (out child, it)) {
+				RemoveChildren (child);
+				object childData = store.GetValue (child, ExtensibleTreeView.DataItemColumn);
+				if (childData != null)
+					UnregisterNode (childData, child, null);
+				store.Remove (ref child);
+			}
+		}
+		
+		internal void CleanupNodeOptions ()
+		{
+			Dictionary<Gtk.TreeIter, TreeOptions> newOps = new Dictionary<Gtk.TreeIter, TreeOptions> ();
+			foreach (KeyValuePair<Gtk.TreeIter,TreeOptions> val in nodeOptions) {
+				if (store.IterIsValid (val.Key))
+					newOps [val.Key] = val.Value;
+			}
+			nodeOptions = newOps;
+		}
 				
 		void NotifyNodeRemoved (object dataObject, NodeBuilder[] chain)
 		{
@@ -1337,7 +1355,7 @@ namespace MonoDevelop.Ide.Gui.Components
 			TreeOptions result = null;
 			Gtk.TreeIter it = iter;
 			do {
-				result = nodeOptions [it] as TreeOptions;
+				nodeOptions.TryGetValue (it, out result);
 			} while (result == null && store.IterParent (out it, it));
 
 			if (result == null)
@@ -1355,8 +1373,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		{
 			if (nodeOptions.Count == 0)
 				return;
-				
-			ArrayList toDelete = new ArrayList ();
+
+			List<Gtk.TreeIter> toDelete = new List<Gtk.TreeIter> ();
 			string path = store.GetPath (iter).ToString () + ":";
 			
 			foreach (Gtk.TreeIter nit in nodeOptions.Keys) {
@@ -1365,7 +1383,7 @@ namespace MonoDevelop.Ide.Gui.Components
 					toDelete.Add (nit);
 			}
 
-			foreach (object ob in toDelete)
+			foreach (Gtk.TreeIter ob in toDelete)
 				nodeOptions.Remove (ob);
 		}
 		
@@ -1417,12 +1435,22 @@ namespace MonoDevelop.Ide.Gui.Components
 		internal TreeOptions GetNodeOptions (ITreeNavigator nav)
 		{
 			TreeNode node = nav.CurrentPosition._node as TreeNode;
-			if (node != null && node.HasIter)
-				return nodeOptions [node.NodeIter] as TreeOptions;
+			if (node != null && node.HasIter) {
+				TreeOptions ops;
+				if (nodeOptions.TryGetValue (node.NodeIter, out ops))
+					return ops;
+				else
+					return null;
+			}
 			else if (node != null)
 				return null;
-			else
-				return nodeOptions [nav.CurrentPosition._iter] as TreeOptions;
+			else {
+				TreeOptions ops;
+				if (nodeOptions.TryGetValue (nav.CurrentPosition._iter, out ops))
+					return ops;
+				else
+					return null;
+			}
 		}
 		
 		internal bool GetIterFromNamePath (string path, out Gtk.TreeIter iter)
@@ -1479,7 +1507,6 @@ namespace MonoDevelop.Ide.Gui.Components
 		
 		NodeAttributes GetNodeAttributes (TreeNodeNavigator node)
 		{
-			Gtk.TreeIter iter = node.CurrentPosition._iter;
 			object dataObject = node.DataItem;
 			NodeAttributes attributes = NodeAttributes.None;
 			
@@ -1519,6 +1546,8 @@ namespace MonoDevelop.Ide.Gui.Components
 		void ShowPopup ()
 		{
 			ITreeNavigator tnav = GetSelectedNode ();
+			if (tnav == null)
+				return;
 			TypeNodeBuilder nb = GetTypeNodeBuilder (tnav.CurrentPosition._iter);
 			string menuPath = nb != null && nb.ContextMenuAddinPath != null ? nb.ContextMenuAddinPath : contextMenuPath;
 			if (menuPath == null) {
@@ -1703,6 +1732,37 @@ namespace MonoDevelop.Ide.Gui.Components
 		protected override void OnDestroyed ()
 		{
 			PropertyService.PropertyChanged -= PropertyChanged;
+			if (pix_render != null) {
+				pix_render.Destroy ();
+				pix_render = null;
+			}
+			if (complete_column != null) {
+				complete_column.Destroy ();
+				complete_column = null;
+			}
+			if (text_render != null) {
+				text_render.Destroy ();
+				text_render = null;
+			}
+			
+			if (store != null) {
+				Clear ();
+				store.Dispose ();
+				store = null;
+			}
+			
+			if (builders != null) {
+				foreach (NodeBuilder nb in builders) {
+					try {
+						nb.Dispose ();
+					} catch (Exception ex) {
+						LoggingService.LogError (ex.ToString ());
+					}
+				}
+				builders = null;
+			}
+			builderChains.Clear ();
+			
 			base.OnDestroyed ();
 		}
 		
@@ -1766,7 +1826,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				return itable [compositionKey] as Gdk.Pixbuf;
 			}
 			
-			public void CacheComposedIcon (Gdk.Pixbuf baseIcon, object compositionKey, Gdk.Pixbuf composedIcon)
+			public Gdk.Pixbuf CacheComposedIcon (Gdk.Pixbuf baseIcon, object compositionKey, Gdk.Pixbuf composedIcon)
 			{
 				Hashtable itable = composedIcons [baseIcon] as Hashtable;
 				if (itable == null) {
@@ -1774,6 +1834,7 @@ namespace MonoDevelop.Ide.Gui.Components
 					composedIcons [baseIcon] = itable;
 				}
 				itable [compositionKey] = composedIcon;
+				return composedIcon;
 			}
 			
 			public ITreeNavigator GetTreeNavigator (object dataObject)
@@ -1812,4 +1873,35 @@ namespace MonoDevelop.Ide.Gui.Components
 			return target;
 		}
 	}
+
+	class IterComparer: IEqualityComparer<Gtk.TreeIter>
+	{
+		Gtk.TreeStore store;
+
+		public IterComparer (Gtk.TreeStore store)
+		{
+			this.store = store;
+		}
+		public bool Equals (Gtk.TreeIter x, Gtk.TreeIter y)
+		{
+			if (!store.IterIsValid (x) || !store.IterIsValid (y))
+				return false;
+			Gtk.TreePath px = store.GetPath (x);
+			Gtk.TreePath py = store.GetPath (y);
+			if (px == null || py == null)
+				return false;
+			return x.Equals (y);
+		}
+
+		public int GetHashCode (Gtk.TreeIter obj)
+		{
+			if (!store.IterIsValid (obj))
+				return 0;
+			Gtk.TreePath p = store.GetPath (obj);
+			if (p == null)
+				return 0;
+			else
+				return p.ToString ().GetHashCode ();
+		}
+	}
 }
