@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 
@@ -48,6 +49,7 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 		static readonly int boldCol = 4;
 		static readonly int iconCol = 5;
 		static readonly int iconVisibleCol = 6;
+		static readonly int visibleCol = 7;
 		
 		bool accelIncomplete = false;
 		bool accelComplete = false;
@@ -57,12 +59,18 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 		bool internalUpdate;
 		List<KeyBindingScheme> schemes;
 		
+		TreeModelFilter filterModel;
+		bool filterChanged;
+		string[] processedFilterTerms;
+		bool filterTimeoutRunning;
+		
 		public KeyBindingsPanel ()
 		{
 			this.Build ();
 			
-			keyStore = new TreeStore (typeof (Command), typeof (string), typeof (string), typeof (string), typeof (int), typeof(string), typeof(bool));
-			keyTreeView.Model = keyStore;
+			keyStore = new TreeStore (typeof (Command), typeof (string), typeof (string), typeof (string), typeof (int), typeof(string), typeof(bool), typeof (bool));
+			keyTreeView.Model = filterModel = new TreeModelFilter (keyStore, null);
+			filterModel.VisibleColumn = visibleCol;
 			
 			TreeViewColumn col = new TreeViewColumn ();
 			col.Title = GettextCatalog.GetString ("Command");
@@ -77,7 +85,18 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			col.AddAttribute (crt, "weight", boldCol);
 			keyTreeView.AppendColumn (col);
 			
-			keyTreeView.AppendColumn (GettextCatalog.GetString ("Key Binding"), new CellRendererText (), "text", bindingCol);
+			TreeViewColumn bindingTVCol = new TreeViewColumn ();
+			bindingTVCol.Title = GettextCatalog.GetString ("Key Binding");
+			CellRendererText bindingRenderer = new CellRendererText ();
+			bindingTVCol.PackStart (bindingRenderer, false);
+			bindingTVCol.SetCellDataFunc (bindingRenderer, delegate (TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter) {
+				string binding = (model.GetValue (iter, bindingCol) as string) ?? "";
+				((CellRendererText)cell).Text = binding.Length > 0
+					? KeyBindingManager.BindingToDisplayLabel (binding, false)
+					: binding;
+			});
+			keyTreeView.AppendColumn (bindingTVCol);
+			
 			keyTreeView.AppendColumn (GettextCatalog.GetString ("Description"), new CellRendererText (), "text", descCol);
 			
 			keyTreeView.Selection.Changed += new EventHandler (OnKeysTreeViewSelectionChange);
@@ -99,6 +118,81 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 
 			SelectCurrentScheme ();
 			schemeCombo.Changed += OnKeyBindingSchemeChanged;
+			
+			searchEntry.Changed += delegate {
+				processedFilterTerms = searchEntry.Text.Split (new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries)
+					.Select (s => s.ToLower ()).ToArray ();;
+				filterChanged = true;
+				if (!filterTimeoutRunning) {
+					filterTimeoutRunning = true;
+					GLib.Timeout.Add (300, delegate {
+						if (!filterChanged) {
+							if (filterTimeoutRunning)
+								Refilter ();
+							filterTimeoutRunning = false;
+							return false;
+						}
+						filterChanged = false;
+						return true;
+					});
+				};
+			};
+			
+			clearFilterButton.Clicked += delegate {
+				searchEntry.Text = "";
+				Refilter ();
+				//stop the timeout from refiltering, if it's already running
+				filterTimeoutRunning = false;
+			};
+		}
+		
+		void Refilter ()
+		{
+			keyTreeView.Model = null;
+			TreeIter iter;
+			bool allVisible = processedFilterTerms == null || processedFilterTerms.Length == 0;
+			if (keyStore.GetIterFirst (out iter))
+				Refilter (iter, allVisible);
+			keyTreeView.Model = filterModel;
+			keyTreeView.ExpandAll ();
+			keyTreeView.ColumnsAutosize ();
+		}
+		
+		bool Refilter (TreeIter iter, bool allVisible)
+		{
+			int visibleCount = 0;
+			
+			do {
+				TreeIter child;
+				if (keyStore.IterChildren (out child, iter)) {
+					bool catAllVisible = allVisible || IsSearchMatch ((string) keyStore.GetValue (iter, labelCol));
+					bool childVisible = Refilter (child, catAllVisible);
+					keyStore.SetValue (iter, visibleCol, childVisible);
+					if (childVisible)
+						visibleCount++;
+				} else {
+					bool visible = allVisible
+						|| IsSearchMatch ((string) keyStore.GetValue (iter, labelCol))
+						|| IsSearchMatch ((string) keyStore.GetValue (iter, descCol));
+					keyStore.SetValue (iter, visibleCol, visible);
+					if (visible)
+						visibleCount++;
+				}
+			} while (keyStore.IterNext (ref iter));
+			
+			return visibleCount > 0;
+		}
+		
+		bool IsSearchMatch (string cmp)
+		{
+			if (cmp == null)
+				return false;
+			
+			var lower = cmp.ToLower ();
+			foreach (var term in processedFilterTerms)
+				if (!lower.Contains (term))
+					return false;
+			return true;
 		}
 
 		void SelectCurrentScheme ()
@@ -168,12 +262,13 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 				if (currentCat != cmd.Category) {
 					currentCat = cmd.Category;
 					string name = currentCat.Length == 0? translatedOther : currentCat;
-					icat = keyStore.AppendValues (null, name, String.Empty, String.Empty, (int) Pango.Weight.Bold, null, false);
+					icat = keyStore.AppendValues (null, name, String.Empty, String.Empty, (int) Pango.Weight.Bold, null, false, true);
 				}
 				string label = cmd.Text.Replace ("_", String.Empty);
-				keyStore.AppendValues (icat, cmd, label, cmd.AccelKey != null ? cmd.AccelKey : String.Empty, cmd.Description, (int) Pango.Weight.Normal, cmd.Icon, true);
+				keyStore.AppendValues (icat, cmd, label, cmd.AccelKey != null ? cmd.AccelKey : String.Empty, cmd.Description, (int) Pango.Weight.Normal, cmd.Icon, true, true);
 			}
 			UpdateGlobalWarningLabel ();
+			Refilter ();
 			return this;
 		}
 		
@@ -219,27 +314,26 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			
 			if (sel.GetSelected (out model, out iter) && model.GetValue (iter,commandCol) != null) {
 				accelEntry.Sensitive = true;
-				accelEntry.Text = (string) model.GetValue (iter, bindingCol);
+				CurrentBinding = (string) model.GetValue (iter, bindingCol);
 				accelEntry.GrabFocus ();
 				accelIncomplete = false;
 				accelComplete = true;
 			} else {
 				accelEntry.Sensitive = false;
-				accelEntry.Text = string.Empty;
+				CurrentBinding = string.Empty;
 			}
 		}
 		
 		[GLib.ConnectBefore]
 		void OnAccelEntryKeyPress (object sender, KeyPressEventArgs e)
 		{
-			Gdk.ModifierType mod = e.Event.State;
 			Gdk.Key key = e.Event.Key;
 			string accel;
 			
 			e.RetVal = true;
 			
 			if (accelComplete) {
-				accelEntry.Text = String.Empty;
+				CurrentBinding = String.Empty;
 				accelIncomplete = false;
 				accelComplete = false;
 				mode = null;
@@ -249,57 +343,83 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			}
 			
 			accelComplete = false;
-			if ((accel = KeyBindingManager.AccelFromKey (key, mod)) != null) {
-				accelEntry.Text = KeyBindingManager.Binding (mode, accel);
+			bool combinationComplete;
+			accel = KeyBindingManager.AccelFromKey (e.Event, out combinationComplete);
+			if (combinationComplete) {
+				CurrentBinding = KeyBindingManager.Binding (mode, accel);
 				accelIncomplete = false;
 				if (mode != null)
 					accelComplete = true;
 				else
 					mode = accel;
 			} else {
-				accel = mode != null ? mode + "|" : String.Empty;
+				accel = (mode != null ? mode + "|" : String.Empty) + accel;
 				accelIncomplete = true;
-				
-				if ((mod & Gdk.ModifierType.ControlMask) != 0)
-					accel += "Control+";
-				if ((mod & Gdk.ModifierType.Mod1Mask) != 0 ||
-				    (key.Equals (Gdk.Key.Meta_L) || key.Equals (Gdk.Key.Meta_R)))
-					accel += "Alt+";
-				if ((mod & Gdk.ModifierType.ShiftMask) != 0)
-					accel += "Shift+";
-				
-				if (key.Equals (Gdk.Key.Control_L) || key.Equals (Gdk.Key.Control_R))
-					accel += "Control+";
-				else if (key.Equals (Gdk.Key.Alt_L) || key.Equals (Gdk.Key.Alt_R))
-					accel += "Alt+";
-				else if (key.Equals (Gdk.Key.Shift_L) || key.Equals (Gdk.Key.Shift_R))
-					accel += "Shift+";
-				
-				accelEntry.Text = accel;
+				CurrentBinding = accel;
+			}
+		}
+		
+		string _realBinding;
+		string CurrentBinding {
+			get {
+				return _realBinding;
+			}
+			set {
+				_realBinding = value;
+				accelEntry.Text = _realBinding == null? "" : KeyBindingManager.BindingToDisplayLabel (_realBinding, false, true);
 			}
 		}
 		
 		void OnAccelEntryKeyRelease (object sender, KeyReleaseEventArgs e)
 		{
 			if (accelIncomplete)
-				accelEntry.Text = mode != null ? mode : String.Empty;
+				CurrentBinding = mode != null ? mode : String.Empty;
 		}
 		
 		void OnUpdateButtonClick (object sender, EventArgs e)
 		{
-			TreeSelection sel = keyTreeView.Selection;
 			TreeIter iter;
-			
-			if (sel.GetSelected (out iter)) {
-				Command cmd = (Command) keyStore.GetValue (iter, commandCol);
-				if (cmd != null) {
-					keyStore.SetValue (iter, bindingCol, accelEntry.Text);
-					currentBindings.SetBinding (cmd, accelEntry.Text);
-					SelectCurrentScheme ();
-				}
+			Command cmd;
+			if (GetSelectedCommandIter (out iter, out cmd)) {
+				keyStore.SetValue (iter, bindingCol, CurrentBinding);
+				currentBindings.SetBinding (cmd, CurrentBinding);
+				SelectCurrentScheme ();
 			}
 			UpdateGlobalWarningLabel ();
 		}
+		
+		bool GetSelectedCommandIter (out TreeIter iter, out Command cmd)
+		{
+			TreeSelection sel = keyTreeView.Selection;
+			if (!sel.GetSelected (out iter)) {
+				cmd = null;
+				return false;
+			}
+			
+			cmd = (Command)filterModel.GetValue (iter, commandCol);
+			if (cmd == null)
+				return false;
+			
+			if (keyStore.GetIterFirst (out iter) && FindIterForCommand (cmd, iter, out iter))
+				return true;
+			
+			throw new Exception ("Did not find command in underlying model");
+		}
+		
+		bool FindIterForCommand (object cmd, TreeIter iter, out TreeIter found)
+		{
+			do {
+				TreeIter child;
+				if (keyStore.IterChildren (out child, iter) && FindIterForCommand (cmd, child, out found))
+					return true;
+				if (keyStore.GetValue (iter, commandCol) == cmd) {
+					found = iter;
+					return true;
+				}
+			} while (keyStore.IterNext (ref iter));
+			found = TreeIter.Zero;
+			return false;
+		}
 
 		void UpdateGlobalWarningLabel ()
 		{
@@ -355,7 +475,7 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 
 		void UpdateWarningLabel ()
 		{
-			if (accelEntry.Text.Length == 0) {
+			if (CurrentBinding.Length == 0) {
 				labelMessage.Visible = false;
 				return;
 			}
@@ -370,7 +490,7 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 				return;
 			}
 			
-			var bindings = FindBindings (accelEntry.Text);
+			var bindings = FindBindings (CurrentBinding);
 			bindings.Remove (cmd);
 			
 			if (bindings.Count > 0) {
@@ -393,7 +513,7 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 				model.IterChildren (out citer, iter);
 				do {
 					string binding = (string) model.GetValue (citer, bindingCol);
-					if (binding == accel) {
+					if (Conflicts (binding, accel)) {
 						Command command = (Command) model.GetValue (citer, commandCol);
 						bindings.Add (command);
 					}
@@ -402,6 +522,19 @@ namespace MonoDevelop.Ide.Gui.OptionPanels
 			return bindings;
 		}
 		
+		bool Conflicts (string b1, string b2)
+		{
+			// Control+X conflicts with Control+X, Control+X|Y
+			// Control+X|Y conflicts with Control+X|Y, Control+X
+			if (b1 == b2)
+				return true;
+			int i = b1.IndexOf ('|');
+			if (i == -1)
+				return b2.StartsWith (b1 + "|");
+			else
+				return b1.Substring (0, i) == b2;
+		}
+		
 		public bool ValidateChanges ()
 		{
 			return true;
