@@ -43,7 +43,6 @@ using MonoDevelop.Components;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Components.Commands;
-using MonoDevelop.Ide.Gui.Search;
 using MonoDevelop.Ide.Gui.Components;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
@@ -140,9 +139,15 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			
 			if (dataObject is ProjectFile) {
 				ProjectFile file = (ProjectFile) dataObject;
-				return Path.GetDirectoryName (file.Name) != targetPath && file.DependsOnFile == null;
-			} if (dataObject is ProjectFolder) {
-				return ((ProjectFolder)dataObject).Path != targetPath;
+				return (Path.GetDirectoryName (file.Name) != targetPath || operation == DragOperation.Copy) && file.DependsOnFile == null;
+			}
+			else if (dataObject is ProjectFolder) {
+				return ((ProjectFolder)dataObject).Path != targetPath || operation == DragOperation.Copy;
+			}
+			else if (dataObject is Gtk.SelectionData) {
+				SelectionData data = (SelectionData) dataObject;
+				if (data.Type == "text/uri-list")
+					return true;
 			}
 			return false;
 		}
@@ -157,8 +162,9 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		
 		void DropNode (Set<SolutionEntityItem> projectsToSave, object dataObject, DragOperation operation)
 		{
-			string targetPath = GetFolderPath (CurrentNode.DataItem);
-			string what, source;
+			FilePath targetPath = GetFolderPath (CurrentNode.DataItem);
+			FilePath source;
+			string what;
 			Project targetProject = (Project) CurrentNode.GetParentDataItem (typeof(Project), true);
 			Project sourceProject;
 			System.Collections.Generic.IEnumerable<ProjectFile> groupedChildren = null;
@@ -173,14 +179,38 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 			else if (dataObject is ProjectFile) {
 				ProjectFile file = (ProjectFile) dataObject;
-				source = file.Name;
+				source = file.FilePath;
 				sourceProject = file.Project;
 				groupedChildren = file.DependentChildren;
 				what = null;
 				ask = false;
-			} else {
+			}
+			else if (dataObject is Gtk.SelectionData) {
+				SelectionData data = (SelectionData) dataObject;
+				if (data.Type != "text/uri-list")
+					return;
+				string sources = System.Text.Encoding.UTF8.GetString (data.Data);
+				string[] files = sources.Split (new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
+				for (int n=0; n<files.Length; n++) {
+					Uri uri = new Uri (files[n]);
+					if (uri.Scheme != "file")
+						return;
+					if (Directory.Exists (uri.LocalPath))
+						return;
+					files[n] = uri.LocalPath;
+				}
+				
+				IdeApp.ProjectOperations.AddFilesToProject (targetProject, files, targetPath);
+				projectsToSave.Add (targetProject);
 				return;
 			}
+			else
+				return;
+
+			targetPath = targetPath.Combine (source.FileName);
+			// If copying to the same directory, make a copy with a different name
+			if (targetPath == source)
+				targetPath = GetTargetCopyName (targetPath, dataObject is ProjectFolder);
 			
 			if (ask) {
 				string q;
@@ -188,7 +218,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 					if (targetPath == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, Path.GetFileName (targetPath));
+						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, targetPath.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Move))
 						return;
 				}
@@ -196,7 +226,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 					if (targetPath == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, Path.GetFileName (targetPath));
+						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, targetPath.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Copy))
 						return;
 				}
@@ -264,6 +294,49 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 		}
 		
+		internal static FilePath GetTargetCopyName (FilePath path, bool isFolder)
+		{
+			int n=1;
+			// First of all try to find an existing copy tag
+			string fn = path.FileNameWithoutExtension;
+			for (int i=1; i<100; i++) {
+				string copyTag = GetCopyTag (i); 
+				if (fn.EndsWith (copyTag)) {
+					string newfn = fn.Substring (0, fn.Length - copyTag.Length);
+					if (newfn.Trim ().Length > 0) {
+						n = i + 1;
+						path = path.ParentDirectory.Combine (newfn + path.Extension);
+						break;
+					}
+				}
+			}
+			FilePath basePath = path;
+			while ((!isFolder && File.Exists (path)) || (isFolder && Directory.Exists (path))) {
+				string copyTag = GetCopyTag (n);
+				path = basePath.ParentDirectory.Combine (basePath.FileNameWithoutExtension + copyTag + basePath.Extension);
+				n++;
+			}
+			return path;
+		}
+		
+		static string GetCopyTag (int n)
+		{
+			string sc;
+			switch (n) {
+				case 1: sc = GettextCatalog.GetString ("copy"); break;
+				case 2: sc = GettextCatalog.GetString ("another copy"); break;
+				case 3: sc = GettextCatalog.GetString ("3rd copy"); break;
+				case 4: sc = GettextCatalog.GetString ("4th copy"); break;
+				case 5: sc = GettextCatalog.GetString ("5th copy"); break;
+				case 6: sc = GettextCatalog.GetString ("6th copy"); break;
+				case 7: sc = GettextCatalog.GetString ("7th copy"); break;
+				case 8: sc = GettextCatalog.GetString ("8th copy"); break;
+				case 9: sc = GettextCatalog.GetString ("9th copy"); break;
+				default: sc = GettextCatalog.GetString ("copy {0}"); break;
+			}
+			return " (" + string.Format (sc, n) + ")";
+		}
+		
 		[CommandHandler (ProjectCommands.AddFiles)]
 		public void AddFilesToProject()
 		{
@@ -272,6 +345,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			FileSelector fdiag  = new FileSelector (GettextCatalog.GetString ("Add files"));
 			fdiag.SetCurrentFolder (GetFolderPath (CurrentNode.DataItem));
 			fdiag.SelectMultiple = true;
+			fdiag.TransientFor = IdeApp.Workbench.RootWindow;
 			
 			//add a combo that can be used to override the default build action
 			ComboBox combo = new ComboBox (project.GetBuildActions ());
