@@ -29,6 +29,7 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 
 using MonoDevelop.Projects;
 using MonoDevelop.Core;
@@ -37,6 +38,7 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide.Gui.Dialogs;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
@@ -68,14 +70,17 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			IdeApp.Workspace.FileRemovedFromProject += fileRemovedHandler;
 			IdeApp.Workspace.FileRenamedInProject += fileRenamedHandler;
 			IdeApp.Workspace.FilePropertyChangedInProject += filePropertyChangedHandler;
+			
+			IdeApp.Workspace.ActiveConfigurationChanged += IdeAppWorkspaceActiveConfigurationChanged;
 		}
-		
+
 		public override void Dispose ()
 		{
 			IdeApp.Workspace.FileAddedToProject -= fileAddedHandler;
 			IdeApp.Workspace.FileRemovedFromProject -= fileRemovedHandler;
 			IdeApp.Workspace.FileRenamedInProject -= fileRenamedHandler;
 			IdeApp.Workspace.FilePropertyChangedInProject -= filePropertyChangedHandler;
+			IdeApp.Workspace.ActiveConfigurationChanged -= IdeAppWorkspaceActiveConfigurationChanged;
 		}
 
 		public override void OnNodeAdded (object dataObject)
@@ -112,8 +117,11 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			if (p is DotNetProject && ((DotNetProject)p).LanguageBinding == null) {
 				iconName = Gtk.Stock.DialogError;
 				label = GettextCatalog.GetString ("{0} <span foreground='red' size='small'>(Unknown language '{1}')</span>", p.Name, ((DotNetProject)p).LanguageName);
+			} else if (p is UnknownProject) {
+				iconName = Gtk.Stock.DialogError;
+				label = GettextCatalog.GetString ("{0} <span foreground='red' size='small'>(Unknown project type)</span>", p.Name);
 			} else {
-				iconName = Services.Icons.GetImageForProjectType (p.ProjectType);
+				iconName = p.StockIcon;
 				if (p.ParentSolution != null && p.ParentSolution.SingleStartup && p.ParentSolution.StartupItem == p)
 					label = "<b>" + p.Name + "</b>";
 				else
@@ -121,6 +129,15 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 			
 			icon = Context.GetIcon (iconName);
+			
+			SolutionConfiguration conf = p.ParentSolution.GetConfiguration (IdeApp.Workspace.ActiveConfiguration);
+			if (conf == null || !conf.BuildEnabledForItem (p)) {
+				Gdk.Pixbuf ticon = Context.GetComposedIcon (icon, "project-no-build");
+				if (ticon == null)
+					ticon = Context.CacheComposedIcon (icon, "project-no-build", ImageService.MakeTransparent (icon, 0.5));
+				icon = ticon;
+				label = "<span foreground='gray'>" + label + "</span>";
+			}
 		}
 
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -296,6 +313,20 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			ITreeBuilder tb = Context.GetTreeBuilder (args.Project);
 			if (tb != null) tb.UpdateAll ();
 		}
+		
+		void IdeAppWorkspaceActiveConfigurationChanged (object sender, EventArgs e)
+		{
+			foreach (Project p in IdeApp.Workspace.GetAllProjects ()) {
+				ITreeBuilder tb = Context.GetTreeBuilder (p);
+				if (tb != null) {
+					tb.Update ();
+					SolutionConfiguration conf = p.ParentSolution.GetConfiguration (IdeApp.Workspace.ActiveConfiguration);
+					if (conf == null || !conf.BuildEnabledForItem (p))
+						tb.Expanded = false;
+				}
+			}
+		}
+		
 	}
 	
 	public class ProjectNodeCommandHandler: FolderCommandHandler
@@ -307,14 +338,8 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		
 		public override void RenameItem (string newName)
 		{
-			if (!FileService.IsValidFileName(newName)) {
-				MessageService.ShowError (GettextCatalog.GetString ("Illegal project name.\nOnly use letters, digits, space, '.' or '_'."));
-				return;
-			}
-			
 			Project project = (Project) CurrentNode.DataItem;
-			project.Name = newName;
-			IdeApp.Workspace.Save();
+			IdeApp.ProjectOperations.RenameItem (project, newName);
 		}
 		
 		public override void ActivateItem ()
@@ -329,19 +354,13 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			Project project = CurrentNode.DataItem as Project;
 			project.ParentSolution.SingleStartup = true;
 			project.ParentSolution.StartupItem = project;
+			IdeApp.ProjectOperations.Save (project.ParentSolution);
 		}
 		
 		public override void DeleteItem ()
 		{
 			Project prj = CurrentNode.DataItem as Project;
-			bool yes = MessageService.Confirm (GettextCatalog.GetString ("Do you really want to remove project '{0}' from '{1}'?", prj.Name, prj.ParentFolder.Name), AlertButton.Remove);
-
-			if (yes) {
-				Solution sol = prj.ParentSolution;
-				prj.ParentFolder.Items.Remove (prj);
-				prj.Dispose ();
-				IdeApp.ProjectOperations.Save (sol);
-			}
+			IdeApp.ProjectOperations.RemoveSolutionItem (prj);
 		}
 		
 		[CommandHandler (ProjectCommands.AddReference)]
