@@ -63,23 +63,26 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			switch (pref.ReferenceType) {
 				case ReferenceType.Project:
 					label = pref.Reference;
+					icon = Context.GetIcon ("md-reference-project");
 					break;
 				case ReferenceType.Assembly:
 					label = Path.GetFileName(pref.Reference);
+					icon = Context.GetIcon ("md-reference-folder");
 					break;
 				case ReferenceType.Gac:
 					label = pref.Reference.Split(',')[0];
+					icon = Context.GetIcon ("md-reference-package");
 					break;
 				default:
 					label = pref.Reference;
+					icon = Context.GetIcon (Stock.Reference);
 					break;
 			}
 
 			if (!pref.IsValid) {
 				label = "<span color='red'>" + label + "</span>";
 				icon = Context.GetIcon ("md-reference-warning");
-			} else
-				icon = Context.GetIcon (Stock.Reference);
+			}
 		}
 		
 		public override bool HasChildNodes (MonoDevelop.Ide.Gui.Components.ITreeBuilder builder, object dataObject)
@@ -94,6 +97,26 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			if (!pref.IsValid)
 				treeBuilder.AddChild (new TreeViewItem (pref.ValidationErrorMessage, Gtk.Stock.DialogWarning));
 		}
+		
+		public override void OnNodeAdded (object dataObject)
+		{
+			ProjectReference pref = (ProjectReference) dataObject;
+			pref.StatusChanged += ReferenceStatusChanged;
+		}
+		
+		public override void OnNodeRemoved (object dataObject)
+		{
+			ProjectReference pref = (ProjectReference) dataObject;
+			pref.StatusChanged -= ReferenceStatusChanged;
+		}
+
+		void ReferenceStatusChanged (object sender, EventArgs e)
+		{
+			ITreeBuilder tb = Context.GetTreeBuilder (sender);
+			if (tb != null)
+				tb.UpdateAll ();
+		}
+
 
 	}
 	
@@ -161,6 +184,46 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 		}
 		
+		[CommandHandler (ProjectCommands.SpecificAssemblyVersion)]
+		[AllowMultiSelection]
+		public void RequireSpecificAssemblyVersion ()
+		{
+			Dictionary<Project,Project> projects = new Dictionary<Project,Project> ();
+			ProjectReference firstRef = null;
+			foreach (ITreeNavigator node in CurrentNodes) {
+				ProjectReference pref = (ProjectReference) node.DataItem;
+				if (firstRef == null) {
+					firstRef = pref;
+					pref.SpecificVersion = !pref.SpecificVersion;
+				} else
+					pref.SpecificVersion = firstRef.SpecificVersion;
+				Project project = node.GetParentDataItem (typeof(Project), false) as Project;
+				projects [project] = project;
+			}
+			foreach (Project p in projects.Values)
+				IdeApp.ProjectOperations.Save (p);
+		}
+		
+		[CommandUpdateHandler (ProjectCommands.SpecificAssemblyVersion)]
+		public void UpdateRequireSpecificAssemblyVersion (CommandInfo info)
+		{
+			ProjectReference lastRef = null;
+			foreach (ITreeNavigator node in CurrentNodes) {
+				ProjectReference pref = (ProjectReference) node.DataItem;
+				if (pref.ReferenceType == ReferenceType.Gac) {
+					if (lastRef == null || lastRef.LocalCopy == pref.LocalCopy) {
+						lastRef = pref;
+						info.Checked = pref.SpecificVersion;
+					} else
+						info.CheckedInconsistent = true;
+				}
+				else {
+					info.Checked = false;
+					info.Enabled = false;
+				}
+			}
+		}
+		
 		public override DragOperation CanDragNode ()
 		{
 			return DragOperation.Copy;
