@@ -59,7 +59,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				icon = Context.GetIcon (MonoDevelop.Core.Gui.Stock.Project);
 				Gdk.Pixbuf gicon = Context.GetComposedIcon (icon, "fade");
 				if (gicon == null) {
-					gicon = Services.Icons.MakeTransparent (icon, 0.5);
+					gicon = ImageService.MakeTransparent (icon, 0.5);
 					Context.CacheComposedIcon (icon, "fade", gicon);
 				}
 				icon = gicon;
@@ -77,7 +77,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		{
 			UnknownSolutionItem entry = (UnknownSolutionItem) dataObject;
 			if (entry.LoadError.Length > 0)
-				treeBuilder.AddChild (new TreeViewItem (entry.LoadError));
+				treeBuilder.AddChild (new TreeViewItem (GLib.Markup.EscapeText (entry.LoadError)));
 		}
 
 		public override string GetNodeName (ITreeNavigator thisNode, object dataObject)
