@@ -1,22 +1,32 @@
-//  ErrorListPad.cs
-//
-//  This file was derived from a file from #Develop. 
-//
-//  Copyright (C) 2001-2007 Mike Krüger <mkrueger@novell.com>
-// 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-// 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
+// ErrorListPad.cs
 //  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Author:
+//       Todd Berman <tberman@sevenl.net>
+//       David Makovský <yakeen@sannyas-on.net>
+//       Lluis Sanchez Gual <lluis@novell.com>
+// 
+// Copyright (c) 2004 Todd Berman
+// Copyright (c) 2006 David Makovský
+// Copyright (c) 2009 Novell, Inc (http://www.novell.com)
+// 
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+// 
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
+
 
 using System;
 using System.Drawing;
@@ -33,6 +43,7 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Tasks;
 
 using Gtk;
+using System.Text;
 
 namespace MonoDevelop.Ide.Gui.Pads
 {
@@ -44,10 +55,12 @@ namespace MonoDevelop.Ide.Gui.Pads
 		ListStore store;
 		TreeModelFilter filter;
 		ToggleToolButton errorBtn, warnBtn, msgBtn;
-		Gtk.Tooltips tips = new Gtk.Tooltips ();
 		Hashtable tasks = new Hashtable ();
 //		IPadWindow window;
 		bool initializeLocation = true;
+		int errorCount;
+		int warningCount;
+		int infoCount;
 
 		Menu menu;
 		Dictionary<ToggleAction, int> columnsActions = new Dictionary<ToggleAction, int> ();
@@ -56,7 +69,6 @@ namespace MonoDevelop.Ide.Gui.Pads
 		Gdk.Pixbuf iconWarning;
 		Gdk.Pixbuf iconError;
 		Gdk.Pixbuf iconInfo;
-		Gdk.Pixbuf iconQuestion;
 		
 		const string showErrorsPropertyName = "SharpDevelop.TaskList.ShowErrors";
 		const string showWarningsPropertyName = "SharpDevelop.TaskList.ShowWarnings";
@@ -86,6 +98,8 @@ namespace MonoDevelop.Ide.Gui.Pads
 		
 		public Gtk.Widget Control {
 			get {
+				if (control == null)
+					CreateControl ();
 				return control;
 			}
 		}
@@ -100,6 +114,10 @@ namespace MonoDevelop.Ide.Gui.Pads
 		}
 
 		public ErrorListPad ()
+		{
+		}
+		
+		void CreateControl ()
 		{
 			control = new VBox ();
 
@@ -113,7 +131,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			errorBtn.IconWidget = new Gtk.Image (Gtk.Stock.DialogError, Gtk.IconSize.Button);
 			errorBtn.IsImportant = true;
 			errorBtn.Toggled += new EventHandler (FilterChanged);
-			errorBtn.SetTooltip (tips, GettextCatalog.GetString ("Show Errors"), GettextCatalog.GetString ("Show Errors"));
+			errorBtn.TooltipText = GettextCatalog.GetString ("Show Errors");
 			toolbar.Insert (errorBtn, -1);
 			
 			toolbar.Insert (new SeparatorToolItem (), -1);
@@ -124,7 +142,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			warnBtn.IconWidget = new Gtk.Image (Gtk.Stock.DialogWarning, Gtk.IconSize.Button);
 			warnBtn.IsImportant = true;
 			warnBtn.Toggled += new EventHandler (FilterChanged);
-			warnBtn.SetTooltip (tips, GettextCatalog.GetString ("Show Warnings"), GettextCatalog.GetString ("Show Warnings"));
+			warnBtn.TooltipText = GettextCatalog.GetString ("Show Warnings");
 			toolbar.Insert (warnBtn, -1);
 			
 			toolbar.Insert (new SeparatorToolItem (), -1);
@@ -135,7 +153,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			msgBtn.IconWidget = new Gtk.Image (Gtk.Stock.DialogInfo, Gtk.IconSize.Button);
 			msgBtn.IsImportant = true;
 			msgBtn.Toggled += new EventHandler (FilterChanged);
-			msgBtn.SetTooltip (tips, GettextCatalog.GetString ("Show Messages"), GettextCatalog.GetString ("Show Messages"));
+			msgBtn.TooltipText = GettextCatalog.GetString ("Show Messages");
 			toolbar.Insert (msgBtn, -1);
 			
 			store = new Gtk.ListStore (typeof (Gdk.Pixbuf), // image - type
@@ -149,40 +167,32 @@ namespace MonoDevelop.Ide.Gui.Pads
 			                           typeof (bool),       // read?
 			                           typeof (int));       // read? -- use Pango weight
 
-			TreeIterCompareFunc sortFunc = new TreeIterCompareFunc (TaskSortFunc);
-			store.SetSortFunc ((int)Columns.Task, sortFunc);
-			store.DefaultSortFunc = sortFunc;
-			store.SetSortColumnId ((int)Columns.Task, SortType.Ascending);
-			
 			TreeModelFilterVisibleFunc filterFunct = new TreeModelFilterVisibleFunc (FilterTaskTypes);
 			filter = new TreeModelFilter (store, null);
-            filter.VisibleFunc = filterFunct;
+			filter.VisibleFunc = filterFunct;
 			
 			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (filter);
 			view.RulesHint = true;
 			view.PopupMenu += new PopupMenuHandler (OnPopupMenu);
 			view.ButtonPressEvent += new ButtonPressEventHandler (OnButtonPressed);
-			view.HeadersClickable = true;
 			AddColumns ();
 			LoadColumnsVisibility ();
 			
 			sw = new Gtk.ScrolledWindow ();
 			sw.ShadowType = ShadowType.None;
 			sw.Add (view);
-			
-			Services.TaskService.TaskRemoved      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (ShowResults));
-			Services.TaskService.TaskAdded        += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskAdded));
-			Services.TaskService.TaskChanged      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskChanged));
+			TaskService.Errors.TasksRemoved      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (ShowResults));
+			TaskService.Errors.TasksAdded        += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskAdded));
+			TaskService.Errors.TasksChanged      += (TaskEventHandler) DispatchService.GuiDispatch (new TaskEventHandler (TaskChanged));
 			
 			IdeApp.Workspace.FirstWorkspaceItemOpened += OnCombineOpen;
 			IdeApp.Workspace.LastWorkspaceItemClosed += OnCombineClosed;
 			
-			view.RowActivated            += new RowActivatedHandler (OnRowActivated);
-						
+			view.RowActivated += new RowActivatedHandler (OnRowActivated);
+			
 			iconWarning = sw.RenderIcon (Gtk.Stock.DialogWarning, Gtk.IconSize.Menu, "");
 			iconError = sw.RenderIcon (Gtk.Stock.DialogError, Gtk.IconSize.Menu, "");
 			iconInfo = sw.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.Menu, "");
-			iconQuestion = sw.RenderIcon (Gtk.Stock.DialogQuestion, Gtk.IconSize.Menu, "");
 			
 			control.Add (sw);
 			toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
@@ -192,13 +202,12 @@ namespace MonoDevelop.Ide.Gui.Pads
 			CreateMenu ();
 
 			// Load existing tasks
-			foreach (Task t in Services.TaskService.Tasks) {
+			foreach (Task t in TaskService.Errors) {
 				AddTask (t);
 			}
 
 			control.FocusChain = new Gtk.Widget [] { sw };
 		}
-		
 		void LoadColumnsVisibility ()
 		{
 			string columns = (string)PropertyService.Get ("Monodevelop.ErrorListColumns", "TRUE;TRUE;TRUE;TRUE;TRUE;TRUE;TRUE");
@@ -366,10 +375,30 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			Task task = SelectedTask;
 			if (task != null) {
+				StringBuilder text = new StringBuilder ();
+				if (!string.IsNullOrEmpty (task.FileName)) {
+					text.Append (task.FileName);
+					if (task.Line >= 1) {
+						text.Append ("(").Append (task.Column);
+						if (task.Column >= 0)
+							text.Append (",").Append (task.Column);
+						text.Append (")");
+					}
+					text.Append (": ");
+				}
+				text.Append (task.Severity);
+				if (!string.IsNullOrEmpty (task.Code)) {
+					text.Append (" ").Append (task.Code);
+				}
+				text.Append (": ");
+				text.Append (task.Description);
+				if (task.WorkspaceObject != null)
+					text.Append (" (").Append (task.WorkspaceObject.Name).Append (")");
+				
 				clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
-				clipboard.Text = task.ToString ();
+				clipboard.Text = text.ToString ();
 				clipboard = Clipboard.Get (Gdk.Atom.Intern ("PRIMARY", false));
-				clipboard.Text = task.ToString ();
+				clipboard.Text = text.ToString ();
 			}
 		}
 
@@ -385,8 +414,8 @@ namespace MonoDevelop.Ide.Gui.Pads
 		bool GetSelectedErrorReference (out string reference)
 		{
 			Task task = SelectedTask;
-			if (task != null && !String.IsNullOrEmpty (task.ErrorNumber)) {
-				reference = task.ErrorNumber;
+			if (task != null && !String.IsNullOrEmpty (task.Code)) {
+				reference = task.Code;
 				return true;
 			}
 			reference = null;
@@ -427,9 +456,6 @@ namespace MonoDevelop.Ide.Gui.Pads
 			
 			TreeViewColumn col;
 			col = view.AppendColumn ("!", iconRender, "pixbuf", Columns.Type);
-			col.Clickable = true;
-			col.Clicked += new EventHandler (OnResortTasks);
-			col.SortIndicator = true;
 			view.AppendColumn ("", toggleRender, "active", Columns.Marked);
 			view.AppendColumn (GettextCatalog.GetString ("Line"), view.TextRenderer, "text", Columns.Line, "weight", Columns.Weight);
 			col = view.AppendColumn (GettextCatalog.GetString ("Description"), view.TextRenderer, "text", Columns.Description, "weight", Columns.Weight, "strikethrough", Columns.Marked);
@@ -481,9 +507,9 @@ namespace MonoDevelop.Ide.Gui.Pads
 				Task task = store.GetValue (iter, (int)Columns.Task) as Task;
 				if (task == null)
 					return true;
-				if (task.TaskType == TaskType.Error && errorBtn.Active) canShow = true;
-				else if (task.TaskType == TaskType.Warning && warnBtn.Active) canShow = true;
-				else if (task.TaskType == TaskType.Comment && msgBtn.Active) canShow = true;
+				if (task.Severity == TaskSeverity.Error && errorBtn.Active) canShow = true;
+				else if (task.Severity == TaskSeverity.Warning && warnBtn.Active) canShow = true;
+				else if (task.Severity == TaskSeverity.Information && msgBtn.Active) canShow = true;
 			} catch {
 				//Not yet fully added
 				return false;
@@ -496,12 +522,13 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			Clear();
 
-			foreach (Task t in Services.TaskService.Tasks)
+			foreach (Task t in TaskService.Errors)
 				AddTask (t);
 		}
 
 		private void Clear()
 		{
+			errorCount = warningCount = infoCount = 0;
 			store.Clear ();
 			tasks.Clear ();
 			UpdateErrorsNum ();
@@ -558,44 +585,30 @@ namespace MonoDevelop.Ide.Gui.Pads
 		
 		void AddTaskInternal (Task t)
 		{
-			if (t.TaskType == TaskType.Comment) 
-				return;
-				
 			if (tasks.Contains (t)) return;
 			
-			switch (t.TaskType) {
-				case TaskType.Error:
+			Gdk.Pixbuf stock;
+			
+			switch (t.Severity) {
+				case TaskSeverity.Error:
+					stock = iconError;
+					errorCount++;
 					UpdateErrorsNum ();
 					break; 
-				case TaskType.Warning:
+				case TaskSeverity.Warning:
+					stock = iconWarning;
+					warningCount++;
 					UpdateWarningsNum ();	
 					break;
 				default:
+					stock = iconInfo;
+					infoCount++;
 					UpdateMessagesNum ();
 					break;
 			}
 		
 			tasks [t] = t;
 			
-			Gdk.Pixbuf stock;
-			switch (t.TaskType) {
-				case TaskType.Warning:
-					stock = iconWarning;
-					break;
-				case TaskType.Error:
-					stock = iconError;
-					break;
-				case TaskType.Comment:
-					stock = iconInfo;
-					break;
-				case TaskType.SearchResult:
-					stock = iconQuestion;
-					break;
-				default:
-					stock = null;
-					break;
-			}
-			
 			string tmpPath = t.FileName;
 			if (t.WorkspaceObject != null)
 				tmpPath = FileService.AbsoluteToRelativePath (t.WorkspaceObject.BaseDirectory, t.FileName);
@@ -604,16 +617,19 @@ namespace MonoDevelop.Ide.Gui.Pads
 			string path     = tmpPath;
 			
 			try {
-				fileName = Path.GetFileName(tmpPath);
+				fileName = Path.GetFileName (tmpPath);
 			} catch (Exception) {}
-			
-			try {
-				path = Path.GetDirectoryName(tmpPath);
-			} catch (Exception) {}
-			
+
+			if (tmpPath != null && tmpPath.Contains (Path.DirectorySeparatorChar.ToString ()))
+			{
+				try{
+					path = Path.GetDirectoryName (tmpPath);
+				}
+				catch (Exception) { }
+			}	
 			string project;
-			if (t.OwnerItem is SolutionItem)
-				project = ((SolutionItem)t.OwnerItem).Name;
+			if (t.WorkspaceObject is SolutionItem)
+				project = t.WorkspaceObject.Name;
 			else
 				project = string.Empty;
 			
@@ -632,17 +648,17 @@ namespace MonoDevelop.Ide.Gui.Pads
 
 		void UpdateErrorsNum () 
 		{
-			errorBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Error", "{0} Errors", IdeApp.Services.TaskService.ErrorsCount), IdeApp.Services.TaskService.ErrorsCount);
+			errorBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Error", "{0} Errors", errorCount), errorCount);
 		}
 
 		void UpdateWarningsNum ()
 		{
-			warnBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Warning", "{0} Warnings", IdeApp.Services.TaskService.WarningsCount), IdeApp.Services.TaskService.WarningsCount); 
+			warnBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Warning", "{0} Warnings", warningCount), warningCount); 
 		}
 
 		void UpdateMessagesNum ()
 		{
-			msgBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Message", "{0} Messages", IdeApp.Services.TaskService.MessagesCount), IdeApp.Services.TaskService.MessagesCount);
+			msgBtn.Label = " " + string.Format(GettextCatalog.GetPluralString("{0} Message", "{0} Messages", infoCount), infoCount);
 		}
 
 		private void ItemToggled (object o, ToggledArgs args)
@@ -654,48 +670,6 @@ namespace MonoDevelop.Ide.Gui.Pads
 			}
 		}
 
-		private SortType ReverseSortOrder (TreeViewColumn col) {
-			if (col.SortIndicator) {
-				if (col.SortOrder == SortType.Ascending)
-					return SortType.Descending;
-				else
-					return SortType.Ascending;
-			} else {
-				return SortType.Ascending;
-			}
-		}
-
-		private void OnResortTasks (object sender, EventArgs args)
-		{
-			TreeViewColumn col = sender as TreeViewColumn;
-			col.SortOrder = ReverseSortOrder (col);
-			col.SortIndicator = true;
-			store.SetSortColumnId ((int)Columns.Task, col.SortOrder);
-		}
-
-		private int TaskSortFunc (TreeModel model, TreeIter iter1, TreeIter iter2)
-		{
-			Task task1 = model.GetValue (iter1, (int)Columns.Task) as Task;
-			Task task2 = model.GetValue (iter2, (int)Columns.Task) as Task;
-
-			if (task1 == null && task2 == null) return 0;
-			else if (task1 == null) return -1;
-			else if (task2 == null) return 1;
-
-			int compare = ((int)task1.TaskType).CompareTo ((int)task2.TaskType);
-			if (compare != 0)
-				return compare;
-
-			if (task1.FileName != null || task2.FileName != null) {
-				if (task1.FileName == null) return -1;
-				if (task2.FileName == null) return 1;
-				compare = task1.FileName.CompareTo (task2.FileName);
-				if (compare == 0)
-					compare = task1.Line.CompareTo (task2.Line);
-			}
-			return compare;
-		}
-
 		public virtual bool GetNextLocation (out string file, out int line, out int column)
 		{
 			bool hasNext;
@@ -743,17 +717,22 @@ namespace MonoDevelop.Ide.Gui.Pads
 
 		public virtual bool GetPreviousLocation (out string file, out int line, out int column)
 		{
-			bool hasNext, hasSel;
+			bool hasNext;
 			TreeIter iter;
+			TreeModel model;
 			TreeIter selIter = TreeIter.Zero;
 			TreeIter prevIter = TreeIter.Zero;
 			
-			hasSel = !initializeLocation && view.Selection.GetSelected (out selIter);
+			TreePath selPath = null;
+			
+			if (!initializeLocation && view.Selection.GetSelected (out model, out selIter))
+				selPath = view.Model.GetPath (selIter);
+
 			hasNext = view.Model.GetIterFirst (out iter);
 			initializeLocation = false;
 			
 			while (hasNext) {
-				if (hasSel && iter.Equals (selIter))
+				if (selPath != null && view.Model.GetPath (iter).Equals (selPath))
 					break;
 				prevIter = iter;
 				hasNext = view.Model.IterNext (ref iter);
@@ -794,10 +773,10 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			if (t == null)
 				IdeApp.Workbench.StatusBar.ShowMessage (GettextCatalog.GetString ("No more errors or warnings"));
-			else if (t.TaskType == TaskType.Error)
-				IdeApp.Workbench.StatusBar.ShowError (t.ErrorNumber + " - " + t.Description);
-			else if (t.TaskType == TaskType.Warning)
-				IdeApp.Workbench.StatusBar.ShowError (t.ErrorNumber + " - " + t.Description);
+			else if (t.Severity == TaskSeverity.Error)
+				IdeApp.Workbench.StatusBar.ShowError (t.Code + " - " + t.Description);
+			else if (t.Severity == TaskSeverity.Warning)
+				IdeApp.Workbench.StatusBar.ShowError (t.Code + " - " + t.Description);
 			else
 				IdeApp.Workbench.StatusBar.ShowMessage (t.Description);
 		}
