@@ -32,10 +32,13 @@ using System.Linq;
 using Gdk;
 using Gtk;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Core.Gui;
+
 
 namespace MonoDevelop.Ide
 {
-	public partial class DocumentSwitcher : Gtk.Window
+	
+	internal partial class DocumentSwitcher : Gtk.Window
 	{
 		Gtk.ListStore padListStore;
 		Gtk.ListStore documentListStore;
@@ -59,7 +62,7 @@ namespace MonoDevelop.Ide
 			Gtk.TreeIter iter;
 			if (treeviewPads.Selection.GetSelected (out iter)) {
 				MonoDevelop.Ide.Gui.Pad pad = padListStore.GetValue (iter, 2) as MonoDevelop.Ide.Gui.Pad;
-				ShowType (MonoDevelop.Ide.Gui.IdeApp.Services.Resources.GetIcon (!string.IsNullOrEmpty (pad.Icon) ? pad.Icon : MonoDevelop.Core.Gui.Stock.MiscFiles, Gtk.IconSize.Dialog),
+				ShowType (ImageService.GetPixbuf (!string.IsNullOrEmpty (pad.Icon) ? pad.Icon : MonoDevelop.Core.Gui.Stock.MiscFiles, Gtk.IconSize.Dialog),
 				          pad.Title,
 				          "",
 				          "");
@@ -69,13 +72,11 @@ namespace MonoDevelop.Ide
 		Pixbuf GetIconForDocument (MonoDevelop.Ide.Gui.Document document, Gtk.IconSize iconSize)
 		{
 			if (!string.IsNullOrEmpty (document.Window.ViewContent.StockIconId))
-				return IdeApp.Services.Resources.GetBitmap (document.Window.ViewContent.StockIconId, iconSize);
+				return ImageService.GetPixbuf (document.Window.ViewContent.StockIconId, iconSize);
 			if (string.IsNullOrEmpty (document.FileName)) 
-				return IdeApp.Services.Resources.GetBitmap (MonoDevelop.Core.Gui.Stock.MiscFiles, iconSize);
-			string icName = MonoDevelop.Projects.Gui.Services.Icons.GetImageForFile (document.FileName);
-			if (icName != MonoDevelop.Core.Gui.Stock.MiscFiles || !System.IO.File.Exists (document.FileName))
-				return IdeApp.Services.Resources.GetIcon (icName, Gtk.IconSize.Menu);
-			return IdeApp.Services.PlatformService.GetPixbufForFile (document.FileName, iconSize);
+				return ImageService.GetPixbuf (MonoDevelop.Core.Gui.Stock.MiscFiles, iconSize);
+			
+			return DesktopService.GetPixbufForFile (document.FileName, iconSize);
 		}
 		
 		void ShowSelectedDocument ()
@@ -115,17 +116,13 @@ namespace MonoDevelop.Ide
 			treeviewPads.AppendColumn ("text", new Gtk.CellRendererText (), "text", 1);
 			treeviewPads.HeadersVisible = false;
 			
-			treeviewPads.Selection.Changed += delegate {
-				ShowSelectedPad ();
-			};
+			treeviewPads.Selection.Changed += TreeviewPadsSelectionChanged;
 			documentListStore = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (Document));
 			treeviewDocuments.Model = documentListStore;
 			treeviewDocuments.AppendColumn ("icon", new Gtk.CellRendererPixbuf (), "pixbuf", 0);
 			treeviewDocuments.AppendColumn ("text", new Gtk.CellRendererText (), "text", 1);
 			treeviewDocuments.HeadersVisible = false;
-			treeviewDocuments.Selection.Changed += delegate {
-				ShowSelectedDocument ();
-			};
+			treeviewDocuments.Selection.Changed += TreeviewDocumentsSelectionChanged;
 			
 			FillLists ();
 			this.labelFileName.Ellipsize = Pango.EllipsizeMode.Start;
@@ -136,6 +133,17 @@ namespace MonoDevelop.Ide
 				SwitchToPad ();
 			}
 		}
+
+		void TreeviewPadsSelectionChanged (object sender, EventArgs e)
+		{
+			ShowSelectedPad ();
+		}
+
+		void TreeviewDocumentsSelectionChanged (object sender, EventArgs e)
+		{
+			ShowSelectedDocument ();
+		}
+		
 		bool documentFocus = true;
 		Gtk.TreeIter selectedPadIter, selectedDocumentIter;
 		
@@ -274,7 +282,7 @@ namespace MonoDevelop.Ide
 			foreach (Pad pad in IdeApp.Workbench.Pads) {
 				if (!pad.Visible)
 					continue;
-				padListStore.AppendValues (IdeApp.Services.Resources.GetBitmap (!String.IsNullOrEmpty (pad.Icon) ? pad.Icon : MonoDevelop.Core.Gui.Stock.MiscFiles, IconSize.Menu),
+				padListStore.AppendValues (ImageService.GetPixbuf (!String.IsNullOrEmpty (pad.Icon) ? pad.Icon : MonoDevelop.Core.Gui.Stock.MiscFiles, IconSize.Menu),
 				                           pad.Title,
 				                           pad);
 			}
@@ -285,6 +293,8 @@ namespace MonoDevelop.Ide
 				                                doc);
 			}
 		}
+		
+		//FIXME: get ctrl(-shift)-tab keybindings from the Switch(Next|Previous)Document commands
 		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
 		{
 			bool next = (evnt.State & Gdk.ModifierType.ShiftMask) != ModifierType.ShiftMask;
@@ -326,16 +336,23 @@ namespace MonoDevelop.Ide
 		{
 			bool ret;
 			if (evnt.Key == Gdk.Key.Control_L || evnt.Key == Gdk.Key.Control_R) {
+				Gdk.Window focusTarget = null;
 				Document doc = SelectedDocument;
 				if (doc != null) {
 					doc.Select ();
+					focusTarget = doc.ActiveView.Control.Toplevel.GdkWindow;
 				} else {
 					Pad pad = SelectedPad;
-					if (pad != null)
-						pad.BringToFront ();
+					if (pad != null) {
+						pad.BringToFront (true);
+						focusTarget = pad.Window.Content.Control.Toplevel.GdkWindow;
+					}
 				}
 				ret = base.OnKeyReleaseEvent (evnt);
+				Gtk.Window parent = this.TransientFor;
 				this.Destroy ();
+				
+				(focusTarget ?? parent.GdkWindow).Focus (0);
 			} else {
 				ret = base.OnKeyReleaseEvent (evnt);
 			}
