@@ -34,9 +34,9 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Search;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
+using MonoDevelop.Ide.FindInFiles;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -167,6 +167,11 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		public ISearchProgressMonitor GetSearchProgressMonitor (bool bringToFront)
+		{
+			return GetSearchProgressMonitor (bringToFront, false);
+		}
+		
+		public ISearchProgressMonitor GetSearchProgressMonitor (bool bringToFront, bool focusPad)
 		{
 			Pad pad = null;
 			string title = GettextCatalog.GetString ("Search Results");
@@ -186,7 +191,7 @@ namespace MonoDevelop.Ide.Gui
 				}
 			}
 			if (pad != null) {
-				if (bringToFront) pad.BringToFront ();
+				if (bringToFront) pad.BringToFront (focusPad);
 				return new SearchProgressMonitor ((SearchResultPad) pad.Content, pad.Title);
 			}
 			
@@ -198,10 +203,11 @@ namespace MonoDevelop.Ide.Gui
 			if (instanceNum > 0)
 				title += " (" + (instanceNum+1) + ")";
 			
-			SearchResultPad monitorPad = new SearchResultPad (instanceNum);
+			SearchResultPad monitorPad = new SearchResultPad (instanceNum) { FocusPad = focusPad };
+			
 			pad = IdeApp.Workbench.ShowPad (monitorPad, newPadId, title, basePadId + "/Center Bottom", MonoDevelop.Core.Gui.Stock.FindIcon);
 			pad.Sticky = true;
-		    searchMonitors.Add (pad);
+			searchMonitors.Add (pad);
 
 			if (searchMonitors.Count > 1) {
 				// Additional search pads will be destroyed when hidden
@@ -212,7 +218,7 @@ namespace MonoDevelop.Ide.Gui
 			}
 			
 			if (bringToFront)
-			    pad.BringToFront ();
+				pad.BringToFront (focusPad);
 
 			return new SearchProgressMonitor (monitorPad, pad.Title);
 		}
