@@ -28,6 +28,7 @@
 
 
 using System;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 using System.IO;
@@ -51,7 +52,7 @@ using MonoDevelop.Core.Gui.ProgressMonitoring;
 using MonoDevelop.Core.Gui.Dialogs;
 using MonoDevelop.Ide.Tasks;
 using MonoDevelop.Ide.Gui.Dialogs;
-using Mono.Debugging.Client;
+using MonoDevelop.Core.Assemblies;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -77,6 +78,7 @@ namespace MonoDevelop.Ide.Gui
 		internal ProjectOperations ()
 		{
 			IdeApp.Workspace.WorkspaceItemUnloaded += OnWorkspaceItemUnloaded;
+			IdeApp.Workspace.ItemUnloading += IdeAppWorkspaceItemUnloading;
 		}
 		
 		public BuildResult LastCompilerResult {
@@ -212,7 +214,7 @@ namespace MonoDevelop.Ide.Gui
 			return (GetDeclaredFile(item) != null);
 		}*/
 		
-		public bool CanJumpToDeclaration (MonoDevelop.Projects.Dom.IDomVisitable visitable)
+		public bool CanJumpToDeclaration (MonoDevelop.Projects.Dom.INode visitable)
 		{
 			if (visitable is MonoDevelop.Projects.Dom.IType) 
 				return ((MonoDevelop.Projects.Dom.IType)visitable).CompilationUnit != null;
@@ -224,7 +226,7 @@ namespace MonoDevelop.Ide.Gui
 			return member.DeclaringType.CompilationUnit != null;
 		}
 
-		public void JumpToDeclaration (MonoDevelop.Projects.Dom.IDomVisitable visitable)
+		public void JumpToDeclaration (MonoDevelop.Projects.Dom.INode visitable)
 		{
 			if (visitable is LocalVariable) {
 				LocalVariable var = (LocalVariable)visitable;
@@ -239,11 +241,18 @@ namespace MonoDevelop.Ide.Gui
 				return;
 			string fileName;
 			if (member is MonoDevelop.Projects.Dom.IType) {
-				fileName = ((MonoDevelop.Projects.Dom.IType)member).CompilationUnit.FileName;
+				try {
+					fileName = ((MonoDevelop.Projects.Dom.IType)member).CompilationUnit.FileName;
+				} catch (Exception e) {
+					LoggingService.LogError ("Can't get file name for type:" + member + ". Try to restart monodevelop.", e);
+					fileName = null;
+				}
 			} else {
 				if (member.DeclaringType == null) 
 					return;
 				fileName = member.DeclaringType.CompilationUnit.FileName;
+				if (member is ExtensionMethod)
+					fileName = ((ExtensionMethod)member).OriginalMethod.DeclaringType.CompilationUnit.FileName;
 			}
 			Document doc = MonoDevelop.Ide.Gui.IdeApp.Workbench.OpenDocument (fileName, member.Location.Line, member.Location.Column, true);
 			if (doc != null) {
@@ -252,16 +261,17 @@ namespace MonoDevelop.Ide.Gui
 					handler.Open (member.HelpUrl);
 			}
 		}
-//		string GetClassFileName (IType cls)
-//		{
-//			if (cls.CompilationUnit.FileName != null)
-//				return cls.CompilationUnit.FileName;
-///*			if (cls.DeclaredIn is IType)
-//				return GetClassFileName ((IType) cls.DeclaredIn);
-//			else*/
-//				return null;
-//		}
-
+		
+		public void RenameItem (IWorkspaceFileObject item, string newName)
+		{
+			ProjectOptionsDialog.RenameItem (item, newName);
+			if (item is SolutionItem) {
+				Save (((SolutionItem)item).ParentSolution);
+			} else {
+				IdeApp.Workspace.Save ();
+				IdeApp.Workspace.SavePreferences ();
+			}
+		}
 		
 		public void Export (IWorkspaceObject item)
 		{
@@ -272,6 +282,7 @@ namespace MonoDevelop.Ide.Gui
 		{
 			ExportProjectDialog dlg = new ExportProjectDialog (entry, format);
 			try {
+				dlg.TransientFor = IdeApp.Workbench.RootWindow;
 				if (dlg.Run () == (int) Gtk.ResponseType.Ok) {
 					
 					using (IProgressMonitor mon = IdeApp.Workbench.ProgressMonitors.GetOutputProgressMonitor (GettextCatalog.GetString ("Export Project"), null, true, true)) {
@@ -288,28 +299,24 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void Save (IEnumerable<SolutionEntityItem> entries)
 		{
-			int count = 0;
-			IEnumerator<SolutionEntityItem> e = entries.GetEnumerator ();
-			while (e.MoveNext ())
-				count++;
-			
-			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (true);
-			try {
-				monitor.BeginTask (null, count);
-				foreach (SolutionEntityItem entry in entries) {
-					entry.Save (monitor);
-					monitor.Step (1);
-				}
-				monitor.ReportSuccess (GettextCatalog.GetString ("Project saved."));
-			} catch (Exception ex) {
-				monitor.ReportError (GettextCatalog.GetString ("Save failed."), ex);
-			} finally {
-				monitor.Dispose ();
-			}
+			List<IWorkspaceFileObject> items = new List<IWorkspaceFileObject> ();
+			foreach (IWorkspaceFileObject it in entries)
+				items.Add (it);
+			Save (items);
 		}
 		
 		public void Save (SolutionEntityItem entry)
 		{
+			if (!entry.FileFormat.CanWrite (entry)) {
+				IWorkspaceFileObject itemContainer = GetContainer (entry);
+				if (SelectValidFileFormat (itemContainer))
+					Save (itemContainer);
+				return;
+			}
+			
+			if (!AllowSave (entry))
+				return;
+			
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (true);
 			try {
 				entry.Save (monitor);
@@ -323,6 +330,14 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void Save (Solution item)
 		{
+			if (!item.FileFormat.CanWrite (item)) {
+				if (!SelectValidFileFormat (item))
+					return;
+			}
+			
+			if (!AllowSave (item))
+				return;
+			
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (true);
 			try {
 				item.Save (monitor);
@@ -336,16 +351,48 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void Save (IEnumerable<IWorkspaceFileObject> items)
 		{
-			int count = 0;
-			IEnumerator<IWorkspaceFileObject> e = items.GetEnumerator ();
-			while (e.MoveNext ())
-				count++;
+			int count = items.Count ();
+			if (count == 0)
+				return;
+			
+			// Verify that the file format for each item is still valid
+			
+			HashSet<IWorkspaceFileObject> fixedItems = new HashSet<IWorkspaceFileObject> ();
+			HashSet<IWorkspaceFileObject> failedItems = new HashSet<IWorkspaceFileObject> ();
+			
+			foreach (IWorkspaceFileObject entry in items) {
+				IWorkspaceFileObject itemContainer = GetContainer (entry);
+				if (fixedItems.Contains (itemContainer) || failedItems.Contains (itemContainer))
+					continue;
+				if (!entry.FileFormat.CanWrite (entry)) {
+					// Can't save the project using this format. Try to find a valid format for the whole solution
+					if (SelectValidFileFormat (itemContainer))
+						fixedItems.Add (itemContainer);
+					else
+						failedItems.Add (itemContainer);
+				}
+			}
+			if (fixedItems.Count > 0)
+				Save (fixedItems);
+			
+			if (failedItems.Count > 0 || fixedItems.Count > 0) {
+				// Some file format changes were required, and some items were saved.
+				// Get a list of items not yet saved.
+				List<IWorkspaceFileObject> notSavedEntries = new List<IWorkspaceFileObject> ();
+				foreach (IWorkspaceFileObject entry in items) {
+					IWorkspaceFileObject itemContainer = GetContainer (entry);
+					if (!fixedItems.Contains (itemContainer) && !failedItems.Contains (itemContainer))
+						notSavedEntries.Add (entry);
+				}
+				items = notSavedEntries;
+			}
 			
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (true);
 			try {
 				monitor.BeginTask (null, count);
 				foreach (IWorkspaceFileObject item in items) {
-					item.Save (monitor);
+					if (AllowSave (item))
+						item.Save (monitor);
 					monitor.Step (1);
 				}
 				monitor.EndTask ();
@@ -364,6 +411,16 @@ namespace MonoDevelop.Ide.Gui
 			else if (item is Solution)
 				Save ((Solution)item);
 			
+			if (!item.FileFormat.CanWrite (item)) {
+				IWorkspaceFileObject ci = GetContainer (item);
+				if (SelectValidFileFormat (ci))
+					Save (ci);
+				return;
+			}
+			
+			if (!AllowSave (item))
+				return;
+			
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor (true);
 			try {
 				item.Save (monitor);
@@ -375,6 +432,52 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
+		bool AllowSave (IWorkspaceFileObject item)
+		{
+			if (HasChanged (item))
+				return MessageService.Confirm (
+				    GettextCatalog.GetString ("Some project files have been changed from outside MonoDevelop. Do you want to overwrite them?"),
+				    GettextCatalog.GetString ("Changes done in those files will be overwritten by MonoDevelop."),
+				    AlertButton.OverwriteFile);
+			else
+				return true;
+		}
+		
+		bool HasChanged (IWorkspaceFileObject item)
+		{
+			if (item.ItemFilesChanged)
+				return true;
+			if (item is WorkspaceItem) {
+				foreach (SolutionEntityItem eitem in ((WorkspaceItem)item).GetAllSolutionItems<SolutionEntityItem> ())
+					if (eitem.ItemFilesChanged)
+						return true;
+			}
+			return false;
+		}
+
+		IWorkspaceFileObject GetContainer (IWorkspaceFileObject item)
+		{
+			SolutionEntityItem si = item as SolutionEntityItem;
+			if (si != null && si.ParentSolution != null && !si.ParentSolution.FileFormat.SupportsMixedFormats)
+				return si.ParentSolution;
+			else
+				return item;
+		}
+		
+		bool SelectValidFileFormat (IWorkspaceFileObject item)
+		{
+			SelectFileFormatDialog dlg = new SelectFileFormatDialog (item);
+			try {
+				if (dlg.Run () == (int) Gtk.ResponseType.Ok && dlg.Format != null) {
+					item.ConvertToFormat (dlg.Format, true);
+					return true;
+				}
+				return false;
+			} finally {
+				dlg.Destroy ();
+			}
+		}
+		
 		public void MarkFileDirty (string filename)
 		{
 			Project entry = IdeApp.Workspace.GetProjectContainingFile (filename);
@@ -394,7 +497,7 @@ namespace MonoDevelop.Ide.Gui
 				SolutionEntityItem selectedProject = (SolutionEntityItem) entry;
 				
 				ProjectOptionsDialog optionsDialog = new ProjectOptionsDialog (IdeApp.Workbench.RootWindow, selectedProject);
-				SolutionItemConfiguration conf = selectedProject.GetActiveConfiguration (IdeApp.Workspace.ActiveConfiguration);
+				SolutionItemConfiguration conf = selectedProject.GetConfiguration (IdeApp.Workspace.ActiveConfiguration);
 				optionsDialog.CurrentConfig = conf != null ? conf.Name : null;
 				try {
 					if (panelId != null)
@@ -409,6 +512,7 @@ namespace MonoDevelop.Ide.Gui
 							}
 						}
 						Save (selectedProject);
+						IdeApp.Workspace.SavePreferences ();
 					}
 				} finally {
 					optionsDialog.Destroy ();
@@ -417,12 +521,14 @@ namespace MonoDevelop.Ide.Gui
 				Solution solution = (Solution) entry;
 				
 				CombineOptionsDialog optionsDialog = new CombineOptionsDialog (IdeApp.Workbench.RootWindow, solution);
-				optionsDialog.CurrentConfig = IdeApp.Workspace.ActiveConfiguration;
+				optionsDialog.CurrentConfig = IdeApp.Workspace.ActiveConfigurationId;
 				try {
 					if (panelId != null)
 						optionsDialog.SelectPanel (panelId);
-					if (optionsDialog.Run () == (int) Gtk.ResponseType.Ok)
-						IdeApp.Workspace.Save ();
+					if (optionsDialog.Run () == (int) Gtk.ResponseType.Ok) {
+						Save (solution);
+						IdeApp.Workspace.SavePreferences (solution);
+					}
 				} finally {
 					optionsDialog.Destroy ();
 				}
@@ -442,6 +548,7 @@ namespace MonoDevelop.Ide.Gui
 							if (si.ParentSolution != null)
 								Save (si.ParentSolution);
 						}
+						IdeApp.Workspace.SavePreferences ();
 					}
 				} finally {
 					optionsDialog.Destroy ();
@@ -450,8 +557,15 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		public void NewSolution ()
+		{
+			NewSolution (null);
+		}
+		
+		public void NewSolution (string defaultTemplate)
 		{
 			NewProjectDialog pd = new NewProjectDialog (null, true, null);
+			if (defaultTemplate != null)
+				pd.SelectTemplate (defaultTemplate);
 			pd.Run ();
 			pd.Destroy ();
 		}
@@ -610,13 +724,13 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		public bool SelectProjectReferences (ProjectReferenceCollection references, TargetFramework targetVersion)
+		public bool SelectProjectReferences (ProjectReferenceCollection references, AssemblyContext ctx, TargetFramework targetVersion)
 		{
 			try {
 				if (selDialog == null)
 					selDialog = new SelectReferenceDialog ();
 				
-				selDialog.SetReferenceCollection (references, targetVersion);
+				selDialog.SetReferenceCollection (references, ctx, targetVersion);
 
 				if (selDialog.Run() == (int)Gtk.ResponseType.Ok) {
 					references.Clear ();
@@ -630,14 +744,75 @@ namespace MonoDevelop.Ide.Gui
 					selDialog.Hide ();
 			}
 		}
-
 		
+		public void RemoveSolutionItem (SolutionItem item)
+		{
+			string question = GettextCatalog.GetString ("Do you really want to remove project '{0}' from '{1}'?", item.Name, item.ParentFolder.Name);
+			string secondaryText = GettextCatalog.GetString ("The Delete option physically removes the project files from disc.");
+			
+			SolutionEntityItem prj = item as SolutionEntityItem;
+			if (prj == null) {
+				if (MessageService.Confirm (question, AlertButton.Remove) && IdeApp.Workspace.RequestItemUnload (item))
+					RemoveItemFromSolution (prj);
+				return;
+			}
+			
+			AlertButton result = MessageService.AskQuestion (question, secondaryText,
+			                                                 AlertButton.Delete, AlertButton.Cancel, AlertButton.Remove);
+			if (result == AlertButton.Delete) {
+				if (!IdeApp.Workspace.RequestItemUnload (prj))
+					return;
+				ConfirmProjectDeleteDialog dlg = new ConfirmProjectDeleteDialog (prj);
+				if (dlg.Run () == (int) Gtk.ResponseType.Ok) {
+					
+					// Remove the project before removing the files to avoid unnecessary events
+					RemoveItemFromSolution (prj);
+					
+					List<FilePath> files = dlg.GetFilesToDelete ();
+					dlg.Destroy ();
+					using (IProgressMonitor monitor = new MonoDevelop.Core.Gui.ProgressMonitoring.MessageDialogProgressMonitor (true)) {
+						monitor.BeginTask (GettextCatalog.GetString ("Deleting Files..."), files.Count);
+						foreach (FilePath file in files) {
+							try {
+								if (Directory.Exists (file))
+									FileService.DeleteDirectory (file);
+								else
+									FileService.DeleteFile (file);
+							} catch (Exception ex) {
+								monitor.ReportError (GettextCatalog.GetString ("The file or directory '{0}' could not be deleted.", file), ex);
+							}
+							monitor.Step (1);
+						}
+						monitor.EndTask ();
+					}
+				} else
+					dlg.Destroy ();
+			}
+			else if (result == AlertButton.Remove && IdeApp.Workspace.RequestItemUnload (prj)) {
+				RemoveItemFromSolution (prj);
+			}
+		}
+		
+		void RemoveItemFromSolution (SolutionItem prj)
+		{
+			Solution sol = prj.ParentSolution;
+			prj.ParentFolder.Items.Remove (prj);
+			prj.Dispose ();
+			IdeApp.ProjectOperations.Save (sol);
+		}
+
 		public bool CanExecute (IBuildTarget entry)
 		{
-			ExecutionContext context = new ExecutionContext (new DefaultExecutionHandlerFactory (), IdeApp.Workbench.ProgressMonitors);
+			ExecutionContext context = new ExecutionContext (Runtime.ProcessService.DefaultExecutionHandler, IdeApp.Workbench.ProgressMonitors);
 			return CanExecute (entry, context);
 		}
 		
+		public bool CanExecute (IBuildTarget entry, IExecutionHandler handler)
+		{
+			ExecutionContext context = new ExecutionContext (handler, IdeApp.Workbench.ProgressMonitors);
+			return entry.CanExecute (context, IdeApp.Workspace.ActiveConfiguration);
+		}
+		
 		public bool CanExecute (IBuildTarget entry, ExecutionContext context)
 		{
 			return entry.CanExecute (context, IdeApp.Workspace.ActiveConfiguration);
@@ -645,7 +820,12 @@ namespace MonoDevelop.Ide.Gui
 		
 		public IAsyncOperation Execute (IBuildTarget entry)
 		{
-			ExecutionContext context = new ExecutionContext (new DefaultExecutionHandlerFactory (), IdeApp.Workbench.ProgressMonitors);
+			return Execute (entry, Runtime.ProcessService.DefaultExecutionHandler);
+		}
+		
+		public IAsyncOperation Execute (IBuildTarget entry, IExecutionHandler handler)
+		{
+			ExecutionContext context = new ExecutionContext (handler, IdeApp.Workbench.ProgressMonitors);
 			return Execute (entry, context);
 		}
 		
@@ -709,7 +889,12 @@ namespace MonoDevelop.Ide.Gui
 		
 		public bool CanExecuteFile (string file)
 		{
-			ExecutionContext context = new ExecutionContext (new DefaultExecutionHandlerFactory (), IdeApp.Workbench.ProgressMonitors);
+			return CanExecuteFile (file, Runtime.ProcessService.DefaultExecutionHandler);
+		}
+		
+		public bool CanExecuteFile (string file, IExecutionHandler handler)
+		{
+			ExecutionContext context = new ExecutionContext (handler, IdeApp.Workbench.ProgressMonitors);
 			return CanExecuteFile (file, context);
 		}
 		
@@ -725,6 +910,12 @@ namespace MonoDevelop.Ide.Gui
 				return false;
 		}
 		
+		public IAsyncOperation ExecuteFile (string file, IExecutionHandler handler)
+		{
+			ExecutionContext context = new ExecutionContext (handler, IdeApp.Workbench.ProgressMonitors);
+			return ExecuteFile (file, context);
+		}
+		
 		public IAsyncOperation ExecuteFile (string file, ExecutionContext context)
 		{
 			Project tempProject = projectService.CreateSingleFileProject (file);
@@ -791,7 +982,7 @@ namespace MonoDevelop.Ide.Gui
 					break;
 				case BeforeCompileAction.PromptForSave:
 					foreach (Document doc in IdeApp.Workbench.Documents) {
-						if (doc.IsDirty) {
+						if (doc.IsDirty && doc.Project != null) {
 							if (MessageService.AskQuestion (
 						            GettextCatalog.GetString ("Save changed documents before building?"),
 							        GettextCatalog.GetString ("Some of the open documents have unsaved changes."),
@@ -805,7 +996,9 @@ namespace MonoDevelop.Ide.Gui
 					}
 					break;
 				case BeforeCompileAction.SaveAllFiles:
-					IdeApp.Workbench.SaveAll ();
+					foreach (Document doc in new List<Document> (IdeApp.Workbench.Documents))
+						if (doc.IsDirty && doc.Project != null)
+							doc.Save ();
 					break;
 				default:
 					System.Diagnostics.Debug.Assert(false);
@@ -815,7 +1008,7 @@ namespace MonoDevelop.Ide.Gui
 
 		void BeginBuild (IProgressMonitor monitor)
 		{
-			Services.TaskService.ClearExceptCommentTasks ();
+			TaskService.Errors.ClearByOwner (this);
 			if (StartBuild != null)
 				StartBuild (this, new BuildEventArgs (monitor, true));
 		}
@@ -831,10 +1024,12 @@ namespace MonoDevelop.Ide.Gui
 					monitor.Log.WriteLine (GettextCatalog.GetString ("---------------------- Done ----------------------"));
 					
 					tasks = new Task [result.Errors.Count];
-					for (int n=0; n<tasks.Length; n++)
+					for (int n=0; n<tasks.Length; n++) {
 						tasks [n] = new Task (result.Errors [n]);
+						tasks [n].Owner = this;
+					}
 
-					Services.TaskService.AddRange (tasks);
+					TaskService.Errors.AddRange (tasks);
 					
 					string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", result.ErrorCount, result.ErrorCount);
 					string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", result.WarningCount, result.WarningCount);
@@ -963,26 +1158,28 @@ namespace MonoDevelop.Ide.Gui
 			return newfilename;
 		}		
 
-		public void TransferFiles (IProgressMonitor monitor, Project sourceProject, string sourcePath, Project targetProject, string targetPath, bool removeFromSource, bool copyOnlyProjectFiles)
+		public void TransferFiles (IProgressMonitor monitor, Project sourceProject, FilePath sourcePath, Project targetProject, FilePath targetPath, bool removeFromSource, bool copyOnlyProjectFiles)
 		{
 			// When transfering directories, targetPath is the directory where the source
-			// directory will be transfered, not including the destination directory name.
+			// directory will be transfered, including the destination directory or file name.
 			// For example, if sourcePath is /a1/a2/a3 and targetPath is /b1/b2, the
-			// new folder will be /b1/b2/a3
+			// new folder or file will be /b1/b2
 			
 			if (targetProject == null)
 				throw new ArgumentNullException ("targetProject");
 
-			if (!targetPath.StartsWith (targetProject.BaseDirectory))
+			if (!targetPath.IsChildPathOf (targetProject.BaseDirectory))
 				throw new ArgumentException ("Invalid project folder: " + targetPath);
 
-			if (sourceProject != null && !sourcePath.StartsWith (sourceProject.BaseDirectory))
+			if (sourceProject != null && !sourcePath.IsChildPathOf (sourceProject.BaseDirectory))
 				throw new ArgumentException ("Invalid project folder: " + sourcePath);
 				
 			if (copyOnlyProjectFiles && sourceProject == null)
 				throw new ArgumentException ("A source project must be specified if copyOnlyProjectFiles is True");
+			
+			bool sourceIsFolder = Directory.Exists (sourcePath);
 
-			bool movingFolder = (removeFromSource && Directory.Exists (sourcePath) && (
+			bool movingFolder = (removeFromSource && sourceIsFolder && (
 					!copyOnlyProjectFiles ||
 					IsDirectoryHierarchyEmpty (sourcePath)));
 
@@ -1024,10 +1221,9 @@ namespace MonoDevelop.Ide.Gui
 			// Ensure that the destination folder is created, even if no files
 			// are copied
 			
-			string newFolder = Path.Combine (targetPath, Path.GetFileName (sourcePath));
 			try {
-				if (Directory.Exists (sourcePath) && !Directory.Exists (newFolder) && !movingFolder)
-					FileService.CreateDirectory (newFolder);
+				if (sourceIsFolder && !Directory.Exists (targetPath) && !movingFolder)
+					FileService.CreateDirectory (targetPath);
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Could not create directory '{0}'.", targetPath), ex);
 				return;
@@ -1047,25 +1243,24 @@ namespace MonoDevelop.Ide.Gui
 			
 			if (movingFolder) {
 				try {
-					FileService.MoveDirectory (sourcePath, newFolder);
+					FileService.MoveDirectory (sourcePath, targetPath);
 				} catch (Exception ex) {
 					monitor.ReportError (GettextCatalog.GetString ("Directory '{0}' could not be moved.", sourcePath), ex);
 					return;
 				}
 			}
 
-			string basePath = Path.GetDirectoryName (sourcePath);
 			monitor.BeginTask (GettextCatalog.GetString ("Copying files..."), filesToMove.Count);
 			
 			foreach (ProjectFile file in filesToMove) {
-				string sourceFile = file.Name;
-				string newFile = targetPath + sourceFile.Substring (basePath.Length);
+				FilePath sourceFile = file.FilePath;
+				FilePath newFile = sourceIsFolder ? targetPath.Combine (sourceFile.ToRelative (sourcePath)) : targetPath;
 				
 				ProjectFile oldProjectFile = oldProjectFiles != null ? oldProjectFiles.GetFile (sourceFile) : null;
 				
 				if (!movingFolder) {
 					try {
-						string fileDir = Path.GetDirectoryName (newFile);
+						FilePath fileDir = newFile.ParentDirectory;
 						if (!Directory.Exists (fileDir))
 							FileService.CreateDirectory (fileDir);
 						if (removeFromSource)
@@ -1130,6 +1325,18 @@ namespace MonoDevelop.Ide.Gui
 				EndBuild (this, new BuildEventArgs (monitor, success));
 			}
 		}
+
+		void IdeAppWorkspaceItemUnloading (object sender, ItemUnloadingEventArgs args)
+		{
+			if (IsBuilding (args.Item))
+				CurrentBuildOperation.Cancel ();
+			if (IsRunning (args.Item)) {
+				if (MessageService.Confirm (GettextCatalog.GetString ("The project '{0}' is currently running. It will have to be stopped. Do you want to continue?", currentRunOperationOwner.Name), AlertButton.Yes)) {
+					CurrentRunOperation.Cancel ();
+				} else
+					args.Cancel = true;
+			}
+		}
 		
 		void OnWorkspaceItemUnloaded (object s, WorkspaceItemEventArgs args)
 		{
@@ -1139,10 +1346,6 @@ namespace MonoDevelop.Ide.Gui
 				CurrentSelectedWorkspaceItem = null;
 			if ((currentItem is IBuildTarget) && ContainsTarget (args.Item, ((IBuildTarget)currentItem)))
 				CurrentSelectedItem = null;
-			if (IsBuilding (args.Item))
-				CurrentBuildOperation.Cancel ();
-			if (IsRunning (args.Item))
-				CurrentRunOperation.Cancel ();
 		}
 		
 		protected virtual void OnCurrentSelectedSolutionChanged(SolutionEventArgs e)
@@ -1177,13 +1380,13 @@ namespace MonoDevelop.Ide.Gui
 	{
 		public IProgressMonitor CreateProgressMonitor ()
 		{
-			return new BackgroundProgressMonitor ("Code Completion Database Generation", "md-parser");
+			return new BackgroundProgressMonitor (GettextCatalog.GetString ("Code completion database generation"), "md-parser");
 		}
 	}
 	
 	class OpenDocumentFileProvider: ITextFileProvider
 	{
-		public IEditableTextFile GetEditableTextFile (string filePath)
+		public IEditableTextFile GetEditableTextFile (FilePath filePath)
 		{
 			foreach (Document doc in IdeApp.Workbench.Documents) {
 				if (doc.FileName == filePath) {
