@@ -1,21 +1,26 @@
 //  SdiWorkspaceLayout.cs
 //
-//  This file was derived from a file from #Develop. 
+// Author:
+//   Mike Krüger
+//   Lluis Sanchez Gual
+//
+//  This file was derived from a file from #Develop 2.0
 //
 //  Copyright (C) 2001-2007 Mike Krüger <mkrueger@novell.com>
+//  Copyright (C) 2006 Novell, Inc (http://www.novell.com)
 // 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+//  This library is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU Lesser General Public License as
+//  published by the Free Software Foundation; either version 2.1 of the
+//  License, or (at your option) any later version.
 // 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
+//  This library is distributed in the hope that it will be useful, but
+//  WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+//  Lesser General Public License for more details.
+// 
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
 using System;
@@ -25,7 +30,7 @@ using System.Collections.Generic;
 using System.Drawing;
 using System.Xml;
 using System.Xml.Serialization;
-
+using System.Linq;
 using MonoDevelop.Core;
 
 using Gtk;
@@ -81,6 +86,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		bool initialized;
 		IWorkbenchWindow lastActive;
+		const int MAX_LASTACTIVEWINDOWS = 10;
+		LinkedList<IWorkbenchWindow> lastActiveWindows = new LinkedList<IWorkbenchWindow> ();
 		bool ignorePageSwitch;
 		bool isInFullViewMode = false;
 
@@ -111,8 +118,8 @@ namespace MonoDevelop.Ide.Gui
 			wbWindow = (Window) workbench;
 			
 			rootWidget = fullViewVBox;
-
-			fullViewVBox.PackStart (workbench.TopMenu, false, false, 0);
+			
+			InstallMenuBar ();
 			
 			toolbarFrame = new CommandFrame (IdeApp.CommandService);
 			fullViewVBox.PackStart (toolbarFrame, true, true, 0);
@@ -124,7 +131,6 @@ namespace MonoDevelop.Ide.Gui
 			}
 			
 			toolBars = workbench.ToolBars;
-			menubar = workbench.TopMenu;
 			
 			// Create the docking widget and add it to the window.
 			dock = new DockFrame ();
@@ -134,7 +140,9 @@ namespace MonoDevelop.Ide.Gui
 			tabControl = new SdiDragNotebook ();
 			tabControl.Scrollable = true;
 			tabControl.SwitchPage += new SwitchPageHandler (ActiveMdiChanged);
-			
+			tabControl.PageAdded += delegate { ActiveMdiChanged (null, null); };
+			tabControl.PageRemoved += delegate { ActiveMdiChanged (null, null); };
+		
 			tabControl.ButtonPressEvent += delegate(object sender, ButtonPressEventArgs e) {
 				int tab = tabControl.FindTabAtPosition (e.Event.XRoot, e.Event.YRoot);
 				if (tab < 0)
@@ -187,10 +195,8 @@ namespace MonoDevelop.Ide.Gui
 			dit.DefaultVisible = false;
 
 			workbench.Add (fullViewVBox);
-			
-			fullViewVBox.PackEnd (IdeApp.Workbench.StatusBar.Control, false, true, 0);
 			fullViewVBox.ShowAll ();
-			IdeApp.Workbench.StatusBar.Control.ShowAll ();
+			fullViewVBox.PackEnd (IdeApp.Workbench.StatusBar, false, true, 0);
 			
 			foreach (IViewContent content in workbench.ViewContentCollection)
 				ShowView (content);
@@ -211,6 +217,11 @@ namespace MonoDevelop.Ide.Gui
 			workbench.ContextChanged += contextChangedHandler;
 		}
 		
+		public void ResetToolbars ()
+		{
+			toolbarFrame.ResetToolbarPositions ();
+		}
+		
 		void ShowPopup ()
 		{
 			Gtk.Menu contextMenu = IdeApp.CommandService.CreateMenu ("/MonoDevelop/Ide/ContextMenu/DocumentTab");
@@ -237,26 +248,20 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		public ICustomXmlSerializer CreateMemento()
-		{
-			return new SdiWorkbenchLayoutMemento (initialized ? toolbarFrame.GetStatus () : new DockToolbarFrameStatus ()).ToProperties ();
-		}
-		
-		public void SetMemento (ICustomXmlSerializer memento)
-		{
-			initialized = true;
-			SdiWorkbenchLayoutMemento m = new SdiWorkbenchLayoutMemento ((Properties)memento);
-			toolbarFrame.SetStatus (m.Status);
+		public ICustomXmlSerializer Memento {
+			get {
+				return new SdiWorkbenchLayoutMemento (initialized ? toolbarFrame.GetStatus () : new DockToolbarFrameStatus ()).ToProperties ();
+			}
+			set {
+				initialized = true;
+				SdiWorkbenchLayoutMemento m = new SdiWorkbenchLayoutMemento ((Properties)value);
+				toolbarFrame.SetStatus (m.Status);
+			}
 		}
 		
 		void OnTabsReordered (Widget widget, int oldPlacement, int newPlacement)
 		{
-			lock (workbench.ViewContentCollection) {
-				IViewContent content = workbench.ViewContentCollection[oldPlacement];
-				workbench.InternalViewContentCollection.RemoveAt (oldPlacement);
-				workbench.InternalViewContentCollection.Insert (newPlacement, content);
-				
-			}
+			IdeApp.Workbench.ReorderDocuments (oldPlacement, newPlacement);
 		}
 
 		void OnContextChanged (object o, EventArgs e)
@@ -395,7 +400,7 @@ namespace MonoDevelop.Ide.Gui
 						if (it != null)
 							it.Visible = true;
 					}
-					it.Present ();
+					it.Present (false);
 				}
 			} catch (Exception ex) {
 				LoggingService.LogError (ex.ToString ());
@@ -428,17 +433,18 @@ namespace MonoDevelop.Ide.Gui
 			workbench.ContextChanged -= contextChangedHandler;
 
 			dock.SaveLayouts (configFile);
-			rootWidget.Remove(((DefaultWorkbench)workbench).TopMenu);
+			UninstallMenuBar ();
 			wbWindow.Remove(rootWidget);
 			activePadCollection = null;
 		}
 		
 		void CreatePadContent (bool force, PadCodon padCodon, PadWindow window, DockItem item)
 		{
+
 			if (force || item.Content == null) {
 				IPadContent newContent = padCodon.PadContent;
 				newContent.Initialize (window);
-			
+
 				Gtk.Widget pcontent;
 				if (newContent is Widget) {
 					pcontent = newContent.Control;
@@ -498,6 +504,13 @@ namespace MonoDevelop.Ide.Gui
 					window.NotifyHidden ();
 			};
 			
+			item.ContentVisibleChanged += delegate {
+				if (item.ContentVisible)
+					window.NotifyContentShown ();
+				else
+					window.NotifyContentHidden ();
+			};
+			
 			if (!activePadCollection.Contains (padCodon))
 				activePadCollection.Add (padCodon);
 		}
@@ -527,6 +540,13 @@ namespace MonoDevelop.Ide.Gui
 				AddPad (content, content.DefaultPlacement);
 		}
 		
+		public void AddPad (PadCodon content)
+		{
+			DockItem item = GetDockItem (content);
+			if (item == null)
+				AddPad (content, content.DefaultPlacement);
+		}
+		
 		public void RemovePad (PadCodon content)
 		{
 			PadWindow win = (PadWindow) padWindows [content];
@@ -540,6 +560,8 @@ namespace MonoDevelop.Ide.Gui
 			
 			foreach (List<PadCodon> pads in padCollections.Values) 
 				pads.Remove (content);
+			
+			Counters.PadsLoaded--;
 		}
 		
 		public bool IsVisible (PadCodon padContent)
@@ -550,6 +572,14 @@ namespace MonoDevelop.Ide.Gui
 			return false;
 		}
 		
+		public bool IsContentVisible (PadCodon padContent)
+		{
+			DockItem item = GetDockItem (padContent);
+			if (item != null)
+				return item.ContentVisible;
+			return false;
+		}
+		
 		public void HidePad (PadCodon padContent)
 		{
 			DockItem item = GetDockItem (padContent);
@@ -557,11 +587,28 @@ namespace MonoDevelop.Ide.Gui
 				item.Visible = false;
 		}
 		
-		public void ActivatePad (PadCodon padContent)
+		public void ActivatePad (PadCodon padContent, bool giveFocus)
 		{
 			DockItem item = GetDockItem (padContent);
 			if (item != null)
-				item.Present ();
+				item.Present (giveFocus);
+		}
+		
+		public bool IsSticky (PadCodon padContent)
+		{
+			DockItem item = GetDockItem (padContent);
+			return item != null && (item.Behavior & DockItemBehavior.Sticky) != 0;
+		}
+
+		public void SetSticky (PadCodon padContent, bool sticky)
+		{
+			DockItem item = GetDockItem (padContent);
+			if (item != null) {
+				if (sticky)
+					item.Behavior |= DockItemBehavior.Sticky;
+				else
+					item.Behavior &= ~DockItemBehavior.Sticky;
+			}
 		}
 		
 		public void RedrawAllComponents()
@@ -584,17 +631,35 @@ namespace MonoDevelop.Ide.Gui
 				toolbarFrame.CurrentLayout = cl;
 			}
 
-			if (wb.TopMenu != menubar) {
-				Gtk.Box parent = (Gtk.Box) menubar.Parent;
-				int pos = ((Gtk.Box.BoxChild) parent [menubar]).Position;
-				
-				parent.PackStart (wb.TopMenu, false, false, 0);
-				((Gtk.Box.BoxChild) parent [wb.TopMenu]).Position = pos;
+			InstallMenuBar ();
+		}
+		
+		void InstallMenuBar ()
+		{
+			DefaultWorkbench wb = (DefaultWorkbench) workbench;
+			
+			if (wb.TopMenu == menubar)
+				return;
+			
+			if (menubar != null)
+				rootWidget.Remove (menubar);
+			
+			if (wb.TopMenu != null) {
+				((VBox)rootWidget).PackStart (wb.TopMenu, false, false, 0);
+				((Gtk.Box.BoxChild) rootWidget [wb.TopMenu]).Position = 0;
 				wb.TopMenu.ShowAll ();
-				
-				parent.Remove (menubar);
-				menubar = wb.TopMenu;
 			}
+			
+			menubar = wb.TopMenu;
+		}
+		
+		void UninstallMenuBar ()
+		{
+			if (menubar == null)
+				return;
+			
+			rootWidget.Remove(((DefaultWorkbench)workbench).TopMenu);
+			menubar = null;
 		}
 		
 		public IPadWindow GetPadWindow (PadCodon content)
@@ -602,17 +667,35 @@ namespace MonoDevelop.Ide.Gui
 			return padWindows [content];
 		}
 		
-		public void CloseWindowEvent (object sender, EventArgs e)
+		bool SelectLastActiveWindow (IWorkbenchWindow cur)
+		{
+			if (lastActiveWindows.Count == 0)
+				return false;
+			IWorkbenchWindow last = null;
+			do {
+				last = lastActiveWindows.Last.Value;
+				lastActiveWindows.RemoveLast ();
+			} while (lastActiveWindows.Count > 0 && (last == cur || last == null || (last != null && last.ViewContent == null)));
+			if (last != null) {
+				last.SelectWindow ();
+				return true;
+			}
+			return false;
+		}
+		
+		public void CloseWindowEvent (object sender, WorkbenchWindowEventArgs e)
 		{
 			SdiWorkspaceWindow f = (SdiWorkspaceWindow) sender;
 			
 			// Unsubscribe events to avoid memory leaks
-			f.TabLabel.CloseClicked -= new EventHandler (closeClicked);			
-
+			f.TabLabel.CloseClicked -= new EventHandler (closeClicked);
+			
 			if (f.ViewContent != null) {
 				((IWorkbench)wbWindow).CloseContent (f.ViewContent);
-				ActiveMdiChanged(this, null);
+				if (e.WasActive && !SelectLastActiveWindow (f))
+					ActiveMdiChanged(this, null);
 			}
+			lastActiveWindows.Remove (f);
 		}
 		
 		public IWorkbenchWindow ShowView (IViewContent content)
@@ -623,19 +706,18 @@ namespace MonoDevelop.Ide.Gui
 				mimeimage = new Gtk.Image (content.StockIconId, IconSize.Menu );
 			}
 			else if (content.IsUntitled && content.UntitledName == null) {
-				mimeimage = new Gtk.Image (IdeApp.Services.PlatformService.GetPixbufForType ("gnome-fs-regular", 16));
+				mimeimage = new Gtk.Image (DesktopService.GetPixbufForType ("gnome-fs-regular", Gtk.IconSize.Menu));
 			} else {
-				mimeimage = new Gtk.Image (IdeApp.Services.PlatformService.GetPixbufForFile (content.ContentName, 16));
+				mimeimage = new Gtk.Image (DesktopService.GetPixbufForFile (content.ContentName ?? content.UntitledName, Gtk.IconSize.Menu));
 			}			
 
 			TabLabel tabLabel = new TabLabel (new Label (), mimeimage != null ? mimeimage : new Gtk.Image (""));
 			tabLabel.CloseClicked += new EventHandler (closeClicked);			
 			tabLabel.ClearFlag (WidgetFlags.CanFocus);
 			SdiWorkspaceWindow sdiWorkspaceWindow = new SdiWorkspaceWindow (workbench, content, tabControl, tabLabel);
-
-			sdiWorkspaceWindow.Closed += new EventHandler (CloseWindowEvent);
+			sdiWorkspaceWindow.TitleChanged += delegate { SetWorkbenchTitle (); };
+			sdiWorkspaceWindow.Closed += CloseWindowEvent;
 			tabControl.InsertPage (sdiWorkspaceWindow, tabLabel, -1);
-			
 			tabLabel.Show ();
 			return sdiWorkspaceWindow;
 		}
@@ -659,6 +741,7 @@ namespace MonoDevelop.Ide.Gui
 				ignorePageSwitch = true;
 				IWorkbenchWindow w = ActiveWorkbenchwindow;
 				tabControl.RemovePage (pageNum);
+				ignorePageSwitch = false;
 				if (w != ActiveWorkbenchwindow)
 					ActiveMdiChanged (null, null);
 			} finally {
@@ -686,43 +769,56 @@ namespace MonoDevelop.Ide.Gui
 		{
 			if (ignorePageSwitch)
 				return;
-
+			
 			if (lastActive == ActiveWorkbenchwindow)
 				return;
-				
+			
+			if (lastActiveWindows.Count > MAX_LASTACTIVEWINDOWS)
+				lastActiveWindows.RemoveFirst ();
+			lastActiveWindows.AddLast (lastActive);
 			lastActive = ActiveWorkbenchwindow;
-
-			try {
-				if (ActiveWorkbenchwindow != null) {
-					if (ActiveWorkbenchwindow.ViewContent.IsUntitled) {
-						((Gtk.Window)workbench).Title = "MonoDevelop";
-					} else {
-						string post = String.Empty;
-						if (ActiveWorkbenchwindow.ViewContent.IsDirty) {
-							post = "*";
-						}
-						if (ActiveWorkbenchwindow.ViewContent.Project != null)
-						{
-							((Gtk.Window)workbench).Title = ActiveWorkbenchwindow.ViewContent.Project.Name + " - " + ActiveWorkbenchwindow.ViewContent.PathRelativeToProject + post + " - MonoDevelop";
-						}
-						else
-						{
-							((Gtk.Window)workbench).Title = ActiveWorkbenchwindow.ViewContent.ContentName + post + " - MonoDevelop";
-						}
-					}
-				} else {
-					((Gtk.Window)workbench).Title = "MonoDevelop";
-					if (isInFullViewMode) 
-						this.ToggleFullViewMode ();
-				}
-			} catch {
-				((Gtk.Window)workbench).Title = "MonoDevelop";
-			}
+			SetWorkbenchTitle ();
 			if (ActiveWorkbenchWindowChanged != null) {
 				ActiveWorkbenchWindowChanged(this, e);
 			}
 		}
+
+		public void SetWorkbenchTitle ()
+		{
+			try {
+				IWorkbenchWindow window = ActiveWorkbenchwindow;
+				if (window != null) {
+					if (window.ViewContent.IsUntitled) {
+						SetDefaultTitle ();
+					} else {
+						string post = String.Empty;
+						if (window.ViewContent.IsDirty) {
+							post = "*";
+						}
+						if (window.ViewContent.Project != null) {
+							((Gtk.Window)workbench).Title = window.ViewContent.Project.Name + " - " + window.ViewContent.PathRelativeToProject + post + " - MonoDevelop";
+						} else {
+							((Gtk.Window)workbench).Title = window.ViewContent.ContentName + post + " - MonoDevelop";
+						}
+					}
+				} else {
+					SetDefaultTitle ();
+					if (isInFullViewMode)
+						this.ToggleFullViewMode ();
+				}
+			} catch (Exception) {
+				SetDefaultTitle ();
+			}
+		}
 		
+		void SetDefaultTitle ()
+		{
+			if (IdeApp.ProjectOperations.CurrentSelectedProject != null) {
+				((Gtk.Window)workbench).Title = IdeApp.ProjectOperations.CurrentSelectedProject.Name + " - MonoDevelop";
+			} else {
+				((Gtk.Window)workbench).Title = "MonoDevelop";
+			}
+		}
 		public event EventHandler ActiveWorkbenchWindowChanged;
 		
 		
@@ -788,5 +884,33 @@ namespace MonoDevelop.Ide.Gui
 			SdiWorkspaceWindow win = (SdiWorkspaceWindow) CurrentPageWidget;
 			return win != null ? win.CommandHandler : null;
 		}
+		
+		protected override bool OnExposeEvent (Gdk.EventExpose evnt)
+		{
+			bool res = base.OnExposeEvent (evnt);
+			if (Children.Length == 0) {
+				Gdk.Rectangle rect = Allocation;
+				using (Cairo.Context cr = Gdk.CairoHelper.Create (evnt.Window)) {
+					cr.NewPath ();
+					cr.MoveTo (rect.X, rect.Y);
+					cr.RelLineTo (rect.Width, 0);
+					cr.RelLineTo (0, rect.Height);
+					cr.RelLineTo (-rect.Width, 0);
+					cr.RelLineTo (0, -rect.Height);
+					cr.ClosePath ();
+					Cairo.Gradient pat = new Cairo.LinearGradient (rect.X, rect.Y, rect.X, rect.Y + rect.Height);
+					Gdk.Color gdkcol = Style.Mid (StateType.Normal);
+					Cairo.Color color1 = new Cairo.Color (gdkcol.Red / (double) ushort.MaxValue, gdkcol.Green / (double) ushort.MaxValue, gdkcol.Blue / (double) ushort.MaxValue);
+					color1.A = 0;
+					pat.AddColorStop (0, color1);
+					color1.A = 0.6;
+					pat.AddColorStop (1, color1);
+					cr.Pattern = pat;
+					cr.FillPreserve ();
+				}
+			}
+			return res;
+		}
+
 	}
 }
