@@ -1,25 +1,32 @@
 //  SdiWorkspaceWindow.cs
 //
-//  This file was derived from a file from #Develop. 
+// Author:
+//   Mike Krüger
+//   Lluis Sanchez Gual
+//
+//  This file was derived from a file from #Develop 2.0
 //
 //  Copyright (C) 2001-2007 Mike Krüger <mkrueger@novell.com>
+//  Copyright (C) 2006 Novell, Inc (http://www.novell.com)
 // 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+//  This library is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU Lesser General Public License as
+//  published by the Free Software Foundation; either version 2.1 of the
+//  License, or (at your option) any later version.
 // 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
+//  This library is distributed in the hope that it will be useful, but
+//  WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+//  Lesser General Public License for more details.
+// 
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 using System;
+using System.Linq;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using Gtk;
 
@@ -103,6 +110,11 @@ namespace MonoDevelop.Ide.Gui
 			get { return tabLabel; }
 		}
 
+		public Document Document {
+			get;
+			set;
+		}
+		
 		public bool ShowNotification {
 			get {
 				return show_notification;
@@ -127,7 +139,6 @@ namespace MonoDevelop.Ide.Gui
 				if (fileName == null) {
 					fileName = content.UntitledName;
 				}
-				
 				OnTitleChanged(null);
 			}
 		}
@@ -167,14 +178,42 @@ namespace MonoDevelop.Ide.Gui
 				return;
 			int toSelect = tabControl.PageNum (this);
 			tabControl.CurrentPage = toSelect;
+			if (tabControl.FocusChild != null) {
+				tabControl.FocusChild.GrabFocus ();
+			} else {
+				DeepGrabFocus (this.ActiveViewContent.Control);
+			}
 		}
 		
+		static void DeepGrabFocus (Gtk.Widget widget)
+		{
+			Widget first = null;
+			foreach (var f in GetFocussableWidgets (widget)) {
+				if (f.HasFocus)
+					return;
+				if (first == null)
+					first = f;
+			}
+			if (first != null)
+				first.GrabFocus ();
+		}
+		
+		static IEnumerable<Gtk.Widget> GetFocussableWidgets (Gtk.Widget widget)
+		{
+			var c = widget as Container;
+			if (widget.CanFocus)
+				yield return widget;
+			if (c != null) {
+				foreach (var f in c.FocusChain.SelectMany (x => GetFocussableWidgets (x)).Where (y => y != null))
+					yield return f;
+			}
+		}
 
 		void BeforeSave(object sender, EventArgs e)
 		{
-			ISecondaryViewContent secondaryViewContent = ActiveViewContent as ISecondaryViewContent;
+			IAttachableViewContent secondaryViewContent = ActiveViewContent as IAttachableViewContent;
 			if (secondaryViewContent != null) {
-				secondaryViewContent.NotifyBeforeSave();
+				secondaryViewContent.BeforeSave ();
 			}
 		}
 		
@@ -204,10 +243,9 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void SetTitleEvent(object sender, EventArgs e)
 		{
-			if (content == null) {
+			if (content == null)
 				return;
-			}
-		
+				
 			string newTitle = "";
 			if (content.ContentName == null) {
 				if (myUntitledTitle == null) {
@@ -242,7 +280,6 @@ namespace MonoDevelop.Ide.Gui
 			} else if (content.IsReadOnly) {
 				newTitle += "+";
 			}
-			
 			if (newTitle != Title) {
 				Title = newTitle;
 			}
@@ -251,7 +288,7 @@ namespace MonoDevelop.Ide.Gui
 		public void OnContentChanged (object o, EventArgs e)
 		{
 			if (subViewContents != null) {
-				foreach (ISecondaryViewContent subContent in subViewContents)
+				foreach (IAttachableViewContent subContent in subViewContents)
 				{
 					subContent.BaseContentChanged ();
 				}
@@ -260,7 +297,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		public bool CloseWindow (bool force, bool fromMenu, int pageNum)
 		{
-			WorkbenchWindowEventArgs args = new WorkbenchWindowEventArgs (force);
+			bool wasActive = workbench.WorkbenchLayout.ActiveWorkbenchwindow == this;
+			WorkbenchWindowEventArgs args = new WorkbenchWindowEventArgs (force, wasActive);
 			args.Cancel = false;
 			OnClosing (args);
 			if (args.Cancel)
@@ -278,7 +316,7 @@ namespace MonoDevelop.Ide.Gui
 			content.WorkbenchWindow = null;
 			
 			if (subViewContents != null) {
-				foreach (ISecondaryViewContent sv in subViewContents) {
+				foreach (IAttachableViewContent sv in subViewContents) {
 					subViewNotebook.Remove (sv.Control);
 					sv.Dispose ();
 				}
@@ -294,7 +332,7 @@ namespace MonoDevelop.Ide.Gui
 			this.separatorItem = null;
 			DetachFromPathedDocument ();
 
-			OnClosed (null);
+			OnClosed (args);
 			
 			this.content = null;
 			this.subViewNotebook = null;
@@ -376,9 +414,10 @@ namespace MonoDevelop.Ide.Gui
 		
 		#endregion
 		
-		public void AttachSecondaryViewContent(ISecondaryViewContent subViewContent)
+			
+		public void AttachViewContent (IAttachableViewContent subViewContent)
 		{
-			// need to create child Notebook when first ISecondaryViewContent is added
+			// need to create child Notebook when first IAttachableViewContent is added
 			CheckCreateSubViewContents ();
 			
 			subViewContents.Add (subViewContent);
@@ -536,14 +575,14 @@ namespace MonoDevelop.Ide.Gui
 		protected void subViewNotebookIndexChanged(object sender, SwitchPageArgs e)
 		{
 			if (oldIndex > 0) {
-				ISecondaryViewContent secondaryViewContent = subViewContents[oldIndex - 1] as ISecondaryViewContent;
+				IAttachableViewContent secondaryViewContent = subViewContents[oldIndex - 1] as IAttachableViewContent;
 				if (secondaryViewContent != null) {
 					secondaryViewContent.Deselected();
 				}
 			}
 			
 			if (subViewNotebook.CurrentPage > 0) {
-				ISecondaryViewContent secondaryViewContent = subViewContents[subViewNotebook.CurrentPage - 1] as ISecondaryViewContent;
+				IAttachableViewContent secondaryViewContent = subViewContents[subViewNotebook.CurrentPage - 1] as IAttachableViewContent;
 				if (secondaryViewContent != null) {
 					secondaryViewContent.Selected();
 				}
@@ -587,11 +626,11 @@ namespace MonoDevelop.Ide.Gui
 					tabLabel.Icon = new Gtk.Image ( content.StockIconId, IconSize.Menu );
 				}
 				else if (content.ContentName != null && content.ContentName.IndexOfAny (new char[] { '*', '+'}) == -1) {
-					tabLabel.Icon.Pixbuf = IdeApp.Services.PlatformService.GetPixbufForFile (content.ContentName, Gtk.IconSize.Menu);
+					tabLabel.Icon.Pixbuf = DesktopService.GetPixbufForFile (content.ContentName, Gtk.IconSize.Menu);
 				}
 			} catch (Exception ex) {
 				LoggingService.LogError (ex.ToString ());
-				tabLabel.Icon.Pixbuf = IdeApp.Services.PlatformService.GetPixbufForType ("gnome-fs-regular", Gtk.IconSize.Menu);
+				tabLabel.Icon.Pixbuf = DesktopService.GetPixbufForType ("gnome-fs-regular", Gtk.IconSize.Menu);
 			}
 
 			if (TitleChanged != null) {
@@ -606,7 +645,7 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 
-		protected virtual void OnClosed (EventArgs e)
+		protected virtual void OnClosed (WorkbenchWindowEventArgs e)
 		{
 			if (Closed != null) {
 				Closed (this, e);
@@ -620,7 +659,7 @@ namespace MonoDevelop.Ide.Gui
 		}
 
 		public event EventHandler TitleChanged;
-		public event EventHandler Closed;
+		public event WorkbenchWindowEventHandler Closed;
 		public event WorkbenchWindowEventHandler Closing;
 		public event ActiveViewContentEventHandler ActiveViewContentChanged;
 	}
