@@ -28,9 +28,12 @@
 
 
 using System;
+using System.Linq;
 using System.IO;
+using System.Xml;
 using System.Collections.Generic;
 using System.Collections.ObjectModel;
+using System.Threading;
 
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Projects;
@@ -54,11 +57,11 @@ namespace MonoDevelop.Ide.Gui
 		ProgressMonitorManager monitors = new ProgressMonitorManager ();
 		DefaultWorkbench workbench;
 		RecentOpen recentOpen = null;
-		IStatusBarService statusBarService;
-		DisplayBindingService displayBindingService;
 		
 		public event EventHandler ActiveDocumentChanged;
 		public event EventHandler LayoutChanged;
+		public event EventHandler GuiLocked;
+		public event EventHandler GuiUnlocked;
 		
 		internal void Initialize (IProgressMonitor monitor)
 		{
@@ -76,13 +79,17 @@ namespace MonoDevelop.Ide.Gui
 				((Gtk.Window)workbench).Visible = false;
 				workbench.ActiveWorkbenchWindowChanged += new EventHandler (OnDocumentChanged);
 				PropertyService.PropertyChanged += new EventHandler<PropertyChangedEventArgs> (TrackPropertyChanges);
+				FileService.FileRemoved += (EventHandler<FileEventArgs>) DispatchService.GuiDispatch (new EventHandler<FileEventArgs> (IdeApp.Workbench.RecentOpen.InformFileRemoved));
+				FileService.FileRenamed += (EventHandler<FileCopyEventArgs>) DispatchService.GuiDispatch (new EventHandler<FileCopyEventArgs> (IdeApp.Workbench.RecentOpen.InformFileRenamed));
+				IdeApp.Workspace.StoringUserPreferences += OnStoringWorkspaceUserPreferences;
+				IdeApp.Workspace.LoadingUserPreferences += OnLoadingWorkspaceUserPreferences;
 				
-				if (Services.DebuggingService != null) {
-					Services.DebuggingService.PausedEvent += (EventHandler) DispatchService.GuiDispatch (new EventHandler (OnDebuggerPaused));
-				}
-				
-				FileService.FileRemoved += (EventHandler<FileEventArgs>) DispatchService.GuiDispatch (new EventHandler<FileEventArgs> (IdeApp.Workbench.RecentOpen.FileRemoved));
-				FileService.FileRenamed += (EventHandler<FileCopyEventArgs>) DispatchService.GuiDispatch (new EventHandler<FileCopyEventArgs> (IdeApp.Workbench.RecentOpen.FileRenamed));
+				IdeApp.CommandService.ApplicationFocusOut += delegate(object o, EventArgs args) {
+					SaveFileStatus ();
+				};
+				IdeApp.CommandService.ApplicationFocusIn += delegate(object o, EventArgs args) {
+					CheckFileStatus ();
+				};
 				
 				pads = null;	// Make sure we get an up to date pad list.
 				monitor.Step (1);
@@ -109,16 +116,17 @@ namespace MonoDevelop.Ide.Gui
 		internal void Show (string workbenchMemento)
 		{
 			RootWindow.Realize ();
-			workbench.SetMemento (PropertyService.Get (workbenchMemento, new Properties ()));
+			workbench.Memento = PropertyService.Get (workbenchMemento, new Properties ());
 			RootWindow.Visible = true;
 			workbench.Context = WorkbenchContext.Edit;
 			
 			// now we have an layout set notify it
 			if (LayoutChanged != null)
 				LayoutChanged (this, EventArgs.Empty);
-
-			workbench.RedrawAllComponents ();
-			RootWindow.Present ();
+			
+			monitors.Initialize ();
+			
+			Present ();
 		}
 		
 		internal bool Close ()
@@ -139,14 +147,17 @@ namespace MonoDevelop.Ide.Gui
 		}
 
 		public Document ActiveDocument {
-			get { return WrapDocument (workbench.ActiveWorkbenchWindow); }
+			get {
+				if (workbench == null || workbench.ActiveWorkbenchWindow == null)
+					return null;
+				return WrapDocument (workbench.ActiveWorkbenchWindow); 
+			}
 		}
 		
-		public Document GetDocument (string fileName)
+		public Document GetDocument (string name)
 		{
-			fileName = FileService.GetFullPath (fileName);
 			foreach (Document doc in documents) {
-				if (FileService.GetFullPath (doc.FileName) == fileName)
+				if (FileService.GetFullPath (doc.Name) == name)
 					return doc;
 			}
 			return null;
@@ -168,6 +179,29 @@ namespace MonoDevelop.Ide.Gui
 			get { return (Gtk.Window) workbench; }
 		}
 		
+		/// <summary>
+		/// Whether the root window or any undocked part of it has toplevel focus. 
+		/// </summary>
+		public bool HasToplevelFocus {
+			get {
+				var toplevel = Gtk.Window.ListToplevels ().Where (x => x.HasToplevelFocus).FirstOrDefault ();
+				if (toplevel == null)
+					return false;
+				if (toplevel == RootWindow)
+					return true;
+				//FIXME: don't depend on type name string
+				var c = toplevel.Child;
+				return c != null && c.GetType ().FullName.StartsWith ("MonoDevelop.Components.Docking");
+			}
+		}
+		
+		public void Present ()
+		{
+			//FIXME: Present is broken on Mac GTK+. It maximises the window.
+			if (!PropertyService.IsMac)
+				RootWindow.Present ();
+		}
+		
 		public WorkbenchContext Context {
 			get { return workbench.Context; }
 			set {
@@ -184,7 +218,7 @@ namespace MonoDevelop.Ide.Gui
 		}
 		
 		public string CurrentLayout {
-			get { return workbench.WorkbenchLayout != null ? workbench.WorkbenchLayout.CurrentLayout : ""; }
+			get { return workbench != null && workbench.WorkbenchLayout != null ? workbench.WorkbenchLayout.CurrentLayout : ""; }
 			set {
 				if (value != workbench.WorkbenchLayout.CurrentLayout)
 				{
@@ -203,19 +237,9 @@ namespace MonoDevelop.Ide.Gui
 			get { return monitors; }
 		}
 		
-		public DisplayBindingService DisplayBindings {
+		public MonoDevelopStatusBar StatusBar {
 			get {
-				if (displayBindingService == null)
-					displayBindingService = new DisplayBindingService ();
-				return displayBindingService; 
-			}
-		}
-		
-		public IStatusBarService StatusBar {
-			get {
-				if (statusBarService == null)
-					statusBarService = new DefaultStatusBarService ();
-				return statusBarService;
+				return workbench.StatusBar;
 			}
 		}
 		
@@ -237,11 +261,15 @@ namespace MonoDevelop.Ide.Gui
 		public void LockGui ()
 		{
 			IdeApp.CommandService.LockAll ();
+			if (GuiLocked != null)
+				GuiLocked (this, EventArgs.Empty);
 		}
 		
 		public void UnlockGui ()
 		{
 			IdeApp.CommandService.UnlockAll ();
+			if (GuiUnlocked != null)
+				GuiUnlocked (this, EventArgs.Empty);
 		}
 		
 		public void SaveAll ()
@@ -275,56 +303,83 @@ namespace MonoDevelop.Ide.Gui
 			workbench.ShowPad (content);
 			return WrapPad (content);
 		}
+
+		internal Pad AddPad (PadCodon content)
+		{
+			workbench.AddPad (content);
+			return WrapPad (content);
+		}
+
+		public Pad AddPad (IPadContent padContent, string id, string label, string defaultPlacement, string icon)
+		{
+			return AddPad (new PadCodon (padContent, id, label, defaultPlacement, icon));
+		}
 		
 		public Pad ShowPad (IPadContent padContent, string id, string label, string defaultPlacement, string icon)
 		{
 			return ShowPad (new PadCodon (padContent, id, label, defaultPlacement, icon));
 		}
-		
-		public FileViewer[] GetFileViewers (string fileName)
+
+		public FileViewer[] GetFileViewers (FilePath fileName)
 		{
 			List<FileViewer> list = new List<FileViewer> ();
 			
-			string mimeType = IdeApp.Services.PlatformService.GetMimeTypeForUri (fileName);
+			string mimeType = DesktopService.GetMimeTypeForUri (fileName);
 
-			IDisplayBinding[] bindings = DisplayBindings.GetBindingsForMimeType (mimeType);
-			foreach (IDisplayBinding bin in bindings)
+			foreach (IDisplayBinding bin in DisplayBindingService.GetBindingsForMimeType (mimeType))
 				list.Add (new FileViewer (bin));
 
-			foreach (DesktopApplication app in IdeApp.Services.PlatformService.GetAllApplications (mimeType))
-				if (app.Command != "monodevelop")
+			foreach (DesktopApplication app in DesktopService.GetAllApplications (mimeType))
+				if (app.IsValid && app.Command != "monodevelop")
 					list.Add (new FileViewer (app));
 				
 			return list.ToArray ();
 		}
-		
-		public Document OpenDocument (string fileName)
+
+		public Document OpenDocument (FilePath fileName)
 		{
 			return OpenDocument (fileName, true);
 		}
-		
-		public Document OpenDocument (string fileName, string encoding)
+
+		public Document OpenDocument (FilePath fileName, string encoding)
 		{
 			return OpenDocument (fileName, -1, -1, true, encoding, null);
 		}
-		
-		public Document OpenDocument (string fileName, bool bringToFront)
+
+		public Document OpenDocument (FilePath fileName, bool bringToFront)
 		{
 			return OpenDocument (fileName, -1, -1, bringToFront);
 		}
-		
-		public Document OpenDocument (string fileName, int line, int column, bool bringToFront)
+
+		public Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront)
 		{
 			return OpenDocument (fileName, line, column, bringToFront, null, null);
 		}
 		
-		public Document OpenDocument (string fileName, int line, int column, bool bringToFront, string encoding)
+		public Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront, bool highlightCaretLine)
+		{
+			return OpenDocument (fileName, line, column, bringToFront, null, null, highlightCaretLine);
+		}
+
+		public Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront, string encoding)
 		{
 			return OpenDocument (fileName, line, column, bringToFront, encoding, null);
 		}
-		
-		internal Document OpenDocument (string fileName, int line, int column, bool bringToFront, string encoding, IDisplayBinding binding)
+
+		public Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront, string encoding, bool highlightCaretLine)
 		{
+			return OpenDocument (fileName, line, column, bringToFront, encoding, null, highlightCaretLine);
+		}
+
+		internal Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront, string encoding, IDisplayBinding binding)
+		{
+			return OpenDocument (fileName, line, column, bringToFront, encoding, binding, true);
+		}
+		
+		internal Document OpenDocument (FilePath fileName, int line, int column, bool bringToFront, string encoding, IDisplayBinding binding, bool highlightCaretLine)
+		{
+			NavigationHistoryService.LogActiveDocument ();
+			
 			foreach (Document doc in Documents) {
 				IBaseViewContent vcFound = null;
 				int vcIndex = 0;
@@ -332,15 +387,7 @@ namespace MonoDevelop.Ide.Gui
 				//search all ViewContents to see if they can "re-use" this filename
 				if (doc.Window.ViewContent.CanReuseView (fileName))
 					vcFound = doc.Window.ViewContent;
-				else if (doc.Window.SubViewContents != null) {
-					for (int i = 0; i < doc.Window.SubViewContents.Count; i++) {
-						ISecondaryViewContent vc = (ISecondaryViewContent) doc.Window.SubViewContents [i];
-						if (vc.CanReuseView (fileName)) {
-							vcIndex = i +1;
-							vcFound = vc;
-						}
-					}
-				}
+				
 				
 				//old method as fallback
 				if ((vcFound == null) && (doc.FileName == fileName))
@@ -351,13 +398,15 @@ namespace MonoDevelop.Ide.Gui
 					if (bringToFront) {
 						doc.Select ();
 						doc.Window.SwitchView (vcIndex);
+						Present ();
 					}
 					
-					IPositionable ipos = (IPositionable) vcFound.GetContent (typeof(IPositionable));
+					IEditableTextBuffer ipos = (IEditableTextBuffer) vcFound.GetContent (typeof(IEditableTextBuffer));
 					if (line >= 1 && ipos != null) {
-						ipos.JumpTo (line, column >= 1 ? column : 1);
+						ipos.SetCaretTo (line, column >= 1 ? column : 1, highlightCaretLine);
 					}
 					
+					NavigationHistoryService.LogActiveDocument ();
 					return doc;
 				}
 			}
@@ -371,20 +420,28 @@ namespace MonoDevelop.Ide.Gui
 			openFileInfo.Column = column;
 			openFileInfo.DisplayBinding = binding;
 			openFileInfo.Encoding = encoding;
+			openFileInfo.HighlightCaretLine = highlightCaretLine;
 			RealOpenFile (openFileInfo);
 			
 			if (!pm.AsyncOperation.Success)
 				return null;
 			
-			if (openFileInfo.NewContent != null)
-				return WrapDocument (openFileInfo.NewContent.WorkbenchWindow);
-			else
+			if (openFileInfo.NewContent != null) {
+				Document doc = WrapDocument (openFileInfo.NewContent.WorkbenchWindow);
+				NavigationHistoryService.LogActiveDocument ();
+				if (bringToFront)
+					Present ();
+				return doc;
+			} else {
 				return null;
+			}
 		}
 		
 		public Document OpenDocument (IViewContent content, bool bringToFront)
 		{
 			workbench.ShowView (content, bringToFront);
+			if (bringToFront)
+				Present ();
 			return WrapDocument (content.WorkbenchWindow);
 		}
 		
@@ -406,7 +463,7 @@ namespace MonoDevelop.Ide.Gui
 		
 		public Document NewDocument (string defaultName, string mimeType, Stream content)
 		{
-			IDisplayBinding binding = DisplayBindings.GetBindingForMimeType (mimeType);
+			IDisplayBinding binding = DisplayBindingService.GetBinding (null, mimeType);
 			IViewContent newContent;
 			
 			if (binding != null) {
@@ -422,8 +479,7 @@ namespace MonoDevelop.Ide.Gui
 				newContent.UntitledName = defaultName;
 				newContent.IsDirty = true;
 				workbench.ShowView(newContent, true);
-				
-				DisplayBindings.AttachSubWindows(newContent.WorkbenchWindow);
+				DisplayBindingService.AttachSubWindows (newContent.WorkbenchWindow);
 			} else {
 				throw new ApplicationException("Can't create display binding for mime type: " + mimeType);				
 			}
@@ -433,7 +489,7 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow)
 		{
-			ShowGlobalPreferencesDialog (null, null);
+			ShowGlobalPreferencesDialog (parentWindow, null);
 		}
 		
 		public void ShowGlobalPreferencesDialog (Gtk.Window parentWindow, string panelId)
@@ -441,15 +497,46 @@ namespace MonoDevelop.Ide.Gui
 			if (parentWindow == null)
 				parentWindow = IdeApp.Workbench.RootWindow;
 
-			TreeViewOptions ops = new TreeViewOptions (
+			OptionsDialog ops = new OptionsDialog (
 				parentWindow,
 				TextEditorProperties.Properties,
-				AddinManager.GetExtensionNode ("/MonoDevelop/Ide/OptionsDialogPanels"));
+				"/MonoDevelop/Ide/GlobalOptionsDialog");
 
-			if (panelId != null)
-				ops.SelectPanel (panelId);
-			if (ops.Run () == (int) Gtk.ResponseType.Ok)
-				PropertyService.SaveProperties ();
+			try {
+				if (panelId != null)
+					ops.SelectPanel (panelId);
+				ops.TransientFor = RootWindow;
+				if (ops.Run () == (int) Gtk.ResponseType.Ok) {
+					PropertyService.SaveProperties ();
+				}
+			} finally {
+				ops.Destroy ();
+			}
+		}
+		
+		public void ShowDefaultPoliciesDialog (Gtk.Window parentWindow)
+		{
+			ShowDefaultPoliciesDialog (parentWindow, null);
+		}
+		
+		public void ShowDefaultPoliciesDialog (Gtk.Window parentWindow, string panelId)
+		{
+			if (parentWindow == null)
+				parentWindow = IdeApp.Workbench.RootWindow;
+
+			MonoDevelop.Projects.Gui.Dialogs.DefaultPolicyOptionsDialog ops
+				= new MonoDevelop.Projects.Gui.Dialogs.DefaultPolicyOptionsDialog (parentWindow);
+
+			try {
+				if (panelId != null)
+					ops.SelectPanel (panelId);
+				ops.TransientFor = RootWindow;
+				if (ops.Run () == (int) Gtk.ResponseType.Ok) {
+					MonoDevelop.Projects.Policies.PolicyService.SaveDefaultPolicies ();
+				}
+			} finally {
+				ops.Destroy ();
+			}
 		}
 		
 		internal void ShowNext ()
@@ -458,7 +545,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			Pad pad = GetLocationListPad ();
 			if (pad != null) {
-				pad.BringToFront ();
+				pad.BringToFront (true);
 				ILocationListPad loc = (ILocationListPad) pad.Content;
 				string file;
 				int lin, col;
@@ -475,7 +562,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			Pad pad = GetLocationListPad ();
 			if (pad != null) {
-				pad.BringToFront ();
+				pad.BringToFront (true);
 				ILocationListPad loc = (ILocationListPad) pad.Content;
 				string file;
 				int lin, col;
@@ -505,10 +592,13 @@ namespace MonoDevelop.Ide.Gui
 			
 			return active;
 		}
+		
 		void OnDocumentChanged (object s, EventArgs a)
 		{
 			if (ActiveDocumentChanged != null)
 				ActiveDocumentChanged (s, a);
+			if (ActiveDocument != null)
+				ActiveDocument.LastTimeActive = DateTime.Now;
 		}
 		
 		internal Document WrapDocument (IWorkbenchWindow window)
@@ -518,7 +608,7 @@ namespace MonoDevelop.Ide.Gui
 			if (doc != null) return doc;
 			doc = new Document (window);
 			window.Closing += new WorkbenchWindowEventHandler (OnWindowClosing);
-			window.Closed += new EventHandler (OnWindowClosed);
+			window.Closed += OnWindowClosed;
 			documents.Add (doc);
 			
 			doc.OnDocumentAttached ();
@@ -528,6 +618,12 @@ namespace MonoDevelop.Ide.Gui
 		
 		Pad WrapPad (PadCodon padContent)
 		{
+			if (pads == null) {
+				foreach (Pad p in Pads) {
+					if (p.InternalContent == padContent)
+						return p;
+				}
+			}
 			Pad pad = new Pad (workbench, padContent);
 			Pads.Add (pad);
 			pad.Window.PadDestroyed += delegate {
@@ -540,27 +636,14 @@ namespace MonoDevelop.Ide.Gui
 		{
 			IWorkbenchWindow window = (IWorkbenchWindow) sender;
 			if (!args.Forced && window.ViewContent != null && window.ViewContent.IsDirty) {
-				
-				QuestionResponse response = Services.MessageService.AskQuestionWithCancel (GettextCatalog.GetString ("Do you want to save the current changes?"));
-				
-				if (response == QuestionResponse.Cancel) {
-					args.Cancel = true;
-					return;
-				}
-
-				if (response == QuestionResponse.Yes) {
+				AlertButton result = MessageService.GenericAlert (Stock.Warning,
+				                                                  GettextCatalog.GetString ("Save the changes to document '{0}' before closing?", window.ViewContent.IsUntitled ? window.ViewContent.UntitledName : System.IO.Path.GetFileName (window.ViewContent.ContentName)), 
+				                                                  GettextCatalog.GetString ("If you don't save, all changes will be permanently lost."),
+				                                                  AlertButton.CloseWithoutSave, AlertButton.Cancel, window.ViewContent.IsUntitled ? AlertButton.SaveAs : AlertButton.Save);
+				if (result == AlertButton.Save || result == AlertButton.SaveAs) {
 					if (window.ViewContent.ContentName == null) {
-						while (true) {
-							FindDocument (window).Save ();
-							if (window.ViewContent.IsDirty) {
-								if (Services.MessageService.AskQuestion(GettextCatalog.GetString ("Do you really want to discard your changes?"))) {
-									break;
-								}
-							} else {
-								break;
-							}
-						}
-						
+						FindDocument (window).Save ();
+						args.Cancel = window.ViewContent.IsDirty;
 					} else {
 						try {
 							if (window.ViewContent.IsFile)
@@ -570,14 +653,16 @@ namespace MonoDevelop.Ide.Gui
 						}
 						catch (Exception ex) {
 							args.Cancel = true;
-							IdeApp.Services.MessageService.ShowError (ex, GettextCatalog.GetString ("The document could not be saved."));
+							MessageService.ShowException (ex, GettextCatalog.GetString ("The document could not be saved."));
 						}
 					}
+				} else {
+					args.Cancel = result != AlertButton.CloseWithoutSave;
 				}
 			}
 		}
 		
-		void OnWindowClosed (object sender, EventArgs args)
+		void OnWindowClosed (object sender, WorkbenchWindowEventArgs args)
 		{
 			IWorkbenchWindow window = (IWorkbenchWindow) sender;
 			documents.Remove (FindDocument (window)); 
@@ -585,34 +670,34 @@ namespace MonoDevelop.Ide.Gui
 		
 		void RealOpenFile (object openFileInfo)
 		{
-			string fileName;
+			FilePath fileName;
 			FileInformation oFileInfo = openFileInfo as FileInformation;
 			IProgressMonitor monitor = oFileInfo.ProgressMonitor;
 
 			using (monitor)
 			{
-				fileName = oFileInfo.FileName;
-				
-				if (fileName == null) {
+				string origName = oFileInfo.FileName;
+
+				if (origName == null) {
 					monitor.ReportError (GettextCatalog.GetString ("Invalid file name"), null);
 					return;
 				}
-	
-				string origName = fileName;
-	
-				if (fileName.StartsWith ("file://"))
-					fileName = new Uri(fileName).LocalPath;
-	
-				if (!fileName.StartsWith ("http://"))
-					fileName = System.IO.Path.GetFullPath (fileName);
+
+				if (origName.StartsWith ("file://"))
+					fileName = new Uri (origName).LocalPath;
+				else
+					fileName = origName;
+
+				if (!origName.StartsWith ("http://"))
+					fileName = fileName.FullPath;
 				
-				//Debug.Assert(FileService.IsValidFileName(fileName));
+				//Debug.Assert(FileService.IsValidPath(fileName));
 				if (FileService.IsDirectory (fileName)) {
 					monitor.ReportError (GettextCatalog.GetString ("{0} is a directory", fileName), null);
 					return;
 				}
 				// test, if file fileName exists
-				if (!fileName.StartsWith("http://")) {
+				if (!origName.StartsWith("http://")) {
 					// test, if an untitled file should be opened
 					if (!System.IO.Path.IsPathRooted(origName)) { 
 						foreach (Document doc in Documents) {
@@ -633,9 +718,9 @@ namespace MonoDevelop.Ide.Gui
 					if (doc.FileName == fileName) {
 						if (oFileInfo.BringToFront) {
 							doc.Select ();
-							IPositionable ipos = doc.GetContent <IPositionable> ();
+							IEditableTextBuffer ipos = doc.GetContent <IEditableTextBuffer> ();
 							if (oFileInfo.Line != -1 && ipos != null) {
-								ipos.JumpTo (oFileInfo.Line, oFileInfo.Column != -1 ? oFileInfo.Column : 0);
+								ipos.SetCaretTo (oFileInfo.Line, oFileInfo.Column != -1 ? oFileInfo.Column : 0, oFileInfo.HighlightCaretLine);
 							}
 						}
 						oFileInfo.NewContent = doc.Window.ViewContent;
@@ -644,13 +729,35 @@ namespace MonoDevelop.Ide.Gui
 				}
 				
 				IDisplayBinding binding;
-				if (oFileInfo.DisplayBinding != null)
+				
+				if (oFileInfo.DisplayBinding != null) {
 					binding = oFileInfo.DisplayBinding;
-				else
-					binding = DisplayBindings.GetBindingPerFileName(fileName);
+				} else {
+					binding = DisplayBindingService.GetBinding (fileName, DesktopService.GetMimeTypeForUri (fileName));
+				}
 				
 				if (binding != null) {
-					Project project = IdeApp.ProjectOperations.CurrentOpenCombine != null ? IdeApp.ProjectOperations.CurrentOpenCombine.GetProjectContainingFile (fileName) : null;
+					// When looking for the project to which the file belongs, look first
+					// in the active project, then the active solution, and so on
+					Project project = null;
+					if (IdeApp.ProjectOperations.CurrentSelectedProject != null) {
+						if (IdeApp.ProjectOperations.CurrentSelectedProject.Files.GetFile (fileName) != null)
+							project = IdeApp.ProjectOperations.CurrentSelectedProject;
+					}
+					if (project == null && IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem != null) {
+						project = IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem.GetProjectContainingFile (fileName);
+						if (project == null) {
+							WorkspaceItem it = IdeApp.ProjectOperations.CurrentSelectedWorkspaceItem.ParentWorkspace;
+							while (it != null && project == null) {
+								project = it.GetProjectContainingFile (fileName);
+								it = it.ParentWorkspace;
+							}
+						}
+					}
+					if (project == null) {
+						project = IdeApp.Workspace.GetProjectContainingFile (fileName);
+					}
+					
 					LoadFileWrapper fw = new LoadFileWrapper (workbench, binding, project, oFileInfo);
 					fw.Invoke (fileName);
 					RecentOpen.AddLastFile (fileName, project != null ? project.Name : null);
@@ -661,23 +768,87 @@ namespace MonoDevelop.Ide.Gui
 						//if (mimetype != null) {
 						//	mimetype.DefaultAction.Launch ();
 						//} else {
-							IdeApp.Services.PlatformService.ShowUrl ("file://" + fileName);
+							DesktopService.ShowUrl ("file://" + fileName);
 						//}
-					} catch {
-						LoadFileWrapper fw = new LoadFileWrapper (workbench, DisplayBindings.LastBinding, null, oFileInfo);
-						fw.Invoke (fileName);
-						RecentOpen.AddLastFile (fileName, null);
+					} catch (Exception ex) {
+						LoggingService.LogError ("Error opening file: " + fileName, ex);
+						MessageService.ShowError (GettextCatalog.GetString ("File '{0}' could not be opened", fileName));
 					}
 				}
 			}
 		}
 		
-		void OnDebuggerPaused (object o, EventArgs e)
+		void OnStoringWorkspaceUserPreferences (object s, UserPreferencesEventArgs args)
 		{
-			IDebuggingService dbgr = Services.DebuggingService;
-			if (dbgr != null) {
-				if (dbgr.CurrentFilename != String.Empty)
-					IdeApp.Workbench.OpenDocument (dbgr.CurrentFilename);
+			WorkbenchUserPrefs prefs = new WorkbenchUserPrefs ();
+			
+			foreach (Document document in Documents) {
+				if (!String.IsNullOrEmpty (document.FileName)) {
+					DocumentUserPrefs dp = new DocumentUserPrefs ();
+					dp.FileName = FileService.AbsoluteToRelativePath (args.Item.BaseDirectory, document.FileName);
+					if (document.TextEditor != null) {
+						dp.Line = document.TextEditor.CursorLine;
+						dp.Column = document.TextEditor.CursorColumn;
+					}
+					prefs.Files.Add (dp);
+				}
+			}
+			
+			foreach (Pad pad in Pads) {
+				IMementoCapable mc = pad.GetMementoCapable ();
+				if (mc != null) {
+					ICustomXmlSerializer mem = mc.Memento;
+					if (mem != null) {
+						PadUserPrefs data = new PadUserPrefs ();
+						data.Id = pad.Id;
+						StringWriter w = new StringWriter ();
+						XmlTextWriter tw = new XmlTextWriter (w);
+						mem.WriteTo (tw);
+						XmlDocument doc = new XmlDocument ();
+						doc.LoadXml (w.ToString ());
+						data.State = doc.DocumentElement;
+						prefs.Pads.Add (data);
+					}
+				}
+			}
+			
+			if (ActiveDocument != null)
+				prefs.ActiveDocument = FileService.AbsoluteToRelativePath (args.Item.BaseDirectory, ActiveDocument.FileName);
+			
+			args.Properties.SetValue ("MonoDevelop.Ide.Workbench", prefs);
+		}
+		
+		void OnLoadingWorkspaceUserPreferences (object s, UserPreferencesEventArgs args)
+		{
+			WorkbenchUserPrefs prefs = args.Properties.GetValue<WorkbenchUserPrefs> ("MonoDevelop.Ide.Workbench");
+			if (prefs == null)
+				return;
+			
+			string currentFileName = prefs.ActiveDocument != null ? Path.GetFullPath (Path.Combine (args.Item.BaseDirectory, prefs.ActiveDocument)) : null;
+			
+			foreach (DocumentUserPrefs doc in prefs.Files) {
+				FilePath fileName = args.Item.BaseDirectory.Combine (doc.FileName).FullPath;
+				if (File.Exists (fileName))
+					IdeApp.Workbench.OpenDocument (fileName, doc.Line, doc.Column, fileName == currentFileName, null, null, false);
+			}
+			
+			foreach (PadUserPrefs pi in prefs.Pads) {
+				foreach (Pad pad in IdeApp.Workbench.Pads) {
+					if (pi.Id == pad.Id && pad.Content is IMementoCapable) {
+						try {
+							string xml = pi.State.OuterXml;
+							IMementoCapable m = (IMementoCapable) pad.Content; 
+							XmlReader innerReader = new XmlTextReader (new StringReader (xml));
+							innerReader.MoveToContent ();
+							ICustomXmlSerializer cs = (ICustomXmlSerializer)m.Memento;
+							if (cs != null)
+								m.Memento = cs.ReadFrom (innerReader);
+						} catch (Exception ex) {
+							LoggingService.LogError ("Error loading view memento.", ex);
+						}
+						break;
+					}
+				}
 			}
 		}
 		
@@ -696,8 +867,102 @@ namespace MonoDevelop.Ide.Gui
 					return pad;
 			return null;
 		}
+		
+		internal void ReorderDocuments (int oldPlacement, int newPlacement)
+		{
+			IViewContent content = workbench.ViewContentCollection[oldPlacement];
+			workbench.InternalViewContentCollection.RemoveAt (oldPlacement);
+			workbench.InternalViewContentCollection.Insert (newPlacement, content);
+			
+			Document doc = documents [oldPlacement];
+			documents.RemoveAt (oldPlacement);
+			documents.Insert (newPlacement, doc);
+		}
+		
+		internal void ResetToolbars ()
+		{
+			workbench.ResetToolbars ();
+		}
+		
+		List<FileData> fileStatus;
+		object fileStatusLock = new object ();
+		
+		internal void SaveFileStatus ()
+		{
+			fileStatus = new List<FileData> ();
+			
+//			DateTime t = DateTime.Now;
+			List<FilePath> files = new List<FilePath> (GetKnownFiles ());
+//			Console.WriteLine ("SaveFileStatus(0) " + (DateTime.Now - t).TotalMilliseconds + "ms " + files.Count);
+			
+			ThreadPool.QueueUserWorkItem (delegate {
+//				t = DateTime.Now;
+				lock (fileStatusLock) {
+					foreach (FilePath file in files) {
+						try {
+							FileInfo fi = new FileInfo (file);
+							FileData fd = new FileData (file, fi.Exists ? fi.LastWriteTime : DateTime.MinValue);
+							fileStatus.Add (fd);
+						} catch {
+							// Ignore
+						}
+					}
+				}
+//				Console.WriteLine ("SaveFileStatus " + (DateTime.Now - t).TotalMilliseconds + "ms " + fileStatus.Count);
+			});
+		}
+		
+		internal void CheckFileStatus ()
+		{
+			if (fileStatus == null)
+				return;
+			
+			ThreadPool.QueueUserWorkItem (delegate {
+				lock (fileStatusLock) {
+//					DateTime t = DateTime.Now;
+					if (fileStatus == null)
+						return;
+					foreach (FileData fd in fileStatus) {
+						try {
+							FileInfo fi = new FileInfo (fd.File);
+							if (fi.Exists) {
+								if (fi.LastWriteTime != fd.Time)
+									FileService.NotifyFileChanged (fd.File);
+							} else if (fd.Time != DateTime.MinValue) {
+								FileService.NotifyFileRemoved (fd.File);
+							}
+						} catch {
+							// Ignore
+						}
+					}
+//					Console.WriteLine ("CheckFileStatus " + (DateTime.Now - t).TotalMilliseconds + "ms " + fileStatus.Count);
+					fileStatus = null;
+				}
+			});
+		}
+		
+		IEnumerable<FilePath> GetKnownFiles ()
+		{
+			foreach (WorkspaceItem item in IdeApp.Workspace.Items) {
+				foreach (FilePath file in item.GetItemFiles (true))
+					yield return file;
+			}
+		}
+		
+		struct FileData
+		{
+			public FileData (FilePath file, DateTime time)
+			{
+				this.File = file;
+				this.Time = time;
+			}
+			
+			public FilePath File;
+			public DateTime Time;
+		}
 	}
 	
+
 	class FileInformation
 	{
 		public IProgressMonitor ProgressMonitor;
@@ -708,6 +973,7 @@ namespace MonoDevelop.Ide.Gui
 		public IDisplayBinding DisplayBinding;
 		public IViewContent NewContent;
 		public string Encoding;
+		public bool HighlightCaretLine;
 	}
 	
 	class LoadFileWrapper
@@ -733,15 +999,27 @@ namespace MonoDevelop.Ide.Gui
 			this.project = project;
 		}
 		
-		public void Invoke(string fileName)
+		public void Invoke (string fileName)
 		{
 			try {
-				newContent = binding.CreateContentForFile (fileName);
+				if (binding.CanCreateContentForUri (fileName)) {
+					newContent = binding.CreateContentForUri (fileName);
+				} else {
+					string mimeType = DesktopService.GetMimeTypeForUri (fileName);
+					if (!binding.CanCreateContentForMimeType (mimeType)) {
+						fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
+						return;
+					}
+					newContent = binding.CreateContentForMimeType (mimeType, null);
+				}
 				if (newContent == null) {
 					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
 					return;
 				}
-
+				
+				if (project != null)
+					newContent.Project = project;
+				
 				IEncodedTextContent etc = (IEncodedTextContent) newContent.GetContent (typeof(IEncodedTextContent));
 				if (fileInfo.Encoding != null && etc != null)
 					etc.Load (fileName, fileInfo.Encoding);
@@ -751,14 +1029,19 @@ namespace MonoDevelop.Ide.Gui
 				fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), ex);
 				return;
 			}
-
-			if (project != null)
-				newContent.Project = project;
-
-			workbench.ShowView (newContent, fileInfo.BringToFront);
-			IdeApp.Workbench.DisplayBindings.AttachSubWindows(newContent.WorkbenchWindow);
+			// content got re-used
+			if (newContent.WorkbenchWindow != null) {
+				newContent.WorkbenchWindow.SelectWindow ();
+				fileInfo.NewContent = newContent;
+				return;
+			}
 			
-			IPositionable ipos = (IPositionable) newContent.GetContent (typeof(IPositionable));
+			workbench.ShowView (newContent, fileInfo.BringToFront);
+			DisplayBindingService.AttachSubWindows (newContent.WorkbenchWindow);
+			
+			newContent.WorkbenchWindow.DocumentType = binding.Name;
+			
+			IEditableTextBuffer ipos = (IEditableTextBuffer) newContent.GetContent (typeof(IEditableTextBuffer));
 			if (fileInfo.Line != -1 && ipos != null) {
 				GLib.Timeout.Add (10, new GLib.TimeoutHandler (JumpToLine));
 			}
@@ -767,8 +1050,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		public bool JumpToLine ()
 		{
-			IPositionable ipos = (IPositionable) newContent.GetContent (typeof(IPositionable));
-			ipos.JumpTo (Math.Max(1, fileInfo.Line), Math.Max(1, fileInfo.Column));
+			IEditableTextBuffer ipos = (IEditableTextBuffer) newContent.GetContent (typeof(IEditableTextBuffer));
+			ipos.SetCaretTo (Math.Max(1, fileInfo.Line), Math.Max(1, fileInfo.Column), fileInfo.HighlightCaretLine);
 			return false;
 		}
 	}
