@@ -81,9 +81,10 @@ namespace MonoDevelop.Ide.Templates
 		
 		public virtual bool IsValidName (string name, string language)
 		{
-			if (name.Length > 0) {
+			return name.IndexOfAny (Path.GetInvalidFileNameChars ()) == -1;
+/*			if (name.Length > 0) {
 				if (language != null && language.Length > 0) {
-					IDotNetLanguageBinding binding = MonoDevelop.Projects.Services.Languages.GetBindingPerLanguageName (language) as IDotNetLanguageBinding;
+					IDotNetLanguageBinding binding = LanguageBindingService.GetBindingPerLanguageName (language) as IDotNetLanguageBinding;
 					if (binding != null) {
 						System.CodeDom.Compiler.CodeDomProvider provider = binding.GetCodeDomProvider ();
 						if (provider != null)
@@ -93,7 +94,7 @@ namespace MonoDevelop.Ide.Templates
 				return name.IndexOfAny (Path.GetInvalidFileNameChars ()) == -1;
 			}
 			else
-				return false;
+				return false;*/
 		}
 		
 		public virtual bool SupportsProject (Project project, string projectPath)
