@@ -36,62 +36,40 @@ using MonoDevelop.Components;
 
 namespace MonoDevelop.Projects.Gui.Completion
 {
-	public class CompletionWindowManager
+	public class CompletionListWindow : ListWindow, IListDataProvider
 	{
-		static CompletionListWindow wnd;
+		const int declarationWindowMargin = 3;
 		
-		static CompletionWindowManager ()
-		{
-			wnd = new CompletionListWindow ();
-		}
-		
-		public static bool ShowWindow (char firstChar, ICompletionDataList list, ICompletionWidget completionWidget,
-		                               ICodeCompletionContext completionContext, CompletionDelegate closedDelegate)
-		{
-			try {
-				if (!wnd.ShowListWindow (firstChar, list,  completionWidget, completionContext, closedDelegate)) {
-					if (list is IDisposable)
-						((IDisposable)list).Dispose ();
-					return false;
-				}
-				return true;
-			} catch (Exception ex) {
-				LoggingService.LogError (ex.ToString ());
-				return false;
-			}
-		}
-		
-		public static bool ProcessKeyEvent (Gdk.Key key, Gdk.ModifierType modifier)
-		{
-			if (!wnd.Visible) return false;
-			return wnd.ProcessKeyEvent (key, modifier);
-		}
-		
-		public static void HideWindow ()
-		{
-			wnd.Hide ();
-		}
-	}
-	
-	internal class CompletionListWindow : ListWindow, IListDataProvider
-	{
-		internal ICompletionWidget completionWidget;
-		ICodeCompletionContext completionContext;
 		DeclarationViewWindow declarationviewwindow = new DeclarationViewWindow ();
 		ICompletionData currentData;
-		ICompletionDataList completionDataList;
-		IMutableCompletionDataList mutableList;
 		Widget parsingMessage;
-		CompletionDelegate closedDelegate;
+		System.Action closedDelegate;
 		int initialWordLength;
 		int previousWidth = -1, previousHeight = -1;
 		
-		const int declarationWindowMargin = 3;
+		public CodeCompletionContext CodeCompletionContext {
+			get;
+			set;
+		}
 		
-		internal CompletionListWindow ()
+		public ICompletionWidget CompletionWidget {
+			get;
+			set;
+		}
+		
+		IMutableCompletionDataList mutableList;
+		ICompletionDataList completionDataList;
+		public ICompletionDataList CompletionDataList {
+			get { return this.completionDataList; }
+			set { this.completionDataList = value; }
+		}
+		
+		public CompletionListWindow ()
 		{
 			SizeAllocated += new SizeAllocatedHandler (ListSizeChanged);
+			Events = Gdk.EventMask.PropertyChangeMask;
 			WindowTransparencyDecorator.Attach (this);
+			DataProvider = this;
 		}
 		
 		protected override void OnDestroyed ()
@@ -103,101 +81,127 @@ namespace MonoDevelop.Projects.Gui.Completion
 			base.OnDestroyed ();
 		}
 
-		public bool ProcessKeyEvent (Gdk.Key key, Gdk.ModifierType modifier)
+		public void PostProcessKeyEvent (KeyActions ka)
 		{
-			ListWindow.KeyAction ka = ProcessKey (key, modifier);
+			if ((ka & KeyActions.Complete) != 0) 
+				CompleteWord ();
+		}
+		
+		public bool PreProcessKeyEvent (Gdk.Key key, char keyChar, Gdk.ModifierType modifier, out KeyActions ka)
+		{
+			ka = ProcessKey (key, keyChar, modifier);
 			
-			if ((ka & ListWindow.KeyAction.CloseWindow) != 0)
-				Hide ();
-				
-			if ((ka & ListWindow.KeyAction.Complete) != 0) {
-				UpdateWord ();
+			if ((ka & KeyActions.Complete) != 0) {
+				CompleteWord ();
 			}
-			
-			if ((ka & ListWindow.KeyAction.Ignore) != 0)
+
+			if ((ka & KeyActions.CloseWindow) != 0)
+				CompletionWindowManager.HideWindow ();
+
+			if ((ka & KeyActions.Ignore) != 0)
 				return true;
 
-			if ((ka & ListWindow.KeyAction.Process) != 0) {
+			if ((ka & KeyActions.Process) != 0) {
 				if (key == Gdk.Key.Left || key == Gdk.Key.Right) {
 					// Close if there's a modifier active EXCEPT lock keys and Modifiers
-					// Makes an exception for Mod1Mask (usually alt), shift and control
+					// Makes an exception for Mod1Mask (usually alt), shift, control, meta and super
 					// This prevents the window from closing if the num/scroll/caps lock are active
 					// FIXME: modifier mappings depend on X server settings
-					if ((modifier & ~(Gdk.ModifierType.LockMask | (Gdk.ModifierType.ModifierMask 
-					    & ~Gdk.ModifierType.Mod1Mask & ~Gdk.ModifierType.ControlMask & ~Gdk.ModifierType.ShiftMask))
-					) != 0) {
-						Hide ();
+					if ((modifier & ~(Gdk.ModifierType.LockMask | (Gdk.ModifierType.ModifierMask & ~(Gdk.ModifierType.ShiftMask | Gdk.ModifierType.Mod1Mask | Gdk.ModifierType.ControlMask | Gdk.ModifierType.MetaMask | Gdk.ModifierType.SuperMask)))) != 0) {
+						CompletionWindowManager.HideWindow ();
 						return false;
 					}
-					
+
 					if (declarationviewwindow.Multiple) {
 						if (key == Gdk.Key.Left)
 							declarationviewwindow.OverloadLeft ();
 						else
 							declarationviewwindow.OverloadRight ();
 						UpdateDeclarationView ();
+					} else {
+						CompletionWindowManager.HideWindow ();
+						return false;
 					}
 					return true;
 				}
+				if (completionDataList != null && completionDataList.CompletionSelectionMode == CompletionSelectionMode.OwnTextField)
+					return true;
 			}
-
 			return false;
 		}
 		
-		internal bool ShowListWindow (char firstChar, ICompletionDataList list, ICompletionWidget completionWidget,
-		                              ICodeCompletionContext completionContext, CompletionDelegate closedDelegate)
+		public override void SelectEntry (string s)
+		{
+			base.SelectEntry (s);
+			UpdateDeclarationView ();
+		}
+		
+		internal bool ShowListWindow (char firstChar, ICompletionDataList list, ICompletionWidget completionWidget, CodeCompletionContext completionContext, System.Action closedDelegate)
 		{
 			if (mutableList != null) {
 				mutableList.Changing -= OnCompletionDataChanging;
 				mutableList.Changed -= OnCompletionDataChanged;
 				HideFooter ();
 			}
-			
 			//initialWordLength = 0;
 			this.completionDataList = list;
-			this.completionContext = completionContext;
+			this.CompleteWithSpaceOrPunctuation = MonoDevelop.Core.PropertyService.Get ("CompleteWithSpaceOrPunctuation", true);
+			
+			this.CodeCompletionContext = completionContext;
 			this.closedDelegate = closedDelegate;
 			mutableList = completionDataList as IMutableCompletionDataList;
-			
+			List.PreviewCompletionString = completionDataList.CompletionSelectionMode == CompletionSelectionMode.OwnTextField;
+
 			if (mutableList != null) {
 				mutableList.Changing += OnCompletionDataChanging;
 				mutableList.Changed += OnCompletionDataChanged;
-			
+
 				if (mutableList.IsChanging)
 					OnCompletionDataChanging (null, null);
 			}
-			
-			this.completionWidget = completionWidget;
+
+			this.CompletionWidget = completionWidget;
 
 			if (FillList ()) {
-				Reset (true);
-				
+// not neccessarry, because list window is not reused anymore:
+//				Reset (true);
+				this.AutoSelect = list.AutoSelect;
+				this.AutoCompleteEmptyMatch = list.AutoCompleteEmptyMatch;
 				// makes control-space in midle of words to work
 				string text = completionWidget.GetCompletionText (completionContext);
+				DefaultCompletionString = completionDataList.DefaultCompletionString ?? "";
 				if (text.Length == 0) {
-					text = completionDataList.DefaultCompletionString;
-					SelectEntry (text);
+					UpdateWordSelection ();
 					initialWordLength = completionWidget.SelectedLength;
-					Show ();
+					ResetSizes ();
+					ShowAll ();
+					SetScrollbarVisibilty ();
+					//if there is only one matching result we take it by default
+					if (completionDataList.AutoCompleteUniqueMatch && IsUniqueMatch && !IsChanging) {
+						CompleteWord ();
+						CompletionWindowManager.HideWindow ();
+					}
 					return true;
 				}
-				
+
 				initialWordLength = text.Length + completionWidget.SelectedLength;
-				PartialWord = text; 
+				PartialWord = text;
+				UpdateWordSelection ();
+				
 				//if there is only one matching result we take it by default
-				if (completionDataList.AutoCompleteUniqueMatch && IsUniqueMatch && !IsChanging)
-				{	
-					UpdateWord ();
-					Hide ();
+				if (completionDataList.AutoCompleteUniqueMatch && IsUniqueMatch && !IsChanging) {
+					CompleteWord ();
+					CompletionWindowManager.HideWindow ();
 				} else {
-					Show ();
+					ResetSizes ();
+					ShowAll ();
+					SetScrollbarVisibilty ();
 				}
 				return true;
 			}
-			else {
-				Hide ();
-				return false;
-			}
+			CompletionWindowManager.HideWindow ();
+			
+			return false;
 		}
 		
 		class DataItemComparer : IComparer<ICompletionData>
@@ -206,7 +210,7 @@ namespace MonoDevelop.Projects.Gui.Completion
 			{
 				return ((a.DisplayFlags & DisplayFlags.Obsolete) == (b.DisplayFlags & DisplayFlags.Obsolete))
 					? StringComparer.OrdinalIgnoreCase.Compare (a.DisplayText, b.DisplayText)
-					: (a.DisplayFlags & DisplayFlags.Obsolete) != 0? 1 : -1;
+					: (a.DisplayFlags & DisplayFlags.Obsolete) != 0 ? 1 : -1;
 			}
 		}
 		
@@ -215,14 +219,13 @@ namespace MonoDevelop.Projects.Gui.Completion
 			if ((completionDataList.Count == 0) && !IsChanging)
 				return false;
 			
-			this.Style = completionWidget.GtkStyle;
+			this.Style = CompletionWidget.GtkStyle;
 			
 			//sort, sinking obsolete items to the bottoms
 			//the string comparison is ordinal as that makes it an order of magnitude faster, which 
 			//which makes completion triggering noticeably more responsive
-			completionDataList.Sort (new DataItemComparer ());
-			
-			DataProvider = this;
+			if (!completionDataList.IsSorted)
+				completionDataList.Sort (new DataItemComparer ());
 			
 			Reposition (true);
 			
@@ -231,27 +234,28 @@ namespace MonoDevelop.Projects.Gui.Completion
 		
 		void Reposition (bool force)
 		{
-			int x = completionContext.TriggerXCoord - TextOffset;
-			int y = completionContext.TriggerYCoord;
-			
+			int x = CodeCompletionContext.TriggerXCoord - TextOffset;
+			int y = CodeCompletionContext.TriggerYCoord;
+
 			int w, h;
 			GetSize (out w, out h);
-			
+
 			if (!force && previousHeight != h && previousWidth != w)
 				return;
-			
+
 			previousHeight = h;
 			previousWidth = w;
-			
-			if ((x + w) > Screen.Width)
+
+			if (x + w > Screen.Width)
 				x = Screen.Width - w;
-			
-			if ((y + h) > Screen.Height)
-			{
-				y = y - completionContext.TriggerTextHeight - h;
+
+			if (y + h > Screen.Height) {
+				y = y - CodeCompletionContext.TriggerTextHeight - h;
 			}
-			
+			curXPos = x;
+			curYPos = y;
 			Move (x, y);
+			UpdateDeclarationView ();
 		}
 		
 		//smaller lists get size reallocated after FillList, so we have to reposition them
@@ -259,53 +263,56 @@ namespace MonoDevelop.Projects.Gui.Completion
 		protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 		{
 			base.OnSizeAllocated (allocation);
-			Reposition (false);
+			Reposition (true);
 		}
 		
-		void UpdateWord ()
+		
+		public void CompleteWord ()
 		{
-			if (Selection == -1 || SelectionDisabled)
+			if (SelectionIndex == -1 || completionDataList == null)
 				return;
-			
-			ICompletionData item = currentData ?? completionDataList[Selection];
+			ICompletionData item = completionDataList[SelectionIndex];
 			if (item == null)
 				return;
-			string word = item.CompletionText;
+			
 			IActionCompletionData ac = item as IActionCompletionData;
 			if (ac != null) {
-				ac.InsertCompletionText (completionWidget, completionContext);
+				ac.InsertCompletionText (CompletionWidget, CodeCompletionContext);
 				return;
 			}
-			string partialWord = PartialWord;
-			int partialWordLength = partialWord != null ? partialWord.Length : 0;
-			int replaceLen = completionContext.TriggerWordLength + partialWordLength - initialWordLength;
-			string pword = completionWidget.GetText (completionContext.TriggerOffset, completionContext.TriggerOffset + replaceLen);
 			
-			completionWidget.SetCompletionText (completionContext, pword, word);
+			int partialWordLength = PartialWord != null ? PartialWord.Length : 0;
+			int replaceLength = CodeCompletionContext.TriggerWordLength + partialWordLength - initialWordLength;
+			string currentWord   = CompletionWidget.GetText (CodeCompletionContext.TriggerOffset, CodeCompletionContext.TriggerOffset + replaceLength);
+			string completedWord = item.CompletionText;
+			AddWordToHistory (completedWord);
+			CompletionWidget.SetCompletionText (CodeCompletionContext, currentWord, completedWord);
 		}
 		
-		public new void Hide ()
+		public override void Destroy ()
 		{
-			base.Hide ();
-			declarationviewwindow.HideAll ();
+			if (declarationviewwindow != null) {
+				declarationviewwindow.Destroy ();
+				declarationviewwindow = null;
+			}
 			
 			if (mutableList != null) {
 				mutableList.Changing -= OnCompletionDataChanging;
 				mutableList.Changed -= OnCompletionDataChanged;
 				mutableList = null;
 			}
-			
+
 			if (completionDataList != null) {
-				if (completionDataList is IDisposable) {
+				if (completionDataList is IDisposable) 
 					((IDisposable)completionDataList).Dispose ();
-				}
 				completionDataList = null;
 			}
-			
+
 			if (closedDelegate != null) {
 				closedDelegate ();
 				closedDelegate = null;
 			}
+			base.Destroy ();
 		}
 		
 		void ListSizeChanged (object obj, SizeAllocatedArgs args)
@@ -315,8 +322,8 @@ namespace MonoDevelop.Projects.Gui.Completion
 
 		protected override void DoubleClick ()
 		{
-			UpdateWord ();
-			Hide ();
+			CompleteWord ();
+			CompletionWindowManager.HideWindow ();
 		}
 		
 		protected override void OnSelectionChanged ()
@@ -330,26 +337,39 @@ namespace MonoDevelop.Projects.Gui.Completion
 			if (completionDataList == null || List.Selection >= completionDataList.Count || List.Selection == -1)
 				return;
 
-			if (List.GdkWindow == null) return;
+			if (List.GdkWindow == null)
+				return;
 			Gdk.Rectangle rect = List.GetRowArea (List.Selection);
 			int listpos_x = 0, listpos_y = 0;
 			while (listpos_x == 0 || listpos_y == 0)
 				GetPosition (out listpos_x, out listpos_y);
 			int vert = listpos_y + rect.Y;
-			
+
 			int lvWidth = 0, lvHeight = 0;
 			while (lvWidth == 0)
 				this.GdkWindow.GetSize (out lvWidth, out lvHeight);
+			
 			if (vert >= listpos_y + lvHeight - 2) {
 				vert = listpos_y + lvHeight - rect.Height;
 			} else if (vert < listpos_y) {
 				vert = listpos_y;
 			}
 
-			ICompletionData data = completionDataList[List.Selection];
+			// no selection, try to find a selection
+			if (List.SelectionIndex < 0 || List.SelectionIndex >= completionDataList.Count) {
+				List.CompletionString = PartialWord;
+				bool hasMismatches;
+				List.Selection = FindMatchedEntry (List.CompletionString, out hasMismatches);
+			}
+			// no success, hide declaration view
+			if (List.SelectionIndex < 0 || List.SelectionIndex >= completionDataList.Count) {
+				declarationviewwindow.Hide ();
+				return;
+			}
+			ICompletionData data = completionDataList[List.SelectionIndex];
 			IOverloadedCompletionData overloadedData = data as IOverloadedCompletionData;
-			
-			IList<ICompletionData> overloads; 
+
+			IList<ICompletionData> overloads;
 			if (overloadedData != null) {
 				overloads = new List<ICompletionData> (overloadedData.GetOverloadedData ());
 			} else {
@@ -373,18 +393,21 @@ namespace MonoDevelop.Projects.Gui.Completion
 			currentData = data;
 			if (declarationviewwindow.DescriptionMarkup.Length == 0)
 				return;
-
-			int dvwWidth, dvwHeight;
-
-			declarationviewwindow.Move (this.Screen.Width+1, vert);
+			
+			declarationviewwindow.Move (this.Screen.Width + 1, vert);
 			
 			declarationviewwindow.SetFixedWidth (-1);
 			declarationviewwindow.ReshowWithInitialSize ();
 			declarationviewwindow.ShowAll ();
 			declarationviewwindow.Multiple = (overloadedData != null && overloadedData.IsOverloaded);
 
-			declarationviewwindow.GdkWindow.GetSize (out dvwWidth, out dvwHeight);
-
+			Requisition req = declarationviewwindow.SizeRequest ();
+			int dvwWidth = req.Width;
+//			int dvwHeight = req.Height;
+			
+//			int dvwWidth, dvwHeight;
+//			declarationviewwindow.GdkWindow.GetSize (out dvwWidth, out dvwHeight);
+		
 			int horiz = listpos_x + lvWidth + declarationWindowMargin;
 			if (this.Screen.Width - horiz >= lvWidth) {
 				if (this.Screen.Width - horiz < dvwWidth)
@@ -394,7 +417,7 @@ namespace MonoDevelop.Projects.Gui.Completion
 					declarationviewwindow.SetFixedWidth (listpos_x - declarationWindowMargin);
 					dvwWidth = declarationviewwindow.SizeRequest ().Width;
 				}
-				horiz = listpos_x - dvwWidth - declarationWindowMargin;
+				horiz = curXPos - dvwWidth - declarationWindowMargin;
 			}
 
 			declarationviewwindow.Move (horiz, vert);
@@ -433,7 +456,7 @@ namespace MonoDevelop.Projects.Gui.Completion
 			string iconName = completionDataList[n].Icon;
 			if (string.IsNullOrEmpty (iconName))
 				return null;
-			return MonoDevelop.Core.Gui.Services.Resources.GetIcon (iconName, Gtk.IconSize.Menu);
+			return ImageService.GetPixbuf (iconName, Gtk.IconSize.Menu);
 		}
 		
 		#endregion
@@ -461,25 +484,20 @@ namespace MonoDevelop.Projects.Gui.Completion
 		
 		void OnCompletionDataChanged (object s, EventArgs args)
 		{
+			ResetSizes ();
 			//try to capture full selection state so as not to interrupt user
 			string last = null;
-			if (Visible) {
-				if (SelectionDisabled)
-					last = PartialWord;
-				else
-					last = CompleteWord;
-			}
+			if (Visible)
+				last = List.AutoSelect ? CurrentCompletionText : PartialWord;
 
 			HideFooter ();
 			if (Visible) {
 				//don't reset the user-entered word when refilling the list
 				Reset (false);
 				FillList ();
-				if (last != null)
+				if (last != null )
 					SelectEntry (last);
 			}
 		}
 	}
-	
-	public delegate void CompletionDelegate ();
 }
