@@ -35,26 +35,8 @@ namespace MonoDevelop.Projects.Gui
 {
 	public class Services
 	{
-		static IconService icons;
-
-		internal static LanguageBindingService Languages {
-			get { return MonoDevelop.Projects.Services.Languages; }
-		}
-	
 		internal static ProjectService ProjectService {
 			get { return MonoDevelop.Projects.Services.ProjectService; }
 		}
-	
-		internal static ResourceService Resources {
-			get { return MonoDevelop.Core.Gui.Services.Resources; }
-		}
-	
-		public static IconService Icons {
-			get {
-				if (icons == null)
-					icons = new IconService ();
-				return icons;
-			}
-		}
 	}
 }
