@@ -38,10 +38,11 @@ using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Projects.CodeGeneration;
+using System.Text;
 
 namespace MonoDevelop.Projects.CodeGeneration
 {
-	public abstract class BaseRefactorer: IRefactorer
+	public abstract class BaseRefactorer: IRefactorer, INameValidator
 	{
 		public virtual RefactorOperations SupportedOperations {
 			get { return RefactorOperations.All ^ RefactorOperations.AddFoldingRegion; }
@@ -100,7 +101,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		public virtual IEnumerable<MemberReference> FindClassReferences (RefactorerContext ctx, string file, IType cls)
+		public virtual IEnumerable<MemberReference> FindClassReferences (RefactorerContext ctx, string file, IType cls, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -108,17 +109,17 @@ namespace MonoDevelop.Projects.CodeGeneration
 		public virtual IMember AddMember (RefactorerContext ctx, IType cls, CodeTypeMember member)
 		{
 			IEditableTextFile buffer = ctx.GetFile (cls.CompilationUnit.FileName);
-			// update class information,
-			cls = GetGeneratedClass (ctx, buffer, cls);
+			
 			int pos = GetNewMemberPosition (buffer, cls, member);
 			string code = GenerateCodeFromMember (member);
 			
 			int line, col;
 			buffer.GetLineColumnFromPosition (pos, out line, out col);
 			
-			string indent = GetLineIndent (buffer, line);
+			string indent = GetLineIndent (buffer, cls.Location.Line) + "\t";
 			code = Indent (code, indent, false);
-			
+//			code = code.Trim (' ', '\t');
+			code += "\n";
 			buffer.InsertText (pos, code);
 			
 			return FindGeneratedMember (ctx, buffer, cls, member, line);
@@ -134,8 +135,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			}
 		}
 		
-		public virtual void AddMembers (RefactorerContext ctx, IType cls, 
-		                                                IEnumerable<CodeTypeMember> members, string foldingRegionName)
+		public virtual void AddMembers (RefactorerContext ctx, IType cls, IEnumerable<CodeTypeMember> members, string foldingRegionName)
 		{
 			//no region name, so distribute them with like members
 			if (string.IsNullOrEmpty (foldingRegionName)) {
@@ -156,7 +156,6 @@ namespace MonoDevelop.Projects.CodeGeneration
 			} else {
 				pos = AddFoldingRegion (ctx, cls, foldingRegionName);
 			}
-			
 			AddMembersAtPosition (ctx, cls, members, buffer, pos);
 		}
 		
@@ -167,22 +166,22 @@ namespace MonoDevelop.Projects.CodeGeneration
 			buffer.GetLineColumnFromPosition (pos, out line, out col);
 			
 			string indent = GetLineIndent (buffer, line);
-			List<int> positions = new List<int> ();
 			
-			bool first = true;
+			StringBuilder generatedString = new StringBuilder ();
+			bool isFirst = true;
 			foreach (CodeTypeMember member in members) {
-				positions.Add (pos);
-				string code = GenerateCodeFromMember (member);
-					//spacing between inserted members
-				if (first)
-					first = false;
-				else
-					code = "\n\n" + code;
-				
-				code = Indent (code, indent, false);
-				buffer.InsertText (pos, code);
-				pos += code.Length;
+				generatedString.Append (Indent (GenerateCodeFromMember (member), indent, isFirst));
+				generatedString.AppendLine ();
+				generatedString.Append (indent);
+				isFirst = false;
 			}
+			// remove last new line + indent
+			generatedString.Length -= indent.Length + Environment.NewLine.Length;
+			// remove indent from last generated code member
+			generatedString.Length -= indent.Length;
+			if (buffer.GetCharAt (pos) == '\n')
+				pos++;
+			buffer.InsertText (pos, generatedString.ToString ());
 		}
 		
 		public virtual int AddFoldingRegion (RefactorerContext ctx, IType cls, string regionName)
@@ -527,16 +526,16 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return FindGeneratedMember (ctx, file, cls, memberInfo, member.Location.Line);
 		}
 		
-		public virtual IEnumerable<MemberReference> FindMemberReferences (RefactorerContext ctx, string fileName, IType cls, IMember member)
+		public virtual IEnumerable<MemberReference> FindMemberReferences (RefactorerContext ctx, string fileName, IType cls, IMember member, bool includeXmlComment)
 		{
 			if (member is IField)
-				return FindFieldReferences (ctx, fileName, cls, (IField) member);
+				return FindFieldReferences (ctx, fileName, cls, (IField) member, includeXmlComment);
 			else if (member is IMethod)
-				return FindMethodReferences (ctx, fileName, cls, (IMethod) member);
+				return FindMethodReferences (ctx, fileName, cls, (IMethod) member, includeXmlComment);
 			else if (member is IProperty)
-				return FindPropertyReferences (ctx, fileName, cls, (IProperty) member);
+				return FindPropertyReferences (ctx, fileName, cls, (IProperty) member, includeXmlComment);
 			else if (member is IEvent)
-				return FindEventReferences (ctx, fileName, cls, (IEvent) member);
+				return FindEventReferences (ctx, fileName, cls, (IEvent) member, includeXmlComment);
 			else
 				return null;
 		}
@@ -620,7 +619,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		protected virtual IEnumerable<MemberReference> FindMethodReferences (RefactorerContext ctx, string fileName, IType cls, IMethod method)
+		protected virtual IEnumerable<MemberReference> FindMethodReferences (RefactorerContext ctx, string fileName, IType cls, IMethod method, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -633,7 +632,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		protected virtual IEnumerable<MemberReference> FindFieldReferences (RefactorerContext ctx, string fileName, IType cls, IField field)
+		protected virtual IEnumerable<MemberReference> FindFieldReferences (RefactorerContext ctx, string fileName, IType cls, IField field, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -646,7 +645,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		protected virtual IEnumerable<MemberReference> FindPropertyReferences (RefactorerContext ctx, string fileName, IType cls, IProperty property)
+		protected virtual IEnumerable<MemberReference> FindPropertyReferences (RefactorerContext ctx, string fileName, IType cls, IProperty property, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -658,7 +657,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		protected virtual IEnumerable<MemberReference> FindEventReferences (RefactorerContext ctx, string fileName, IType cls, IEvent evnt)
+		protected virtual IEnumerable<MemberReference> FindEventReferences (RefactorerContext ctx, string fileName, IType cls, IEvent evnt, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -703,7 +702,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			int pos = -1;
 			
 			// It'd be nice if we didn't have to worry about this being null
-			if (member.DeclaringType.CompilationUnit.FileName != null) {
+			if (!member.DeclaringType.CompilationUnit.FileName.IsNull) {
 				if ((file = ctx.GetFile (member.DeclaringType.CompilationUnit.FileName)) != null)
 					pos = GetParameterNamePosition (file, param);
 			}
@@ -740,7 +739,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return true;
 		}
 
-		public virtual IEnumerable<MemberReference> FindParameterReferences (RefactorerContext ctx, string fileName, IParameter param)
+		public virtual IEnumerable<MemberReference> FindParameterReferences (RefactorerContext ctx, string fileName, IParameter param, bool includeXmlComment)
 		{
 			return null;
 		}
@@ -795,6 +794,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 			CodeDomProvider provider = GetCodeDomProvider ();
 			StringWriter sw = new StringWriter ();
 			provider.GenerateCodeFromType (type, sw, GetOptions (member is CodeMemberMethod));
+			
 			string code = sw.ToString ();
 			int i = code.IndexOf ('{');
 			int j = code.LastIndexOf ('}');
@@ -802,7 +802,23 @@ namespace MonoDevelop.Projects.CodeGeneration
 			if (member is CodeMemberMethod)
 				if ((i = code.IndexOf ('(')) != -1)
 					code = code.Insert (i, " ");
-			return RemoveIndent (code);
+			
+			code = code.TrimEnd ('\n', '\r', ' ', '\t');
+			
+			// remove empty preceeding lines
+			string eol;
+			string[] lines = SplitLines (code, out eol);
+			
+			int firstLine = -1;
+			for (int k = 0; k < lines.Length; k++) {
+				if (lines[k].Trim ().Length != 0)
+					break;
+				firstLine = k + 1;
+			}
+			if (firstLine >= 0) 
+				code = String.Join (eol, lines, firstLine, lines.Length - firstLine);
+			
+			return RemoveIndent (code) + eol;
 		}
 		
 
@@ -814,13 +830,12 @@ namespace MonoDevelop.Projects.CodeGeneration
 			// Don't get the class from the parse results because in that class the types are not resolved.
 			// Get the class from the database instead.
 			ParsedDocument doc = ProjectDomService.Parse (ctx.ParserContext.Project, buffer.Name, null, delegate () { return buffer.Text; });
-			IType result =  ctx.ParserContext.GetType (cls.FullName, null, true, true);
+			IType result = ctx.ParserContext.GetType (cls.FullName, cls.TypeParameters.Count, true, true);
 			if (result is CompoundType) {
 				IType hintType = doc.CompilationUnit.GetType (cls.FullName, cls.TypeParameters.Count);
 				if (hintType != null)
 					((CompoundType)result).SetMainPart (buffer.Name, hintType.Location);
 			}
-			
 			return result;
 		}
 		
@@ -851,54 +866,99 @@ namespace MonoDevelop.Projects.CodeGeneration
 			return null;
 		}
 		
-		protected string RemoveIndent (string code)
+		static string[] SplitLines (string code, out string eol)
 		{
-			string[] lines = code.Split ('\n');
-			int minInd = int.MaxValue;
+			List<string> lines = new List<string> ();
+			eol = null;
+			int lastLineOffset = 0;
+			for (int i = 0; i < code.Length; i++) {
+				int additionalEolChars = 0;
+				switch (code[i]) {
+				case '\r':
+					if (i + 1 < code.Length && code[i + 1] == '\n') {
+						i++;
+						if (eol == null)
+							eol = "\r\n";
+						additionalEolChars = 1;
+					}
+					if (eol == null)
+						eol = "\r";
+					break;
+				case '\n':
+					if (eol == null)
+						eol = "\n";
+					break;
+				default:
+					continue;
+				}
+				lines.Add (code.Substring (lastLineOffset, i - lastLineOffset - additionalEolChars));
+				lastLineOffset = i + 1;
+			}
+			if (lastLineOffset < code.Length)
+				lines.Add (code.Substring (lastLineOffset, code.Length - lastLineOffset));
+			if (eol == null)
+				eol = Environment.NewLine;
+			return lines.ToArray ();
+		}
+		
+		static int GetBlockIndent (IEnumerable<string> lines) 
+		{
+			int result = int.MaxValue;
 			
-			for (int n=0; n<lines.Length; n++) {
-				string line = lines [n];
-				for (int i=0; i<line.Length; i++) {
-					char c = line [i];
-					if (c != ' ' && c != '\t') {
-						if (i < minInd)
-							minInd = i;
+			foreach (string line in lines) {
+//				System.Console.WriteLine (">" + line.Replace("\n", "\\n").Replace ("\r", "\\r").Replace ("\t", "\\t"));
+				for (int i = 0; i < line.Length; i++) {
+					char ch = line[i];
+					if (ch != ' ' && ch != '\t') {
+						if (i < result)
+							result = i;
 						break;
 					}
 				}
 			}
 			
-			if (minInd == int.MaxValue)
-				minInd = 0;
-			
-			int firstLine = -1, lastLine = -1;
-			
-			for (int n=0; n<lines.Length; n++) {
-				if (minInd >= lines[n].Length)
-					continue;
-					
-				if (lines[n].Trim (' ','\t') != "") {
-					if (firstLine == -1)
-						firstLine = n;
-					lastLine = n;
-				}
-				
-				lines [n] = lines [n].Substring (minInd);
-			}
-			
-			if (firstLine == -1)
-				return "";
-			
-			return string.Join ("\n", lines, firstLine, lastLine - firstLine + 1);
+			return result == int.MaxValue ? 0 : result;
 		}
 		
-		protected string Indent (string code, string indent, bool indentFirstLine)
+		public static string RemoveIndent (string code)
 		{
-			code = code.Replace ("\n", "\n" + indent);
+			string eol;
+			string[] lines = SplitLines (code, out eol);
+			int minInd = GetBlockIndent (lines);
+			
+			for (int i = 0; i < lines.Length; i++) {
+				if (minInd > lines[i].Length)
+					continue;
+				
+				lines[i] = lines[i].Substring (minInd);
+			}
+			
+			StringBuilder result = new StringBuilder ();
+			foreach (string line in lines) {
+				result.Append (line);
+				result.Append (eol);
+			}
+			return result.ToString ();
+		}
+		
+		public static string Indent (string code, string indent, bool indentFirstLine)
+		{
+			StringBuilder result = new StringBuilder ();
 			if (indentFirstLine)
-				return indent + code;
-			else
-				return code;
+				result.Append (indent);
+			for (int i = 0; i < code.Length; i++) {
+				result.Append (code[i]);
+				if (code[i] == '\r') {
+					if (i + 1 < code.Length && code[i + 1] == '\n') {
+						result.Append ('\n');
+						i++;
+					}
+					result.Append (indent);
+				} else if (code[i] == '\n')
+					result.Append (indent);
+			}
+			
+			return result.ToString ();
 		}
 		
 		protected int EnsurePositionIsNotInRegionsAndIndented (Project p, IEditableTextFile buffer, string indent, int position)
@@ -915,27 +975,30 @@ namespace MonoDevelop.Projects.CodeGeneration
 			}
 			
 			int result = buffer.GetPositionFromLineColumn (line, column);
-			string eolMarker = Environment.NewLine;
-			buffer.InsertText (result, eolMarker);
-			result += eolMarker.Length;
+			
+			if (column != 1) {
+				string eolMarker = Environment.NewLine;
+				buffer.InsertText (result, eolMarker);
+				result += eolMarker.Length;
+			}
+			
 			buffer.InsertText (result, indent);
-			buffer.GetLineColumnFromPosition (result + indent.Length, out line, out column);
-
-			return result + indent.Length;
+			result += indent.Length;
+			
+			return result;
 		}
 		
 		protected virtual int GetNewMemberPosition (IEditableTextFile buffer, IType cls, CodeTypeMember member)
 		{
 			if (member is CodeMemberField)
 				return GetNewFieldPosition (buffer, cls);
-			else if (member is CodeMemberMethod)
-				return  GetNewMethodPosition (buffer, cls);
-			else if (member is CodeMemberEvent)
+			if (member is CodeMemberMethod)
+				return GetNewMethodPosition (buffer, cls);
+			if (member is CodeMemberEvent)
 				return GetNewEventPosition (buffer, cls);
-			else if (member is CodeMemberProperty)
+			if (member is CodeMemberProperty)
 				return GetNewPropertyPosition (buffer, cls);
-			else
-				throw new InvalidOperationException ("Invalid member type: " + member);
+			throw new InvalidOperationException ("Invalid member type: " + member);
 		}
 		
 		protected static IType GetMainPart (IType t)
@@ -973,7 +1036,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 					string s = buffer.GetText (sp, ep);
 					int i = s.IndexOf ('}');
 					if (i == -1) return -1;
-					buffer.InsertText (sp + i, "\n" + ind + "\t\n" + ind);
+//					buffer.InsertText (sp + i, "\n" + ind + "\t\n" + ind);
 					pos = sp + i + ind.Length + 2;
 				} else {
 					pos = GetNextLine (buffer, pos);
@@ -1000,15 +1063,19 @@ namespace MonoDevelop.Projects.CodeGeneration
 				int pos;
 				if (!m.BodyRegion.IsEmpty && m.BodyRegion.End.Line > 0) {
 					pos = buffer.GetPositionFromLineColumn (m.BodyRegion.End.Line, m.BodyRegion.End.Column);
+					pos = GetNextLine (buffer, pos);
+					pos = SkipBlankLine (buffer, pos);
 				} else {
 					// Abstract or P/Inboke methods don't have a body
 					pos = buffer.GetPositionFromLineColumn (m.Location.Line, m.Location.Column);
 					pos = GetNextLine (buffer, pos);
 				}
 				
-				buffer.InsertText (pos++, "\n");
+//				buffer.InsertText (pos++, "\n");
 				string ind = GetLineIndent (buffer, m.Location.Line);
-				return EnsurePositionIsNotInRegionsAndIndented (cls.SourceProject as Project, buffer, ind, pos);
+				pos = EnsurePositionIsNotInRegionsAndIndented (cls.SourceProject as Project, buffer, ind, pos);
+				
+				return pos;
 			}
 		}
 		
@@ -1027,7 +1094,8 @@ namespace MonoDevelop.Projects.CodeGeneration
 				IProperty m = cls.Properties.Last ();
 				
 				int pos = buffer.GetPositionFromLineColumn (m.BodyRegion.End.Line, m.BodyRegion.End.Column);
-				buffer.InsertText (pos++, "\n");
+				pos = GetNextLine (buffer, pos);
+				pos = SkipBlankLine (buffer, pos);
 				string indent = GetLineIndent (buffer, m.Location.Line);
 				return EnsurePositionIsNotInRegionsAndIndented (cls.SourceProject as Project, buffer, indent, pos);
 			}
@@ -1048,34 +1116,65 @@ namespace MonoDevelop.Projects.CodeGeneration
 				IEvent m = GetMainPart (cls).Events.Last ();
 				
 				int pos;
-				if (!m.BodyRegion.IsEmpty)
+				if (!m.BodyRegion.IsEmpty) {
 					pos = buffer.GetPositionFromLineColumn (m.BodyRegion.End.Line, m.BodyRegion.End.Column);
-				else {
+					pos = GetNextLine (buffer, pos);
+					pos = SkipBlankLine (buffer, pos);
+				} else {
 					pos = buffer.GetPositionFromLineColumn (m.Location.Line, m.Location.Column);
 					pos = GetNextLine (buffer, pos);
 				}
 
-				buffer.InsertText (pos++, "\n");
+//				buffer.InsertText (pos++, "\n");
 				string ind = GetLineIndent (buffer, m.Location.Line);
 				return EnsurePositionIsNotInRegionsAndIndented (cls.SourceProject as Project, buffer, ind, pos);
 			}
 		}
 		
+		protected virtual int SkipBlankLine (IEditableTextFile buffer, int pos)
+		{
+			int i = pos;
+			while (i < buffer.Length) {
+				char ch = buffer.GetCharAt (i);
+				switch (ch) {
+				case '\n':
+					return i + 1;
+				case '\r':
+					if (i + 1 < buffer.Length && buffer.GetCharAt (i + 1) == '\n')
+						i++;
+					return i + 1;
+				case ' ':
+				case '\t':
+					i++;
+					break;
+				default:
+					return pos;
+				}
+			}
+			return pos;
+		}
+		
 		protected virtual int GetNextLine (IEditableTextFile buffer, int pos)
 		{
+			if (pos < 0)
+				return 0;
 			while (pos < buffer.Length) {
-				string s = buffer.GetText (pos, pos + 1);
-				if (s == "\n") {
-//					buffer.InsertText (pos + 1, "\n");
+				char ch = buffer.GetCharAt (pos);
+				switch (ch) {
+				case '\n':
 					return pos + 1;
-				}
-				if (s != " " && s == "\t") {
-//					buffer.InsertText (pos, "\n\n");
+				case '\r':
+					if (pos + 1 < buffer.Length && buffer.GetCharAt (pos + 1) == '\n')
+						pos++;
 					return pos + 1;
+/*				case ' ':
+				case '\t':
+					pos++;
+					break;*/
+				default:
+					pos++;
+					continue;
 				}
-				if (s == "}")
-					return pos;
-				pos++;
 			}
 			return pos;
 		}
@@ -1083,13 +1182,17 @@ namespace MonoDevelop.Projects.CodeGeneration
 		protected string GetLineIndent (IEditableTextFile buffer, int line)
 		{
 			int pos = buffer.GetPositionFromLineColumn (line, 1);
-			int ipos = pos;
-			string s = buffer.GetText (pos, pos + 1);
-			while ((s == " " || s == "\t") && pos < buffer.Length) {
-				pos++;
-				s = buffer.GetText (pos, pos + 1);
+			StringBuilder result = new StringBuilder ();
+			while (pos < buffer.Length) {
+				char ch = buffer.GetCharAt (pos);
+				if (ch == ' ' || ch == '\t') {
+					result.Append (ch);
+					pos++;
+					continue;
+				}
+				break;
 			}
-			return buffer.GetText (ipos, pos);
+			return result.ToString ();
 		}
 		
 		protected virtual CodeGeneratorOptions GetOptions (bool isMethod)
@@ -1100,6 +1203,9 @@ namespace MonoDevelop.Projects.CodeGeneration
 				ops.BracingStyle = "C";
 			return ops;
 		}
-
+		public abstract void AddGlobalNamespaceImport (RefactorerContext ctx, string fileName, string nsName);
+		public abstract void AddLocalNamespaceImport (RefactorerContext ctx, string fileName, string nsName, DomLocation caretLocation);
+		public abstract DomLocation CompleteStatement (RefactorerContext ctx, string fileName, DomLocation caretLocation);
+		public abstract ValidationResult ValidateName (INode visitable, string name);
 	}
 }
