@@ -43,34 +43,58 @@ namespace MonoDevelop.Projects.Dom
 		
 		static readonly ReadOnlyCollection<ITypeParameter> emptyParamList = new List<ITypeParameter> ().AsReadOnly ();
 		static readonly ReadOnlyCollection<IReturnType> emptyTypeList = new List<IReturnType> ().AsReadOnly ();
+		static readonly IList<IMember> emptyMembers = new List<IMember> ();
 		
-		List<ITypeParameter> typeParameters      = null;
-		List<IMember> members                   = new List<IMember> ();
-		List<IReturnType> implementedInterfaces = null;
+		protected List<ITypeParameter> typeParameters      = null;
+		List<IMember> members                    = null;
+		List<IReturnType> implementedInterfaces  = null;
 		
 		protected ClassType classType = ClassType.Unknown;
 		protected string nameSpace;
 		
+		public override MemberType MemberType {
+			get {
+				return MemberType.Type;
+			}
+		}
+
+		public virtual TypeKind Kind {
+			get {
+				return TypeKind.Definition;
+			}
+		}
+		
+		public virtual TypeModifier TypeModifier {
+			get;
+			set;
+		}
 		
 		protected override string CalculateFullName ()
 		{
 			base.fullNameIsDirty = false;
 			if (DeclaringType != null) 
 				return DeclaringType.FullName + "." + Name;
-			return !String.IsNullOrEmpty (Namespace) ? Namespace + "." + Name : Name;
+			return !string.IsNullOrEmpty (Namespace) ? Namespace + "." + Name : Name;
 		}
 		
 		public string DecoratedFullName {
 			get {
-				string res;
-				if (DeclaringType != null)
-					res = ((DomType)DeclaringType).DecoratedFullName + "." + Name;
-				else
-					res = FullName;
-				if (TypeParameters.Count > 0)
-					return res + "`" + TypeParameters.Count;
-				else
-					return res;
+				StringBuilder result = new StringBuilder ();
+				if (DeclaringType != null) {
+					result.Append (DeclaringType.DecoratedFullName);
+				} else {
+					if (!string.IsNullOrEmpty (Namespace))
+						result.Append (Namespace);
+				}
+				if (result.Length > 0)
+					result.Append (".");
+				result.Append (Name);
+				if (TypeParameters.Count > 0) {
+					result.Append ('`');
+					result.Append (TypeParameters.Count);
+				}
+				
+				return result.ToString ();
 			}
 		}
 		protected void SetName (string fullName)
@@ -170,14 +194,14 @@ namespace MonoDevelop.Projects.Dom
 		
 		public virtual IEnumerable<IMember> Members {
 			get {
-				return members;
+				return members ?? emptyMembers;
 			}
 		}
 		
 		public virtual IEnumerable<IType> InnerTypes {
 			get {
 				foreach (IMember item in Members)
-					if (item is IType)
+					if (item.MemberType == MemberType.Type)
 						yield return (IType)item;
 			}
 		}
@@ -185,7 +209,7 @@ namespace MonoDevelop.Projects.Dom
 		public virtual IEnumerable<IField> Fields {
 			get {
 				foreach (IMember item in Members)
-					if (item is IField)
+					if (item.MemberType == MemberType.Field)
 						yield return (IField)item;
 			}
 		}
@@ -193,7 +217,7 @@ namespace MonoDevelop.Projects.Dom
 		public virtual IEnumerable<IProperty> Properties {
 			get {
 				foreach (IMember item in Members)
-					if (item is IProperty)
+					if (item.MemberType == MemberType.Property)
 						yield return (IProperty)item;
 			}
 		}
@@ -201,7 +225,7 @@ namespace MonoDevelop.Projects.Dom
 		public virtual IEnumerable<IMethod> Methods {
 			get {
 				foreach (IMember item in Members)
-					if (item is IMethod)
+					if (item.MemberType == MemberType.Method)
 						yield return (IMethod)item;
 			}
 		}
@@ -209,7 +233,7 @@ namespace MonoDevelop.Projects.Dom
 		public virtual IEnumerable<IEvent> Events {
 			get {
 				foreach (IMember item in Members)
-					if (item is IEvent)
+					if (item.MemberType == MemberType.Event)
 						yield return (IEvent)item;
 			}
 		}
@@ -372,11 +396,14 @@ namespace MonoDevelop.Projects.Dom
 		
 		public bool HasOverriden (IMember member)
 		{
-			if (member is IMethod)
-				return HasOverriden (member as IMethod);
-			if (member is IProperty)
-				return HasOverriden (member as IProperty);
-			return false;
+			switch (member.MemberType) {
+				case MemberType.Method:
+					return HasOverriden (member as IMethod);
+				case MemberType.Property:
+					return HasOverriden (member as IProperty);
+				default:
+					return false;
+			}
 		}
 		
 		public bool IsBaseType (IReturnType type)
@@ -411,8 +438,19 @@ namespace MonoDevelop.Projects.Dom
 			}
 			return false;
 		}
-		
-		public static DomType CreateDelegate (ICompilationUnit compilationUnit, string name, DomLocation location, IReturnType type, List<IParameter> parameters)
+
+		public override bool IsAccessibleFrom (MonoDevelop.Projects.Dom.Parser.ProjectDom dom, IType calledType, IMember member, bool includeProtected)
+		{
+			if (calledType != null) {
+				foreach (IType baseType in dom.GetInheritanceTree (calledType)) {
+					if (baseType.FullName == calledType.FullName) 
+						return true;
+				}
+			}
+			return base.IsAccessibleFrom (dom, calledType, member, includeProtected);
+		}
+
+		public static DomType CreateDelegate (ICompilationUnit compilationUnit, string name, DomLocation location, IReturnType type, IEnumerable<IParameter> parameters)
 		{
 			DomType result = new DomType ();
 			result.compilationUnit = compilationUnit;
@@ -420,8 +458,7 @@ namespace MonoDevelop.Projects.Dom
 			result.classType = MonoDevelop.Projects.Dom.ClassType.Delegate;
 			DomMethod delegateMethod = new DomMethod ("Invoke", Modifiers.None, MethodModifier.None, location, DomRegion.Empty, type);
 			delegateMethod.Add (parameters);
-			result.members.Add (delegateMethod);
-			result.methodCount = 1;
+			result.Add (delegateMethod);
 			return result;
 		}
 		
@@ -469,59 +506,40 @@ namespace MonoDevelop.Projects.Dom
 			}
 		}
 		
-		public void Add (IField member)
-		{
-			fieldCount++;
-			((AbstractMember)member).DeclaringType = this;
-			this.members.Add (member);
-		}
-		public void Add (IMethod member)
-		{
-			if (member.IsConstructor) {
-				constructorCount++;
-			} else {
-				methodCount++;
-			}
-			((AbstractMember)member).DeclaringType = this;
-			this.members.Add (member);
-		}
-		public void Add (IProperty member)
-		{
-			if (member.IsIndexer) {
-				indexerCount++;
-			} else {
-				propertyCount++;
-			}
-			((AbstractMember)member).DeclaringType = this;
-			this.members.Add (member);
-		}
-		public void Add (IEvent member)
-		{
-			eventCount++;
-			((AbstractMember)member).DeclaringType = this;
-			this.members.Add (member);
-		}
-		public void Add (IType member)
-		{
-			innerTypeCount++;
-			((AbstractMember)member).DeclaringType = this;
-			this.members.Add (member);
-		}
-		
 		public void Add (IMember member)
 		{
-			if (member is IField)
-				Add ((IField) member);
-			else if (member is IMethod)
-				Add ((IMethod) member);
-			else if (member is IProperty)
-				Add ((IProperty) member);
-			else if (member is IEvent)
-				Add ((IEvent) member);
-			else if (member is IType)
-				Add ((IType) member);
-			else
-				throw new InvalidOperationException ();
+			member.DeclaringType = this;
+			
+			switch (member.MemberType) {
+				case MemberType.Field:
+					fieldCount++;
+					break;
+				case MemberType.Method:
+					if (((IMethod)member).IsConstructor) {
+						constructorCount++;
+					} else {
+						methodCount++;
+					}
+					break;
+				case MemberType.Property:
+					if (((IProperty)member).IsIndexer) {
+						indexerCount++;
+					} else {
+						propertyCount++;
+					}
+					break;
+				case MemberType.Event:
+					eventCount++;
+					break;
+				case MemberType.Type:
+					innerTypeCount++;
+					break;
+				default:
+					throw new InvalidOperationException ();
+			}
+			if (this.members == null)
+				this.members = new List<IMember> ();
+			this.members.Add (member);
 		}
 		
 		protected void ClearInterfaceImplementations ()
@@ -594,19 +612,24 @@ namespace MonoDevelop.Projects.Dom
 			return type.FullName + "~" + type.TypeParameters.Count;
 		}
 		
-		internal static IType CreateInstantiatedGenericTypeInternal (IType type, IList<IReturnType> genericArguments)
+		public static IType CreateInstantiatedGenericTypeInternal (IType type, IList<IReturnType> genericArguments)
 		{
 			// This method is now internal. The public one has been moved to ProjectDom, which take cares of caching
 			// instantiated generic types.
-			
 			if (type is InstantiatedType)
 				return type;
 			string name = GetInstantiatedTypeName (type.Name, genericArguments);
 			GenericTypeInstanceResolver resolver = new GenericTypeInstanceResolver ();
 			if (genericArguments != null) {
-				string fullTypeName = ((DomType)type).DecoratedFullName;
-				for (int i = 0; i < type.TypeParameters.Count && i < genericArguments.Count; i++)
-					resolver.Add (fullTypeName + "." + type.TypeParameters[i].Name, genericArguments[i]);
+				int j = genericArguments.Count - 1;
+				IType curType = type;
+				while (curType != null) {
+					string fullTypeName = curType.DecoratedFullName;
+					for (int i = curType.TypeParameters.Count - 1; i >= 0 && j >= 0; i--, j--) {
+						resolver.Add (fullTypeName + "." + curType.TypeParameters[i].Name, genericArguments[j]);
+					}
+					curType = curType.DeclaringType;
+				}
 			}
 			InstantiatedType result = (InstantiatedType) type.AcceptVisitor (resolver, type);
 			if (result.typeParameters != null)
@@ -617,8 +640,47 @@ namespace MonoDevelop.Projects.Dom
 			result.GenericParameters = genericArguments;
 			result.UninstantiatedType = type;
 			result.DeclaringType = type.DeclaringType;
+			CreateInstantiatedSubtypes (result, type, genericArguments);
 			return result;
 		}
+
+		static void CreateInstantiatedSubtypes (InstantiatedType result, IType curType, IList<IReturnType> genericArguments)
+		{
+			foreach (IType innerType in curType.InnerTypes) {
+
+				List<IReturnType> newArguments = new List<IReturnType> ();
+				List<int> removeInheritedArguments = new List<int> ();
+				for (int i = 0; i < innerType.TypeParameters.Count; i++) {
+					ITypeParameter curParameter = innerType.TypeParameters[i];
+					bool found = false;
+					for (int j = curType.TypeParameters.Count - 1; j >= 0 ; j--) {
+						if (curType.TypeParameters[j].Name == curParameter.Name) {
+							removeInheritedArguments.Add (newArguments.Count);
+							newArguments.Add (genericArguments[j]);
+							found = true;
+							break;
+						}
+					}
+					if (!found)
+						newArguments.Add (new DomReturnType (curParameter.Name));
+				}
+
+				InstantiatedType innerInstantiatedType = (InstantiatedType)CreateInstantiatedGenericTypeInternal (innerType, newArguments);
+				for (int i = 0, j = 0; i < innerInstantiatedType.TypeParameters.Count && j < innerInstantiatedType.TypeParameters.Count; i++,j++) {
+					if (curType.TypeParameters[i].Name == innerInstantiatedType.TypeParameters[j].Name) {
+						innerInstantiatedType.typeParameters.RemoveAt (j);
+						j--;
+					}
+				}
+
+				result.Add (innerInstantiatedType);
+				CreateInstantiatedSubtypes (innerInstantiatedType, innerType, newArguments);
+				foreach (int i in removeInheritedArguments) {
+					if (i >= 0 && i < newArguments.Count)
+						newArguments.RemoveAt (i);
+				}
+			}
+		}
 		
 		internal class GenericTypeInstanceResolver: CopyDomVisitor<IType>
 		{
@@ -629,16 +691,41 @@ namespace MonoDevelop.Projects.Dom
 				typeTable.Add (name, type);
 			}
 			
-			public override IDomVisitable Visit (IReturnType type, IType typeToInstantiate)
+			IType currentType = null;
+			public override INode Visit (IType type, IType data)
+			{
+				if (currentType != null)
+					return null;
+				currentType = type;
+				return base.Visit (type, data);
+			}
+			
+			IReturnType LookupReturnType (string decoratedName, IReturnType type, IType typeToInstantiate) 
 			{
-				DomReturnType copyFrom = (DomReturnType) type;
-				
 				IReturnType res;
-				if (typeTable.TryGetValue (copyFrom.DecoratedFullName, out res)) {
-					if (type.ArrayDimensions == 0 && type.GenericArguments.Count == 0)
+				if (typeTable.TryGetValue (decoratedName, out res)) {
+					if (type.ArrayDimensions == 0 && type.PointerNestingLevel == 0) {
 						return res;
+					}
+					DomReturnType copy = (DomReturnType)base.Visit (res, typeToInstantiate);
+					copy.PointerNestingLevel = type.PointerNestingLevel;
+					copy.SetDimensions (type.GetDimensions ());
+					return copy;
 				}
-				return base.Visit (type, typeToInstantiate);
+				return null;
+			}
+			
+			public override INode Visit (IReturnType type, IType typeToInstantiate)
+			{
+				DomReturnType copyFrom = (DomReturnType) type; 
+				string decoratedName = copyFrom.DecoratedFullName;
+				IReturnType result = LookupReturnType (decoratedName, type, typeToInstantiate);
+				IType curType = currentType;
+				while (result == null && curType != null) {
+					result = LookupReturnType (curType.DecoratedFullName + "." + decoratedName, type, typeToInstantiate);
+					curType = curType.DeclaringType;
+				}
+				return result ?? base.Visit (type, typeToInstantiate);
 			}
 			
 			protected override DomType CreateInstance (IType type, IType typeToInstantiate)
@@ -649,7 +736,7 @@ namespace MonoDevelop.Projects.Dom
 					return base.CreateInstance (type, typeToInstantiate);
 			}
 		}
- 		
+		
 		/// <summary>
 		/// Returns a list of types that contains extension methods
 		/// </summary>
@@ -681,8 +768,9 @@ namespace MonoDevelop.Projects.Dom
 			List<IMethod> result = new List<IMethod> ();
 			foreach (IType staticType in accessibleExtensionTypes) {
 				foreach (IMethod method in staticType.Methods) {
-					if (method.Extends (this.SourceProjectDom, this)) {
-						result.Add (method);
+					IMethod extMethod = method.Extends (this.SourceProjectDom, this);
+					if (extMethod != null) {
+						result.Add (extMethod);
 					}
 				}
 			}
@@ -712,27 +800,73 @@ namespace MonoDevelop.Projects.Dom
 		public override int GetHashCode ()
 		{
 			IType a = this is InstantiatedType ? ((InstantiatedType)this).UninstantiatedType : this;
-			return a.ToString ().GetHashCode ();
+			return a.DecoratedFullName.GetHashCode ();
 		}
+		
+		public static IReturnType GetComponentType (ProjectDom dom, IReturnType returnType)
+		{
+			if (dom == null || returnType == null)
+				return null;
+			if (returnType.FullName == DomReturnType.String.FullName && returnType.ArrayDimensions == 0)
+				return DomReturnType.Char;
+			if (returnType.ArrayDimensions > 0)
+				return new DomReturnType (returnType.FullName);
+			
+			IType resolvedType = dom.GetType (returnType);
+			if (resolvedType != null) {
+				foreach (IType curType in dom.GetInheritanceTree (resolvedType)) {
+					foreach (IReturnType baseType in curType.BaseTypes) {
+						if (baseType.FullName == "System.Collections.Generic.IEnumerable" && baseType.GenericArguments.Count == 1)
+							return baseType.GenericArguments[0];
+					}
+					
+					foreach (IProperty property in curType.Properties) {
+						if (property.IsIndexer && !property.IsExplicitDeclaration)
+							return property.ReturnType;
+					}
+				}
+			}
 
+			if (returnType.GenericArguments.Count > 0)
+				return returnType.GenericArguments[0];
+
+			return null;
+		}
+		
+		public virtual IMember GetMemberAt (int line, int column)
+		{
+			IMember result = Members.FirstOrDefault (member => member.BodyRegion.Contains (line, column));
+			if (result is IType) {
+				IMember containingMember = ((IType)result).GetMemberAt (line, column);
+				if (containingMember != null)
+					return containingMember;
+			}
+			if (result == null)
+				result = Members.FirstOrDefault (member => member.Location.Line == line);
+			return result;
+		}
 		
 		public bool Equals (IType other)
 		{
-			IType a = this is InstantiatedType ? ((InstantiatedType)this).UninstantiatedType : this;
-			IType b = other is InstantiatedType ? ((InstantiatedType)other).UninstantiatedType : other;
-			return a.TypeParameters.Count == b.TypeParameters.Count && a.FullName == b.FullName;
+			IType         a = this is InstantiatedType ? ((InstantiatedType)this).UninstantiatedType : this;
+			int typeParamsA = this is InstantiatedType ? ((InstantiatedType)this).UninstantiatedType.TypeParameters.Count : a.TypeParameters.Count;
+			
+			IType         b = other is InstantiatedType ? ((InstantiatedType)other).UninstantiatedType : other;
+			int typeParamsB = other is InstantiatedType ? ((InstantiatedType)other).UninstantiatedType.TypeParameters.Count : other.TypeParameters.Count;
+			return typeParamsA == typeParamsB && a.FullName == b.FullName;
 		}
 	}
 	
 	internal sealed class Stock 
 	{
-		public static readonly string Error = "gtk-error";
+		public static readonly string Error = "gtk-dialog-error";
 		public static readonly string Class = "md-class";
 		public static readonly string Enum = "md-enum";
 		public static readonly string Event = "md-event";
 		public static readonly string Field = "md-field";
 		public static readonly string Interface = "md-interface";
 		public static readonly string Method = "md-method";
+		public static readonly string ExtensionMethod = "md-extensionmethod";
 		public static readonly string Property = "md-property";
 		public static readonly string Struct = "md-struct";
 		public static readonly string Delegate = "md-delegate";
@@ -745,6 +879,7 @@ namespace MonoDevelop.Projects.Dom
 		public static readonly string InternalField = "md-internal-field";
 		public static readonly string InternalInterface = "md-internal-interface";
 		public static readonly string InternalMethod = "md-internal-method";
+		public static readonly string InternalExtensionMethod = "md-internal-extensionmethod";
 		public static readonly string InternalProperty = "md-internal-property";
 		public static readonly string InternalStruct = "md-internal-struct";
 		
@@ -755,6 +890,7 @@ namespace MonoDevelop.Projects.Dom
 		public static readonly string PrivateField = "md-private-field";
 		public static readonly string PrivateInterface = "md-private-interface";
 		public static readonly string PrivateMethod = "md-private-method";
+		public static readonly string PrivateExtensionMethod = "md-private-extensionmethod";
 		public static readonly string PrivateProperty = "md-private-property";
 		public static readonly string PrivateStruct = "md-private-struct";
 		
@@ -765,6 +901,7 @@ namespace MonoDevelop.Projects.Dom
 		public static readonly string ProtectedField = "md-protected-field";
 		public static readonly string ProtectedInterface = "md-protected-interface";
 		public static readonly string ProtectedMethod = "md-protected-method";
+		public static readonly string ProtectedExtensionMethod = "md-protected-extensionmethod";
 		public static readonly string ProtectedProperty = "md-protected-property";
 		public static readonly string ProtectedStruct = "md-protected-struct";
 		
