@@ -51,7 +51,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 			get { return (DotNetProject) Item; }
 		}
 		
-		protected override BuildResult OnClean (IProgressMonitor monitor, string configuration)
+		protected override BuildResult OnClean (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			DotNetProject project = Project;
 			DotNetProjectConfiguration conf = (DotNetProjectConfiguration) project.GetConfiguration (configuration);
@@ -59,8 +59,9 @@ namespace MonoDevelop.Projects.Formats.MD1
 			// Delete the generated debug info
 			string file = project.GetOutputFileName (configuration);
 			if (file != null) {
-				if (File.Exists (file + ".mdb"))
-					FileService.DeleteFile (file + ".mdb");
+				string mdb = conf.TargetRuntime.GetAssemblyDebugInfoFile (file);
+				if (File.Exists (mdb))
+					FileService.DeleteFile (mdb);
 			}
 
 			List<string> cultures = new List<string> ();
@@ -96,7 +97,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 		}
 
 		
-		protected override BuildResult OnBuild (IProgressMonitor monitor, string configuration)
+		protected override BuildResult OnBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			DotNetProject project = Project;
 			
@@ -185,35 +186,21 @@ namespace MonoDevelop.Projects.Formats.MD1
 			}
 			buildData.Configuration = (DotNetProjectConfiguration) conf.Clone ();
 			buildData.Configuration.SetParentItem (project);
+			buildData.ConfigurationSelector = configuration;
 
 			return ProjectExtensionUtil.Compile (monitor, project, buildData, delegate {
 				ProjectItemCollection items = buildData.Items;
 				BuildResult res = BuildResources (buildData.Configuration, ref items, monitor);
 				if (res != null)
 					return res;
-				
-				List<string> supportAssemblies = new List<string> ();
-				CopySupportAssemblies (supportAssemblies, configuration);
 	
-				try {
-					res = project.LanguageBinding.Compile (items, buildData.Configuration, monitor);
-					if (refres != null) {
-						refres.Append (res);
-						return refres;
-					}
-					else
-						return res;
+				res = project.LanguageBinding.Compile (items, buildData.Configuration, buildData.ConfigurationSelector, monitor);
+				if (refres != null) {
+					refres.Append (res);
+					return refres;
 				}
-				finally {
-					// Delete support assemblies
-					foreach (string s in supportAssemblies) {
-						try {
-							FileService.DeleteFile (s);
-						} catch {
-							// Ignore
-						}
-					}
-				}		
+				else
+					return res;
 			});
 		}		
 
@@ -221,13 +208,8 @@ namespace MonoDevelop.Projects.Formats.MD1
 		// and sets @projectItems to a cloned collection minus such resource files.
 		private BuildResult BuildResources (DotNetProjectConfiguration configuration, ref ProjectItemCollection projectItems, IProgressMonitor monitor)
 		{
-			string resgen = "resgen";
-			if (System.Environment.Version.Major >= 2) {
-				switch (configuration.TargetFramework.ClrVersion) {
-					case ClrVersion.Net_2_0: resgen = "resgen2"; break;
-					case ClrVersion.Net_1_1: resgen = "resgen1"; break;
-				}
-			}
+			string resgen = configuration.TargetRuntime.GetToolPath (configuration.TargetFramework, "resgen");
+			
 			bool cloned = false;
 			Dictionary<string, string> resourcesByCulture = new Dictionary<string, string> ();
 			foreach (ProjectFile finfo in projectItems.GetAll<ProjectFile> ()) {
@@ -264,7 +246,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 			}
 
 			string al = configuration.TargetFramework.ClrVersion == ClrVersion.Net_2_0 ? "al2" : "al";
-			CompilerError err = GenerateSatelliteAssemblies (resourcesByCulture, configuration.OutputDirectory, al, Path.GetFileNameWithoutExtension (configuration.OutputAssembly), monitor);
+			CompilerError err = GenerateSatelliteAssemblies (resourcesByCulture, configuration.OutputDirectory, al, Path.GetFileName (configuration.OutputAssembly), monitor);
 			if (err != null) {
 				CompilerResults cr = new CompilerResults (new TempFileCollection ());
 				cr.Errors.Add (err);
@@ -275,46 +257,6 @@ namespace MonoDevelop.Projects.Formats.MD1
 			return null;
 		}
 		
-		void CopySupportAssemblies (List<string> files, string configuration)
-		{
-			foreach (ProjectReference projectReference in Project.References) {
-				if (projectReference.ReferenceType == ReferenceType.Project) {
-					// It is a project reference. If this project depends
-					// on other (non-gac) assemblies there may be a compilation problem because
-					// the compiler won't be able to indirectly find them.
-					// The solution is to copy them in the project directory, and delete
-					// them after compilation.
-					DotNetProject p = Project.ParentSolution.FindProjectByName (projectReference.Reference) as DotNetProject;
-					if (p == null)
-						continue;
-					
-					string tdir = Path.GetDirectoryName (p.GetOutputFileName (configuration));
-					CopySupportAssemblies (p, tdir, files, configuration);
-				}
-			}
-		}
-		
-		void CopySupportAssemblies (DotNetProject prj, string targetDir, List<string> files, string configuration)
-		{
-			foreach (ProjectReference pref in prj.References) {
-				if (pref.ReferenceType == ReferenceType.Gac)
-					continue;
-				foreach (string referenceFileName in pref.GetReferencedFileNames (configuration)) {
-					string asmName = Path.GetFileName (referenceFileName);
-					asmName = Path.Combine (targetDir, asmName);
-					if (!File.Exists (asmName)) {
-						File.Copy (referenceFileName, asmName);
-						files.Add (asmName);
-					}
-				}
-				if (pref.ReferenceType == ReferenceType.Project) {
-					DotNetProject sp = Project.ParentSolution.FindProjectByName (pref.Reference) as DotNetProject;
-					if (sp != null)
-						CopySupportAssemblies (sp, targetDir, files, configuration);
-				}
-			}
-		}
-		
 		CompilerError GetResourceId (ProjectFile finfo, ref string fname, string resgen, out string resourceId, IProgressMonitor monitor)
 		{
 			resourceId = finfo.ResourceId;
@@ -333,6 +275,12 @@ namespace MonoDevelop.Projects.Formats.MD1
 				fname = Path.ChangeExtension (fname, ".resources");
 				return null;
 			}
+			
+			if (resgen == null) {
+				string msg = GettextCatalog.GetString ("Unable to find 'resgen' tool.");
+				monitor.ReportError (msg, null);
+				return new CompilerError (fname, 0, 0, String.Empty, msg);
+			}
 
 			using (StringWriter sw = new StringWriter ()) {
 				LoggingService.LogDebug ("Compiling resources\n{0}$ {1} /compile {2}", Path.GetDirectoryName (fname), resgen, fname);
