@@ -34,8 +34,11 @@ using System.Reflection;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
+using MonoDevelop.Core.Assemblies;
 using MonoDevelop.Projects.Formats.MD1;
 using MonoDevelop.Projects.Extensions;
+using MonoDevelop.Core.Execution;
+using Mono.Addins;
 
 namespace MonoDevelop.Projects.Formats.MSBuild
 {
@@ -46,6 +49,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		IResourceHandler customResourceHandler;
 		List<string> subtypeGuids = new List<string> ();
 		const string Unspecified = null;
+		RemoteProjectBuilder projectBuilder;
 		
 		SolutionEntityItem EntityItem {
 			get { return (SolutionEntityItem) Item; }
@@ -75,15 +79,78 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		{
 			if (import != null && import.Trim().Length > 0)
 				this.targetImports.AddRange (import.Split (':'));
+			
+			Runtime.SystemAssemblyService.DefaultRuntimeChanged += OnDefaultRuntimeChanged;
+		}
+		
+		void OnDefaultRuntimeChanged (object o, EventArgs args)
+		{
+			// If the default runtime changes, the project builder for this project may change
+			// so it has to be created again.
+			if (projectBuilder != null) {
+				MSBuildProjectService.ReleaseProjectBuilder (projectBuilder);
+				projectBuilder = null;
+			}
+		}
+		
+		RemoteProjectBuilder ProjectBuilder {
+			get {
+				if (projectBuilder == null)
+					projectBuilder = MSBuildProjectService.GetProjectBuilder (((DotNetProject)Item).TargetRuntime);
+				return projectBuilder;
+			}
+		}
+		
+		public override void Dispose ()
+		{
+			base.Dispose ();
+			if (projectBuilder != null) {
+				MSBuildProjectService.ReleaseProjectBuilder (projectBuilder);
+				projectBuilder = null;
+			}
+			Runtime.SystemAssemblyService.DefaultRuntimeChanged -= OnDefaultRuntimeChanged;
 		}
 
-		public override BuildResult RunTarget (IProgressMonitor monitor, string target, string configuration)
+		
+		public override BuildResult RunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration)
 		{
-			if (Item is DotNetProject) {
-				MD1DotNetProjectHandler handler = new MD1DotNetProjectHandler ((DotNetProject)Item);
-				return handler.RunTarget (monitor, target, configuration);
-			} else
-				return null;
+			if (PropertyService.Get ("MonoDevelop.Ide.BuildWithMSBuild", false)) {
+				SolutionEntityItem item = Item as SolutionEntityItem;
+				if (item != null) {
+					TargetRuntime runtime = null;
+					TargetFramework fx;
+					if (item is DotNetProject) {
+						runtime = ((DotNetProject) item).TargetRuntime;
+						fx = ((DotNetProject) item).TargetFramework;
+					}
+					else {
+						runtime = Runtime.SystemAssemblyService.CurrentRuntime;
+						fx = Services.ProjectService.DefaultTargetFramework;
+					}
+					
+					SolutionItemConfiguration configObject = item.GetConfiguration (configuration);
+				
+					LogWriter logWriter = new LogWriter (monitor.Log);
+					MSBuildResult[] results = ProjectBuilder.RunTarget (item.FileName, target, configObject.Name, configObject.Platform, runtime.GetMSBuildBinPath (fx), logWriter);
+					System.Runtime.Remoting.RemotingServices.Disconnect (logWriter);
+					
+					BuildResult br = new BuildResult ();
+					foreach (MSBuildResult res in results) {
+						if (res.IsWarning)
+							br.AddWarning (res.File, res.Line, res.Column, res.Code, res.Message);
+						else
+							br.AddError (res.File, res.Line, res.Column, res.Code, res.Message);
+					}
+					return br;
+				}
+			}
+			else {
+				if (Item is DotNetProject) {
+					MD1DotNetProjectHandler handler = new MD1DotNetProjectHandler ((DotNetProject)Item);
+					return handler.RunTarget (monitor, target, configuration);
+				}
+			}
+			return null;
 		}
 		
 		public string GetDefaultResourceId (ProjectFile file)
@@ -126,29 +193,29 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if (projectTypeGuids != null) {
 				foreach (string guid in projectTypeGuids.Split (';')) {
 					string sguid = guid.Trim ();
-					if (sguid.Length > 0 && sguid != TypeGuid)
+					if (sguid.Length > 0 && string.Compare (sguid, TypeGuid, true) != 0)
 						subtypeGuids.Add (guid);
 				}
 			}
 			
-			Item = CreateSolutionItem (language, projectTypeGuids, itemType, itemClass);
-
-			Item.SetItemHandler (this);
-			MSBuildProjectService.SetId (Item, itemGuid);
-			
-			SolutionEntityItem it = (SolutionEntityItem) Item;
-			
-			it.FileName = fileName;
-			it.Name = System.IO.Path.GetFileNameWithoutExtension (fileName);
-			
 			try {
 				ProjectExtensionUtil.BeginLoadOperation ();
+				Item = CreateSolutionItem (language, projectTypeGuids, itemType, itemClass);
+	
+				Item.SetItemHandler (this);
+				MSBuildProjectService.SetId (Item, itemGuid);
+				
+				SolutionEntityItem it = (SolutionEntityItem) Item;
+				
+				it.FileName = fileName;
+				it.Name = System.IO.Path.GetFileNameWithoutExtension (fileName);
+			
 				Load (monitor, p);
+				return it;
+				
 			} finally {
 				ProjectExtensionUtil.EndLoadOperation ();
 			}
-			
-			return it;
 		}
 		
 		SolutionItem CreateSolutionItem (string language, string typeGuids, string itemType, Type itemClass)
@@ -163,13 +230,14 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					item = st.CreateInstance (language);
 					if (!string.IsNullOrEmpty (st.Import))
 						targetImports.AddRange (st.Import.Split (':'));
-				}
+				} else
+					throw new InvalidOperationException ("Unknown solution item type.");
 			}
 			if (item == null && itemClass != null)
 				item = (SolutionItem) Activator.CreateInstance (itemClass);
 			
 			if (item == null && !string.IsNullOrEmpty (language))
-				item = new DotNetProject (language);
+				item = new DotNetAssemblyProject (language);
 			
 			if (item == null) {
 				if (string.IsNullOrEmpty (itemType))
@@ -203,8 +271,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			
 			DotNetProject dotNetProject = Item as DotNetProject;
 			
-			string assemblyName = null;
-			string frameworkVersion = "2.0";
+			string frameworkVersion = null;
 			
 			// Read all items
 			
@@ -215,8 +282,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 			
 			if (dotNetProject != null) {
-				// Get the common assembly name
-				assemblyName = globalGroup.GetPropertyValue ("AssemblyName");
 				frameworkVersion = globalGroup.GetPropertyValue ("TargetFrameworkVersion");
 				if (frameworkVersion != null && frameworkVersion.StartsWith ("v"))
 					frameworkVersion = frameworkVersion.Substring (1);
@@ -234,6 +299,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			List<ConfigData> configData = GetConfigData (msproject);
 			List<ConfigData> readConfigData = new List<ConfigData> ();
 			
+			MSBuildPropertyGroup mergedToProjectProperties = ExtractMergedtoprojectProperties (ser, globalGroup, EntityItem.CreateConfiguration ("Dummy"));
+			configData.Insert (0, new ConfigData (Unspecified, Unspecified, mergedToProjectProperties));
+			
 			foreach (ConfigData cgrp in configData) {
 				readConfigData.Add (cgrp);
 
@@ -250,11 +318,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				MSBuildPropertyGroup grp = CreateMergedConfiguration (readConfigData, conf, platform);
 				SolutionItemConfiguration config = EntityItem.CreateConfiguration (conf);
 				
-				if (config is DotNetProjectConfiguration) {
-					// Clean the default assembly name
-					((DotNetProjectConfiguration)config).OutputAssembly = string.Empty;
-				}
-				
 				config.Platform = platform;
 				DataItem data = ReadPropertyGroupMetadata (ser, grp, config);
 				ser.Deserialize (config, data);
@@ -266,9 +329,6 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 						data = ReadPropertyGroupMetadata (ser, grp, dpc.CompilationParameters);
 						ser.Deserialize (dpc.CompilationParameters, data);
 					}
-					
-					if (!string.IsNullOrEmpty (assemblyName) && string.IsNullOrEmpty (dpc.OutputAssembly))
-						dpc.OutputAssembly = assemblyName;
 				}
 			}
 			
@@ -285,16 +345,43 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			ser.Deserialize (Item, globalData);
 
 			// Final initializations
-
-			string fx = Item.ExtendedProperties ["InternalTargetFrameworkVersion"] as string;
-			if (fx != null && dotNetProject != null) {
-				dotNetProject.TargetFramework = Runtime.SystemAssemblyService.GetTargetFramework (fx);
-				Item.ExtendedProperties.Remove ("InternalTargetFrameworkVersion");
+			
+			if (dotNetProject != null && string.IsNullOrEmpty (frameworkVersion)) {
+				string fx = Item.ExtendedProperties ["InternalTargetFrameworkVersion"] as string;
+				if (fx != null) {
+					dotNetProject.TargetFramework = Runtime.SystemAssemblyService.GetTargetFramework (fx);
+					Item.ExtendedProperties.Remove ("InternalTargetFrameworkVersion");
+				} else {
+					// If no framework is specified, it means the format is VS2005, so the default framework is 2.0.
+					dotNetProject.TargetFramework = Runtime.SystemAssemblyService.GetTargetFramework ("2.0");
+				}
 			}
 			
 			Item.NeedsReload = false;
 		}
+
+		MSBuildPropertyGroup ExtractMergedtoprojectProperties (MSBuildSerializer ser, MSBuildPropertyGroup pgroup, object ob)
+		{
+			XmlDocument doc = new XmlDocument ();
+			MSBuildPropertyGroup res = new MSBuildPropertyGroup (doc.CreateElement ("PropGroup"));
+			
+			foreach (string propName in GetMergeToProjectProperties (ser, ob)) {
+				MSBuildProperty bp = pgroup.GetProperty (propName);
+				if (bp != null)
+					res.SetPropertyValue (bp.Name, bp.Value);
+			}
+			return res;
+		}
 		
+		IEnumerable<string> GetMergeToProjectProperties (MSBuildSerializer ser, object ob)
+		{
+			ClassDataType dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (ob.GetType ());
+			foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, ob)) {
+				if (IsMergeToProjectProperty (prop))
+					yield return prop.Name;
+			}
+		}
+
 		ProjectItem ReadItem (MSBuildSerializer ser, MSBuildItem buildItem)
 		{
 			Project project = Item as Project;
@@ -311,13 +398,22 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				else if (buildItem.Name == "Reference" && dotNetProject != null) {
 					ProjectReference pref;
 					if (buildItem.HasMetadata ("HintPath")) {
-						string path = MSBuildProjectService.FromMSBuildPath (dotNetProject.ItemDirectory, buildItem.GetMetadata ("HintPath"));
-						if (File.Exists (path)) {
+						string hintPath = buildItem.GetMetadata ("HintPath");
+						string path;
+						if (!MSBuildProjectService.FromMSBuildPath (dotNetProject.ItemDirectory, hintPath, out path)) {
 							pref = new ProjectReference (ReferenceType.Assembly, path);
-							pref.LocalCopy = !buildItem.GetMetadataIsFalse ("Private");
+							pref.SetInvalid (GettextCatalog.GetString ("Invalid file path"));
+							pref.ExtendedProperties ["_OriginalMSBuildReferenceInclude"] = buildItem.Include;
+							pref.ExtendedProperties ["_OriginalMSBuildReferenceHintPath"] = hintPath;
+						} else if (File.Exists (path)) {
+							pref = new ProjectReference (ReferenceType.Assembly, path);
+							if (MSBuildProjectService.IsAbsoluteMSBuildPath (hintPath))
+								pref.ExtendedProperties ["_OriginalMSBuildReferenceIsAbsolute"] = true;
 						} else {
 							pref = new ProjectReference (ReferenceType.Gac, buildItem.Include);
+							pref.ExtendedProperties ["_OriginalMSBuildReferenceHintPath"] = hintPath;
 						}
+						pref.LocalCopy = !buildItem.GetMetadataIsFalse ("Private");
 					} else {
 						string asm = buildItem.Include;
 						// This is a workaround for a VS bug. Looks like it is writing this assembly incorrectly
@@ -325,6 +421,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 							asm = "System.Configuration";
 						else if (asm == "System.XML")
 							asm = "System.Xml";
+						else if (asm == "system")
+							asm = "System";
 						pref = new ProjectReference (ReferenceType.Gac, asm);
 					}
 					pref.Condition = buildItem.Condition;
@@ -333,11 +431,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					return pref;
 				}
 				else if (buildItem.Name == "ProjectReference" && dotNetProject != null) {
-					string name = buildItem.GetMetadata ("Name");
-					// The name of the project is the first word of the string (it may contain other stuff).
-					int i = name.IndexOf (' ');
-					if (i != -1)
-						name = name.Substring (0, i);
+					// Get the project name from the path, since the Name attribute may other stuff other than the name
+					string path = MSBuildProjectService.FromMSBuildPath (project.ItemDirectory, buildItem.Include);
+					string name = Path.GetFileNameWithoutExtension (path);
 					ProjectReference pref = new ProjectReference (ReferenceType.Project, name);
 					pref.LocalCopy = !buildItem.GetMetadataIsFalse ("Private");
 					pref.Condition = buildItem.Condition;
@@ -345,7 +441,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				}
 				else if (dt == null && !string.IsNullOrEmpty (buildItem.Include)) {
 					// Unknown item. Must be a file.
-					if (!UnsupportedItems.Contains (buildItem.Name))
+					if (!UnsupportedItems.Contains (buildItem.Name) && IsValidFile (buildItem.Include))
 						return ReadProjectFile (ser, project, buildItem, typeof(ProjectFile));
 				}
 			}
@@ -358,6 +454,18 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			
 			return null;
 		}
+
+		bool IsValidFile (string path)
+		{
+			// If it is an absolute uri, it's not a valid file
+			try {
+				return !Uri.IsWellFormedUriString (path, UriKind.Absolute);
+			} catch {
+				// Old mono versions may crash in IsWellFormedUriString if the path
+				// is not an uri.
+				return true;
+			}
+		}
 		
 		class ConfigData
 		{
@@ -509,18 +617,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 
 			WritePropertyGroupMetadata (globalGroup, ser.InternalItemProperties.ItemData, ser, Item, langParams);
 			
-			// Find a common assembly name for all configurations
-			
-			string assemblyName = null;
+			// Convert debug property
 			
 			foreach (SolutionItemConfiguration conf in eitem.Configurations) {
 				DotNetProjectConfiguration cp = conf as MonoDevelop.Projects.DotNetProjectConfiguration;
 				if (cp != null) {
-					if (assemblyName == null)
-						assemblyName = cp.OutputAssembly;
-					else if (assemblyName != cp.OutputAssembly)
-						assemblyName = string.Empty;
-					
 					if (newProject)
 						cp.ExtendedProperties ["ErrorReport"] = "prompt";
 					
@@ -534,61 +635,76 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				}
 			}
 			
-			if (!string.IsNullOrEmpty (assemblyName))
-				SetGroupProperty (globalGroup, "AssemblyName", assemblyName, false);
-			else
-				globalGroup.RemoveProperty ("AssemblyName");
-			
 			// Configurations
 
-			List<ConfigData> configData = GetConfigData (msproject);
-			
-			foreach (SolutionItemConfiguration conf in eitem.Configurations) {
-				bool newConf = false;
-				MSBuildPropertyGroup propGroup = FindPropertyGroup (configData, conf);
-				if (propGroup == null) {
-					propGroup = msproject.AddNewPropertyGroup (false);
-					propGroup.Condition = BuildConfigCondition (conf.Name, conf.Platform);
-					newConf = true;
-				}
+			if (eitem.Configurations.Count > 0) {
+				List<ConfigData> configData = GetConfigData (msproject);
+				Dictionary<string,string> mergeToProjectProperties = new Dictionary<string,string> ();
+				List<string> mergeToProjectPropertyNames = new List<string> (GetMergeToProjectProperties (ser, eitem.Configurations [0]));
+				List<string> mergeToProjectPropertyNamesCopy = new List<string> (mergeToProjectPropertyNames);
 				
-				DotNetProjectConfiguration netConfig = conf as DotNetProjectConfiguration;
-				
-				DataItem ditem = (DataItem) ser.Serialize (conf);
-				
-				if (netConfig != null) {
-					// Remove all compilation parameters properties from the data item, since we are going to write them again.
-					ClassDataType dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (netConfig.CompilationParameters.GetType ());
-					foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, netConfig.CompilationParameters)) {
-						DataNode n = ditem.ItemData [prop.Name];
-						if (n != null)
-							ditem.ItemData.Remove (n);
+				foreach (SolutionItemConfiguration conf in eitem.Configurations) {
+					bool newConf = false;
+					MSBuildPropertyGroup propGroup = FindPropertyGroup (configData, conf);
+					if (propGroup == null) {
+						propGroup = msproject.AddNewPropertyGroup (false);
+						propGroup.Condition = BuildConfigCondition (conf.Name, conf.Platform);
+						ConfigData cd = new ConfigData (conf.Name, conf.Platform, propGroup);
+						configData.Add (cd);
+						newConf = true;
 					}
-					DataItem ditemComp = (DataItem) ser.Serialize (netConfig.CompilationParameters);
-					ditem.ItemData.AddRange (ditemComp.ItemData);
+					
+					DotNetProjectConfiguration netConfig = conf as DotNetProjectConfiguration;
+					
+					DataItem ditem = (DataItem) ser.Serialize (conf);
+					
+					if (netConfig != null) {
+						// Remove all compilation parameters properties from the data item, since we are going to write them again.
+						ClassDataType dt = (ClassDataType) ser.DataContext.GetConfigurationDataType (netConfig.CompilationParameters.GetType ());
+						foreach (ItemProperty prop in dt.GetProperties (ser.SerializationContext, netConfig.CompilationParameters)) {
+							DataNode n = ditem.ItemData [prop.Name];
+							if (n != null)
+								ditem.ItemData.Remove (n);
+						}
+						DataItem ditemComp = (DataItem) ser.Serialize (netConfig.CompilationParameters);
+						ditem.ItemData.AddRange (ditemComp.ItemData);
+					}
+	
+					if (newConf)
+						ditem.ItemData.Sort (configOrder);
+					
+					WritePropertyGroupMetadata (propGroup, ditem.ItemData, ser, conf, netConfig != null ? netConfig.CompilationParameters : null);
+					
+					UnmergeBaseConfiguration (configData, propGroup, conf.Name, conf.Platform);
+					
+					CollectMergetoprojectProperties (propGroup, mergeToProjectPropertyNames, mergeToProjectProperties);
 				}
-
-				if (newConf)
-					ditem.ItemData.Sort (configOrder);
 				
-				WritePropertyGroupMetadata (propGroup, ditem.ItemData, ser, conf, netConfig != null ? netConfig.CompilationParameters : null);
-				
-				if (!string.IsNullOrEmpty (assemblyName))
-					propGroup.RemoveProperty ("AssemblyName");
-
-				UnmergeBaseConfiguration (configData, propGroup, conf.Name, conf.Platform);
+				// Move properties with common values from configurations to the main
+				// property group
+				foreach (KeyValuePair<string,string> prop in mergeToProjectProperties)
+					globalGroup.SetPropertyValue (prop.Key, prop.Value);
+				foreach (string prop in mergeToProjectPropertyNamesCopy) {
+					if (!mergeToProjectProperties.ContainsKey (prop))
+						globalGroup.RemoveProperty (prop);
+				}
+				foreach (SolutionItemConfiguration conf in eitem.Configurations) {
+					MSBuildPropertyGroup propGroup = FindPropertyGroup (configData, conf);
+					foreach (string mp in mergeToProjectProperties.Keys)
+						propGroup.RemoveProperty (mp);
+				}
 			}
 			
 			// Remove old items
-			ArrayList list = new ArrayList ();
-			foreach (object ob in msproject.GetAllItems ())
-				list.Add (ob);
+			Dictionary<string,MSBuildItem> oldItems = new Dictionary<string, MSBuildItem> ();
+			foreach (MSBuildItem item in msproject.GetAllItems ())
+				oldItems [item.Name + "<" + item.Include] = item;
 			
 			// Add the new items
 			foreach (object ob in ((SolutionEntityItem)Item).Items)
-				SaveItem (monitor, ser, msproject, ob);
+				SaveItem (monitor, ser, msproject, ob, oldItems);
 			
-			foreach (MSBuildItem buildItem in list)
+			foreach (MSBuildItem buildItem in oldItems.Values)
 				msproject.RemoveItem (buildItem);
 		
 			if (dotNetProject != null) {
@@ -601,9 +717,17 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				
 			}
 
-			if (newProject) {
-				foreach (string import in TargetImports)
-					msproject.AddNewImport (import, null);
+			// Impdate the imports section
+			
+			List<string> currentImports = msproject.Imports;
+			List<string> newImports = GetImports ();
+			foreach (string imp in newImports) {
+				if (!currentImports.Contains (imp))
+					msproject.AddNewImport (imp, null);
+			}
+			foreach (string imp in currentImports) {
+				if (!newImports.Contains (imp))
+					msproject.RemoveImport (imp);
 			}
 			
 			DataItem extendedData = ser.ExternalItemProperties;
@@ -617,24 +741,54 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			
 			msproject.Save (eitem.FileName);
 		}
+
+		void CollectMergetoprojectProperties (MSBuildPropertyGroup pgroup, List<String> propertyNames, Dictionary<string,string> mergeToProjectProperties)
+		{
+			// This method checks every property in pgroup which has the MergeToProject flag.
+			// If the value of this property is the same as the one stored in mergeToProjectProperties
+			// it means that the property can be merged to the main project property group (so far).
+			
+			foreach (string pname in new List<String> (propertyNames)) {
+				MSBuildProperty prop = pgroup.GetProperty (pname);
+				
+				string mvalue;
+				if (!mergeToProjectProperties.TryGetValue (pname, out mvalue)) {
+					if (prop != null) {
+						// This is the first time the value is checked. Just assign it.
+						mergeToProjectProperties.Add (pname, prop.Value);
+						continue;
+					}
+					// If there is no value, it can't be merged
+				}
+				else if (prop != null && prop.Value == mvalue)
+					// Same value. It can be merged.
+					continue;
+
+				// The property can't be merged because different configurations have different
+				// values for it. Remove it from the list.
+				propertyNames.Remove (pname);
+				mergeToProjectProperties.Remove (pname);
+			}
+		}
+
 		
-		void SaveItem (MonoDevelop.Core.IProgressMonitor monitor, MSBuildSerializer ser, MSBuildProject msproject, object ob)
+		void SaveItem (MonoDevelop.Core.IProgressMonitor monitor, MSBuildSerializer ser, MSBuildProject msproject, object ob, Dictionary<string,MSBuildItem> oldItems)
 		{
 			if (ob is ProjectReference) {
-				SaveReference (monitor, ser, msproject, (ProjectReference) ob);
+				SaveReference (monitor, ser, msproject, (ProjectReference) ob, oldItems);
 			}
 			else if (ob is ProjectFile) {
-				SaveProjectFile (ser, msproject, (ProjectFile) ob);
+				SaveProjectFile (ser, msproject, (ProjectFile) ob, oldItems);
 			}
 			else {
 				DataType dt = ser.DataContext.GetConfigurationDataType (ob.GetType ());
 				string itemName = dt.Name;
 				MSBuildItem buildItem = msproject.AddNewItem (itemName, "");
-				WriteBuildItemMetadata (ser, buildItem, ob);
+				WriteBuildItemMetadata (ser, buildItem, ob, oldItems);
 			}
 		}
 		
-		void SaveProjectFile (MSBuildSerializer ser, MSBuildProject msproject, ProjectFile file)
+		void SaveProjectFile (MSBuildSerializer ser, MSBuildProject msproject, ProjectFile file, Dictionary<string,MSBuildItem> oldItems)
 		{
 			string itemName = (file.Subtype == Subtype.Directory)? "Folder" : file.BuildAction;
 
@@ -646,8 +800,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			if ((file.Subtype == Subtype.Directory) && path[path.Length-1] != '\\')
 				path = path + "\\";
 			
-			MSBuildItem buildItem = msproject.AddNewItem (itemName, path);
-			WriteBuildItemMetadata (ser, buildItem, file);
+			MSBuildItem buildItem = AddOrGetBuildItem (msproject, oldItems, itemName, path);
+			WriteBuildItemMetadata (ser, buildItem, file, oldItems);
 			
 			if (!string.IsNullOrEmpty (file.DependsOn))
 				buildItem.SetMetadata ("DependentUpon", MSBuildProjectService.ToMSBuildPath (Path.GetDirectoryName (file.FilePath), file.DependsOn));
@@ -680,28 +834,44 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			}
 		}
 		
-		void SaveReference (MonoDevelop.Core.IProgressMonitor monitor, MSBuildSerializer ser, MSBuildProject msproject, ProjectReference pref)
+		void SaveReference (MonoDevelop.Core.IProgressMonitor monitor, MSBuildSerializer ser, MSBuildProject msproject, ProjectReference pref, Dictionary<string,MSBuildItem> oldItems)
 		{
 			MSBuildItem buildItem;
 			if (pref.ReferenceType == ReferenceType.Assembly) {
 				string asm = null;
-				if (File.Exists (pref.Reference)) {
-					try {
-						asm = AssemblyName.GetAssemblyName (pref.Reference).FullName;
-					} catch (Exception ex) {
-						string msg = string.Format ("Could not get full name for assembly '{0}'.", pref.Reference);
-						monitor.ReportWarning (msg);
-						LoggingService.LogError (msg, ex);
+				string hintPath = null;
+				if (pref.ExtendedProperties.Contains ("_OriginalMSBuildReferenceInclude")) {
+					asm = (string) pref.ExtendedProperties ["_OriginalMSBuildReferenceInclude"];
+					hintPath = (string) pref.ExtendedProperties ["_OriginalMSBuildReferenceHintPath"];
+				}
+				else {
+					if (File.Exists (pref.Reference)) {
+						try {
+							asm = AssemblyName.GetAssemblyName (pref.Reference).FullName;
+						} catch (Exception ex) {
+							string msg = string.Format ("Could not get full name for assembly '{0}'.", pref.Reference);
+							monitor.ReportWarning (msg);
+							LoggingService.LogError (msg, ex);
+						}
 					}
+					string basePath = Item.ItemDirectory;
+					if (pref.ExtendedProperties.Contains ("_OriginalMSBuildReferenceIsAbsolute"))
+						basePath = null;
+					hintPath = MSBuildProjectService.ToMSBuildPath (basePath, pref.Reference);
 				}
 				if (asm == null)
 					asm = Path.GetFileNameWithoutExtension (pref.Reference);
-				buildItem = msproject.AddNewItem ("Reference", asm);
+				
+				buildItem = AddOrGetBuildItem (msproject, oldItems, "Reference", asm);
 				if (!pref.SpecificVersion)
 					buildItem.SetMetadata ("SpecificVersion", "False");
-				buildItem.SetMetadata ("HintPath", MSBuildProjectService.ToMSBuildPath (Item.ItemDirectory, pref.Reference));
+				else
+					buildItem.UnsetMetadata ("SpecificVersion");
+				buildItem.SetMetadata ("HintPath", hintPath);
 				if (!pref.LocalCopy)
 					buildItem.SetMetadata ("Private", "False");
+				else
+					buildItem.UnsetMetadata ("Private");
 			}
 			else if (pref.ReferenceType == ReferenceType.Gac) {
 				string include = pref.StoredReference;
@@ -711,25 +881,37 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					if (i != -1)
 						include = include.Substring (0, i).Trim ();
 				}
-				buildItem = msproject.AddNewItem ("Reference", include);
+				buildItem = AddOrGetBuildItem (msproject, oldItems, "Reference", include);
 				if (!pref.SpecificVersion)
 					buildItem.SetMetadata ("SpecificVersion", "False");
+				else
+					buildItem.UnsetMetadata ("SpecificVersion");
 				IList supportedFrameworks = TargetFormat.FrameworkVersions;
 				if (pkg != null && pkg.IsFrameworkPackage && supportedFrameworks.Contains (pkg.TargetFramework) && pkg.TargetFramework != "2.0" && supportedFrameworks.Count > 1) {
 					TargetFramework fx = Runtime.SystemAssemblyService.GetTargetFramework (pkg.TargetFramework);
 					buildItem.SetMetadata ("RequiredTargetFramework", fx.Id);
-				}
+				} else
+					buildItem.UnsetMetadata ("RequiredTargetFramework");
+				string hintPath = (string) pref.ExtendedProperties ["_OriginalMSBuildReferenceHintPath"];
+				if (hintPath != null)
+					buildItem.SetMetadata ("HintPath", hintPath);
+				else
+					buildItem.UnsetMetadata ("HintPath");
 			}
 			else if (pref.ReferenceType == ReferenceType.Project) {
 				Project refProj = Item.ParentSolution.FindProjectByName (pref.Reference);
 				if (refProj != null) {
-					buildItem = msproject.AddNewItem ("ProjectReference", MSBuildProjectService.ToMSBuildPath (Item.ItemDirectory, refProj.FileName));
+					buildItem = AddOrGetBuildItem (msproject, oldItems, "ProjectReference", MSBuildProjectService.ToMSBuildPath (Item.ItemDirectory, refProj.FileName));
 					MSBuildProjectHandler handler = refProj.ItemHandler as MSBuildProjectHandler;
 					if (handler != null)
 						buildItem.SetMetadata ("Project", handler.Item.ItemId);
+					else
+						buildItem.UnsetMetadata ("Project");
 					buildItem.SetMetadata ("Name", refProj.Name);
 					if (!pref.LocalCopy)
 						buildItem.SetMetadata ("Private", "False");
+					else
+						buildItem.UnsetMetadata ("Private");
 				} else {
 					monitor.ReportWarning (GettextCatalog.GetString ("Reference to unknown project '{0}' ignored.", pref.Reference));
 					return;
@@ -738,11 +920,25 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			else {
 				// Custom
 				DataType dt = ser.DataContext.GetConfigurationDataType (pref.GetType ());
-				buildItem = msproject.AddNewItem (dt.Name, pref.Reference);
+				buildItem = AddOrGetBuildItem (msproject, oldItems, dt.Name, pref.Reference);
 			}
-			WriteBuildItemMetadata (ser, buildItem, pref);
+			WriteBuildItemMetadata (ser, buildItem, pref, oldItems);
 			buildItem.Condition = pref.Condition;
 		}
+		
+		List<string> GetImports ()
+		{
+			List<string> imps = new List<string> ();
+			if (targetImports != null) {
+				foreach (string imp in targetImports)
+					imps.Add (imp);
+			}
+			foreach (IMSBuildImportProvider ip in AddinManager.GetExtensionObjects ("/MonoDevelop/ProjectModel/MSBuildImportProviders")) {
+				foreach (string imp in ip.GetRequiredImports (EntityItem))
+					imps.Add (imp);
+			}
+			return imps;
+		}
 
 		void UnmergeBaseConfiguration (List<ConfigData> configData, MSBuildPropertyGroup propGroup, string conf, string platform)
 		{
@@ -766,10 +962,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		{
 			DataItem ditem = new DataItem ();
 			foreach (ItemProperty prop in ser.GetProperties (dataItem)) {
-				if (prop.Name == "Include")
+				string name = ToMsbuildItemName (prop.Name);
+				if (name == "Include")
 					ditem.ItemData.Add (new DataValue ("Include", buildItem.Include));
-				else if (buildItem.HasMetadata (prop.Name)) {
-					string data = buildItem.GetMetadata (prop.Name);
+				else if (buildItem.HasMetadata (name)) {
+					string data = buildItem.GetMetadata (name);
 					ditem.ItemData.Add (GetDataNode (prop, data));
 				}
 			}
@@ -777,11 +974,16 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			ser.Deserialize (dataItem, ditem);
 		}
 		
-		void WriteBuildItemMetadata (DataSerializer ser, MSBuildItem buildItem, object dataItem)
+		void WriteBuildItemMetadata (DataSerializer ser, MSBuildItem buildItem, object dataItem, Dictionary<string,MSBuildItem> oldItems)
 		{
+			var notWrittenProps = new HashSet<string> ();
+			foreach (ItemProperty prop in ser.GetProperties (dataItem))
+				notWrittenProps.Add (prop.Name);
+			
 			DataItem ditem = (DataItem) ser.Serialize (dataItem, dataItem.GetType ());
 			if (ditem.HasItemData) {
 				foreach (DataNode node in ditem.ItemData) {
+					notWrittenProps.Remove (node.Name);
 					if (node.Name == "Include" && node is DataValue)
 						buildItem.Include = ((DataValue) node).Value;
 					else {
@@ -790,6 +992,19 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					}
 				}
 			}
+			foreach (string prop in notWrittenProps)
+				buildItem.UnsetMetadata (prop);
+		}
+		
+		MSBuildItem AddOrGetBuildItem (MSBuildProject msproject, Dictionary<string,MSBuildItem> oldItems, string name, string include)
+		{
+			MSBuildItem buildItem;
+			string key = name + "<" + include;
+			if (oldItems.TryGetValue (key, out buildItem)) {
+				oldItems.Remove (key);
+				return buildItem;
+			} else
+				return msproject.AddNewItem (name, include);
 		}
 		
 		DataItem ReadPropertyGroupMetadata (DataSerializer ser, MSBuildPropertyGroup propGroup, object dataItem)
@@ -834,6 +1049,11 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			foreach (string prop in notWrittenProps)
 				propGroup.RemoveProperty (prop);
 		}
+		
+		string ToMsbuildItemName (string name)
+		{
+			return name.Replace ('.', '-');
+		}
 
 		void ConvertToMsbuildFormat (DataNode node)
 		{
@@ -968,8 +1188,21 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			return " '$(Configuration)|$(Platform)' == '" + config + "|" + platform + "' ";
 		}
 
+		bool IsMergeToProjectProperty (ItemProperty prop)
+		{
+			foreach (object at in prop.CustomAttributes) {
+				if (at is MergeToProjectAttribute)
+					return true;
+			}
+			return false;
+		}
+
 		string FindClosestSupportedVersion (string version)
 		{
+			TargetFramework sfx = Runtime.SystemAssemblyService.GetTargetFramework (version);
+			if (!string.IsNullOrEmpty (sfx.SubsetOfFramework) && ((IList)TargetFormat.FrameworkVersions).Contains (sfx.SubsetOfFramework))
+				return sfx.SubsetOfFramework;
+			
 			foreach (string supv in TargetFormat.FrameworkVersions) {
 				TargetFramework fx = Runtime.SystemAssemblyService.GetTargetFramework (supv);
 				if (fx.IsCompatibleWithFramework (version))
@@ -1031,7 +1264,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 		// Items generated by VS but which MD is not using and should be ignored
 		
 		internal static readonly IList<string> UnsupportedItems = new string[] {
-			"BootstrapperFile", "AppDesigner"
+			"BootstrapperFile", "AppDesigner", "WebReferences", "WebReferenceUrl"
 		};
 	}
 	
@@ -1058,9 +1291,9 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 					return false;
 			}
 			if (instance is SolutionEntityItem) {
-				if (prop.Name == "Policies" || prop.Name == "BaseDirectory")
+				if (prop.IsExtendedProperty (typeof(SolutionEntityItem)))
 					return true;
-				return prop.IsExtendedProperty (typeof(SolutionEntityItem));
+				return prop.Name != "name" && prop.Name != "Configurations";
 			}
 			if (instance is ProjectFile)
 				return prop.IsExtendedProperty (typeof(ProjectFile));
