@@ -96,17 +96,23 @@ namespace MonoDevelop.Projects
 			return cmd;
 		}
 		
-		public void Execute (IProgressMonitor monitor, IWorkspaceObject entry, string configuration)
+		public void Execute (IProgressMonitor monitor, IWorkspaceObject entry, ConfigurationSelector configuration)
 		{
 			Execute (monitor, entry, null, configuration);
 		}
 		
-		public bool CanExecute (ExecutionContext context, string configuration)
+		public bool CanExecute (ExecutionContext context, ConfigurationSelector configuration)
 		{
-			return context == null || context.ExecutionHandler.CanExecute (command);
+			string exe = command;
+			int i = exe.IndexOf (' ');
+			if (i != -1)
+				exe = command.Substring (0, i);
+			
+			ExecutionCommand cmd = Runtime.ProcessService.CreateCommand (exe);
+			return context == null || context.ExecutionHandler.CanExecute (cmd);
 		}
 		
-		public void Execute (IProgressMonitor monitor, IWorkspaceObject entry, ExecutionContext context, string configuration)
+		public void Execute (IProgressMonitor monitor, IWorkspaceObject entry, ExecutionContext context, ConfigurationSelector configuration)
 		{
 			string [,] customtags = null;
 			
@@ -149,6 +155,9 @@ namespace MonoDevelop.Projects
 				};
 			}
 			
+			if (string.IsNullOrEmpty (command))
+				return;
+			
 			int i = command.IndexOf (' ');
 			string exe;
 			string args = string.Empty;
@@ -162,9 +171,9 @@ namespace MonoDevelop.Projects
 			
 			monitor.Log.WriteLine (GettextCatalog.GetString ("Executing: {0} {1}", exe, args));
 
-			string dir = (string.IsNullOrEmpty (workingdir) ? entry.BaseDirectory : StringParserService.Parse (workingdir, customtags));
-			
-			string localPath = Path.Combine (entry.BaseDirectory, exe);
+			FilePath dir = (string.IsNullOrEmpty (workingdir) ? entry.BaseDirectory : (FilePath) StringParserService.Parse (workingdir, customtags));
+
+			FilePath localPath = entry.BaseDirectory.Combine (exe);
 			if (File.Exists (localPath))
 				exe = localPath;
 			
@@ -177,7 +186,13 @@ namespace MonoDevelop.Projects
 				else
 					console = context.ConsoleFactory.CreateConsole (!pauseExternalConsole);
 
-				oper = context.ExecutionHandler.Execute (exe, args, dir, null, console);
+				ExecutionCommand cmd = Runtime.ProcessService.CreateCommand (exe);
+				ProcessExecutionCommand pcmd = cmd as ProcessExecutionCommand;
+				if (pcmd != null) {
+					pcmd.Arguments = args;
+					pcmd.WorkingDirectory = dir;
+				}
+				oper = context.ExecutionHandler.Execute (cmd, console);
 			}
 			else {
 				if (externalConsole) {
