@@ -1,22 +1,30 @@
-//  ProjectReference.cs
-//
-//  This file was derived from a file from #Develop. 
-//
-//  Copyright (C) 2001-2007 Mike Krüger <mkrueger@novell.com>
 // 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-// 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
+// ProjectReference.cs
 //  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Author:
+//       Lluis Sanchez Gual <lluis@novell.com>
+// 
+// Copyright (c) 2009 Novell, Inc (http://www.novell.com)
+// 
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+// 
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
+
+
 
 using System;
 using System.Collections;
@@ -28,6 +36,7 @@ using MonoDevelop.Core;
 using System.ComponentModel;
 using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
+using MonoDevelop.Core.Assemblies;
 
 namespace MonoDevelop.Projects
 {
@@ -61,6 +70,9 @@ namespace MonoDevelop.Projects
 		bool notFound;
 		string package;
 		SystemPackage cachedPackage;
+		string customError;
+		
+		public event EventHandler StatusChanged;
 		
 		[ItemProperty ("Package", DefaultValue="")]
 		internal string packageName {
@@ -110,6 +122,19 @@ namespace MonoDevelop.Projects
 			UpdateGacReference ();
 		}
 		
+		protected void InitCustomReference (string reference)
+		{
+			Reference = reference;
+			referenceType = ReferenceType.Custom;
+		}
+		
+		public static ProjectReference RenameReference (ProjectReference pref, string newReference)
+		{
+			ProjectReference newRef = (ProjectReference) pref.MemberwiseClone ();
+			newRef.reference = newReference;
+			return newRef;
+		}
+		
 		public Project OwnerProject {
 			get { return ownerProject; }
 		}
@@ -152,9 +177,6 @@ namespace MonoDevelop.Projects
 			get {
 				return loadedReference;
 			}
-			set {
-				loadedReference = value;
-			}
 		}
 
 		public bool SpecificVersion {
@@ -162,7 +184,10 @@ namespace MonoDevelop.Projects
 				return specificVersion;
 			}
 			set {
-				specificVersion = value;
+				if (specificVersion != value) {
+					specificVersion = value;
+					OnStatusChanged ();
+				}
 			}
 		}
 
@@ -173,11 +198,17 @@ namespace MonoDevelop.Projects
 		// Returns the validation error message, or an empty string if everything is ok
 		public virtual string ValidationErrorMessage {
 			get {
+				if (customError != null)
+					return customError;
 				if (ReferenceType == ReferenceType.Gac) {
-					if (notFound)
-						return GettextCatalog.GetString ("Assembly not found");
 					if (!IsExactVersion && SpecificVersion)
 						return GettextCatalog.GetString ("Specified version not found: expected {0}, found {1}", GetVersionNum (StoredReference), GetVersionNum (Reference));
+					if (notFound) {
+						if (ownerProject != null)
+							return GettextCatalog.GetString ("Assembly not available for {0} (in {1})", TargetFramework.Name, TargetRuntime.DisplayName);
+						else
+							return GettextCatalog.GetString ("Assembly not found");
+					}
 				} else if (ReferenceType == ReferenceType.Project) {
 					if (ownerProject != null && ownerProject.ParentSolution != null) {
 						DotNetProject p = ownerProject.ParentSolution.FindProjectByName (reference) as DotNetProject;
@@ -188,17 +219,24 @@ namespace MonoDevelop.Projects
 					}
 				} else if (ReferenceType == ReferenceType.Assembly) {
 					if (!File.Exists (reference))
-						GettextCatalog.GetString ("File not found");
+						return GettextCatalog.GetString ("File not found");
 				}
 				return string.Empty;
 			}
 		}
 		
+		public void SetInvalid (string message)
+		{
+			customError = message;
+			OnStatusChanged ();
+		}
+		
 		public bool IsExactVersion {
 			get {
 				if (ReferenceType == ReferenceType.Gac) {
-					if (StoredReference != Reference && !Reference.StartsWith (StoredReference + ","))
-						return false;
+					string r1 = MonoDevelop.Core.Assemblies.AssemblyContext.NormalizeAsmName (StoredReference);
+					string r2 = MonoDevelop.Core.Assemblies.AssemblyContext.NormalizeAsmName (Reference);
+					return r1 == r2;
 				}
 				return true;
 			}
@@ -223,14 +261,14 @@ namespace MonoDevelop.Projects
 		/// Returns the file name to an assembly, regardless of what 
 		/// type the assembly is.
 		/// </summary>
-		string GetReferencedFileName (string configuration)
+		string GetReferencedFileName (ConfigurationSelector configuration)
 		{
 			switch (ReferenceType) {
 				case ReferenceType.Assembly:
 					return reference;
 				
 				case ReferenceType.Gac:
-					string file = Runtime.SystemAssemblyService.GetAssemblyLocation (Reference, package);
+					string file = AssemblyContext.GetAssemblyLocation (Reference, package, ownerProject != null? ownerProject.TargetFramework : null);
 					return file == null ? reference : file;
 				case ReferenceType.Project:
 					if (ownerProject != null) {
@@ -247,28 +285,50 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		public virtual string[] GetReferencedFileNames (string configuration)
+		public virtual string[] GetReferencedFileNames (ConfigurationSelector configuration)
 		{
 			string s = GetReferencedFileName (configuration);
+	/*		if (referenceType == ReferenceType.Gac) {
+				List<string> result = new List<string> ();
+				result.Add (s);
+				AddRequiredPackages (result, Package);
+				return result.ToArray ();
+			}*/
+			
 			if (s != null)
 				return new string[] { s };
-			else
-				return new string [0];
+			return new string [0];
 		}
+		/*
+		void AddRequiredPackages (List<string> result, SystemPackage fromPackage)
+		{
+			if (fromPackage == null || string.IsNullOrEmpty (fromPackage.Requires))
+				return;
+			foreach (string requiredPackageName in fromPackage.Requires.Split (' ')) {
+				SystemPackage package = AssemblyContext.GetPackage (requiredPackageName);
+				if (package == null)
+					continue;
+				foreach (SystemAssembly assembly in package.Assemblies) {
+					if (assembly == null)
+						continue;
+					string location = AssemblyContext.GetAssemblyLocation (assembly.FullName, ownerProject != null ? ownerProject.TargetFramework : null);
+					result.Add (location);
+				}
+				AddRequiredPackages (result, package);
+			}
+		}*/
 		
 		void UpdateGacReference ()
 		{
-			if (referenceType == ReferenceType.Gac) {
+			if (referenceType == ReferenceType.Gac && ownerProject != null) {
 				notFound = false;
-				string cref = Runtime.SystemAssemblyService.FindInstalledAssembly (reference, package);
-				if (ownerProject != null) {
-					if (cref == null)
-						cref = reference;
-					cref = Runtime.SystemAssemblyService.GetAssemblyNameForVersion (cref, package, ownerProject.TargetFramework);
-					notFound = (cref == null);
-				}
+				string cref = AssemblyContext.FindInstalledAssembly (reference, package, ownerProject.TargetFramework);
+				if (cref == null)
+					cref = reference;
+				cref = AssemblyContext.GetAssemblyNameForVersion (cref, package, ownerProject.TargetFramework);
+				notFound = (cref == null);
 				if (cref != null && cref != reference) {
-					SystemAssembly asm = Runtime.SystemAssemblyService.GetAssemblyFromFullName (cref, package);
+					SystemAssembly asm = AssemblyContext.GetAssemblyFromFullName (cref, package, ownerProject.TargetFramework);
 					bool isFrameworkAssembly = asm != null && asm.Package.IsFrameworkPackage;
 					if (loadedReference == null && !isFrameworkAssembly) {
 						loadedReference = reference;
@@ -276,6 +336,34 @@ namespace MonoDevelop.Projects
 					reference = cref;
 				}
 				cachedPackage = null;
+				OnStatusChanged ();
+			}
+		}
+		
+		IAssemblyContext AssemblyContext {
+			get {
+				if (ownerProject != null)
+					return ownerProject.AssemblyContext;
+				else
+					return Runtime.SystemAssemblyService.DefaultAssemblyContext;
+			}
+		}
+		
+		TargetRuntime TargetRuntime {
+			get {
+				if (ownerProject != null)
+					return ownerProject.TargetRuntime;
+				else
+					return Runtime.SystemAssemblyService.DefaultRuntime;
+			}
+		}
+		
+		TargetFramework TargetFramework {
+			get {
+				if (ownerProject != null)
+					return ownerProject.TargetFramework;
+				else
+					return null;
 			}
 		}
 		
@@ -284,17 +372,21 @@ namespace MonoDevelop.Projects
 				if (referenceType == ReferenceType.Gac) {
 					if (cachedPackage != null)
 						return cachedPackage;
+					
 					if (package != null)
-						return Runtime.SystemAssemblyService.GetPackage (package);
+						return AssemblyContext.GetPackage (package);
 
 					// No package is specified, get any of the registered assemblies, giving priority to gaced assemblies
 					// (because non-gac assemblies should have a package name set)
 					SystemAssembly best = null;
-					foreach (SystemAssembly asm in Runtime.SystemAssemblyService.GetAssembliesFromFullName (reference)) {
-						if (asm.Package.IsGacPackage) {
+					foreach (SystemAssembly asm in AssemblyContext.GetAssembliesFromFullName (reference)) {
+						//highest priority to framework packages
+						if (ownerProject != null && asm.Package.IsFrameworkPackage && asm.Package.TargetFramework == ownerProject.TargetFramework.Id) {
+							return cachedPackage = asm.Package;
+						}
+						if (asm.Package.IsGacPackage)
 							best = asm;
-							break;
-						} else if (best == null)
+						else if (best == null)
 							best = asm;
 					}
 					if (best != null)
@@ -311,7 +403,7 @@ namespace MonoDevelop.Projects
 				reference = loadedReference;
 				loadedReference = null;
 				UpdateGacReference ();
-			} else if (notFound)
+			} else
 				UpdateGacReference ();
 		}
 		
@@ -325,12 +417,23 @@ namespace MonoDevelop.Projects
 			ProjectReference oref = other as ProjectReference;
 			if (oref == null) return false;
 			
-			return reference == oref.reference && referenceType == oref.referenceType && package == oref.package;
+			return StoredReference == oref.StoredReference && referenceType == oref.referenceType && package == oref.package;
 		}
 		
 		public override int GetHashCode ()
 		{
-			return reference.GetHashCode ();
+			return StoredReference.GetHashCode ();
+		}
+		
+		internal void NotifyStatusChanged ()
+		{
+			OnStatusChanged ();
+		}
+		
+		protected virtual void OnStatusChanged ()
+		{
+			if (StatusChanged != null)
+				StatusChanged (this, EventArgs.Empty);
 		}
 	}
 	
