@@ -38,6 +38,7 @@ namespace MonoDevelop.Projects
 {
 	public class Solution: WorkspaceItem, IConfigurationTarget
 	{
+		internal object MemoryProbe = Counters.SolutionsInMemory.CreateMemoryProbe ();
 		SolutionFolder rootFolder;
 		string defaultConfiguration;
 		
@@ -55,14 +56,15 @@ namespace MonoDevelop.Projects
 		[ItemProperty ("description", DefaultValue = "")]
 		string description;
 		
-		[ItemProperty ("version", DefaultValue = "")]
-		string version;
+		[ItemProperty ("version", DefaultValue = "0.1")]
+		string version = "0.1";
 		
 		[ProjectPathItemProperty ("outputpath")]
 		string outputdir     = null;
 		
 		public Solution ()
 		{
+			Counters.SolutionsLoaded++;
 			configurations = new SolutionConfigurationCollection (this);
 		}
 		
@@ -187,7 +189,17 @@ namespace MonoDevelop.Projects
 				multiStartupItems = value;
 			}
 		}
-
+		
+		public override PropertyBag UserProperties {
+			get {
+				return RootFolder.UserProperties;
+			}
+		}
+		
+		public override void LoadUserProperties (PropertyBag properties)
+		{
+			RootFolder.LoadUserProperties (properties);
+		}
 		
 		public void CreateDefaultConfigurations ()
 		{
@@ -212,7 +224,7 @@ namespace MonoDevelop.Projects
 		{
 			SolutionConfiguration conf = new SolutionConfiguration (name);
 			foreach (SolutionEntityItem item in Items) {
-				if (createConfigForItems && item.GetConfiguration (name) == null) {
+				if (createConfigForItems && item.GetConfiguration (new ItemConfigurationSelector (name)) == null) {
 					SolutionItemConfiguration newc = item.CreateConfiguration (name);
 					if (item.DefaultConfiguration != null)
 						newc.CopyFrom (item.DefaultConfiguration);
@@ -232,6 +244,11 @@ namespace MonoDevelop.Projects
 			return configs.AsReadOnly ();
 		}
 		
+		public virtual SolutionConfiguration GetConfiguration (ConfigurationSelector configuration)
+		{
+			return (SolutionConfiguration) configuration.GetConfiguration (this) ?? DefaultConfiguration;
+		}
+		
 		public SolutionItem GetSolutionItem (string itemId)
 		{
 			foreach (SolutionItem item in Items)
@@ -255,17 +272,17 @@ namespace MonoDevelop.Projects
 			return RootFolder.GetAllItems<T> ();
 		}
 		
-		public ReadOnlyCollection<T> GetAllSolutionItemsWithTopologicalSort<T> (string configuration) where T: SolutionItem
+		public ReadOnlyCollection<T> GetAllSolutionItemsWithTopologicalSort<T> (ConfigurationSelector configuration) where T: SolutionItem
 		{
 			return RootFolder.GetAllItemsWithTopologicalSort<T> (configuration);
 		}
 		
-		public ReadOnlyCollection<Project> GetAllProjectsWithTopologicalSort (string configuration)
+		public ReadOnlyCollection<Project> GetAllProjectsWithTopologicalSort (ConfigurationSelector configuration)
 		{
 			return RootFolder.GetAllProjectsWithTopologicalSort (configuration);
 		}
-		
-		public override Project GetProjectContainingFile (string fileName) 
+
+		public override Project GetProjectContainingFile (FilePath fileName) 
 		{
 			return RootFolder.GetProjectContainingFile (fileName);
 		}
@@ -307,7 +324,7 @@ namespace MonoDevelop.Projects
 		
 		string DefaultOutputDirectory {
 			get {
-				return (BaseDirectory != null) ? Path.Combine (BaseDirectory, Path.Combine ("build", "bin")) : null;
+				return (BaseDirectory != FilePath.Null) ? BaseDirectory.Combine ("build", "bin") : FilePath.Null;
 			}
 		}
 		
@@ -344,6 +361,14 @@ namespace MonoDevelop.Projects
 			}
 		}
 
+		public ConfigurationSelector DefaultConfigurationSelector {
+			get {
+				if (defaultConfiguration == null && configurations.Count > 0)
+					DefaultConfigurationId = configurations [0].Id;
+				return new SolutionConfigurationSelector (DefaultConfigurationId);
+			}
+		}
+
 		IItemConfigurationCollection IConfigurationTarget.Configurations {
 			get {
 				return Configurations;
@@ -372,6 +397,10 @@ namespace MonoDevelop.Projects
 			}
 			set {
 				version = value;
+				foreach (SolutionEntityItem item in GetAllSolutionItems<SolutionEntityItem> ()) {
+					if (item.SyncVersionWithSolution)
+						item.Version = value;
+				}
 			}
 		}
 		
@@ -379,6 +408,7 @@ namespace MonoDevelop.Projects
 		{
 			base.Dispose ();
 			RootFolder.Dispose ();
+			Counters.SolutionsLoaded--;
 		}
 
 
@@ -392,14 +422,14 @@ namespace MonoDevelop.Projects
 			}
 		}	 
 		
-		protected override BuildResult OnBuild (IProgressMonitor monitor, string configuration)
+		protected override BuildResult OnBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			return RootFolder.Build (monitor, configuration);
 		}
 		
-		protected override void OnClean (IProgressMonitor monitor, string configuration)
+		protected override void OnClean (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
-			SolutionConfiguration config = Configurations [configuration];
+			SolutionConfiguration config = GetConfiguration (configuration);
 			if (config == null)
 				return;
 			
@@ -411,7 +441,7 @@ namespace MonoDevelop.Projects
 			}
 		}
 
-		protected internal override bool OnGetCanExecute(ExecutionContext context, string configuration)
+		protected internal override bool OnGetCanExecute(ExecutionContext context, ConfigurationSelector configuration)
 		{
 			if (SingleStartup) {
 				if (StartupItem == null)
@@ -426,7 +456,7 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		protected internal override void OnExecute (IProgressMonitor monitor, ExecutionContext context, string configuration)
+		protected internal override void OnExecute (IProgressMonitor monitor, ExecutionContext context, ConfigurationSelector configuration)
 		{
 			if (SingleStartup) {
 				if (StartupItem == null) {
@@ -457,6 +487,7 @@ namespace MonoDevelop.Projects
 							LoggingService.LogError ("Project execution failed", ex);
 						}
 					});
+					t.Name = "Project execution";
 					t.IsBackground = true;
 					t.Start ();
 				}
@@ -467,12 +498,12 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		protected internal override bool OnGetNeedsBuilding (string configuration)
+		protected internal override bool OnGetNeedsBuilding (ConfigurationSelector configuration)
 		{
 			return RootFolder.NeedsBuilding (configuration);
 		}
 		
-		protected internal override void OnSetNeedsBuilding (bool val, string configuration)
+		protected internal override void OnSetNeedsBuilding (bool val, ConfigurationSelector configuration)
 		{
 			RootFolder.SetNeedsBuilding (val, configuration);
 		}
@@ -489,18 +520,16 @@ namespace MonoDevelop.Projects
 			foreach (SolutionItem item in GetAllSolutionItems<SolutionItem> ())
 				ConvertToSolutionFormat (item, convertChildren);
 		}
-		
-		public override List<string> GetItemFiles (bool includeReferencedFiles)
+
+		public override List<FilePath> GetItemFiles (bool includeReferencedFiles)
 		{
-			List<string> files = base.GetItemFiles (includeReferencedFiles);
+			List<FilePath> files = base.GetItemFiles (includeReferencedFiles);
 			if (includeReferencedFiles) {
 				foreach (SolutionEntityItem item in GetAllSolutionItems<SolutionEntityItem> ())
 					files.AddRange (item.GetItemFiles (true));
 			}
 			return files;
 		}
-
-
 		
 #region Notifications from children
 		
@@ -661,6 +690,12 @@ namespace MonoDevelop.Projects
 				EntrySaved (this, args);
 		}
 		
+		internal protected virtual void OnItemReloadRequired (SolutionItemEventArgs args)
+		{
+			if (ItemReloadRequired != null)
+				ItemReloadRequired (this, args);
+		}
+		
 #endregion
 		
 		public event EventHandler StartupItemChanged;
@@ -677,5 +712,6 @@ namespace MonoDevelop.Projects
 		public event ProjectReferenceEventHandler ReferenceRemovedFromProject;
 		public event SolutionItemEventHandler EntryModified;
 		public event SolutionItemEventHandler EntrySaved;
+		public event EventHandler<SolutionItemEventArgs> ItemReloadRequired;
 	}
 }
