@@ -28,12 +28,14 @@
 
 
 using System;
+using System.Threading;
 using MonoDevelop.Core;
+using MonoDevelop.Core.Logging;
 using MonoDevelop.Core.Execution;
 using System.IO;
 using System.Runtime.Remoting;
 using System.Runtime.Remoting.Channels;
-using System.Runtime.Remoting.Channels.Tcp;
+using System.Runtime.Remoting.Channels.Ipc;
 using System.Runtime.Serialization.Formatters.Binary;
 using System.Runtime.Remoting.Lifetime;
 using System.Reflection;
@@ -58,8 +60,8 @@ public class MonoDevelopProcessHost
 			} else
 				input = Console.In;
 			
-			string channel = input.ReadLine ();
 			string sref = input.ReadLine ();
+			string pidToWatch = input.ReadLine ();
 			
 			if (tmpFile != null) {
 				try {
@@ -69,30 +71,17 @@ public class MonoDevelopProcessHost
 				}
 			}
 			
-			string unixPath = null;
-			if (channel == "unix") {
-				unixPath = System.IO.Path.GetTempFileName ();
-				Hashtable props = new Hashtable ();
-				props ["path"] = unixPath;
-				props ["name"] = "__internal_unix";
-				ChannelServices.RegisterChannel (new UnixChannel (props, null, null), false);
-			} else {
-				Hashtable props = new Hashtable ();
-				props ["port"] = 0;
-				props ["name"] = "__internal_tcp";
-				BinaryClientFormatterSinkProvider clientProvider = new BinaryClientFormatterSinkProvider();
-				BinaryServerFormatterSinkProvider serverProvider = new BinaryServerFormatterSinkProvider();
-
-				serverProvider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
-
-				ChannelServices.RegisterChannel (new TcpChannel (props, clientProvider, serverProvider), false);
-			}
+			WatchParentProcess (int.Parse (pidToWatch));
+			
+			string unixPath = RegisterRemotingChannel ();
 			
 			byte[] data = Convert.FromBase64String (sref);
 			MemoryStream ms = new MemoryStream (data);
 			BinaryFormatter bf = new BinaryFormatter ();
 			IProcessHostController pc = (IProcessHostController) bf.Deserialize (ms);
 			
+			LoggingService.AddLogger (new LocalLogger (pc.GetLogger (), args[0]));
+			
 			ProcessHost rp = new ProcessHost (pc);
 			pc.RegisterHost (rp);
 			try {
@@ -114,6 +103,66 @@ public class MonoDevelopProcessHost
 		
 		return 0;
 	}
+		
+	static string RegisterRemotingChannel ()
+	{
+		IDictionary dict = new Hashtable ();
+		BinaryClientFormatterSinkProvider clientProvider = new BinaryClientFormatterSinkProvider();
+		BinaryServerFormatterSinkProvider serverProvider = new BinaryServerFormatterSinkProvider();
+		string unixRemotingFile = Path.GetTempFileName ();
+		dict ["portName"] = Path.GetFileName (unixRemotingFile);
+		serverProvider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
+		ChannelServices.RegisterChannel (new IpcChannel (dict, clientProvider, serverProvider), false);
+		return unixRemotingFile;
+	}
+	
+	static void WatchParentProcess (int pid)
+	{
+		Thread t = new Thread (delegate () {
+			while (true) {
+				try {
+					System.Diagnostics.Process.GetProcessById (pid);
+				} catch {
+					Environment.Exit (1);
+				}
+				Thread.Sleep (1000);
+			}
+		});
+		t.Name = "Parent process watcher";
+		t.IsBackground = true;
+		t.Start ();
+	}
+}
+
+class LocalLogger: ILogger
+{
+	ILogger wrapped;
+	string id;
+	
+	public LocalLogger (ILogger wrapped, string id)
+	{
+		this.wrapped = wrapped;
+		this.id = id;
+	}
+	
+	#region ILogger implementation
+	public void Log (LogLevel level, string message)
+	{
+		wrapped.Log (level, "[" + id + "] " + message);
+	}
+	
+	public EnabledLoggingLevel EnabledLevel {
+		get {
+			return EnabledLoggingLevel.All;
+		}
+	}
+	
+	public string Name {
+		get {
+			return "Local Logger";
+		}
+	}
+	#endregion
 }
 
 public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
@@ -128,11 +177,9 @@ public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
 		lease.Register (this);
 	}
 	
-	public RemoteProcessObject CreateInstance (Type type)
+	public IDisposable CreateInstance (Type type)
 	{
-		RemoteProcessObject proc = (RemoteProcessObject) Activator.CreateInstance (type);
-		proc.Attach (controller);
-		return proc;
+		return (IDisposable) Activator.CreateInstance (type);
 	}
 	
 	public void LoadAddins (string[] addinIds)
@@ -142,7 +189,7 @@ public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
 			AddinManager.LoadAddin (null, ad);
 	}
 	
-	public RemoteProcessObject CreateInstance (string fullTypeName)
+	public IDisposable CreateInstance (string fullTypeName)
 	{
 		try {
 			Type t = Type.GetType (fullTypeName);
@@ -153,13 +200,19 @@ public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
 		}
 	}
 	
-	public RemoteProcessObject CreateInstance (string assemblyPath, string typeName)
+	public IDisposable CreateInstance (string assemblyPath, string typeName)
 	{
 		Assembly asm = Assembly.LoadFrom (assemblyPath);
 		Type t = asm.GetType (typeName);
 		if (t == null) throw new InvalidOperationException ("Type not found: " + typeName);
 		return CreateInstance (t);
 	}
+	
+	public void DisposeObject (IDisposable obj)
+	{
+		obj.Dispose ();
+	}
+
 		
 	public TimeSpan Renewal (ILease lease)
 	{
