@@ -28,30 +28,46 @@
 
 using System;
 using Mono.TextEditor;
+using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 
 namespace MonoDevelop.CSharpBinding.Tests
 {
-	public class TestViewContent : AbstractViewContent, IEditableTextBuffer
+	public class TestViewContent : AbstractViewContent, IEditableTextBuffer, Mono.TextEditor.ITextEditorDataProvider
 	{
+		TextEditorData data;
+		
 		public override Gtk.Widget Control {
 			get {
 				return null;
 			}
 		}
 		
+
+		public TestViewContent ()
+		{
+			document = new Mono.TextEditor.Document ();
+			data = new TextEditorData (document);
+			Name = "";
+		}
+		
 		public override void Load(string fileName)
 		{
 		}
 		
-		public string Name { 
+		public FilePath Name { 
+			get;
+			set;
+		}
+		
+		public int LineCount {
 			get {
-				return "";
+				return document.LineCount;
 			}
 		}
 		
-		Mono.TextEditor.Document document = new Mono.TextEditor.Document ();
+		Mono.TextEditor.Document document;
 		public string Text {
 			get {
 				return document.Text;
@@ -100,22 +116,37 @@ namespace MonoDevelop.CSharpBinding.Tests
 		}
 		
 		public string SelectedText { get { return ""; } set { } }
-		int cursorPosition = 0;
+		
 		public int CursorPosition {
 			get {
-				return cursorPosition;
+				return data.Caret.Offset;
 			}
 			set {
-				cursorPosition = value;
+				data.Caret.Offset = value;
 			}
 		}
 
-		public int SelectionStartPosition { get { return 0; } }
-		public int SelectionEndPosition { get { return 0; } }
+		public int SelectionStartPosition { 
+			get {
+				if (!data.IsSomethingSelected)
+					return data.Caret.Offset;
+				return data.SelectionRange.Offset;
+			}
+		}
+		
+		public int SelectionEndPosition { 
+			get {
+				if (!data.IsSomethingSelected)
+					return data.Caret.Offset;
+				return data.SelectionRange.EndOffset;
+			}
+		}
 		
 		public void Select (int startPosition, int endPosition)
 		{
+			data.SelectionRange = new Segment (startPosition, endPosition - startPosition);
 		}
+		
 		public void ShowPosition (int position)
 		{
 		}
@@ -134,6 +165,9 @@ namespace MonoDevelop.CSharpBinding.Tests
 		public void SetCaretTo (int line, int column)
 		{
 		}
+		public void SetCaretTo (int line, int column, bool highlightCaretLine)
+		{
+		}
 		
 		public void Undo()
 		{
@@ -149,6 +183,10 @@ namespace MonoDevelop.CSharpBinding.Tests
 		{
 		}
 		
+		public TextEditorData GetTextEditorData ()
+		{
+			return data;
+		}
 		public event EventHandler CaretPositionSet;
 		public event EventHandler<TextChangedEventArgs> TextChanged;
 	}
