@@ -86,14 +86,14 @@ namespace MonoDevelop.Projects
 			Assert.AreEqual (1, countSolutionItemAdded);
 			Assert.AreEqual (0, sol.Items.Count);
 			
-			DotNetProject project = new DotNetProject ("C#");
+			DotNetAssemblyProject project = new DotNetAssemblyProject ("C#");
 			project.Name = "project1";
 			sol.RootFolder.Items.Add (project);
 			
 			Assert.AreEqual (2, countSolutionItemAdded);
 			Assert.AreEqual (1, sol.Items.Count);
 			
-			DotNetProject project2 = new DotNetProject ("C#");
+			DotNetAssemblyProject project2 = new DotNetAssemblyProject ("C#");
 			project2.Name = "project2";
 			folder.Items.Add (project2);
 			
@@ -163,27 +163,27 @@ namespace MonoDevelop.Projects
 			
 			sol.Name = "test1";
 			Assert.AreEqual ("test1", sol.Name);
-			Assert.AreEqual ("test1.sln", sol.FileName);
+			Assert.AreEqual ("test1.sln", (string) sol.FileName);
 			Assert.AreEqual (1, nameChanges);
 			
 			sol.Name = "test2";
 			Assert.AreEqual ("test2", sol.Name);
-			Assert.AreEqual ("test2.sln", sol.FileName);
+			Assert.AreEqual ("test2.sln", (string) sol.FileName);
 			Assert.AreEqual (2, nameChanges);
 			
 			sol.FileName = "/tmp/test3.sln";
 			Assert.AreEqual ("test3", sol.Name);
-			Assert.AreEqual ("/tmp/test3.sln", sol.FileName);
+			Assert.AreEqual ("/tmp/test3.sln", (string) sol.FileName);
 			Assert.AreEqual (3, nameChanges);
 			
 			sol.Name = "test4";
 			Assert.AreEqual ("test4", sol.Name);
-			Assert.AreEqual ("/tmp/test4.sln", sol.FileName);
+			Assert.AreEqual ("/tmp/test4.sln", (string) sol.FileName);
 			Assert.AreEqual (4, nameChanges);
 			
 			sol.ConvertToFormat (Util.FileFormatMD1, true);
 			Assert.AreEqual ("test4", sol.Name);
-			Assert.AreEqual ("/tmp/test4.mds", sol.FileName);
+			Assert.AreEqual ("/tmp/test4.mds", (string) sol.FileName);
 			Assert.AreEqual (4, nameChanges);
 		}
 		
@@ -192,34 +192,34 @@ namespace MonoDevelop.Projects
 		{
 			int nameChanges = 0;
 			
-			DotNetProject prj = new DotNetProject ("C#");
+			DotNetAssemblyProject prj = new DotNetAssemblyProject ("C#");
 			prj.NameChanged += delegate {
 				nameChanges++;
 			};
 			
 			prj.Name = "test1";
 			Assert.AreEqual ("test1", prj.Name);
-			Assert.AreEqual ("test1.csproj", prj.FileName);
+			Assert.AreEqual ("test1.csproj", (string) prj.FileName);
 			Assert.AreEqual (1, nameChanges);
 			
 			prj.Name = "test2";
 			Assert.AreEqual ("test2", prj.Name);
-			Assert.AreEqual ("test2.csproj", prj.FileName);
+			Assert.AreEqual ("test2.csproj", (string) prj.FileName);
 			Assert.AreEqual (2, nameChanges);
 			
 			prj.FileName = "/tmp/test3.csproj";
 			Assert.AreEqual ("test3", prj.Name);
-			Assert.AreEqual ("/tmp/test3.csproj", prj.FileName);
+			Assert.AreEqual ("/tmp/test3.csproj", (string) prj.FileName);
 			Assert.AreEqual (3, nameChanges);
 			
 			prj.Name = "test4";
 			Assert.AreEqual ("test4", prj.Name);
-			Assert.AreEqual ("/tmp/test4.csproj", prj.FileName);
+			Assert.AreEqual ("/tmp/test4.csproj", (string) prj.FileName);
 			Assert.AreEqual (4, nameChanges);
 			
 			prj.FileFormat = Util.FileFormatMD1;
 			Assert.AreEqual ("test4", prj.Name);
-			Assert.AreEqual ("/tmp/test4.mdp", prj.FileName);
+			Assert.AreEqual ("/tmp/test4.mdp", (string) prj.FileName);
 			Assert.AreEqual (4, nameChanges);
 			Assert.AreEqual ("MD1", prj.FileFormat.Id);
 			
@@ -227,7 +227,7 @@ namespace MonoDevelop.Projects
 			Solution sol = new Solution ();
 			sol.RootFolder.Items.Add (prj);
 			Assert.AreEqual ("test4", prj.Name);
-			Assert.AreEqual ("/tmp/test4.csproj", prj.FileName);
+			Assert.AreEqual ("/tmp/test4.csproj", (string) prj.FileName);
 			Assert.AreEqual (4, nameChanges);
 			Assert.AreEqual ("MSBuild05", prj.FileFormat.Id);
 
@@ -235,7 +235,7 @@ namespace MonoDevelop.Projects
 			sol.RootFolder.Items.Remove (prj);
 			Assert.AreEqual ("MSBuild05", prj.FileFormat.Id);
 			Assert.AreEqual ("test4", prj.Name);
-			Assert.AreEqual ("/tmp/test4.csproj", prj.FileName);
+			Assert.AreEqual ("/tmp/test4.csproj", (string) prj.FileName);
 			Assert.AreEqual (4, nameChanges);
 		}
 		
@@ -312,7 +312,7 @@ namespace MonoDevelop.Projects
 		{
 			Solution sol = TestProjectsChecks.CreateConsoleSolution ("item-files");
 			
-			List<string> files = sol.GetItemFiles (false);
+			List<FilePath> files = sol.GetItemFiles (false);
 			Assert.AreEqual (1, files.Count);
 			Assert.AreEqual (sol.FileName, files [0]);
 			
@@ -348,19 +348,21 @@ namespace MonoDevelop.Projects
 			DotNetProject lib2 = (DotNetProject) sol.FindProjectByName ("library2");
 			Assert.IsNotNull (lib2);
 			
-			Assert.IsTrue (p.NeedsBuilding ("Debug"));
-			Assert.IsTrue (lib1.NeedsBuilding ("Debug"));
-			Assert.IsTrue (lib2.NeedsBuilding ("Debug"));
+			SolutionConfigurationSelector config = (SolutionConfigurationSelector) "Debug";
+			
+			Assert.IsTrue (p.NeedsBuilding (config));
+			Assert.IsTrue (lib1.NeedsBuilding (config));
+			Assert.IsTrue (lib2.NeedsBuilding (config));
 			
 			// Build the project and the references
 			
-			BuildResult res = p.Build (Util.GetMonitor (), "Debug", true);
+			BuildResult res = p.Build (Util.GetMonitor (), config, true);
 			Assert.AreEqual (0, res.ErrorCount);
 			Assert.AreEqual (0, res.WarningCount);
 			Assert.AreEqual (3, res.BuildCount);
-			Assert.IsFalse (p.NeedsBuilding ("Debug"));
-			Assert.IsFalse (lib1.NeedsBuilding ("Debug"));
-			Assert.IsFalse (lib2.NeedsBuilding ("Debug"));
+			Assert.IsFalse (p.NeedsBuilding (config));
+			Assert.IsFalse (lib1.NeedsBuilding (config));
+			Assert.IsFalse (lib2.NeedsBuilding (config));
 			
 			Assert.IsTrue (File.Exists (Util.Combine (p.BaseDirectory, "bin", "Debug", "console-with-libs-mdp.exe")));
 			Assert.IsTrue (File.Exists (Util.Combine (p.BaseDirectory, "bin", "Debug", "console-with-libs-mdp.exe.mdb")));
@@ -374,17 +376,17 @@ namespace MonoDevelop.Projects
 			p.SetNeedsBuilding (true);
 			lib1.SetNeedsBuilding (true);
 			lib2.SetNeedsBuilding (true);
-			Assert.IsTrue (p.NeedsBuilding ("Debug"));
-			Assert.IsTrue (lib1.NeedsBuilding ("Debug"));
-			Assert.IsTrue (lib2.NeedsBuilding ("Debug"));
+			Assert.IsTrue (p.NeedsBuilding (config));
+			Assert.IsTrue (lib1.NeedsBuilding (config));
+			Assert.IsTrue (lib2.NeedsBuilding (config));
 			
-			res = p.Build (Util.GetMonitor (), "Debug", false);
+			res = p.Build (Util.GetMonitor (), config, false);
 			Assert.AreEqual (0, res.ErrorCount);
 			Assert.AreEqual (0, res.WarningCount);
 			Assert.AreEqual (1, res.BuildCount);
-			Assert.IsTrue (p.NeedsBuilding ("Debug"));   // True because references require building
-			Assert.IsTrue (lib1.NeedsBuilding ("Debug"));
-			Assert.IsTrue (lib2.NeedsBuilding ("Debug"));
+			Assert.IsTrue (p.NeedsBuilding (config));   // True because references require building
+			Assert.IsTrue (lib1.NeedsBuilding (config));
+			Assert.IsTrue (lib2.NeedsBuilding (config));
 		}
 		
 		[Test()]
@@ -458,7 +460,7 @@ namespace MonoDevelop.Projects
 			
 			// Build the solution folder
 			
-			res = folder.Build (Util.GetMonitor (), "Debug");
+			res = folder.Build (Util.GetMonitor (), (SolutionConfigurationSelector) "Debug");
 			Assert.AreEqual (0, res.ErrorCount);
 			Assert.AreEqual (0, res.WarningCount);
 			Assert.AreEqual (1, res.BuildCount);
@@ -472,7 +474,7 @@ namespace MonoDevelop.Projects
 			
 			// Clean the solution folder
 			
-			folder.Clean (Util.GetMonitor (), "Debug");
+			folder.Clean (Util.GetMonitor (), (SolutionConfigurationSelector) "Debug");
 			Assert.IsFalse (File.Exists (Util.Combine (lib2.BaseDirectory, "bin", "Debug", "library2.dll")));
 			Assert.IsFalse (File.Exists (Util.Combine (lib2.BaseDirectory, "bin", "Debug", "library2.dll.mdb")));
 		}
@@ -512,7 +514,7 @@ namespace MonoDevelop.Projects
 			
 			// Add new project
 			
-			Project newp = new DotNetProject ("C#");
+			Project newp = new DotNetAssemblyProject ("C#");
 			Assert.AreEqual ("MSBuild05", newp.FileFormat.Id);
 			
 			sol.RootFolder.Items.Add (newp);
@@ -570,9 +572,10 @@ namespace MonoDevelop.Projects
 			CheckProjectBuildClean (lib4, "Release");
 		}
 		
-		void CheckSolutionBuildClean (Solution sol, string config)
+		void CheckSolutionBuildClean (Solution sol, string configuration)
 		{
-			string tag = "CheckSolutionBuildClean config=" + config;
+			SolutionConfigurationSelector config = (SolutionConfigurationSelector) configuration;
+			string tag = "CheckSolutionBuildClean config=" + configuration;
 			DotNetProject lib1 = (DotNetProject) sol.FindProjectByName ("Lib1");
 			DotNetProject lib2 = (DotNetProject) sol.FindProjectByName ("Lib2");
 			DotNetProject lib3 = (DotNetProject) sol.FindProjectByName ("Lib3");
@@ -600,9 +603,10 @@ namespace MonoDevelop.Projects
 			Assert.IsFalse (File.Exists (lib4.GetOutputFileName (config)), tag);
 		}
 		
-		void CheckProjectReferencesBuildClean (Solution sol, string config)
+		void CheckProjectReferencesBuildClean (Solution sol, string configuration)
 		{
-			string tag = "CheckProjectReferencesBuildClean config=" + config;
+			SolutionConfigurationSelector config = (SolutionConfigurationSelector) configuration;
+			string tag = "CheckProjectReferencesBuildClean config=" + configuration;
 			DotNetProject lib1 = (DotNetProject) sol.FindProjectByName ("Lib1");
 			DotNetProject lib2 = (DotNetProject) sol.FindProjectByName ("Lib2");
 			DotNetProject lib3 = (DotNetProject) sol.FindProjectByName ("Lib3");
@@ -625,9 +629,10 @@ namespace MonoDevelop.Projects
 			sol.Clean (Util.GetMonitor (), config);
 		}
 		
-		void CheckProjectBuildClean (DotNetProject lib, string config)
+		void CheckProjectBuildClean (DotNetProject lib, string configuration)
 		{
-			string tag = "CheckProjectBuildClean lib=" + lib.Name + " config=" + config;
+			SolutionConfigurationSelector config = (SolutionConfigurationSelector) configuration;
+			string tag = "CheckProjectBuildClean lib=" + lib.Name + " config=" + configuration;
 			
 			Assert.IsFalse (File.Exists (lib.GetOutputFileName (config)), tag);
 			
@@ -643,7 +648,7 @@ namespace MonoDevelop.Projects
 		
 		string GetConfigFolderName (DotNetProject lib, string conf)
 		{
-			return Path.GetFileName (Path.GetDirectoryName (lib.GetOutputFileName (conf)));
+			return Path.GetFileName (Path.GetDirectoryName (lib.GetOutputFileName ((SolutionConfigurationSelector)conf)));
 		}
 	}
 }
